//=============================================================================
// VisuStella MZ - Battle System - OTB - Order Turn Battle
// VisuMZ_2_BattleSystemOTB.js
//=============================================================================

var Imported = Imported || {};
Imported.VisuMZ_2_BattleSystemOTB = true;

var VisuMZ = VisuMZ || {};
VisuMZ.BattleSystemOTB = VisuMZ.BattleSystemOTB || {};
VisuMZ.BattleSystemOTB.version = 1.04;

//=============================================================================
 /*:
 * @target MZ
 * @plugindesc [RPG Maker MZ] [Tier 2] [Version 1.04] [BattleSystemOTB]
 * @author VisuStella
 * @url http://www.yanfly.moe/wiki/Battle_System_-_OTB_VisuStella_MZ
 * @base VisuMZ_0_CoreEngine
 * @base VisuMZ_1_BattleCore
 * @orderAfter VisuMZ_1_BattleCore
 *
 * @help
 * ============================================================================
 * Introduction
 * ============================================================================
 *
 * This plugin changes the RPG Maker MZ battle system to "Order Turn Battle",
 * a turn-based battle system where actions are executed immediately and the
 * orders for both the current and next turn are not only visible, but also
 * malleable. New mechanics are introduced where the player can manipulate the
 * turn order of an action's user or action's target in various ways they want.
 * 
 * The two Turn Orders are displayed at the top of the top of the screen to
 * give the player a clear understanding of who's turn it will be when it
 * becomes time to act, making it easier and viable for the player to formulate
 * strategies and adapt to the situation in battle.
 * 
 * *NOTE* To use this battle system, you will need the updated version of
 * VisuStella's Core Engine. Go into its Plugin Parameters and change the
 * "Battle System" plugin parameter to "otb".
 *
 * Features include all (but not limited to) the following:
 * 
 * * Utilizes the balanced AGI nature of the Default Turn Battle system.
 * * Allows for actions to execute immediately upon selection.
 * * Two Turn Order Displays appear at the top of the screen, giving the player
 *   an idea of who's turn it will be and when, for both the current turn and
 *   the next turn.
 * * Skills and Items can have an "Instant Use" effect, which allows them to
 *   perform an action immediately without using up a turn.
 * * Skills and Items can manipulate the turn order of the action's user or the
 *   action's target(s). This can apply to either the current turn or the next
 *   turn, depending on the notetags and/or action effects used.
 * * The Turn Order Display will give a preview on how turn orders will change
 *   upon specific skills and/or items being used.
 *
 * ============================================================================
 * Requirements
 * ============================================================================
 *
 * This plugin is made for RPG Maker MZ. This will not work in other iterations
 * of RPG Maker.
 *
 * ------ Required Plugin List ------
 *
 * * VisuMZ_0_CoreEngine
 * * VisuMZ_1_BattleCore
 *
 * This plugin requires the above listed plugins to be installed inside your
 * game's Plugin Manager list in order to work. You cannot start your game with
 * this plugin enabled without the listed plugins.
 *
 * ------ Tier 2 ------
 *
 * This plugin is a Tier 2 plugin. Place it under other plugins of lower tier
 * value on your Plugin Manager list (ie: 0, 1, 2, 3, 4, 5). This is to ensure
 * that your plugins will have the best compatibility with the rest of the
 * VisuStella MZ library.
 *
 * ============================================================================
 * Major Changes
 * ============================================================================
 *
 * This plugin adds some new hard-coded features to RPG Maker MZ's functions.
 * The following is a list of them.
 *
 * ---
 * 
 * Turn Order Displays
 * 
 * The Two Turn Order Displays will capture the battle's current and next turn
 * orders determined by the BattleManager. This feature does not overwrite any
 * functions, but the Turn Order Displays may or may not conflict with any
 * existing HUD elements that are already positioned on the screen. If so, you
 * can choose to offset the Turn Order Display or move it to a different part
 * of the screen through the plugin parameters.
 * 
 * ---
 * 
 * Agility
 * 
 * Agility behaves slightly different from normal when it comes to the Order
 * Turn Battle system. Aside from the first turn in battle, agility will always
 * calculate the turn order for the "Next Turn" when conducted. This means that
 * any changes to agility values will not have any effect on the next turn's
 * already established turn order.
 * 
 * However, this can be remedied by utilizing the notetags provided by this
 * plugin to alter the Next Turn orders for specific targets. In fact, for
 * skill and item "effects" that add AGI Buffs and/or Debuffs, the target's
 * turn position on the Turn Order Display will be manipulated in accordance.
 * This auto-conversion feature can be disabled in the Plugin Parameters.
 * 
 * ---
 * 
 * Action Speed
 * 
 * Because the Order Turn Battle system already calculates agility speeds
 * before selecting an action to perform, the effects of the actioon speed will
 * not work the same way it did with the default battle system. Instead, the
 * Action Speed will be sent through a formula to determine its effect on the
 * following turn, either pushing the user ahead in next turn's turn order
 * (with a positive speed value) or back (with a negative speed value).
 * 
 * This option can have its formula altered or straight up disabled in the
 * Plugin Parameters.
 * 
 * ---
 * 
 * Infinity Speed and Clamping
 * 
 * Since Action Speeds are decided in such a way, enemies that will survive a
 * stun state past two turns will have "Infinity" speed on the recovery turn,
 * allowing them to act first relative to the rest of the battle participants
 * in order to balance out the turns they've lost.
 * 
 * Enemies with "Infinity" speed cannot be overtaken through turn order
 * manipulation while they are on the "Next Turn" order. If anything, battlers
 * who shift their turn order faster will be just trailing behind them, thus
 * the "clamping" effect. However if this occurs during the "Current Turn"
 * order, all is fair game and any battler can overtake them. Plan out your
 * battle system effects carefully with these rules in mind.
 * 
 * If you do not like the idea of Infinity Speed and/or Clamping, you can turn
 * them off in the Plugin Parameters.
 * 
 * This effect does not affect stun states that last only one turn. The effect
 * will only occur with stun states that last 2 turns or more.
 * 
 * ---
 * 
 * Instant Use
 * 
 * Skills and Items can have an "Instant Use" property which allows them to be
 * used immediately without consuming a turn. This can be used for actions that
 * otherwise do not warrant a whole turn. These can be used for minor buffs,
 * debuffs, toggles, etc.
 * 
 * ---
 *
 * ============================================================================
 * VisuStella MZ Compatibility
 * ============================================================================
 *
 * While this plugin is compatible with the majority of the VisuStella MZ
 * plugin library, it is not compatible with specific plugins or specific
 * features. This section will highlight the main plugins/features that will
 * not be compatible with this plugin or put focus on how the make certain
 * features compatible.
 *
 * ---
 * 
 * VisuMZ_2_PartySystem
 * 
 * In battle, the player cannot change entire parties at once from the Party
 * Command Window. The feature will be unaccessible while Order Turn Battle is
 * in play. However, the player can still change party members through the
 * Actor Command Window by having actors replace other actors. Party changing
 * is also available through battle events, Common Events, and script calls.
 * 
 * ---
 *
 * ============================================================================
 * Notetags
 * ============================================================================
 *
 * The following are notetags that have been added through this plugin. These
 * notetags will not work with your game if this plugin is OFF or not present.
 *
 * ---
 * 
 * === General OTB-Related Notetags ===
 * 
 * These notetags are general purpose notetags that have became available
 * through this plugin.
 * 
 * ---
 * 
 * <OTB Help>
 *  description
 *  description
 * </OTB Help>
 *
 * - Used for: Skill, Item Notetags
 * - If your game happens to support the ability to change battle systems, this
 *   notetag lets you change how the skill/item's help description text will
 *   look under OTB.
 * - This is primarily used if the skill behaves differently in OTB versus any
 *   other battle system.
 * - Replace 'description' with help text that's only displayed if the game's
 *   battle system is set to OTB.
 *
 * ---
 * 
 * === OTB Turn Order Display-Related Notetags ===
 * 
 * These notetags affect the OTB Turn Order Display
 * 
 * ---
 *
 * <OTB Turn Order Icon: x>
 *
 * - Used for: Actor, Enemy Notetags
 * - Changes the slot graphic used for the battler to a specific icon.
 * - Replace 'x' with the icon index to be used.
 * 
 * ---
 *
 * <OTB Turn Order Face: filename, index>
 *
 * - Used for: Actor, Enemy Notetags
 * - Changes the slot graphic used for the enemy to a specific face.
 * - Replace 'filename' with the filename of the image.
 *   - Do not include the file extension.
 * - Replace 'index' with the index of the face. Index values start at 0.
 * - Example: <OTB Turn Order Face: Monster, 1>
 * 
 * ---
 * 
 * === Instant Use-Related Notetags ===
 * 
 * ---
 *
 * <OTB Instant>
 * <OTB Instant Use>
 * <OTB Instant Cast>
 *
 * - Used for: Skill, Item Notetags
 * - Allows the skill/item to be used immediately without consuming a turn.
 *
 * ---
 * 
 * === Turn Order Manipulation-Related Notetags ===
 * 
 * ---
 *
 * <OTB User Current Turn: +x>
 * <OTB User Next Turn: +x>
 * <OTB User Follow Turn: +x>
 *
 * <OTB User Current Turn: -x>
 * <OTB User Next Turn: -x>
 * <OTB User Follow Turn: -x>
 *
 * - Used for: Skill, Item Notetags
 * - Changes the user's position in the turn order for the current turn, next
 *   turn, or whichever turn is following.
 * - If using the "Follow" variant, if the user has actions left for the
 *   current turn, it will affect the current turn. If not, it affects the
 *   next turn instead.
 * - Replace 'x' with a number representing the number of slots to change.
 *   - Negative numbers move the user closer to the front.
 *   - Positive numbers move the user towards the back.
 * - This effect only occurs once per skill/item use and at the start of the
 *   action when initializing the skill/item.
 *
 * ---
 *
 * <OTB Target Current Turn: +x>
 * <OTB Target Next Turn: +x>
 * <OTB Target Follow Turn: +x>
 *
 * <OTB Target Current Turn: -x>
 * <OTB Target Next Turn: -x>
 * <OTB Target Follow Turn: -x>
 *
 * - Used for: Skill, Item Notetags
 * - Changes the target's position in the turn order for the current turn, next
 *   turn, or whichever turn is following.
 * - If using the "Follow" variant, if the target has actions left for the
 *   current turn, it will affect the current turn. If not, it affects the
 *   next turn instead.
 * - Replace 'x' with a number representing the number of slots to change.
 *   - Negative numbers move the target closer to the front.
 *   - Positive numbers move the target towards the back.
 * - This effect will occur as many times as there are successfully connected
 *   hits for each target, meaning a target can have its turn order shifted
 *   multiple times.
 * - These are best used with single target skills/items as multi-target skills
 *   may shift multiple targets back and forth with each other if they are
 *   adjacent to one another.
 *
 * ---
 *
 * ============================================================================
 * Plugin Commands
 * ============================================================================
 *
 * The following are Plugin Commands that come with this plugin. They can be
 * accessed through the Plugin Command event command.
 *
 * ---
 * 
 * === Actor Plugin Commands ===
 * 
 * ---
 *
 * Actor: Change OTB Turn Order Icon
 * - Changes the icons used for the specific actor(s) on the OTB Turn Order.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Icon:
 *   - Changes the graphic to this icon.
 *
 * ---
 *
 * Actor: Change OTB Turn Order Face
 * - Changes the faces used for the specific actor(s) on the OTB Turn Order.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Face Name:
 *   - This is the filename for the target face graphic.
 *
 *   Face Index:
 *   - This is the index for the target face graphic.
 *
 * ---
 *
 * Actor: Clear OTB Turn Order Graphic
 * - Clears the OTB Turn Order graphics for the actor(s).
 * - The settings will revert to the Plugin Parameter settings.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 * ---
 * 
 * === Enemy Plugin Commands ===
 * 
 * ---
 *
 * Enemy: Change OTB Turn Order Icon
 * - Changes the icons used for the specific enemy(ies) on the OTB Turn Order.
 *
 *   Enemy Index(es):
 *   - Select which enemy index(es) to affect.
 *
 *   Icon:
 *   - Changes the graphic to this icon.
 *
 * ---
 *
 * Enemy: Change OTB Turn Order Face
 * - Changes the faces used for the specific enemy(ies) on the OTB Turn Order.
 *
 *   Enemy Index(es):
 *   - Select which enemy index(es) to affect.
 *
 *   Face Name:
 *   - This is the filename for the target face graphic.
 *
 *   Face Index:
 *   - This is the index for the target face graphic.
 *
 * ---
 *
 * Enemy: Clear OTB Turn Order Graphic
 * - Clears the OTB Turn Order graphics for the enemy(ies).
 * - The settings will revert to the Plugin Parameter settings.
 *
 *   Enemy Index(es):
 *   - Select which enemy index(es) to affect.
 *
 * ---
 * 
 * === System Plugin Commands ===
 * 
 * ---
 *
 * System: OTB Turn Order Visibility
 * - Determine the visibility of the OTB Turn Order Display.
 *
 *   Visibility:
 *   - Changes the visibility of the OTB Turn Order Display.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Conversion Settings
 * ============================================================================
 *
 * Automatically converts specific mechanics to fit OTB.
 *
 * ---
 *
 * Buffs
 * 
 *   AGI Buff => Current:
 *   - Auto-convert AGI Buff effects for Items/Skills to speed up target's
 *     current Turn Order?
 * 
 *   AGI Buff => Next:
 *   - Auto-convert AGI Buff effects for Items/Skills to speed up target's
 *     next Turn Order?
 *
 * ---
 *
 * Debuffs
 * 
 *   AGI Debuff => Current:
 *   - Auto-convert AGI Debuff effects for Items/Skills to speed up target's
 *     current Turn Order?
 * 
 *   AGI Debuff => Next:
 *   - Auto-convert AGI Debuff effects for Items/Skills to speed up target's
 *     next Turn Order?
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Mechanics Settings
 * ============================================================================
 *
 * Determines the mechanics of Battle System OTB. These range from how Action
 * Times are handled to speed.
 *
 * ---
 *
 * Action Times+
 * 
 *   Enable Action Times?:
 *   - Enable Action Times+ to have an effect on OTB?
 * 
 *     Randomize Order?:
 *     - If enabled, randomize the action order for added actions?
 *
 * ---
 *
 * Speed
 * 
 *   Allow Random Speed?:
 *   - Allow speed to be randomized base off the user's AGI?
 * 
 *   Post-Stun Infinity?:
 *   - After a 2+ turn stun states, battlers have infinity speed for their
 *     recovery turn.
 *   - Once again, this only applies to stun states that last 2+ turns.
 * 
 *     Infinity Clamp?:
 *     - Prevents turn order manipulation from going faster than infinity
 *       speed battlers.
 * 
 *   JS: Initial Speed:
 *   - Code used to calculate initial speed at the start of battle.
 * 
 *   JS: Speed => Order:
 *   - Code used to calculate how action speeds alter next turn's order.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Turn Order Display
 * ============================================================================
 *
 * Turn Order Display settings used for Battle System OTB. These adjust how the
 * two visible turn orders appears in-game.
 *
 * ---
 *
 * General
 * 
 *   Display Position:
 *   - Select where the Turn Order will appear on the screen.
 *     - Top
 *     - Bottom
 * 
 *     Offset X:
 *     - How much to offset the X coordinate by.
 *     - Negative: left. Positive: right.
 * 
 *     Offset Y:
 *     - How much to offset the Y coordinate by.
 *     - Negative: up. Positive: down.
 * 
 *   Reposition for Help?:
 *   - If the display position is at the top, reposition the display when the
 *     help window is open?
 * 
 *     Offset X:
 *     - Reposition the display's X coordinates by this much when the Help
 *       Window is visible.
 * 
 *     Offset Y:
 *     - Reposition the display's Y coordinates by this much when the Help
 *       Window is visible.
 * 
 *   Forward Direction:
 *   - Decide on the direction of the Turn Order.
 *     - Left to Right
 *     - Right to Left
 * 
 *   Subject Distance:
 *   - How far do you want the currently active battler to distance itself from
 *     the rest of the Turn Order?
 * 
 *   Screen Buffer:
 *   - What distance do you want the display to be away from the edge of the
 *     screen by?
 * 
 * ---
 * 
 * UI Background
 * 
 *   Background Style:
 *   - Select the style you want for the background.
 *     - fill
 *     - gradient
 *     - image
 *     - transparent
 * 
 *   Image Filename:
 *   - When using the "image" style, select an image from /img/system/ as the
 *     background image.
 * 
 *     Offset X:
 *     - How much do you want to offset the Background Image's X position?
 * 
 *     Offset Y:
 *     - How much do you want to offset the Background Image's Y position?
 * 
 * ---
 * 
 * UI Text
 * 
 *   Font Size:
 *   - The font size used for parameter values.
 * 
 *   Active Battler Text:
 *   - Text used to display the active battler.
 *   - This text will always be center aligned.
 * 
 *     Offset X:
 *     - How much do you want to offset the text's X position?
 * 
 *     Offset Y:
 *     - How much do you want to offset the text's Y position?
 * 
 *   Current Turn Text:
 *   - Text used to display the current turn.
 * 
 *     Offset X:
 *     - How much do you want to offset the text's X position?
 * 
 *     Offset Y:
 *     - How much do you want to offset the text's Y position?
 * 
 *   Next Turn Text:
 *   - Text used to display the next turn.
 * 
 *     Offset X:
 *     - How much do you want to offset the text's X position?
 * 
 *     Offset Y:
 *     - How much do you want to offset the text's Y position?
 * 
 *   Text Align:
 *   - Text alignment for the Current and Next Turn texts?
 *     - auto
 *     - left
 *     - center
 *     - right
 * 
 * ---
 * 
 * Slots
 * 
 *   Width:
 *   - How many pixels wide should the slots be on the Turn Order display?
 * 
 *   Height:
 *   - How many pixels tall should the slots be on the Turn Order display?
 * 
 *   Preview Scale:
 *   - How much do you want to scale the preview sprites by?
 *   - Use a number between 0 and 1 for the best results.
 * 
 *     Offset X:
 *     - How much do you want to offset the Preview Sprites' X position?
 * 
 *     Offset Y:
 *     - How much do you want to offset the Preview Sprites' Y position?
 * 
 *   Update Frames:
 *   - How many frames should it take for the slots to update their
 *     positions by?
 *
 * ---
 *
 * Slot Border
 * 
 *   Show Border?:
 *   - Show borders for the slot sprites?
 * 
 *   Border Thickness:
 *   - How many pixels thick should the colored portion of the border be?
 * 
 *   Actors
 *   Enemies
 * 
 *     Border Color:
 *     - Use #rrggbb for custom colors or regular numbers for text colors
 *       from the Window Skin.
 * 
 *       Preview Version:
 *       - A different setting is used for the preview version.
 * 
 *     Border Skin:
 *     - Optional. Place a skin on the actor/enemy borders instead of
 *       rendering them?
 * 
 *       Preview Version:
 *       - A different setting is used for the preview version.
 * 
 * ---
 * 
 * Slot Sprites
 * 
 *   Actors
 * 
 *     Sprite Type:
 *     - Select the type of sprite used for the actor graphic.
 *     - Face Graphic - Show the actor's face.
 *     - Icon - Show a specified icon.
 *     - Sideview Actor - Show the actor's sideview battler.
 * 
 *     Default Icon:
 *     - Which icon do you want to use for actors by default?
 * 
 *   Enemies
 * 
 *     Sprite Type:
 *     - Select the type of sprite used for the enemy graphic.
 *     - Face Graphic - Show a specified face graphic.
 *     - Icon - Show a specified icon.
 *     - Enemy - Show the enemy's graphic or sideview battler.
 * 
 *     Default Face Name:
 *     - Use this default face graphic if there is no specified face.
 * 
 *     Default Face Index:
 *     - Use this default face index if there is no specified index.
 * 
 *     Default Icon:
 *     - Which icon do you want to use for enemies by default?
 * 
 *     Match Hue?:
 *     - Match the hue for enemy battlers?
 *     - Does not apply if there's a sideview battler.
 *
 * ---
 *
 * Slot Letter
 * 
 *   Show Enemy Letter?:
 *   - Show the enemy's letter on the slot sprite?
 * 
 *   Font Name:
 *   - The font name used for the text of the Letter.
 *   - Leave empty to use the default game's font.
 * 
 *   Font Size:
 *   - The font size used for the text of the Letter.
 *
 * ---
 *
 * Slot Background
 * 
 *   Show Background?:
 *   - Show the background on the slot sprite?
 * 
 *   Actors
 *   Enemies
 * 
 *     Background Color 1:
 *     Background Color 2:
 *     - Use #rrggbb for custom colors or regular numbers for text colors
 *       from the Window Skin.
 * 
 *       Preview Version:
 *       - A different setting is used for the preview version.
 * 
 *     Background Skin:
 *     - Optional. Use a skin for the actor background instead of
 *       rendering them?
 * 
 *       Preview Version:
 *       - A different setting is used for the preview version.
 *
 * ---
 *
 * ============================================================================
 * Terms of Use
 * ============================================================================
 *
 * 1. These plugins may be used in free or commercial games provided that they
 * have been acquired through legitimate means at VisuStella.com and/or any
 * other official approved VisuStella sources. Exceptions and special
 * circumstances that may prohibit usage will be listed on VisuStella.com.
 * 
 * 2. All of the listed coders found in the Credits section of this plugin must
 * be given credit in your games or credited as a collective under the name:
 * "VisuStella".
 * 
 * 3. You may edit the source code to suit your needs, so long as you do not
 * claim the source code belongs to you. VisuStella also does not take
 * responsibility for the plugin if any changes have been made to the plugin's
 * code, nor does VisuStella take responsibility for user-provided custom code
 * used for custom control effects including advanced JavaScript notetags
 * and/or plugin parameters that allow custom JavaScript code.
 * 
 * 4. You may NOT redistribute these plugins nor take code from this plugin to
 * use as your own. These plugins and their code are only to be downloaded from
 * VisuStella.com and other official/approved VisuStella sources. A list of
 * official/approved sources can also be found on VisuStella.com.
 *
 * 5. VisuStella is not responsible for problems found in your game due to
 * unintended usage, incompatibility problems with plugins outside of the
 * VisuStella MZ library, plugin versions that aren't up to date, nor
 * responsible for the proper working of compatibility patches made by any
 * third parties. VisuStella is not responsible for errors caused by any
 * user-provided custom code used for custom control effects including advanced
 * JavaScript notetags and/or plugin parameters that allow JavaScript code.
 *
 * 6. If a compatibility patch needs to be made through a third party that is
 * unaffiliated with VisuStella that involves using code from the VisuStella MZ
 * library, contact must be made with a member from VisuStella and have it
 * approved. The patch would be placed on VisuStella.com as a free download
 * to the public. Such patches cannot be sold for monetary gain, including
 * commissions, crowdfunding, and/or donations.
 * 
 * 7. If this VisuStella MZ plugin is a paid product, all project team members
 * must purchase their own individual copies of the paid product if they are to
 * use it. Usage includes working on related game mechanics, managing related
 * code, and/or using related Plugin Commands and features. Redistribution of
 * the plugin and/or its code to other members of the team is NOT allowed
 * unless they own the plugin itself as that conflicts with Article 4.
 * 
 * 8. Any extensions and/or addendums made to this plugin's Terms of Use can be
 * found on VisuStella.com and must be followed.
 *
 * ============================================================================
 * Credits
 * ============================================================================
 * 
 * If you are using this plugin, credit the following people in your game:
 * 
 * Team VisuStella
 * * Yanfly
 * * Arisu
 * * Olivia
 * * Irina
 *
 * ============================================================================
 * Changelog
 * ============================================================================
 * 
 * Version 1.04: August 6, 2021
 * * Bug Fixes!
 * ** Enemies with multiple actions will no longer step forward when it's not
 *    their turn. Fix made by Olivia.
 * 
 * Version 1.03: June 25, 2021
 * * Optimization Update!
 * ** Plugin should run more optimized.
 * 
 * Version 1.02: April 16, 2021
 * * Bug Fixes!
 * ** Post-stun infinity clamping should now be adjusted properly for
 *    previewing turn order changes.
 * 
 * Version 1.01: April 9, 2021
 * * Bug Fixes!
 * ** Subsequent battles will properly reset the turn order. Fix by Olivia.
 * 
 * Version 1.00 Official Release Date: April 26, 2021
 * * Finished Plugin!
 *
 * ============================================================================
 * End of Helpfile
 * ============================================================================
 *
 * @ --------------------------------------------------------------------------
 *
 * @command OtbTurnOrderActorIcon
 * @text Actor: Change OTB Turn Order Icon
 * @desc Changes the icons used for the specific actor(s) on the OTB Turn Order.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg IconIndex:num
 * @text Icon
 * @desc Changes the graphic to this icon.
 * @default 84
 *
 * @ --------------------------------------------------------------------------
 *
 * @command OtbTurnOrderActorFace
 * @text Actor: Change OTB Turn Order Face
 * @desc Changes the faces used for the specific actor(s) on the OTB Turn Order.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg FaceName:str
 * @text Face Name
 * @type file
 * @dir img/faces/
 * @desc This is the filename for the target face graphic.
 * @default Actor1
 *
 * @arg FaceIndex:num
 * @text Face Index
 * @type number
 * @desc This is the index for the target face graphic.
 * @default 0
 *
 * @ --------------------------------------------------------------------------
 *
 * @command OtbTurnOrderClearActorGraphic
 * @text Actor: Clear OTB Turn Order Graphic
 * @desc Clears the OTB Turn Order graphics for the actor(s).
 * The settings will revert to the Plugin Parameter settings.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command OtbTurnOrderEnemyIcon
 * @text Enemy: Change OTB Turn Order Icon
 * @desc Changes the icons used for the specific enemy(ies) on the OTB Turn Order.
 *
 * @arg Enemies:arraynum
 * @text Enemy Index(es)
 * @type number[]
 * @desc Select which enemy index(es) to affect.
 * @default ["1"]
 *
 * @arg IconIndex:num
 * @text Icon
 * @desc Changes the graphic to this icon.
 * @default 298
 *
 * @ --------------------------------------------------------------------------
 *
 * @command OtbTurnOrderEnemyFace
 * @text Enemy: Change OTB Turn Order Face
 * @desc Changes the faces used for the specific enemy(ies) on the OTB Turn Order.
 *
 * @arg Enemies:arraynum
 * @text Enemy Index(es)
 * @type number[]
 * @desc Select which enemy index(es) to affect.
 * @default ["1"]
 *
 * @arg FaceName:str
 * @text Face Name
 * @parent EnemySprite
 * @type file
 * @dir img/faces/
 * @desc This is the filename for the target face graphic.
 * @default Monster
 *
 * @arg FaceIndex:num
 * @text Face Index
 * @parent EnemySprite
 * @type number
 * @desc This is the index for the target face graphic.
 * @default 1
 *
 * @ --------------------------------------------------------------------------
 *
 * @command OtbTurnOrderClearEnemyGraphic
 * @text Enemy: Clear OTB Turn Order Graphic
 * @desc Clears the OTB Turn Order graphics for the enemy(ies).
 * The settings will revert to the Plugin Parameter settings.
 *
 * @arg Enemies:arraynum
 * @text Enemy Index(es)
 * @type number[]
 * @desc Select which enemy index(es) to affect.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command SystemTurnOrderVisibility
 * @text System: OTB Turn Order Visibility
 * @desc Determine the visibility of the OTB Turn Order Display.
 *
 * @arg Visible:eval
 * @text Visibility
 * @type boolean
 * @on Visible
 * @off Hidden
 * @desc Changes the visibility of the OTB Turn Order Display.
 * @default true
 *
 * @ --------------------------------------------------------------------------
 *
 * @ ==========================================================================
 * @ Plugin Parameters
 * @ ==========================================================================
 *
 * @param BreakHead
 * @text --------------------------
 * @default ----------------------------------
 *
 * @param BattleSystemOTB
 * @default Plugin Parameters
 *
 * @param ATTENTION
 * @default READ THE HELP FILE
 *
 * @param BreakSettings
 * @text --------------------------
 * @default ----------------------------------
 *
 * @param Conversion:struct
 * @text Conversion Settings
 * @type struct<Conversion>
 * @desc Automatically converts specific mechanics to fit OTB.
 * @default {"Buffs":"","ConvertAgiBuffCurrent:eval":"true","ConvertAgiBuffNext:eval":"true","Debuffs":"","ConvertAgiDebuffCurrent:eval":"true","ConvertAgiDebuffNext:eval":"true"}
 *
 * @param Mechanics:struct
 * @text Mechanics Settings
 * @type struct<Mechanics>
 * @desc Determines the mechanics of Battle System OTB.
 * @default {"Actions":"","EnableActionTimes:eval":"true","RandomizeActionTimesOrder:eval":"true","Speed":"","AllowRandomSpeed:eval":"false","PostStunInfinitySpeed:eval":"true","InfinityClamp:eval":"true","InitialSpeedJS:func":"\"// Declare Constants\\nconst agi = this.subject().agi;\\n\\n// Create Speed\\nlet speed = agi;\\nif (this.allowRandomSpeed()) {\\n    speed += Math.randomInt(Math.floor(5 + agi / 4));\\n}\\n\\n// Return Speed\\nreturn speed;\"","ConvertSpeedJS:func":"\"// Declare Constants\\nconst item = this.item();\\nconst modifier = 50;\\n\\n// Calculate Order Slots Changed\\nlet change = item.speed / (-modifier);\\nchange = (change >= 0) ? Math.ceil(change) : Math.floor(change);\\n\\n// Return Change\\nreturn change || 0;\""}
 *
 * @param TurnOrder:struct
 * @text Turn Order Display
 * @type struct<TurnOrder>
 * @desc Turn Order Display settings used for Battle System OTB.
 * @default {"General":"","DisplayPosition:str":"top","DisplayOffsetX:num":"0","DisplayOffsetY:num":"0","RepositionTopForHelp:eval":"true","RepositionTopHelpX:num":"+0","RepositionTopHelpY:num":"+96","RepositionLogWindow:eval":"true","LogWindowOffsetY:num":"+0","OrderDirection:eval":"false","SubjectDistance:num":"16","ScreenBuffer:num":"36","UiBackground":"","BgDimStyle:str":"gradient","BgImageFilename:str":"","BgImageOffsetX:num":"+0","BgImageOffsetY:num":"+0","UiText":"","UiFontSize:num":"16","UiSubjectText:str":"★","UiSubjectOffsetX:num":"+0","UiSubjectOffsetY:num":"-6","UiCurrentText:str":"✦CURRENT TURN✦","UiCurrentOffsetX:num":"+6","UiCurrentOffsetY:num":"-6","UiNextText:str":"✧NEXT TURN✧","UiNextOffsetX:num":"+6","UiNextOffsetY:num":"-6","UiAlignment:str":"auto","Slots":"","SpriteThin:num":"72","SpriteLength:num":"72","PreviewScale:num":"0.5","PreviewOffsetX:num":"+0","PreviewOffsetY:num":"+0","UpdateFrames:num":"24","Border":"","ShowMarkerBorder:eval":"true","BorderActor":"","ActorBorderColor:str":"4","PreviewActorBorderColor:str":"0","ActorSystemBorder:str":"","PreviewActorSystemBorder:str":"","BorderEnemy":"","EnemyBorderColor:str":"2","PreviewEnemyBorderColor:str":"0","EnemySystemBorder:str":"","PreviewEnemySystemBorder:str":"","BorderThickness:num":"2","Sprite":"","ActorSprite":"","ActorBattlerType:str":"face","ActorBattlerIcon:num":"84","EnemySprite":"","EnemyBattlerType:str":"enemy","EnemyBattlerFaceName:str":"Monster","EnemyBattlerFaceIndex:num":"1","EnemyBattlerIcon:num":"298","EnemyBattlerMatchHue:eval":"true","Letter":"","EnemyBattlerDrawLetter:eval":"true","EnemyBattlerFontFace:str":"","EnemyBattlerFontSize:num":"16","Background":"","ShowMarkerBg:eval":"true","BackgroundActor":"","ActorBgColor1:str":"19","PreviewActorBgColor1:str":"19","ActorBgColor2:str":"9","PreviewActorBgColor2:str":"0","ActorSystemBg:str":"","PreviewActorSystemBg:str":"","BackgroundEnemy":"","EnemyBgColor1:str":"19","PreviewEnemyBgColor1:str":"19","EnemyBgColor2:str":"18","PreviewEnemyBgColor2:str":"0","EnemySystemBg:str":"","PreviewEnemySystemBg:str":""}
 *
 * @param BreakEnd1
 * @text --------------------------
 * @default ----------------------------------
 *
 * @param End Of
 * @default Plugin Parameters
 *
 * @param BreakEnd2
 * @text --------------------------
 * @default ----------------------------------
 *
 */
/* ----------------------------------------------------------------------------
 * Conversion Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~Conversion:
 * 
 * @param Buffs
 *
 * @param ConvertAgiBuffCurrent:eval
 * @text AGI Buff => Current
 * @parent Buffs
 * @type boolean
 * @on Convert
 * @off Don't Convert
 * @desc Auto-convert AGI Buff effects for Items/Skills to speed up target's current Turn Order?
 * @default true
 *
 * @param ConvertAgiBuffNext:eval
 * @text AGI Buff => Next
 * @parent Buffs
 * @type boolean
 * @on Convert
 * @off Don't Convert
 * @desc Auto-convert AGI Buff effects for Items/Skills to speed up target's next Turn Order?
 * @default true
 * 
 * @param Debuffs
 *
 * @param ConvertAgiDebuffCurrent:eval
 * @text AGI Debuff => Current
 * @parent Debuffs
 * @type boolean
 * @on Convert
 * @off Don't Convert
 * @desc Auto-convert AGI Debuff effects for Items/Skills to speed up target's current Turn Order?
 * @default true
 *
 * @param ConvertAgiDebuffNext:eval
 * @text AGI Debuff => Next
 * @parent Debuffs
 * @type boolean
 * @on Convert
 * @off Don't Convert
 * @desc Auto-convert AGI Debuff effects for Items/Skills to speed up target's next Turn Order?
 * @default true
 *
 */
/* ----------------------------------------------------------------------------
 * Mechanics Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~Mechanics:
 *
 * @param Actions
 * @text Action Times+
 *
 * @param EnableActionTimes:eval
 * @text Enable Action Times?
 * @parent Actions
 * @type boolean
 * @on Enable
 * @off Disable
 * @desc Enable Action Times+ to have an effect on OTB?
 * @default true
 *
 * @param RandomizeActionTimesOrder:eval
 * @text Randomize Order?
 * @parent EnableActionTimes:eval
 * @type boolean
 * @on Randomize
 * @off Clumped
 * @desc If enabled, randomize the action order for added actions?
 * @default true
 * 
 * @param Speed
 *
 * @param AllowRandomSpeed:eval
 * @text Allow Random Speed?
 * @parent Speed
 * @type boolean
 * @on Allow
 * @off Disable
 * @desc Allow speed to be randomized base off the user's AGI?
 * @default false
 *
 * @param PostStunInfinitySpeed:eval
 * @text Post-Stun Infinity?
 * @parent Speed
 * @type boolean
 * @on Infinity
 * @off Normal
 * @desc After a 2+ turn stun states, battlers have infinity speed for their recovery turn.
 * @default true
 *
 * @param InfinityClamp:eval
 * @text Infinity Clamp?
 * @parent PostStunInfinitySpeed:eval
 * @type boolean
 * @on Enable Clamp
 * @off Disable Clamp
 * @desc Prevents turn order manipulation from going faster than infinity speed battlers.
 * @default true
 *
 * @param InitialSpeedJS:func
 * @text JS: Initial Speed
 * @parent Speed
 * @type note
 * @desc Code used to calculate initial speed at the start of battle.
 * @default "// Declare Constants\nconst agi = this.subject().agi;\n\n// Create Speed\nlet speed = agi;\nif (this.allowRandomSpeed()) {\n    speed += Math.randomInt(Math.floor(5 + agi / 4));\n}\n\n// Return Speed\nreturn speed;"
 *
 * @param ConvertSpeedJS:func
 * @text JS: Speed => Order
 * @parent Speed
 * @type note
 * @desc Code used to calculate how action speeds alter next turn's order.
 * @default "// Declare Constants\nconst item = this.item();\nconst modifier = 50;\n\n// Calculate Order Slots Changed\nlet change = item.speed / (-modifier);\nchange = (change >= 0) ? Math.ceil(change) : Math.floor(change);\n\n// Return Change\nreturn change || 0;"
 * 
 */
/* ----------------------------------------------------------------------------
 * Turn Order Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~TurnOrder:
 *
 * @param General
 *
 * @param DisplayPosition:str
 * @text Display Position
 * @parent General
 * @type select
 * @option top
 * @option bottom
 * @desc Select where the Turn Order will appear on the screen.
 * @default top
 * 
 * @param DisplayOffsetX:num
 * @text Offset X
 * @parent DisplayPosition:str
 * @desc How much to offset the X coordinate by.
 * Negative: left. Positive: right.
 * @default 0
 * 
 * @param DisplayOffsetY:num
 * @text Offset Y
 * @parent DisplayPosition:str
 * @desc How much to offset the Y coordinate by.
 * Negative: up. Positive: down.
 * @default 0
 *
 * @param RepositionTopForHelp:eval
 * @text Reposition for Help?
 * @parent DisplayPosition:str
 * @type boolean
 * @on Reposition
 * @off Stay
 * @desc If the display position is at the top, reposition the
 * display when the help window is open?
 * @default true
 *
 * @param RepositionTopHelpX:num
 * @text Offset X
 * @parent RepositionTopForHelp:eval
 * @desc Reposition the display's X coordinates by this much when
 * the Help Window is visible.
 * @default +0
 *
 * @param RepositionTopHelpY:num
 * @text Offset Y
 * @parent RepositionTopForHelp:eval
 * @desc Reposition the display's Y coordinates by this much when
 * the Help Window is visible.
 * @default +96
 *
 * @param RepositionLogWindow:eval
 * @text Reposition Log?
 * @parent DisplayPosition:str
 * @type boolean
 * @on Reposition
 * @off Stay
 * @desc If the display position is at the top, reposition the
 * Battle Log Window to be lower?
 * @default true
 *
 * @param LogWindowOffsetY:num
 * @text Offset Y
 * @parent RepositionLogWindow:eval
 * @desc How much do you want to offset the Log Window's Y position?
 * @default +0
 *
 * @param OrderDirection:eval
 * @text Forward Direction
 * @parent General
 * @type boolean
 * @on Left to Right
 * @off Right to Left
 * @desc Decide on the direction of the Turn Order.
 * @default false
 *
 * @param SubjectDistance:num
 * @text Subject Distance
 * @parent General
 * @type number
 * @desc How far do you want the currently active battler to
 * distance itself from the rest of the Turn Order?
 * @default 16
 *
 * @param ScreenBuffer:num
 * @text Screen Buffer
 * @parent General
 * @type number
 * @desc What distance do you want the display to be away
 * from the edge of the screen by?
 * @default 36
 *
 * @param UiBackground
 * @text UI Background
 *
 * @param BgDimStyle:str
 * @text Background Style
 * @parent UiBackground
 * @type select
 * @option fill
 * @option gradient
 * @option image
 * @option transparent
 * @desc Select the style you want for the background.
 * @default gradient
 *
 * @param BgImageFilename:str
 * @text Image Filename
 * @parent UiBackground
 * @type file
 * @dir img/system/
 * @desc When using the "image" style, select an image from /img/system/ as the background image.
 * @default 
 *
 * @param BgImageOffsetX:num
 * @text Offset X
 * @parent BgImageFilename:str
 * @desc How much do you want to offset the Background Image's X position?
 * @default +0
 *
 * @param BgImageOffsetY:num
 * @text Offset Y
 * @parent BgImageFilename:str
 * @desc How much do you want to offset the Background Image's Y position?
 * @default +0
 *
 * @param UiText
 * @text UI Text
 *
 * @param UiFontSize:num
 * @text Font Size
 * @parent UiText
 * @desc The font size used for parameter values.
 * @default 16
 *
 * @param UiSubjectText:str
 * @text Active Battler Text
 * @parent UiText
 * @desc Text used to display the active battler.
 * This text will always be center aligned.
 * @default ★
 *
 * @param UiSubjectOffsetX:num
 * @text Offset X
 * @parent UiSubjectText:str
 * @desc How much do you want to offset the text's X position?
 * @default +0
 *
 * @param UiSubjectOffsetY:num
 * @text Offset Y
 * @parent UiSubjectText:str
 * @desc How much do you want to offset the text's Y position?
 * @default -6
 *
 * @param UiCurrentText:str
 * @text Current Turn Text
 * @parent UiText
 * @desc Text used to display the current turn.
 * @default ✦CURRENT TURN✦
 *
 * @param UiCurrentOffsetX:num
 * @text Offset X
 * @parent UiCurrentText:str
 * @desc How much do you want to offset the text's X position?
 * @default +6
 *
 * @param UiCurrentOffsetY:num
 * @text Offset Y
 * @parent UiCurrentText:str
 * @desc How much do you want to offset the text's Y position?
 * @default -6
 *
 * @param UiNextText:str
 * @text Next Turn Text
 * @parent UiText
 * @desc Text used to display the next turn.
 * @default ✧NEXT TURN✧
 *
 * @param UiNextOffsetX:num
 * @text Offset X
 * @parent UiNextText:str
 * @desc How much do you want to offset the text's X position?
 * @default +6
 *
 * @param UiNextOffsetY:num
 * @text Offset Y
 * @parent UiNextText:str
 * @desc How much do you want to offset the text's Y position?
 * @default -6
 *
 * @param UiAlignment:str
 * @text Text Align
 * @parent UiText
 * @type combo
 * @option auto
 * @option left
 * @option center
 * @option right
 * @desc Text alignment for the Current and Next Turn texts?
 * @default auto
 * 
 * @param Slots
 *
 * @param SpriteThin:num
 * @text Width
 * @parent Slots
 * @type number
 * @min 1
 * @desc How many pixels wide should the slots be on the
 * Turn Order display?
 * @default 72
 *
 * @param SpriteLength:num
 * @text Height
 * @parent Slots
 * @type number
 * @min 1
 * @desc How many pixels tall should the slots be on the
 * Turn Order display?
 * @default 72
 *
 * @param PreviewScale:num
 * @text Preview Scale
 * @parent Slots
 * @desc How much do you want to scale the preview sprites by?
 * Use a number between 0 and 1 for the best results.
 * @default 0.5
 *
 * @param PreviewOffsetX:num
 * @text Offset X
 * @parent PreviewScale:num
 * @desc How much do you want to offset the Preview Sprites' X position?
 * @default +0
 *
 * @param PreviewOffsetY:num
 * @text Offset Y
 * @parent PreviewScale:num
 * @desc How much do you want to offset the Preview Sprites' Y position?
 * @default +0
 *
 * @param UpdateFrames:num
 * @text Update Frames
 * @parent Slots
 * @type number
 * @min 1
 * @desc How many frames should it take for the slots to
 * update their positions by?
 * @default 24
 *
 * @param Border
 * @text Slot Border
 *
 * @param ShowMarkerBorder:eval
 * @text Show Border?
 * @parent Border
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Show borders for the slot sprites?
 * @default true
 *
 * @param BorderThickness:num
 * @text Border Thickness
 * @parent Markers
 * @type number
 * @min 1
 * @desc How many pixels thick should the colored portion of the border be?
 * @default 2
 *
 * @param BorderActor
 * @text Actors
 * @parent Border
 *
 * @param ActorBorderColor:str
 * @text Border Color
 * @parent BorderActor
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 4
 *
 * @param PreviewActorBorderColor:str
 * @text Preview Version
 * @parent ActorBorderColor:str
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 0
 *
 * @param ActorSystemBorder:str
 * @text Border Skin
 * @parent BorderActor
 * @type file
 * @dir img/system/
 * @desc Optional. Place a skin on the actor borders instead of rendering them?
 * @default 
 *
 * @param PreviewActorSystemBorder:str
 * @text Preview Version
 * @parent ActorSystemBorder:str
 * @type file
 * @dir img/system/
 * @desc Optional. Place a skin on the actor borders instead of rendering them?
 * @default 
 *
 * @param BorderEnemy
 * @text Enemies
 * @parent Border
 *
 * @param EnemyBorderColor:str
 * @text Border Color
 * @parent BorderEnemy
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 2
 *
 * @param PreviewEnemyBorderColor:str
 * @text Preview Version
 * @parent EnemyBorderColor:str
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 0
 *
 * @param EnemySystemBorder:str
 * @text Border Skin
 * @parent BorderEnemy
 * @type file
 * @dir img/system/
 * @desc Optional. Place a skin on the enemy borders instead of rendering them?
 * @default 
 *
 * @param PreviewEnemySystemBorder:str
 * @text Preview Version
 * @parent EnemySystemBorder:str
 * @type file
 * @dir img/system/
 * @desc Optional. Place a skin on the enemy borders instead of rendering them?
 * @default 
 *
 * @param Sprite
 * @text Slot Sprites
 *
 * @param ActorSprite
 * @text Actors
 * @parent Sprite
 *
 * @param ActorBattlerType:str
 * @text Sprite Type
 * @parent ActorSprite
 * @type select
 * @option Face Graphic - Show the actor's face.
 * @value face
 * @option Icon - Show a specified icon.
 * @value icon
 * @option Sideview Actor - Show the actor's sideview battler.
 * @value svactor
 * @desc Select the type of sprite used for the actor graphic.
 * @default face
 *
 * @param ActorBattlerIcon:num
 * @text Default Icon
 * @parent ActorSprite
 * @desc Which icon do you want to use for actors by default?
 * @default 84
 *
 * @param EnemySprite
 * @text Enemies
 * @parent Sprite
 *
 * @param EnemyBattlerType:str
 * @text Sprite Type
 * @parent EnemySprite
 * @type select
 * @option Face Graphic - Show a specified face graphic.
 * @value face
 * @option Icon - Show a specified icon.
 * @value icon
 * @option Enemy - Show the enemy's graphic or sideview battler.
 * @value enemy
 * @desc Select the type of sprite used for the enemy graphic.
 * @default enemy
 *
 * @param EnemyBattlerFaceName:str
 * @text Default Face Name
 * @parent EnemySprite
 * @type file
 * @dir img/faces/
 * @desc Use this default face graphic if there is no specified face.
 * @default Monster
 *
 * @param EnemyBattlerFaceIndex:num
 * @text Default Face Index
 * @parent EnemySprite
 * @type number
 * @desc Use this default face index if there is no specified index.
 * @default 1
 *
 * @param EnemyBattlerIcon:num
 * @text Default Icon
 * @parent EnemySprite
 * @desc Which icon do you want to use for enemies by default?
 * @default 298
 *
 * @param EnemyBattlerMatchHue:eval
 * @text Match Hue?
 * @parent EnemySprite
 * @type boolean
 * @on Match
 * @off Don't Match
 * @desc Match the hue for enemy battlers?
 * Does not apply if there's a sideview battler.
 * @default true
 *
 * @param Letter
 * @text Slot Letter
 *
 * @param EnemyBattlerDrawLetter:eval
 * @text Show Enemy Letter?
 * @parent Letter
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Show the enemy's letter on the slot sprite?
 * @default true
 *
 * @param EnemyBattlerFontFace:str
 * @text Font Name
 * @parent Letter
 * @desc The font name used for the text of the Letter.
 * Leave empty to use the default game's font.
 * @default 
 *
 * @param EnemyBattlerFontSize:num
 * @text Font Size
 * @parent Letter
 * @min 1
 * @desc The font size used for the text of the Letter.
 * @default 16
 *
 * @param Background
 * @text Slot Background
 *
 * @param ShowMarkerBg:eval
 * @text Show Background?
 * @parent Background
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Show the background on the slot sprite?
 * @default true
 *
 * @param BackgroundActor
 * @text Actors
 * @parent Background
 *
 * @param ActorBgColor1:str
 * @text Background Color 1
 * @parent BackgroundActor
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 19
 *
 * @param PreviewActorBgColor1:str
 * @text Preview Version
 * @parent ActorBgColor1:str
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 19
 *
 * @param ActorBgColor2:str
 * @text Background Color 2
 * @parent BackgroundActor
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 9
 *
 * @param PreviewActorBgColor2:str
 * @text Preview Version
 * @parent ActorBgColor2:str
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 0
 *
 * @param ActorSystemBg:str
 * @text Background Skin
 * @parent BackgroundActor
 * @type file
 * @dir img/system/
 * @desc Optional. Use a skin for the actor background instead of rendering them?
 * @default 
 *
 * @param PreviewActorSystemBg:str
 * @text Preview Version
 * @parent ActorSystemBg:str
 * @type file
 * @dir img/system/
 * @desc Optional. Use a skin for the actor background instead of rendering them?
 * @default 
 *
 * @param BackgroundEnemy
 * @text Enemies
 * @parent Background
 *
 * @param EnemyBgColor1:str
 * @text Background Color 1
 * @parent BackgroundEnemy
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 19
 *
 * @param PreviewEnemyBgColor1:str
 * @text Preview Version
 * @parent EnemyBgColor1:str
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 19
 *
 * @param EnemyBgColor2:str
 * @text Background Color 2
 * @parent BackgroundEnemy
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 18
 *
 * @param PreviewEnemyBgColor2:str
 * @text Preview Version
 * @parent EnemyBgColor2:str
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 0
 *
 * @param EnemySystemBg:str
 * @text Background Skin
 * @parent BackgroundEnemy
 * @type file
 * @dir img/system/
 * @desc Optional. Use a skin for the enemy background instead of rendering them?
 * @default 
 *
 * @param PreviewEnemySystemBg:str
 * @text Preview Version
 * @parent EnemySystemBg:str
 * @type file
 * @dir img/system/
 * @desc Optional. Use a skin for the enemy background instead of rendering them?
 * @default 
 *
 */
//=============================================================================

const _0x30f4=['BattleManager_battleSys','containerPosition','initMembers','otbGainInstant','_graphicSprite','SideviewBattleUI','update','calculateTargetPositions','pOyqM','changeIconGraphicBitmap','1255312MxjyZI','sZLZf','applyItemUserEffect','removeActor','ARRAYSTR','_otbTurnOrderGraphicType','_index','drawText','xOBMm','onActorOk','BattleManager_makeActionOrders','windowRect','needsSelection','changeSourceArray','onSkillCancel','endTurn','ConvertAgiDebuffNext','IEqMa','uXRGJ','vPSMJ','otbApplyActionTimes','bZFVR','Scene_Battle_onItemCancel','isActiveTpb','processUpdateGraphic','addChildToBack','setBattleSystemOTBTurnOrderVisible','Scene_Battle_commandGuard','filter','note','VisuMZ_3_SideviewBattleUI','UXAWz','randomInt','removeStatesAuto','_previewNext','_otbTurnOrderFaceIndex','commandCancel','createActorCommandWindow','_unit','OtbTurnOrderClearActorGraphic','qNvdW','_actorCommandWindow','Game_Battler_removeState','Game_System_initialize','OTB_CONVERT_AGI_DEBUFF_NEXT_TURN','Game_Battler_addState','findIndex','format','BattleManager_processTurn','Window_Help_setItem','createTurnOrderOTBGraphicFaceIndex','match','index','remove','singleSkill','adBfo','lzbef','updateStateTurns','TargetCurrOrder','children','face','sANms','startActorCommandSelection','currentExt','EnemyBattlerType','registerCommand','Game_Battler_onTurnEnd','Game_Battler_performCollapse','getUnitSideSide','OtbTurnOrderActorIcon','Game_BattlerBase_hide','selectNextActorOTB','getNextSubject','_positionTargetY','grduA','yUWnW','makeNextActionOrdersOTB','updatePosition','commandGuard','OTB_ADDED_RANDOMIZE_ADDED_ACTION_ORDER','RepositionTopForHelp','clearRect','NZejd','Game_Battler_makeSpeed','makeActionTimes','#000000','_subjectX','_otbTimesActedThisTurn','onEnemyCancel','boxHeight','KOwBJ','EnemyBattlerFaceName','OrderDirection','UpdateFrames','removeState','_sourceArray','_otb_createdFirstTurnOrders','battleEnd','canChangeOtbTurnOrder','lineHeight','Scene_Battle_onEnemyOk','setHue','battlerName','_currentTurn','ZxnNP','zDgUz','otbPreviewOrderChange','width','finishActorInput','MUluS','trim','VisuMZ_2_PartySystem','UiCurrentOffsetX','actorCommandSingleSkill','updateGraphicHue','changeFaceGraphicBitmap','addLoadListener','RepositionTopHelpX','commandAttack','_graphicIconIndex','OtbTurnOrderClearEnemyGraphic','makeActionOrders','currentSymbol','sesEi','1208793jobDIx','Game_Action_speed','status','updateSelectionEffect','_hidden','stepForward','_bgImageSprite','zIDeE','ActionBattlersNextFilter','makeOTBSpeed','onBattleStart','repositionLogWindowOTB','name','hasSvBattler','ILiLw','dJifP','ARRAYSTRUCT','members','PreviewOffsetY','GVgCa','removeUnableTurnOrderSprites','battleSys','_letterSprite','STRUCT','Scene_Battle_createActorCommandWindow','anchor','lJiHk','rbWcI','_targetHomeX','BgDimStyle','isAlive','_logWindow','_otbTurnOrderWindow','NXubd','Visible','zEqeE','updatePadding','hide','svBattlerName','cMgFn','BattleManager_finishActorInput','ElNyH','Game_BattlerBase_appear','EnemyBattlerIcon','ActorBattlerIcon','removeSprite','otbCalcTargetCurrentOrderChange','UiFontSize','Scene_Battle_commandAttack','adjustForPreview','_graphicEnemy','parameters','%1-%2','BattleSystemOTB','createGraphicSprite','NUM','_previewCurrent','SystemTurnOrderVisibility','HgHMn','prototype','_contentsBackSprite','EFFECT_ADD_BUFF','scale','_subject','ActorBattlerType','visible','createOTBTurnOrderWindow','UserFollOrder','previewOrderByAction','UserCurrOrder','RepositionLogWindow','OtbTurnOrderActorFace','elZky','_spriteGroupWidth','removeChild','_ogWindowLayerY','loadSvActor','TurnOrderOTBGraphicFaceName','UserNextOrder','ZJFaz','createInitialPositions','ConvertAgiBuffCurrent','JprIH','nlpZY','changeSvActorGraphicBitmap','DisplayPosition','_otbTurnOrderFaceName','call','otbCalcUserCurrentOrderChange','resetFontSettings','FaceName','zwJYL','TsJRZ','OTB_CONVERT_AGI_DEBUFF_CURRENT_TURN','top','_containerWidth','hSzuf','DisplayOffsetY','iNQKe','swWCa','round','YWOIn','otbRemoveCurrentSubject','_phase','ARRAYJSON','DisplayOffsetX','573151eXAHrR','qmVAQ','vfVxY','UiNextOffsetY','frsRC','raVzJ','resumeTurnOrderSprites','initMembersOTB','_windowLayer','initialize','setup','_stateIDs','%1SystemBg','turnOrderChangeOTB','otbShiftNextTurnSpritesToCurrentTurn','FnheE','OtbTurnOrderEnemyIcon','MAYzQ','Scene_Battle_commandFight','onTurnEnd','loadSystem','WcDnM','createAllWindows','_forcedBattlers','IconIndex','ScreenBuffer','processTurn','Game_Battler_onBattleEnd','Game_Action_allowRandomSpeed','getChildIndex','_tempActor','boxWidth','changeEnemyGraphicBitmap','unshift','BattleManager_setup','Instant','isHorz','290769sTWFNM','_spriteContainer','isActor','iBfjn','onEnemyOk','isInfinitySpeedOTB','Scene_Battle_createAllWindows','indexOf','FaceIndex','jPfAF','FUNC','_containerHeight','_actionBattlers','EnemyBattlerFontFace','getStateTooltipBattler','otbPreviewOrderClear','isBattleMember','Scene_Battle_commandCancel','RepositionTopHelpY','ZiQBo','active','getColor','UiCurrentText','ETvJk','processSpriteRemoval','addBattlerToTurnOrderAtEnd','UiNextOffsetX','adWqF','isOTB','constructor','setItem','EVAL','HtvdH','onBattleStartOTB','select','isTurnBased','Scene_Battle_onEnemyCancel','canMove','Qeiow','shiftNextTurnSpritesToCurrentTurn','nNiFu','_preemptive','Wlfim','2214500WyYfrj','actor','clamp','Game_Party_removeActor','onBattleEndOTB','OTB_ADDED_ACTION_TIMES','XKbPB','_otbTurnOrderIconIndex','battlerHue','11474vlLGlI','opacity','_graphicFaceIndex','KdMNx','Enemy','_isAlive','PreviewScale','battler','WDdAN','updateVisibility','UiNextText','AkVoU','enemy','isBattleItemWindowOTB','BattleManager_endAction','SpriteThin','parse','shift','setGuard','onActorCancel','EnemyBattlerFaceIndex','requestUpdateTurnOrders','dtBaZ','makeActions','_otbTurnOrderVisible','_fadeTarget','fHTZV','selectNextActor','cfoLx','battleMembers','_currentX','height','_handlers','eUzMv','faceWidth','_instance','left','containerWindow','moveToPosition','42xvGUBC','SdQUc','UCcKO','iconHeight','UiSubjectOffsetX','push','return\x200','applyGlobal','faceName','svactor','AllowRandomSpeed','SubjectDistance','blt','ConvertSpeedJS','BattleManager_isTpb','otbShiftTurnOrderForSubject','BattleManager_getNextSubject','isSceneBattle','addChildAt','min','BorderThickness','UiSubjectText','map','onTurnEndOTB','padding','STR','processTurnOTB','_graphicType','createTurnOrderSprites','addState','MRmwy','onItemOk','EWXbT','cRfcM','otbCalcTargetNextOrderChange','speed','_scene','isEnemy','splice','OTB_STUN_INFINITY_SPEED','currentAction','subject','KMbsJ','numActions','isUsingSideviewUiLayout','updateOpacity','center','addBattlerToTurnOrderAtStart','_positionTargetX','BgImageOffsetX','bitmapWidth','contents','isNextOtbSubject','transparent','getStateIdWithName','onItemCancel','zKMXZ','defaultPosition','VwIAk','1013903YODgoS','isTpb','mainSprite','removeCurrentSubject','makeDeepCopy','addActor','MTxds','_targetHomeY','Scene_Battle_onSkillCancel','Conversion','isBattleSystemOTBTurnOrderVisible','addChild','TargetFollOrder','otbAddBattlerToTurnOrderAtEnd','additionalTargetXAdjustments','KXNuz','_fadeSpeed','osLtX','_homeX','_nextX','_graphicFaceName','startFade','LuRXj','yPQCR','_statusWindow','jzVdF','dataId','ARRAYNUM','Game_Party_addActor','oKnCm','createTurnOrderOTBGraphicType','startInput','Actors','createChildren','FUrml','Game_Action_applyGlobal','RjVeV','_homeY','CTPca','loadSvEnemy','bind','otbReturnBattlerToTurnOrders','startInputOTB','ARRAYEVAL','_letter','fillRect','_tempBattler','_isBattleOver','OTB_STUN_INFINITY_CLAMP','setOTBGraphicIconIndex','%1\x20is\x20incorrectly\x20placed\x20on\x20the\x20plugin\x20list.\x0aIt\x20is\x20a\x20Tier\x20%2\x20plugin\x20placed\x20over\x20other\x20Tier\x20%3\x20plugins.\x0aPlease\x20reorder\x20the\x20plugin\x20list\x20from\x20smallest\x20to\x20largest\x20tier\x20numbers.','faceHeight','removeActionBattlersOTB','onSkillOk','calculateTargetIndex','_homeDuration','TurnOrderOTBGraphicIconIndex','endAction','preEndActionOTB','setAttack','wOvtU','description','Game_Actor_selectNextCommand','getBorderThickness','onBattleEnd','RPBqA','_backgroundSprite','RrxWO','_isAppeared','refreshTurnOrder','axSPz','Game_Action_applyItemUserEffect','TurnOrderOTBGraphicFaceIndex','PreviewActor','otbCreateNewTurnOrderSprites','BattleManager_isActiveTpb','ceil','makeActionOrdersOTB','Window_Selectable_select','isPartyCommandWindowDisabled','icon','version','createOrderPreviewSprite','GetAllIndicies','_ogWindowLayerX','gradientFillRect','Scene_Battle_actorCommandSingleSkill','BattleManager_selectNextActor','isSideView','commandCancelOTB','%1\x20is\x20missing\x20a\x20required\x20plugin.\x0aPlease\x20install\x20%2\x20into\x20the\x20Plugin\x20Manager.','InitialSpeedJS','Scene_Battle_onSkillOk','rzzCK','XQXax','performCollapse','yuQiB','rivFF','KTcuQ','otbUnshiftBattlerToTurnOrders','_surprise','ShowMarkerBorder','drawUiText','cgyBx','_graphicSv','BgImageFilename','JDpyZ','sortContainer','_enemyWindow','canInput','applyItemTargetEffectOTB','length','_otb_actionBattlersNext','_fadeDuration','Pynri','ZFtBX','%1BorderColor','Mpplj','euGDO','clear','bitmap','includes','Settings','setSkill','ConvertParams','createOrderPreview','BattleManager_startInput','hbtVG','postEndActionOTB','_nextTurn','ActionBattlersFilter','OTB_CONVERT_AGI_BUFF_CURRENT_TURN','_actorWindow','qhugV','otbProcessActionCheck','faceIndex','makeSpeed','2tzofhw','attack','clearTurnOrderOTBGraphics','BattleManager_endTurn','isAppeared','MoveDistance','toUpperCase','item','TurnOrderOTBGraphicType','flAdC','%1BgColor2','JtDDU','XDULd','shiftTurnOrderForSubject','mpIFe','contentsOpacity','drawDimmedArea','gradient','_positionDuration','_speed','Enemies','updateTurnOrders','initBattleSystemOTB','appear','Scene_Battle_onItemOk','guard','Scene_Battle_onActorOk','inputtingAction','drawBgImage','endBattlerActions','sort','dimColor1','rhwWg','SpriteLength','%1\x27s\x20version\x20does\x20not\x20match\x20plugin\x27s.\x20Please\x20update\x20it\x20in\x20the\x20Plugin\x20Manager.','createNewTurnOrderSprites','allBattleMembers','OTB','checkOpacity','auto','otbRemoveUnableTurnOrderSprites','TnXCf','max','right','BattleManager_isTurnBased','RegExp','_offset','ConvertAgiDebuffCurrent','createTurnOrderOTBGraphicFaceName','floor','Ijuuk','bitmapHeight','fontSize','exit','getInfinityClamp','createSpriteContainers','RIXFP','Scene_Battle_onActorCancel','iXprq','OTB_CONVERT_AGI_BUFF_NEXT_TURN','Game_Battler_onBattleStart','commandFight','createActorCommandWindowOTB','TargetNextOrder','createLetterSprite','_requestTurnOrderUpdate','applyBattleItemWindowOTB','Mechanics','UiCurrentOffsetY','GzNvI','uADqG','dimColor2','setText','performActionEndOTB','createBackgroundSprite','otbCalcUserNextOrderChange','_graphicHue','_previewContainer','allowRandomSpeed','EnemyBattlerDrawLetter','rABly','refresh','mainFontFace','code','loadEnemy','effects','uBlgv','FBere','%1SystemBorder','TurnOrder','createTurnOrderOTBGraphicIconIndex','_plural','IconSet'];const _0x55ad4d=_0xbb96;(function(_0x1ef72b,_0x397802){const _0x54343b=_0xbb96;while(!![]){try{const _0x239af1=-parseInt(_0x54343b(0x11e))+-parseInt(_0x54343b(0x243))+parseInt(_0x54343b(0x1e1))*parseInt(_0x54343b(0x208))+-parseInt(_0x54343b(0x188))*-parseInt(_0x54343b(0x2cc))+parseInt(_0x54343b(0x1ad))+-parseInt(_0x54343b(0x333))+parseInt(_0x54343b(0x1d8));if(_0x239af1===_0x397802)break;else _0x1ef72b['push'](_0x1ef72b['shift']());}catch(_0x32a7cb){_0x1ef72b['push'](_0x1ef72b['shift']());}}}(_0x30f4,0xa006f));var label=_0x55ad4d(0x153),tier=tier||0x0,dependencies=[],pluginData=$plugins['filter'](function(_0x8d883e){const _0x36c442=_0x55ad4d;return _0x8d883e[_0x36c442(0x120)]&&_0x8d883e[_0x36c442(0x280)]['includes']('['+label+']');})[0x0];VisuMZ[label][_0x55ad4d(0x2bd)]=VisuMZ[label][_0x55ad4d(0x2bd)]||{},VisuMZ['ConvertParams']=function(_0x35d344,_0x7bcfeb){const _0xb46e55=_0x55ad4d;for(const _0x3ff651 in _0x7bcfeb){if(_0x3ff651[_0xb46e55(0x366)](/(.*):(.*)/i)){const _0x1eb410=String(RegExp['$1']),_0x54193e=String(RegExp['$2'])[_0xb46e55(0x2d2)]()['trim']();let _0x588ec3,_0x3c2d1f,_0x18df10;switch(_0x54193e){case _0xb46e55(0x155):_0x588ec3=_0x7bcfeb[_0x3ff651]!==''?Number(_0x7bcfeb[_0x3ff651]):0x0;break;case _0xb46e55(0x25e):_0x3c2d1f=_0x7bcfeb[_0x3ff651]!==''?JSON['parse'](_0x7bcfeb[_0x3ff651]):[],_0x588ec3=_0x3c2d1f[_0xb46e55(0x21e)](_0x132728=>Number(_0x132728));break;case _0xb46e55(0x1cc):_0x588ec3=_0x7bcfeb[_0x3ff651]!==''?eval(_0x7bcfeb[_0x3ff651]):null;break;case _0xb46e55(0x26e):_0x3c2d1f=_0x7bcfeb[_0x3ff651]!==''?JSON[_0xb46e55(0x1f1)](_0x7bcfeb[_0x3ff651]):[],_0x588ec3=_0x3c2d1f[_0xb46e55(0x21e)](_0x4de727=>eval(_0x4de727));break;case'JSON':_0x588ec3=_0x7bcfeb[_0x3ff651]!==''?JSON[_0xb46e55(0x1f1)](_0x7bcfeb[_0x3ff651]):'';break;case _0xb46e55(0x186):_0x3c2d1f=_0x7bcfeb[_0x3ff651]!==''?JSON['parse'](_0x7bcfeb[_0x3ff651]):[],_0x588ec3=_0x3c2d1f[_0xb46e55(0x21e)](_0x3c37f2=>JSON['parse'](_0x3c37f2));break;case _0xb46e55(0x1b7):_0x588ec3=_0x7bcfeb[_0x3ff651]!==''?new Function(JSON[_0xb46e55(0x1f1)](_0x7bcfeb[_0x3ff651])):new Function(_0xb46e55(0x20e));break;case'ARRAYFUNC':_0x3c2d1f=_0x7bcfeb[_0x3ff651]!==''?JSON['parse'](_0x7bcfeb[_0x3ff651]):[],_0x588ec3=_0x3c2d1f['map'](_0x1f1fbc=>new Function(JSON[_0xb46e55(0x1f1)](_0x1f1fbc)));break;case _0xb46e55(0x221):_0x588ec3=_0x7bcfeb[_0x3ff651]!==''?String(_0x7bcfeb[_0x3ff651]):'';break;case _0xb46e55(0x337):_0x3c2d1f=_0x7bcfeb[_0x3ff651]!==''?JSON[_0xb46e55(0x1f1)](_0x7bcfeb[_0x3ff651]):[],_0x588ec3=_0x3c2d1f[_0xb46e55(0x21e)](_0x28fd88=>String(_0x28fd88));break;case _0xb46e55(0x135):_0x18df10=_0x7bcfeb[_0x3ff651]!==''?JSON[_0xb46e55(0x1f1)](_0x7bcfeb[_0x3ff651]):{},_0x588ec3=VisuMZ[_0xb46e55(0x2bf)]({},_0x18df10);break;case _0xb46e55(0x12e):_0x3c2d1f=_0x7bcfeb[_0x3ff651]!==''?JSON['parse'](_0x7bcfeb[_0x3ff651]):[],_0x588ec3=_0x3c2d1f[_0xb46e55(0x21e)](_0x3ad9ee=>VisuMZ[_0xb46e55(0x2bf)]({},JSON[_0xb46e55(0x1f1)](_0x3ad9ee)));break;default:continue;}_0x35d344[_0x1eb410]=_0x588ec3;}}return _0x35d344;},(_0xbe20ec=>{const _0x1be638=_0x55ad4d,_0x7b58a1=_0xbe20ec[_0x1be638(0x12a)];for(const _0xe41eee of dependencies){if(!Imported[_0xe41eee]){alert(_0x1be638(0x29d)[_0x1be638(0x362)](_0x7b58a1,_0xe41eee)),SceneManager['exit']();break;}}const _0x464e26=_0xbe20ec[_0x1be638(0x280)];if(_0x464e26[_0x1be638(0x366)](/\[Version[ ](.*?)\]/i)){const _0x90b32b=Number(RegExp['$1']);_0x90b32b!==VisuMZ[label][_0x1be638(0x294)]&&(alert(_0x1be638(0x2ee)[_0x1be638(0x362)](_0x7b58a1,_0x90b32b)),SceneManager[_0x1be638(0x301)]());}if(_0x464e26[_0x1be638(0x366)](/\[Tier[ ](\d+)\]/i)){if(_0x1be638(0x139)===_0x1be638(0x2c8))this[_0x1be638(0x295)](_0x432bd9,![],_0x3268d7);else{const _0x21b280=Number(RegExp['$1']);_0x21b280<tier?'fMUFe'!=='qnNof'?(alert(_0x1be638(0x275)[_0x1be638(0x362)](_0x7b58a1,_0x21b280,tier)),SceneManager[_0x1be638(0x301)]()):this[_0x1be638(0x174)]=this[_0x1be638(0x2fc)]():tier=Math['max'](_0x21b280,tier);}}VisuMZ['ConvertParams'](VisuMZ[label][_0x1be638(0x2bd)],_0xbe20ec[_0x1be638(0x151)]);})(pluginData),PluginManager['registerCommand'](pluginData['name'],_0x55ad4d(0x378),_0x260679=>{const _0x4fe424=_0x55ad4d;VisuMZ[_0x4fe424(0x2bf)](_0x260679,_0x260679);const _0x57539f=_0x260679[_0x4fe424(0x263)],_0x83200d=_0x260679[_0x4fe424(0x1a0)];for(const _0x38dd4b of _0x57539f){if(_0x4fe424(0x13f)!==_0x4fe424(0x13f))_0x3fbd8e[_0x4fe424(0x26c)](this);else{const _0x205a54=$gameActors[_0x4fe424(0x1d9)](_0x38dd4b);if(!_0x205a54)continue;_0x205a54['_otbTurnOrderGraphicType']=_0x4fe424(0x293),_0x205a54[_0x4fe424(0x1df)]=_0x83200d;}}}),PluginManager['registerCommand'](pluginData[_0x55ad4d(0x12a)],_0x55ad4d(0x165),_0x3ca958=>{const _0x559fd9=_0x55ad4d;VisuMZ[_0x559fd9(0x2bf)](_0x3ca958,_0x3ca958);const _0x44ac27=_0x3ca958['Actors'],_0x253935=_0x3ca958['FaceName'],_0x2a8317=_0x3ca958[_0x559fd9(0x1b5)];for(const _0x3ece0f of _0x44ac27){const _0x2e063a=$gameActors[_0x559fd9(0x1d9)](_0x3ece0f);if(!_0x2e063a)continue;_0x2e063a['_otbTurnOrderGraphicType']=_0x559fd9(0x36f),_0x2e063a[_0x559fd9(0x174)]=_0x253935,_0x2e063a[_0x559fd9(0x356)]=_0x2a8317;}}),PluginManager[_0x55ad4d(0x374)](pluginData[_0x55ad4d(0x12a)],_0x55ad4d(0x35a),_0xdf3e4d=>{const _0x340751=_0x55ad4d;VisuMZ['ConvertParams'](_0xdf3e4d,_0xdf3e4d);const _0x2b3729=_0xdf3e4d['Actors'];for(const _0x439608 of _0x2b3729){if(_0x340751(0x269)!==_0x340751(0x12d)){const _0x419db6=$gameActors['actor'](_0x439608);if(!_0x419db6)continue;_0x419db6[_0x340751(0x2ce)]();}else this[_0x340751(0x15d)]=this['_currentTurn'][_0x42a0d9],this[_0x340751(0x109)][_0x5ae3db][_0x340751(0x330)](),this[_0x340751(0x109)]['splice'](_0x2a96b1,0x1);}}),PluginManager[_0x55ad4d(0x374)](pluginData[_0x55ad4d(0x12a)],_0x55ad4d(0x198),_0x1546d4=>{const _0x4f668a=_0x55ad4d;VisuMZ[_0x4f668a(0x2bf)](_0x1546d4,_0x1546d4);const _0x46d446=_0x1546d4[_0x4f668a(0x2e0)],_0x4b366b=_0x1546d4[_0x4f668a(0x1a0)];for(const _0x1d15bf of _0x46d446){if('mSTMw'!==_0x4f668a(0x37d)){const _0x32cf77=$gameTroop[_0x4f668a(0x12f)]()[_0x1d15bf];if(!_0x32cf77)continue;_0x32cf77[_0x4f668a(0x338)]='icon',_0x32cf77[_0x4f668a(0x1df)]=_0x4b366b;}else _0x2943fd[_0x4f668a(0x26c)](this);}}),PluginManager[_0x55ad4d(0x374)](pluginData['name'],'OtbTurnOrderEnemyFace',_0x3a7376=>{const _0x1f6b0c=_0x55ad4d;VisuMZ['ConvertParams'](_0x3a7376,_0x3a7376);const _0x12d3b2=_0x3a7376[_0x1f6b0c(0x2e0)],_0x5eb501=_0x3a7376[_0x1f6b0c(0x178)],_0x293c59=_0x3a7376[_0x1f6b0c(0x1b5)];for(const _0x5e3898 of _0x12d3b2){const _0x1b66c3=$gameTroop[_0x1f6b0c(0x12f)]()[_0x5e3898];if(!_0x1b66c3)continue;_0x1b66c3[_0x1f6b0c(0x338)]=_0x1f6b0c(0x36f),_0x1b66c3[_0x1f6b0c(0x174)]=_0x5eb501,_0x1b66c3['_otbTurnOrderFaceIndex']=_0x293c59;}}),PluginManager['registerCommand'](pluginData[_0x55ad4d(0x12a)],_0x55ad4d(0x11a),_0x4e7531=>{const _0x5b6ffe=_0x55ad4d;VisuMZ[_0x5b6ffe(0x2bf)](_0x4e7531,_0x4e7531);const _0x43a41b=_0x4e7531[_0x5b6ffe(0x2e0)];for(const _0x2f4a12 of _0x43a41b){if(_0x5b6ffe(0x1ec)!=='AkVoU')_0x187577[_0x5b6ffe(0x153)][_0x5b6ffe(0x379)][_0x5b6ffe(0x175)](this),_0x197b95[_0x5b6ffe(0x277)]();else{const _0x3facda=$gameTroop[_0x5b6ffe(0x12f)]()[_0x2f4a12];if(!_0x3facda)continue;_0x3facda['clearTurnOrderOTBGraphics']();}}}),PluginManager[_0x55ad4d(0x374)](pluginData[_0x55ad4d(0x12a)],_0x55ad4d(0x157),_0x12716f=>{const _0x44ab58=_0x55ad4d;VisuMZ[_0x44ab58(0x2bf)](_0x12716f,_0x12716f);const _0x4decd8=_0x12716f[_0x44ab58(0x140)];$gameSystem[_0x44ab58(0x34d)](_0x4decd8);}),VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2f9)]={'Instant':/<OTB (?:INSTANT|INSTANT CAST|INSTANT USE)>/i,'UserFollOrder':/<OTB USER FOLLOW TURN: ([\+\-]\d+)>/i,'UserCurrOrder':/<OTB USER CURRENT TURN: ([\+\-]\d+)>/i,'UserNextOrder':/<OTB USER NEXT TURN: ([\+\-]\d+)>/i,'TargetFollOrder':/<OTB TARGET FOLLOW TURN: ([\+\-]\d+)>/i,'TargetCurrOrder':/<OTB TARGET CURRENT TURN: ([\+\-]\d+)>/i,'TargetNextOrder':/<OTB TARGET NEXT TURN: ([\+\-]\d+)>/i},DataManager[_0x55ad4d(0x23e)]=function(_0x1df6a8){const _0x4fd471=_0x55ad4d;_0x1df6a8=_0x1df6a8[_0x4fd471(0x2d2)]()[_0x4fd471(0x110)](),this[_0x4fd471(0x193)]=this[_0x4fd471(0x193)]||{};if(this[_0x4fd471(0x193)][_0x1df6a8])return this['_stateIDs'][_0x1df6a8];for(const _0x3b5460 of $dataStates){if('lSbLM'!==_0x4fd471(0x228)){if(!_0x3b5460)continue;this[_0x4fd471(0x193)][_0x3b5460[_0x4fd471(0x12a)][_0x4fd471(0x2d2)]()['trim']()]=_0x3b5460['id'];}else this[_0x4fd471(0x150)]=_0x44d734[_0x4fd471(0x108)](),_0x574c63=_0x35348a[_0x4fd471(0x26a)](this['_graphicEnemy']),_0x4473c4['addLoadListener'](this['changeEnemyGraphicBitmap'][_0x4fd471(0x26b)](this,_0x1d4420));}return this[_0x4fd471(0x193)][_0x1df6a8]||0x0;},SceneManager[_0x55ad4d(0x219)]=function(){const _0x2b420b=_0x55ad4d;return this[_0x2b420b(0x22c)]&&this['_scene'][_0x2b420b(0x1ca)]===Scene_Battle;},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x1aa)]=BattleManager[_0x55ad4d(0x192)],BattleManager[_0x55ad4d(0x192)]=function(_0x3cc1d1,_0x5ab4c7,_0x2c640a){const _0x1893fe=_0x55ad4d;VisuMZ[_0x1893fe(0x153)][_0x1893fe(0x1aa)]['call'](this,_0x3cc1d1,_0x5ab4c7,_0x2c640a),this['initMembersOTB']();},BattleManager[_0x55ad4d(0x18f)]=function(){const _0xc950ce=_0x55ad4d;if(!this[_0xc950ce(0x1c9)]())return;this[_0xc950ce(0x2b3)]=[],this[_0xc950ce(0x102)]=![];},VisuMZ[_0x55ad4d(0x153)]['BattleManager_battleSys']=BattleManager[_0x55ad4d(0x133)],BattleManager[_0x55ad4d(0x133)]=function(){const _0x4ad13c=_0x55ad4d;if(this[_0x4ad13c(0x1c9)]())return _0x4ad13c(0x2f1);return VisuMZ['BattleSystemOTB'][_0x4ad13c(0x329)][_0x4ad13c(0x175)](this);},BattleManager[_0x55ad4d(0x1c9)]=function(){const _0x249e19=_0x55ad4d;return $gameSystem['getBattleSystem']()===_0x249e19(0x2f1);},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x216)]=BattleManager[_0x55ad4d(0x244)],BattleManager[_0x55ad4d(0x244)]=function(){const _0xac2743=_0x55ad4d;if(this[_0xac2743(0x1c9)]())return![];return VisuMZ[_0xac2743(0x153)][_0xac2743(0x216)][_0xac2743(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x28e)]=BattleManager['isActiveTpb'],BattleManager[_0x55ad4d(0x34a)]=function(){const _0x2e2190=_0x55ad4d;if(this[_0x2e2190(0x1c9)]())return![];return VisuMZ['BattleSystemOTB'][_0x2e2190(0x28e)][_0x2e2190(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2f8)]=BattleManager[_0x55ad4d(0x1d0)],BattleManager['isTurnBased']=function(){const _0x22a9d5=_0x55ad4d;if(this[_0x22a9d5(0x1c9)]())return!![];return VisuMZ[_0x22a9d5(0x153)]['BattleManager_isTurnBased'][_0x22a9d5(0x175)](this);},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x2c1)]=BattleManager[_0x55ad4d(0x262)],BattleManager['startInput']=function(){const _0x4bbe9c=_0x55ad4d;VisuMZ[_0x4bbe9c(0x153)]['BattleManager_startInput']['call'](this),this['isOTB']()&&$gameParty['canInput']()&&!this[_0x4bbe9c(0x2a7)]&&this[_0x4bbe9c(0x26d)]();},BattleManager[_0x55ad4d(0x26d)]=function(){this['startTurn']();},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x363)]=BattleManager[_0x55ad4d(0x1a2)],BattleManager[_0x55ad4d(0x1a2)]=function(){const _0xcdddf4=_0x55ad4d;this[_0xcdddf4(0x1c9)]()?this[_0xcdddf4(0x222)]():_0xcdddf4(0x10f)!==_0xcdddf4(0x1cd)?VisuMZ[_0xcdddf4(0x153)][_0xcdddf4(0x363)][_0xcdddf4(0x175)](this):_0x4386d5['BattleSystemOTB'][_0xcdddf4(0x29a)]['call'](this);},BattleManager[_0x55ad4d(0x222)]=function(){const _0x14ad97=_0x55ad4d,_0x4fc71b=this['_subject'];if(_0x4fc71b[_0x14ad97(0x1af)]()&&_0x4fc71b[_0x14ad97(0x2b0)]()){if(_0x14ad97(0x289)===_0x14ad97(0x284)){const _0x20ceb8=this[_0x14ad97(0x1d9)]()[_0x14ad97(0x350)];if(_0x20ceb8['match'](/<OTB TURN ORDER ICON:[ ](\d+)>/i))return _0x3e5c15(_0x4a359b['$1']);return _0x3ca4e9[_0x14ad97(0x2bd)][_0x14ad97(0x14a)];}else{const _0x41be30=_0x4fc71b[_0x14ad97(0x230)]();if(!_0x41be30)_0x14ad97(0x166)==='cmMBB'?this[_0x14ad97(0x191)](...arguments):VisuMZ[_0x14ad97(0x153)][_0x14ad97(0x363)]['call'](this);else _0x41be30['_forceAction']?VisuMZ[_0x14ad97(0x153)][_0x14ad97(0x363)]['call'](this):(this['_currentActor']=_0x4fc71b,this['startActorInput']());}}else VisuMZ[_0x14ad97(0x153)][_0x14ad97(0x363)][_0x14ad97(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x146)]=BattleManager[_0x55ad4d(0x10e)],BattleManager[_0x55ad4d(0x10e)]=function(){const _0x2e60f6=_0x55ad4d;this[_0x2e60f6(0x1c9)]()?_0x2e60f6(0x125)===_0x2e60f6(0x125)?VisuMZ['BattleSystemOTB']['BattleManager_processTurn'][_0x2e60f6(0x175)](this):(_0x4bf1b3[_0x2e60f6(0x1bc)](),_0x411db6[_0x2e60f6(0x153)][_0x2e60f6(0x106)][_0x2e60f6(0x175)](this)):VisuMZ[_0x2e60f6(0x153)][_0x2e60f6(0x146)][_0x2e60f6(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x29a)]=BattleManager[_0x55ad4d(0x1fc)],BattleManager[_0x55ad4d(0x1fc)]=function(){const _0x3acce3=_0x55ad4d;this[_0x3acce3(0x1c9)]()?this[_0x3acce3(0x37a)]():VisuMZ['BattleSystemOTB'][_0x3acce3(0x29a)][_0x3acce3(0x175)](this);},BattleManager[_0x55ad4d(0x37a)]=function(){this['_currentActor']=null,this['_inputting']=![];},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x1ef)]=BattleManager['endAction'],BattleManager[_0x55ad4d(0x27c)]=function(){const _0x456187=_0x55ad4d;this[_0x456187(0x27d)](),VisuMZ[_0x456187(0x153)][_0x456187(0x1ef)][_0x456187(0x175)](this),this[_0x456187(0x2c3)]();},BattleManager['preEndActionOTB']=function(){const _0x2ee61c=_0x55ad4d;if(!this[_0x2ee61c(0x1c9)]())return;this[_0x2ee61c(0x277)](),this[_0x2ee61c(0x15d)]&&(_0x2ee61c(0x2b5)===_0x2ee61c(0x141)?this['isOTB']()?_0x15172b['BattleSystemOTB'][_0x2ee61c(0x363)][_0x2ee61c(0x175)](this):_0x3f09f2[_0x2ee61c(0x153)][_0x2ee61c(0x146)][_0x2ee61c(0x175)](this):this[_0x2ee61c(0x15d)]['performActionEndOTB']()),this[_0x2ee61c(0x15d)]&&this[_0x2ee61c(0x15d)][_0x2ee61c(0x1d2)]()&&this[_0x2ee61c(0x1b9)][_0x2ee61c(0x2bc)](this[_0x2ee61c(0x15d)])&&this[_0x2ee61c(0x15d)][_0x2ee61c(0x1f8)]();},BattleManager['postEndActionOTB']=function(){const _0x1cedc2=_0x55ad4d;if(!this[_0x1cedc2(0x1c9)]())return;this[_0x1cedc2(0x277)]();if(this[_0x1cedc2(0x15d)]){if(_0x1cedc2(0x1fb)!==_0x1cedc2(0x17a))this[_0x1cedc2(0x2e9)](this[_0x1cedc2(0x15d)]),this[_0x1cedc2(0x15d)]=null;else{if(_0x57dec8[_0x1cedc2(0x17b)])_0xba4386+=0x1;}}this[_0x1cedc2(0x19f)][_0x1cedc2(0x2b2)]>0x0&&(this[_0x1cedc2(0x15d)]=this[_0x1cedc2(0x37b)]());;},BattleManager[_0x55ad4d(0x1dd)]=VisuMZ['BattleSystemOTB'][_0x55ad4d(0x2bd)]['Mechanics']['EnableActionTimes'],BattleManager[_0x55ad4d(0x382)]=VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2bd)][_0x55ad4d(0x30f)]['RandomizeActionTimesOrder'],BattleManager[_0x55ad4d(0x273)]=VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2bd)]['Mechanics']['InfinityClamp'],VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x33d)]=BattleManager[_0x55ad4d(0x11b)],BattleManager[_0x55ad4d(0x11b)]=function(){const _0x1ea865=_0x55ad4d;if(this['isOTB']()){if(_0x1ea865(0x346)!=='QwBDy')this[_0x1ea865(0x290)]();else return _0x19c3f8[_0x1ea865(0x1c9)]();}else VisuMZ['BattleSystemOTB']['BattleManager_makeActionOrders'][_0x1ea865(0x175)](this);},BattleManager[_0x55ad4d(0x290)]=function(){const _0x4f26d9=_0x55ad4d;let _0x5a6a97=this[_0x4f26d9(0x102)]?0x1:0x2;while(_0x5a6a97--){this[_0x4f26d9(0x37f)]();}const _0x2d1c5c=!this[_0x4f26d9(0x102)];this[_0x4f26d9(0x102)]=!![];},BattleManager[_0x55ad4d(0x37f)]=function(){const _0x168ee9=_0x55ad4d;this[_0x168ee9(0x1b9)]=this['_otb_actionBattlersNext'],this['otbShiftNextTurnSpritesToCurrentTurn']();const _0x208f3e=[];_0x208f3e[_0x168ee9(0x20d)](...$gameParty[_0x168ee9(0x1fe)]()),_0x208f3e[_0x168ee9(0x20d)](...$gameTroop[_0x168ee9(0x12f)]());for(const _0x13a650 of _0x208f3e){_0x13a650['makeSpeed']();}_0x208f3e[_0x168ee9(0x2ea)]((_0x306872,_0x5a23a9)=>_0x5a23a9[_0x168ee9(0x22b)]()-_0x306872[_0x168ee9(0x22b)]()),this['_otb_actionBattlersNext']=_0x208f3e,this[_0x168ee9(0x347)](),this[_0x168ee9(0x277)](),this[_0x168ee9(0x28d)]();},BattleManager[_0x55ad4d(0x347)]=function(){const _0x1278c7=_0x55ad4d;if(!BattleManager[_0x1278c7(0x1dd)])return;const _0x2f2061=this[_0x1278c7(0x2b3)],_0x3dece0=this[_0x1278c7(0x2f0)]();for(const _0x1ec11f of _0x3dece0){if(_0x1278c7(0x1e9)===_0x1278c7(0x2ad))return _0x2691d9[_0x1278c7(0xfe)]?-_0x2ee142[_0x1278c7(0x1f0)]:_0xc383ee['SpriteThin'];else{if(!_0x1ec11f)continue;if(!_0x1ec11f[_0x1278c7(0x2d0)]())continue;if(!_0x1ec11f[_0x1278c7(0x13c)]())continue;if(!_0x2f2061['includes'](_0x1ec11f))continue;const _0x4c18aa=_0x2f2061[_0x1278c7(0x1b4)](_0x1ec11f);let _0x64e756=_0x1ec11f[_0x1278c7(0x387)]()-0x1;while(_0x64e756--){let _0x590a28=_0x4c18aa;BattleManager[_0x1278c7(0x382)]&&(_0x590a28=Math[_0x1278c7(0x353)](_0x2f2061[_0x1278c7(0x2b2)]-_0x4c18aa)+_0x4c18aa),_0x2f2061['splice'](_0x590a28,0x0,_0x1ec11f);}}}},BattleManager[_0x55ad4d(0x277)]=function(){const _0xbe1472=_0x55ad4d;if(!this[_0xbe1472(0x1c9)]())return;this[_0xbe1472(0x1b9)]=this['_actionBattlers']||[],this['_actionBattlers']['remove'](null),this['_actionBattlers'][_0xbe1472(0x368)](undefined),this['_actionBattlers']=this[_0xbe1472(0x1b9)]['filter'](_0xc6aa72=>_0xc6aa72[_0xbe1472(0x1bd)]()),this[_0xbe1472(0x1b9)]=this[_0xbe1472(0x1b9)][_0xbe1472(0x34f)](_0x49c925=>VisuMZ['BattleSystemOTB'][_0xbe1472(0x2c5)](_0x49c925)),this['_surprise']&&(_0xbe1472(0x1de)!==_0xbe1472(0x304)?this[_0xbe1472(0x1b9)]=this[_0xbe1472(0x1b9)][_0xbe1472(0x34f)](_0x1d021d=>!_0x1d021d['isActor']()):(this[_0xbe1472(0x32b)](_0x2df85a,_0x14e170,_0x279bbb),_0x1c8ae1[_0xbe1472(0x159)][_0xbe1472(0x191)]['call'](this),this[_0xbe1472(0x1e2)]=0x0,this[_0xbe1472(0x264)](),this[_0xbe1472(0x2f2)]())),this[_0xbe1472(0x1d6)]&&(this['_actionBattlers']=this['_actionBattlers'][_0xbe1472(0x34f)](_0x2beb51=>!_0x2beb51[_0xbe1472(0x22d)]())),this['_otb_actionBattlersNext']=this['_otb_actionBattlersNext']||[],this[_0xbe1472(0x2b3)][_0xbe1472(0x368)](null),this[_0xbe1472(0x2b3)][_0xbe1472(0x368)](undefined),this[_0xbe1472(0x2b3)]=this[_0xbe1472(0x2b3)]['filter'](_0x237a87=>_0x237a87['isBattleMember']()),this[_0xbe1472(0x2b3)]=this[_0xbe1472(0x2b3)]['filter'](_0x47c2d0=>VisuMZ[_0xbe1472(0x153)][_0xbe1472(0x126)](_0x47c2d0)),this[_0xbe1472(0x2f4)](),this[_0xbe1472(0x288)]();},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2c5)]=function(_0x878377){const _0x4311f3=_0x55ad4d;if(!_0x878377)return![];if(!_0x878377[_0x4311f3(0x13c)]())return![];if(!_0x878377[_0x4311f3(0x2d0)]())return![];return _0x878377[_0x4311f3(0x1d2)]();},VisuMZ[_0x55ad4d(0x153)]['ActionBattlersNextFilter']=function(_0x312b15){const _0x42bb04=_0x55ad4d;if(!_0x312b15)return![];const _0x4c6907=JsonEx[_0x42bb04(0x247)](_0x312b15);return _0x4c6907['_tempActor']=!![],_0x4c6907[_0x42bb04(0x271)]=!![],_0x4c6907[_0x42bb04(0x36c)](),_0x4c6907['removeStatesAuto'](0x1),_0x4c6907[_0x42bb04(0x354)](0x2),_0x4c6907[_0x42bb04(0x31d)](),VisuMZ[_0x42bb04(0x153)][_0x42bb04(0x2c5)](_0x4c6907);},BattleManager[_0x55ad4d(0x195)]=function(_0x908c97,_0x264ace,_0x18481b){const _0x359266=_0x55ad4d;if(!_0x264ace)return;const _0x550319=_0x18481b?this['_otb_actionBattlersNext']:this[_0x359266(0x1b9)];if(!_0x550319)return;if(!_0x550319['includes'](_0x908c97))return;const _0x3df0c2=VisuMZ[_0x359266(0x153)][_0x359266(0x296)](_0x908c97,_0x550319),_0x5bc00b=_0x18481b?VisuMZ[_0x359266(0x153)]['getInfinityClamp'](_0x550319):0x0,_0x5603db=_0x3df0c2['length']-0x1;for(let _0x9344a7=_0x5603db;_0x9344a7>=0x0;_0x9344a7--){_0x550319['splice'](_0x3df0c2[_0x9344a7],0x1);}for(var _0x2e2732=0x0;_0x2e2732<_0x3df0c2[_0x359266(0x2b2)];_0x2e2732++){var _0x347e2e=(_0x3df0c2[_0x2e2732]-_0x264ace)[_0x359266(0x1da)](_0x5bc00b,_0x550319[_0x359266(0x2b2)]);_0x550319['splice'](_0x347e2e,0x0,_0x908c97);}this[_0x359266(0x277)](),this['refreshTurnOrder']();},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x296)]=function(_0x13065f,_0x1aa211){const _0x1896f0=_0x55ad4d,_0x995a36=[],_0x28cf4e=_0x1aa211[_0x1896f0(0x2b2)];for(let _0x3b084c=0x0;_0x3b084c<_0x28cf4e;_0x3b084c++){if(_0x1aa211[_0x3b084c]===_0x13065f)_0x995a36[_0x1896f0(0x20d)](_0x3b084c);}return _0x995a36;},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x302)]=function(_0x185152){const _0x3f0de5=_0x55ad4d;if(!BattleManager[_0x3f0de5(0x273)])return 0x0;if(!_0x185152)return 0x0;let _0x484561=0x0;const _0x2042ff=_0x185152['length'];for(let _0x549c60=0x0;_0x549c60<_0x2042ff;_0x549c60++){if(_0x3f0de5(0x312)===_0x3f0de5(0x312)){const _0x51257d=_0x185152[_0x549c60];if(!_0x51257d)continue;if(_0x51257d[_0x3f0de5(0x22b)]()!==Infinity)return _0x549c60;else _0x484561++;}else _0x956ba4[_0x3f0de5(0x101)]&&_0x3bc32f[_0x3f0de5(0x101)][_0x3f0de5(0x368)](_0x5b3ac0),this['_spriteContainer'][_0x3f0de5(0x168)](_0x36e8bf),this[_0x3f0de5(0x319)]['removeChild'](_0x5a32fe);}return _0x484561;},BattleManager[_0x55ad4d(0x196)]=function(){const _0x2a818c=_0x55ad4d;if(!this[_0x2a818c(0x1c9)]())return;const _0x440885=SceneManager[_0x2a818c(0x22c)][_0x2a818c(0x13e)];if(!_0x440885)return;_0x440885[_0x2a818c(0x1d4)]();},BattleManager[_0x55ad4d(0x28d)]=function(){const _0x174f60=_0x55ad4d;if(!this[_0x174f60(0x1c9)]())return;const _0x448419=SceneManager[_0x174f60(0x22c)][_0x174f60(0x13e)];if(!_0x448419)return;_0x448419[_0x174f60(0x2ef)]();},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x218)]=BattleManager[_0x55ad4d(0x37b)],BattleManager[_0x55ad4d(0x37b)]=function(){const _0x2ca7dc=_0x55ad4d;return this[_0x2ca7dc(0x15d)]=VisuMZ[_0x2ca7dc(0x153)][_0x2ca7dc(0x218)][_0x2ca7dc(0x175)](this),this[_0x2ca7dc(0x1c9)]()&&this['_subject']&&this[_0x2ca7dc(0x217)](this[_0x2ca7dc(0x15d)]),this[_0x2ca7dc(0x15d)];},BattleManager[_0x55ad4d(0x217)]=function(_0x235d94){const _0x412586=_0x55ad4d;if(!this[_0x412586(0x1c9)]())return;const _0x366229=SceneManager['_scene']['_otbTurnOrderWindow'];if(!_0x366229)return;if(!_0x235d94)return;_0x366229[_0x412586(0x2d9)](_0x235d94);},BattleManager[_0x55ad4d(0x288)]=function(){const _0x4536a5=_0x55ad4d;if(!this[_0x4536a5(0x1c9)]())return;const _0x509887=SceneManager[_0x4536a5(0x22c)][_0x4536a5(0x13e)];if(!_0x509887)return;_0x509887[_0x4536a5(0x1f6)]();},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2cf)]=BattleManager[_0x55ad4d(0x342)],BattleManager[_0x55ad4d(0x342)]=function(){const _0x28566e=_0x55ad4d;VisuMZ[_0x28566e(0x153)][_0x28566e(0x2cf)]['call'](this);if(this[_0x28566e(0x1c9)]()){if(_0x28566e(0x171)!=='lyBop')this['otbRemoveCurrentSubject']();else{_0x46822d['BattleSystemOTB'][_0x28566e(0x25f)][_0x28566e(0x175)](this,_0x105b48);if(_0x4a197f[_0x28566e(0x111)])return;_0x2ffd8d['isSceneBattle']()&&_0x48e590['isOTB']()&&(_0x1ba4eb[_0x28566e(0x277)](),_0x47f692[_0x28566e(0x26c)](_0x3b661b[_0x28566e(0x1d9)](_0x54c16f)));}}},BattleManager[_0x55ad4d(0x184)]=function(){const _0x4f3ef5=_0x55ad4d;if(!this[_0x4f3ef5(0x1c9)]())return;const _0x3d9563=SceneManager[_0x4f3ef5(0x22c)][_0x4f3ef5(0x13e)];if(!_0x3d9563)return;_0x3d9563[_0x4f3ef5(0x246)]();},BattleManager[_0x55ad4d(0x2f4)]=function(){const _0x3c016e=_0x55ad4d;if(!this[_0x3c016e(0x1c9)]())return;const _0x23a217=SceneManager[_0x3c016e(0x22c)][_0x3c016e(0x13e)];if(!_0x23a217)return;_0x23a217['removeUnableTurnOrderSprites']();},BattleManager['otbReturnBattlerToTurnOrders']=function(_0x820210){const _0x3dec62=_0x55ad4d;if(!_0x820210)return;const _0x79719e=_0x820210[_0x3dec62(0x387)]();_0x820210[_0x3dec62(0x1f8)]();if(!this['_actionBattlers']['includes'](_0x820210)){const _0xc10234=Math[_0x3dec62(0x2f6)](0x0,_0x79719e-(_0x820210['_otbTimesActedThisTurn']||0x0));this['otbAddBattlerToTurnOrderAtEnd'](_0x820210,_0xc10234,this['_actionBattlers']);}if(!this[_0x3dec62(0x2b3)][_0x3dec62(0x2bc)](_0x820210)){const _0x8de79e=_0x79719e;this['otbAddBattlerToTurnOrderAtEnd'](_0x820210,_0x8de79e,this[_0x3dec62(0x2b3)]);}},BattleManager[_0x55ad4d(0x250)]=function(_0x2a1a00,_0x543611,_0x1f67f8){const _0x500f32=_0x55ad4d;if(!this['isOTB']())return;const _0x2839a8=SceneManager['_scene'][_0x500f32(0x13e)];while(_0x543611--){if(_0x500f32(0x1b0)===_0x500f32(0x1b0))_0x1f67f8['push'](_0x2a1a00),_0x2839a8&&_0x2839a8[_0x500f32(0x1c6)](_0x2a1a00,_0x1f67f8);else return _0x4b8c5a['isOTB']()?_0x59e414[_0x500f32(0x153)][_0x500f32(0x2bd)][_0x500f32(0x30f)]['AllowRandomSpeed']:_0x1f889d['BattleSystemOTB'][_0x500f32(0x1a4)]['call'](this);}},BattleManager[_0x55ad4d(0x2a6)]=function(_0x57e0aa){const _0x87e699=_0x55ad4d;if(!_0x57e0aa)return;const _0xf9f629=_0x57e0aa['makeActionTimes']();_0x57e0aa['makeActions']();if(!this[_0x87e699(0x1b9)][_0x87e699(0x2bc)](_0x57e0aa)){const _0x450969=Math[_0x87e699(0x2f6)](0x0,_0xf9f629-(_0x57e0aa[_0x87e699(0x38a)]||0x0));this[_0x87e699(0x237)](_0x57e0aa,_0x450969,this[_0x87e699(0x1b9)]);}if(!this[_0x87e699(0x2b3)][_0x87e699(0x2bc)](_0x57e0aa)){if('raVzJ'!==_0x87e699(0x18d))this[_0x87e699(0x2ab)]=_0xf21f71[_0x87e699(0x144)](),_0x243a27=_0x1a7697[_0x87e699(0x16a)](this['_graphicSv']),_0x21cd05['addLoadListener'](this['changeSvActorGraphicBitmap'][_0x87e699(0x26b)](this,_0x50c67c));else{const _0x50dfa1=_0xf9f629;this[_0x87e699(0x237)](_0x57e0aa,_0x50dfa1,this[_0x87e699(0x2b3)]);}}},BattleManager['otbAddBattlerToTurnOrderAtStart']=function(_0x3d46f6,_0x3b9c89,_0x4aeae0){const _0x124200=_0x55ad4d;if(!this[_0x124200(0x1c9)]())return;const _0x10c57f=SceneManager[_0x124200(0x22c)][_0x124200(0x13e)];while(_0x3b9c89--){_0x124200(0x323)===_0x124200(0x331)?(_0x211bfc['BattleSystemOTB'][_0x124200(0x1a3)][_0x124200(0x175)](this),this[_0x124200(0x1dc)]()):(_0x4aeae0[_0x124200(0x1a9)](_0x3d46f6),_0x10c57f&&(_0x124200(0x254)===_0x124200(0x2a4)?_0x48e628[_0x124200(0x2bc)](_0x4fa15c)&&(_0x1d774f+=_0x3fa5fc(_0x23d40a['$1'])):_0x10c57f['addBattlerToTurnOrderAtStart'](_0x3d46f6,_0x4aeae0)));}},BattleManager['otbPreviewOrderClear']=function(){const _0x58eff3=_0x55ad4d;if(!this[_0x58eff3(0x1c9)]())return;const _0x35d47a=SceneManager[_0x58eff3(0x22c)]['_otbTurnOrderWindow'];if(!_0x35d47a)return;_0x35d47a['previewOrderByAction'](null);},BattleManager[_0x55ad4d(0x10c)]=function(){const _0x368937=_0x55ad4d;if(!this[_0x368937(0x1c9)]())return;const _0x2fe491=SceneManager['_scene'][_0x368937(0x13e)];if(!_0x2fe491)return;_0x2fe491[_0x368937(0x162)](this[_0x368937(0x2e7)]());},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x35e)]=Game_System[_0x55ad4d(0x159)][_0x55ad4d(0x191)],Game_System[_0x55ad4d(0x159)][_0x55ad4d(0x191)]=function(){const _0x413427=_0x55ad4d;VisuMZ[_0x413427(0x153)][_0x413427(0x35e)][_0x413427(0x175)](this),this['initBattleSystemOTB']();},Game_System[_0x55ad4d(0x159)][_0x55ad4d(0x2e2)]=function(){const _0x24e1f7=_0x55ad4d;this[_0x24e1f7(0x1f9)]=!![];},Game_System[_0x55ad4d(0x159)][_0x55ad4d(0x24d)]=function(){const _0x3a4d7c=_0x55ad4d;if(this[_0x3a4d7c(0x1f9)]===undefined){if('YVSGe'==='YVSGe')this['initBattleSystemOTB']();else{const _0x1609ac=_0x154c60[_0x3a4d7c(0x2bd)];return _0x1609ac[_0x3a4d7c(0x1f0)];}}return this['_otbTurnOrderVisible'];},Game_System[_0x55ad4d(0x159)]['setBattleSystemOTBTurnOrderVisible']=function(_0x1dee2b){const _0x408376=_0x55ad4d;this[_0x408376(0x1f9)]===undefined&&this[_0x408376(0x2e2)](),this[_0x408376(0x1f9)]=_0x1dee2b;},Game_Action[_0x55ad4d(0x2c6)]=VisuMZ[_0x55ad4d(0x153)]['Settings'][_0x55ad4d(0x24c)][_0x55ad4d(0x16f)],Game_Action[_0x55ad4d(0x17b)]=VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2bd)]['Conversion'][_0x55ad4d(0x2fb)],Game_Action[_0x55ad4d(0x307)]=VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2bd)][_0x55ad4d(0x24c)]['ConvertAgiBuffNext'],Game_Action[_0x55ad4d(0x35f)]=VisuMZ['BattleSystemOTB']['Settings'][_0x55ad4d(0x24c)][_0x55ad4d(0x343)],VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x11f)]=Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x22b)],Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x22b)]=function(){const _0x4e2c4a=_0x55ad4d;if(BattleManager[_0x4e2c4a(0x1c9)]())return 0x0;else{if(_0x4e2c4a(0x2ec)!==_0x4e2c4a(0x2ec))this['_graphicType']=_0x4e2c4a(0x36f);else return VisuMZ[_0x4e2c4a(0x153)][_0x4e2c4a(0x11f)][_0x4e2c4a(0x175)](this);}},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x266)]=Game_Action['prototype'][_0x55ad4d(0x20f)],Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x20f)]=function(){const _0x16278d=_0x55ad4d;VisuMZ[_0x16278d(0x153)][_0x16278d(0x266)]['call'](this),this['applyGlobalBattleSystemOTB']();},Game_Action[_0x55ad4d(0x159)]['applyGlobalBattleSystemOTB']=function(){const _0x4f1e55=_0x55ad4d;if(!SceneManager['isSceneBattle']())return;if(!BattleManager[_0x4f1e55(0x1c9)]())return;if(!this[_0x4f1e55(0x2d3)]())return;if(!this[_0x4f1e55(0x231)]())return;const _0x4458b2=VisuMZ['BattleSystemOTB'][_0x4f1e55(0x2f9)],_0x3d1f4=this['item']()[_0x4f1e55(0x350)];_0x3d1f4[_0x4f1e55(0x366)](_0x4458b2[_0x4f1e55(0x1ab)])&&this[_0x4f1e55(0x231)]()[_0x4f1e55(0x32c)](0x1);let _0x5c0a20=this[_0x4f1e55(0x176)](),_0x1e1691=this[_0x4f1e55(0x317)]();if(_0x5c0a20!==0x0){if(_0x4f1e55(0x306)!=='vhzhH')BattleManager[_0x4f1e55(0x195)](this['subject'](),-_0x5c0a20,![]);else return![];}_0x1e1691!==0x0&&BattleManager['turnOrderChangeOTB'](this[_0x4f1e55(0x231)](),-_0x1e1691,!![]);},Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x176)]=function(){const _0x4f368e=_0x55ad4d;if(!SceneManager[_0x4f368e(0x219)]())return 0x0;if(!BattleManager[_0x4f368e(0x1c9)]())return 0x0;if(!this[_0x4f368e(0x2d3)]())return 0x0;if(!this[_0x4f368e(0x231)]())return 0x0;if(!this['subject']()[_0x4f368e(0x104)]())return 0x0;const _0x37855=VisuMZ[_0x4f368e(0x153)]['RegExp'],_0x41742e=this[_0x4f368e(0x2d3)]()[_0x4f368e(0x350)],_0x2119d4=BattleManager[_0x4f368e(0x1b9)]||[];let _0x17aa44=0x0;if(_0x41742e['match'](_0x37855['UserFollOrder'])){if(_0x2119d4[_0x4f368e(0x2bc)](this[_0x4f368e(0x231)]())){if(_0x4f368e(0x2d5)!==_0x4f368e(0x322))_0x17aa44+=Number(RegExp['$1']);else return _0x1d2286(_0xfc2613['$1']);}}return _0x41742e[_0x4f368e(0x366)](_0x37855[_0x4f368e(0x163)])&&(_0x17aa44+=Number(RegExp['$1'])),_0x17aa44;},Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x317)]=function(){const _0x33eaf4=_0x55ad4d;if(!SceneManager['isSceneBattle']())return 0x0;if(!BattleManager[_0x33eaf4(0x1c9)]())return 0x0;if(!this[_0x33eaf4(0x2d3)]())return 0x0;if(!this[_0x33eaf4(0x231)]())return 0x0;if(!this['subject']()[_0x33eaf4(0x104)]())return 0x0;const _0x33c7ac=VisuMZ[_0x33eaf4(0x153)]['Settings'][_0x33eaf4(0x30f)],_0x25c0a7=VisuMZ[_0x33eaf4(0x153)][_0x33eaf4(0x2f9)],_0x22b23c=this['item']()[_0x33eaf4(0x350)],_0x547a58=BattleManager[_0x33eaf4(0x2b3)]||[];let _0x139dc2=0x0;if(_0x33c7ac[_0x33eaf4(0x215)]){if('mFyqM'!=='mFyqM'){const _0xa26243=_0x5dbbea[_0x33eaf4(0x1b9)][_0x33eaf4(0x1b4)](_0x3720e4)+0x1;return _0xa26243;}else _0x139dc2+=_0x33c7ac[_0x33eaf4(0x215)][_0x33eaf4(0x175)](this);}if(_0x22b23c[_0x33eaf4(0x366)](_0x25c0a7[_0x33eaf4(0x161)])){if(_0x33eaf4(0x138)!=='lJiHk'){const _0x328430=this['battler']();if(!_0x328430)return this[_0x33eaf4(0x241)]();if(_0x328430===_0x4585c2[_0x33eaf4(0x15d)])return 0x0;if(_0x4b4bd4['_actionBattlers']['includes'](_0x328430)){const _0x852dcd=_0x1fc72a['_actionBattlers'][_0x33eaf4(0x1b4)](_0x328430)+0x1;return _0x852dcd;}return this[_0x33eaf4(0x241)]();}else _0x547a58[_0x33eaf4(0x2bc)](this[_0x33eaf4(0x231)]())&&(_0x139dc2+=Number(RegExp['$1']));}return _0x22b23c['match'](_0x25c0a7[_0x33eaf4(0x16c)])&&(_0x139dc2+=Number(RegExp['$1'])),_0x139dc2;},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x28a)]=Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x335)],Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x335)]=function(_0x375750){const _0x8b632a=_0x55ad4d;VisuMZ[_0x8b632a(0x153)][_0x8b632a(0x28a)]['call'](this,_0x375750),this[_0x8b632a(0x2b1)](_0x375750);},Game_Action['prototype'][_0x55ad4d(0x2b1)]=function(_0x7f8245){const _0xd18f0e=_0x55ad4d;if(!SceneManager[_0xd18f0e(0x219)]())return;if(!BattleManager[_0xd18f0e(0x1c9)]())return;if(!this[_0xd18f0e(0x2d3)]())return;if(!_0x7f8245)return;if(!_0x7f8245[_0xd18f0e(0x104)]())return 0x0;let _0x1d5651=this[_0xd18f0e(0x14c)](_0x7f8245),_0x1e52af=this[_0xd18f0e(0x22a)](_0x7f8245);_0x1d5651!==0x0&&BattleManager['turnOrderChangeOTB'](_0x7f8245,-_0x1d5651,![]),_0x1e52af!==0x0&&BattleManager[_0xd18f0e(0x195)](_0x7f8245,-_0x1e52af,!![]);},Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x14c)]=function(_0x22f9db){const _0x1855d6=_0x55ad4d;if(!SceneManager[_0x1855d6(0x219)]())return 0x0;if(!BattleManager[_0x1855d6(0x1c9)]())return 0x0;if(!this['item']())return 0x0;if(!_0x22f9db)return 0x0;if(!_0x22f9db[_0x1855d6(0x104)]())return 0x0;const _0x1edb9a=VisuMZ[_0x1855d6(0x153)][_0x1855d6(0x2f9)],_0x21801e=this['item']()['note'],_0x252635=BattleManager['_actionBattlers']||[];let _0x631c49=0x0;if(_0x21801e[_0x1855d6(0x366)](_0x1edb9a[_0x1855d6(0x24f)])){if(_0x1855d6(0x226)!==_0x1855d6(0x226)){if(_0x399cc7['OTB_CONVERT_AGI_DEBUFF_NEXT_TURN'])_0x86cd11+=0x1;}else _0x252635[_0x1855d6(0x2bc)](_0x22f9db)&&(_0x631c49+=Number(RegExp['$1']));}_0x21801e[_0x1855d6(0x366)](_0x1edb9a[_0x1855d6(0x36d)])&&(_0x631c49+=Number(RegExp['$1']));const _0x531ee1=this[_0x1855d6(0x2d3)]()[_0x1855d6(0x321)];for(const _0x327f4b of _0x531ee1){if(_0x1855d6(0x1d7)!=='GGycy'){if(!_0x327f4b)continue;if(_0x327f4b[_0x1855d6(0x31f)]===Game_Action[_0x1855d6(0x15b)]&&_0x327f4b[_0x1855d6(0x25d)]===0x6){if(_0x1855d6(0x345)!==_0x1855d6(0x345)){if(this[_0x1855d6(0x2ab)]!==_0x1c2330[_0x1855d6(0x108)]())return this[_0x1855d6(0x34b)]();}else{if(Game_Action[_0x1855d6(0x2c6)])_0x631c49-=0x1;}}if(_0x327f4b[_0x1855d6(0x31f)]===Game_Action['EFFECT_ADD_DEBUFF']&&_0x327f4b[_0x1855d6(0x25d)]===0x6){if(_0x1855d6(0x1fd)!==_0x1855d6(0x1fd))this[_0x1855d6(0x272)]=!![],this[_0x1855d6(0x258)](0x0);else{if(Game_Action[_0x1855d6(0x17b)])_0x631c49+=0x1;}}}else _0x5dfff3[_0x1855d6(0x33a)](this[_0x1855d6(0x26f)][_0x1855d6(0x110)](),_0x14e229*0x1/0x8,_0x21df6a/0x2,_0x222e81,_0x3d346f/0x2,_0x1855d6(0x205));}return _0x631c49;},Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x22a)]=function(_0x4802ca){const _0x4372c8=_0x55ad4d;if(!SceneManager[_0x4372c8(0x219)]())return 0x0;if(!BattleManager[_0x4372c8(0x1c9)]())return 0x0;if(!this[_0x4372c8(0x2d3)]())return 0x0;if(!_0x4802ca)return 0x0;if(!_0x4802ca[_0x4372c8(0x104)]())return 0x0;const _0xc4253=VisuMZ[_0x4372c8(0x153)][_0x4372c8(0x2f9)],_0x54a0e2=this[_0x4372c8(0x2d3)]()['note'],_0x15ec0f=BattleManager[_0x4372c8(0x2b3)]||[];let _0x5c975d=0x0;_0x54a0e2[_0x4372c8(0x366)](_0xc4253[_0x4372c8(0x24f)])&&('nVIxH'==='zciwI'?(_0x563b71[_0x4372c8(0x298)](_0x5b2285,_0x145f4c,_0x54bdb1/0x2,_0x14f25b,_0xd5e23b,_0xf61762,![]),_0x9792fa[_0x4372c8(0x270)](_0x533eab+_0x2e698f/0x2,_0x37441d,_0x15db0e/0x2,_0x156248,_0x13b543),_0x4fd7ff[_0x4372c8(0x298)](_0x266043,_0x3b3f41,_0x952782/0x2,_0x2c6012,_0x1c8efc,_0x20acc8,![]),_0x5aba13[_0x4372c8(0x270)](_0x1e55db+_0x132856/0x2,_0x17146b,_0x296524/0x2,_0x577274,_0x3737d7),_0x5a74d0[_0x4372c8(0x298)](_0x5710ab,_0xd0c34,_0x1688a0/0x2,_0x3d6bc8,_0x31c44e,_0x9460f8,![]),_0x2434ba[_0x4372c8(0x270)](_0x35f65a+_0xc482/0x2,_0x5999a3,_0x14b226/0x2,_0x1e8775,_0x282764)):_0x15ec0f[_0x4372c8(0x2bc)](_0x4802ca)&&(_0x5c975d+=Number(RegExp['$1'])));_0x54a0e2[_0x4372c8(0x366)](_0xc4253[_0x4372c8(0x30b)])&&(_0x5c975d+=Number(RegExp['$1']));const _0x40c548=this[_0x4372c8(0x2d3)]()[_0x4372c8(0x321)];for(const _0x27410d of _0x40c548){if(_0x4372c8(0x2b6)===_0x4372c8(0x131))this[_0x4372c8(0x15d)]['performActionEndOTB']();else{if(!_0x27410d)continue;if(_0x27410d['code']===Game_Action[_0x4372c8(0x15b)]&&_0x27410d['dataId']===0x6){if(_0x4372c8(0x183)!=='HLvYz'){if(Game_Action['OTB_CONVERT_AGI_BUFF_NEXT_TURN'])_0x5c975d-=0x1;}else _0x5c509b=_0x4c6141['max'](0x0,_0x5f532a[_0x4372c8(0x153)][_0x4372c8(0x302)](_0x3d7289));}if(_0x27410d[_0x4372c8(0x31f)]===Game_Action['EFFECT_ADD_DEBUFF']&&_0x27410d[_0x4372c8(0x25d)]===0x6){if(Game_Action['OTB_CONVERT_AGI_DEBUFF_NEXT_TURN'])_0x5c975d+=0x1;}}}return _0x5c975d;},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x2ce)]=function(){const _0x1ff7a7=_0x55ad4d;delete this[_0x1ff7a7(0x338)],delete this['_otbTurnOrderFaceName'],delete this['_otbTurnOrderFaceIndex'],delete this[_0x1ff7a7(0x1df)];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x2d4)]=function(){const _0x1af594=_0x55ad4d;return this[_0x1af594(0x338)]===undefined&&(this['_otbTurnOrderGraphicType']=this['createTurnOrderOTBGraphicType']()),this['_otbTurnOrderGraphicType'];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x261)]=function(){const _0x308232=_0x55ad4d;return Window_OTB_TurnOrder[_0x308232(0x2bd)][_0x308232(0x373)];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x16b)]=function(){const _0x41fd7a=_0x55ad4d;return this[_0x41fd7a(0x174)]===undefined&&(this[_0x41fd7a(0x174)]=this[_0x41fd7a(0x2fc)]()),this[_0x41fd7a(0x174)];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x2fc)]=function(){const _0x2a381c=_0x55ad4d;return Window_OTB_TurnOrder[_0x2a381c(0x2bd)][_0x2a381c(0xfd)];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x28b)]=function(){const _0x30c827=_0x55ad4d;return this[_0x30c827(0x356)]===undefined&&(this[_0x30c827(0x356)]=this[_0x30c827(0x365)]()),this[_0x30c827(0x356)];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x365)]=function(){const _0x4d05aa=_0x55ad4d;return Window_OTB_TurnOrder[_0x4d05aa(0x2bd)][_0x4d05aa(0x1f5)];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x27b)]=function(){const _0x224422=_0x55ad4d;return this[_0x224422(0x1df)]===undefined&&(this[_0x224422(0x1df)]=this['createTurnOrderOTBGraphicIconIndex']()),this[_0x224422(0x1df)];},Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x326)]=function(){const _0x384304=_0x55ad4d;return Window_OTB_TurnOrder['Settings'][_0x384304(0x149)];},Game_BattlerBase['prototype'][_0x55ad4d(0x274)]=function(_0x253154){this['_otbTurnOrderIconIndex']=_0x253154;},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x379)]=Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x143)],Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x143)]=function(){const _0x3f4945=_0x55ad4d;VisuMZ['BattleSystemOTB'][_0x3f4945(0x379)][_0x3f4945(0x175)](this),BattleManager[_0x3f4945(0x277)]();},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x148)]=Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x2e3)],Game_BattlerBase[_0x55ad4d(0x159)][_0x55ad4d(0x2e3)]=function(){const _0x2fe46e=_0x55ad4d,_0x343642=this[_0x2fe46e(0x122)];VisuMZ[_0x2fe46e(0x153)][_0x2fe46e(0x148)]['call'](this),BattleManager[_0x2fe46e(0x1c9)]()&&SceneManager['isSceneBattle']()&&_0x343642&&!this[_0x2fe46e(0x122)]&&BattleManager[_0x2fe46e(0x26c)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x376)]=Game_Battler[_0x55ad4d(0x159)]['performCollapse'],Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x2a2)]=function(){const _0x1c8228=_0x55ad4d;VisuMZ[_0x1c8228(0x153)][_0x1c8228(0x376)][_0x1c8228(0x175)](this),BattleManager[_0x1c8228(0x277)]();},Game_Battler[_0x55ad4d(0x22f)]=VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2bd)][_0x55ad4d(0x30f)]['PostStunInfinitySpeed'],VisuMZ[_0x55ad4d(0x153)]['Game_Battler_onBattleStart']=Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x128)],Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x128)]=function(_0x49fed3){const _0x297dee=_0x55ad4d;VisuMZ[_0x297dee(0x153)][_0x297dee(0x308)]['call'](this,_0x49fed3),this[_0x297dee(0x1ce)](_0x49fed3);},Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x1ce)]=function(_0xf5a743){const _0x3c35ad=_0x55ad4d;if(!BattleManager['isOTB']())return;this[_0x3c35ad(0x38a)]=0x0;},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x1a3)]=Game_Battler['prototype'][_0x55ad4d(0x283)],Game_Battler['prototype'][_0x55ad4d(0x283)]=function(){const _0x1468bc=_0x55ad4d;VisuMZ[_0x1468bc(0x153)][_0x1468bc(0x1a3)][_0x1468bc(0x175)](this),this['onBattleEndOTB']();},Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x1dc)]=function(){const _0x3c616c=_0x55ad4d;if(!BattleManager[_0x3c616c(0x1c9)]())return;this[_0x3c616c(0x38a)]=0x0;},Game_Battler['prototype'][_0x55ad4d(0x315)]=function(){const _0x809289=_0x55ad4d;if(!BattleManager['isOTB']())return;this['_otbTimesActedThisTurn']=this[_0x809289(0x38a)]||0x0,this[_0x809289(0x38a)]++;if(this[_0x809289(0x233)]()>0x0&&this===BattleManager['_subject']){const _0xa6ff57=BattleManager[_0x809289(0x19f)];if(_0xa6ff57['length']>0x0&&_0xa6ff57[0x0]!==this){if(_0x809289(0x334)===_0x809289(0x334))return;else this[_0x809289(0x30d)]=!![];}const _0x214e01=this[_0x809289(0x1e8)]();if(_0x214e01&&BattleManager['isNextOtbSubject'](this))_0x214e01[_0x809289(0x123)]();}},BattleManager[_0x55ad4d(0x23c)]=function(_0x45b9bf){if(!_0x45b9bf)return![];return this['_actionBattlers'][0x0]===_0x45b9bf;},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x375)]=Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x19b)],Game_Battler['prototype'][_0x55ad4d(0x19b)]=function(){const _0x5dea63=_0x55ad4d;VisuMZ[_0x5dea63(0x153)][_0x5dea63(0x375)][_0x5dea63(0x175)](this),this[_0x5dea63(0x21f)]();},Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x21f)]=function(){const _0xb38cc1=_0x55ad4d;if(!BattleManager[_0xb38cc1(0x1c9)]())return;this[_0xb38cc1(0x38a)]=0x0;},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x386)]=Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x2cb)],Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x2cb)]=function(){const _0x3a53ba=_0x55ad4d;if(BattleManager[_0x3a53ba(0x1c9)]())this[_0x3a53ba(0x127)]();else{if(_0x3a53ba(0x2a3)==='ntJLB')return;else VisuMZ[_0x3a53ba(0x153)][_0x3a53ba(0x386)][_0x3a53ba(0x175)](this);}},Game_Battler[_0x55ad4d(0x159)]['makeOTBSpeed']=function(){const _0x45d7ff=_0x55ad4d;if(this[_0x45d7ff(0x1b2)]())this[_0x45d7ff(0x2df)]=Infinity;else{const _0x5c8998=this[_0x45d7ff(0x230)]()||new Game_Action(this);this[_0x45d7ff(0x2df)]=VisuMZ['BattleSystemOTB'][_0x45d7ff(0x2bd)]['Mechanics'][_0x45d7ff(0x29e)]['call'](_0x5c8998);}},Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x1b2)]=function(){const _0xa2a8fa=_0x55ad4d;if(!Game_Battler[_0xa2a8fa(0x22f)])return![];if(!this[_0xa2a8fa(0x13c)]())return![];if(!this[_0xa2a8fa(0x2d0)]())return![];if(this[_0xa2a8fa(0x1d2)]())return![];const _0x6ec5bb=JsonEx[_0xa2a8fa(0x247)](this);return _0x6ec5bb[_0xa2a8fa(0x1a6)]=!![],_0x6ec5bb['_tempBattler']=!![],_0x6ec5bb[_0xa2a8fa(0x36c)](),_0x6ec5bb[_0xa2a8fa(0x354)](0x1),_0x6ec5bb[_0xa2a8fa(0x354)](0x2),_0x6ec5bb[_0xa2a8fa(0x31d)](),_0x6ec5bb['canMove']();},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x1a4)]=Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x31a)],Game_Action[_0x55ad4d(0x159)][_0x55ad4d(0x31a)]=function(){const _0x3125a7=_0x55ad4d;return BattleManager['isOTB']()?VisuMZ[_0x3125a7(0x153)][_0x3125a7(0x2bd)][_0x3125a7(0x30f)][_0x3125a7(0x212)]:VisuMZ[_0x3125a7(0x153)][_0x3125a7(0x1a4)][_0x3125a7(0x175)](this);},Game_Battler[_0x55ad4d(0x159)]['otbGainInstant']=function(_0x29d328){const _0x2dc381=_0x55ad4d;if(!this[_0x2dc381(0x1d2)]())return;this[_0x2dc381(0x38a)]=this[_0x2dc381(0x38a)]||0x0,this['_otbTimesActedThisTurn']--,BattleManager['otbAddBattlerToTurnOrderAtStart'](this,_0x29d328,BattleManager['_actionBattlers']);},Game_Battler[_0x55ad4d(0x159)]['canChangeOtbTurnOrder']=function(){const _0x3e8849=_0x55ad4d;if(this[_0x3e8849(0x22b)]()===Infinity)return![];return!![];},Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x2c9)]=function(_0x408c7b,_0x276ad2){const _0x90c1bb=_0x55ad4d;if(this[_0x90c1bb(0x271)]||this['_tempActor'])return;if(!SceneManager['isSceneBattle']())return;if(!BattleManager[_0x90c1bb(0x1c9)]())return;if(_0x408c7b&&!this[_0x90c1bb(0x1d2)]())BattleManager[_0x90c1bb(0x277)]();else!_0x408c7b&&this[_0x90c1bb(0x1d2)]()&&BattleManager[_0x90c1bb(0x26c)](this);if(this[_0x90c1bb(0x1d2)]()){const _0x35d049=this[_0x90c1bb(0x387)]()-_0x276ad2;if(_0x35d049>0x0){if(_0x90c1bb(0x229)!==_0x90c1bb(0x180))BattleManager[_0x90c1bb(0x250)](this,_0x35d049,BattleManager[_0x90c1bb(0x1b9)]),BattleManager[_0x90c1bb(0x250)](this,_0x35d049,BattleManager[_0x90c1bb(0x2b3)]);else{const _0x34f531=this['bitmapWidth'](),_0x1f8985=this[_0x90c1bb(0x2ff)](),_0x383d91=_0x3f1fbf[_0x90c1bb(0x21b)](_0x34f531,_0x1f8985);this[_0x90c1bb(0x32d)][_0x90c1bb(0x2bb)]=new _0x364d8c(_0x34f531,_0x1f8985);const _0x23a5b4=this[_0x90c1bb(0x32d)][_0x90c1bb(0x2bb)],_0x1b8064=0x9,_0x3aa729=0x6,_0x4ddb70=_0xbf01fe[_0x90c1bb(0x10d)]/_0x1b8064,_0x4ac659=_0x16f04[_0x90c1bb(0x200)]/_0x3aa729,_0x3d7107=_0x57b5a8[_0x90c1bb(0x21b)](0x1,_0x383d91/_0x4ddb70,_0x383d91/_0x4ac659),_0x168e98=_0x4ddb70*_0x3d7107,_0x39ae09=_0x4ac659*_0x3d7107,_0x34c246=_0x2f4511['round']((_0x34f531-_0x168e98)/0x2),_0x3418f0=_0x44efcd['round']((_0x1f8985-_0x39ae09)/0x2);_0x23a5b4[_0x90c1bb(0x214)](_0x4cf4e0,0x0,0x0,_0x4ddb70,_0x4ac659,_0x34c246,_0x3418f0,_0x168e98,_0x39ae09);}}}},VisuMZ['BattleSystemOTB']['Game_Battler_addState']=Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x225)],Game_Battler['prototype'][_0x55ad4d(0x225)]=function(_0x51c3a1){const _0x5506e5=_0x55ad4d,_0x219a72=this[_0x5506e5(0x1d2)](),_0x5a4461=this[_0x5506e5(0x387)]();VisuMZ[_0x5506e5(0x153)][_0x5506e5(0x360)][_0x5506e5(0x175)](this,_0x51c3a1),this[_0x5506e5(0x2c9)](_0x219a72,_0x5a4461);},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x35d)]=Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x100)],Game_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x100)]=function(_0x4ca8ee){const _0x45b93e=_0x55ad4d,_0x1e872b=this[_0x45b93e(0x1d2)](),_0x3de23b=this[_0x45b93e(0x387)]();VisuMZ[_0x45b93e(0x153)][_0x45b93e(0x35d)][_0x45b93e(0x175)](this,_0x4ca8ee),this[_0x45b93e(0x2c9)](_0x1e872b,_0x3de23b);},VisuMZ[_0x55ad4d(0x153)]['Game_Actor_selectNextCommand']=Game_Actor[_0x55ad4d(0x159)]['selectNextCommand'],Game_Actor[_0x55ad4d(0x159)]['selectNextCommand']=function(){const _0x98d0e9=_0x55ad4d;if(BattleManager[_0x98d0e9(0x1c9)]()){if(_0x98d0e9(0x35b)!==_0x98d0e9(0x2a1)){if(this[_0x98d0e9(0x1e8)]())this[_0x98d0e9(0x1e8)]()['stepForward']();return![];}else _0x56b967['bitmap']=_0x356ee6['loadSystem'](_0x1c4628[_0x355118]);}return VisuMZ[_0x98d0e9(0x153)][_0x98d0e9(0x281)][_0x98d0e9(0x175)](this);},Game_Actor[_0x55ad4d(0x159)][_0x55ad4d(0x261)]=function(){const _0x41c733=_0x55ad4d,_0x59e2d0=this[_0x41c733(0x1d9)]()[_0x41c733(0x350)];if(_0x59e2d0[_0x41c733(0x366)](/<OTB TURN ORDER FACE:[ ](.*),[ ](\d+)>/i))return _0x41c733(0x36f);else{if(_0x59e2d0[_0x41c733(0x366)](/<OTB TURN ORDER ICON:[ ](\d+)>/i))return _0x41c733(0x293);}return Window_OTB_TurnOrder[_0x41c733(0x2bd)][_0x41c733(0x15e)];},Game_Actor[_0x55ad4d(0x159)][_0x55ad4d(0x16b)]=function(){const _0x32d3ac=_0x55ad4d,_0x2314c8=this['actor']()[_0x32d3ac(0x350)];if(_0x2314c8[_0x32d3ac(0x366)](/<OTB TURN ORDER FACE:[ ](.*),[ ](\d+)>/i)){if(_0x32d3ac(0x1d3)!==_0x32d3ac(0x1f7))return String(RegExp['$1']);else{const _0x49bffe=this[_0x32d3ac(0x1e3)],_0x23332c=this[_0x32d3ac(0x23a)](),_0x4231f5=this['bitmapHeight'](),_0x213f6a=_0x1be2fe[_0x32d3ac(0x2f6)](_0x23332c,_0x4231f5);this[_0x32d3ac(0x32d)][_0x32d3ac(0x2bb)]=new _0x98826c(_0x23332c,_0x4231f5);const _0x30cf07=this[_0x32d3ac(0x32d)][_0x32d3ac(0x2bb)],_0x11c8e8=_0xebfb8b['faceWidth'],_0x521456=_0x58d1dd['faceHeight'],_0x4b49ba=_0x213f6a/_0x5daa18[_0x32d3ac(0x2f6)](_0x11c8e8,_0x521456),_0x1855a5=_0x251bac[_0x32d3ac(0x203)],_0xf9a14f=_0x5d69ee[_0x32d3ac(0x276)],_0x414d7c=_0x49bffe%0x4*_0x11c8e8+(_0x11c8e8-_0x1855a5)/0x2,_0x81c9e5=_0x8d6fb5[_0x32d3ac(0x2fd)](_0x49bffe/0x4)*_0x521456+(_0x521456-_0xf9a14f)/0x2,_0x20b8dc=(_0x23332c-_0x11c8e8*_0x4b49ba)/0x2,_0x3979a6=(_0x4231f5-_0x521456*_0x4b49ba)/0x2;_0x30cf07['blt'](_0x25ce6f,_0x414d7c,_0x81c9e5,_0x1855a5,_0xf9a14f,_0x20b8dc,_0x3979a6,_0x213f6a,_0x213f6a);}}return this[_0x32d3ac(0x210)]();},Game_Actor[_0x55ad4d(0x159)]['TurnOrderOTBGraphicFaceIndex']=function(){const _0x48c1ed=_0x55ad4d,_0x1631df=this[_0x48c1ed(0x1d9)]()[_0x48c1ed(0x350)];if(_0x1631df['match'](/<OTB TURN ORDER FACE:[ ](.*),[ ](\d+)>/i)){if(_0x48c1ed(0x19d)!=='gYdPc')return Number(RegExp['$2']);else _0x215f56++;}return this[_0x48c1ed(0x2ca)]();},Game_Actor[_0x55ad4d(0x159)]['createTurnOrderOTBGraphicIconIndex']=function(){const _0x1c2f68=_0x55ad4d,_0x5407b9=this[_0x1c2f68(0x1d9)]()[_0x1c2f68(0x350)];if(_0x5407b9[_0x1c2f68(0x366)](/<OTB TURN ORDER ICON:[ ](\d+)>/i)){if(_0x1c2f68(0x17e)===_0x1c2f68(0x179)){if(!this[_0x1c2f68(0x1c9)]())return;this['removeActionBattlersOTB'](),this[_0x1c2f68(0x15d)]&&this[_0x1c2f68(0x15d)][_0x1c2f68(0x315)](),this['_subject']&&this[_0x1c2f68(0x15d)][_0x1c2f68(0x1d2)]()&&this['_actionBattlers'][_0x1c2f68(0x2bc)](this[_0x1c2f68(0x15d)])&&this[_0x1c2f68(0x15d)][_0x1c2f68(0x1f8)]();}else return Number(RegExp['$1']);}return Window_OTB_TurnOrder['Settings'][_0x1c2f68(0x14a)];},Game_Enemy['prototype'][_0x55ad4d(0x261)]=function(){const _0x499c47=_0x55ad4d,_0x37cdfc=this['enemy']()[_0x499c47(0x350)];if(_0x37cdfc[_0x499c47(0x366)](/<OTB TURN ORDER FACE:[ ](.*),[ ](\d+)>/i))return _0x499c47(0x36f);else{if(_0x37cdfc[_0x499c47(0x366)](/<OTB TURN ORDER ICON:[ ](\d+)>/i))return'OxPLr'!==_0x499c47(0x265)?_0x499c47(0x293):_0x2a7555(_0x6183f1['$2']);}return Window_OTB_TurnOrder[_0x499c47(0x2bd)][_0x499c47(0x373)];},Game_Enemy['prototype'][_0x55ad4d(0x2fc)]=function(){const _0x38c099=_0x55ad4d,_0x42b6bb=this[_0x38c099(0x1ed)]()['note'];if(_0x42b6bb[_0x38c099(0x366)](/<OTB TURN ORDER FACE:[ ](.*),[ ](\d+)>/i))return String(RegExp['$1']);return Window_OTB_TurnOrder[_0x38c099(0x2bd)]['EnemyBattlerFaceName'];},Game_Enemy[_0x55ad4d(0x159)]['createTurnOrderOTBGraphicFaceIndex']=function(){const _0x1cb696=_0x55ad4d,_0x23997e=this[_0x1cb696(0x1ed)]()['note'];if(_0x23997e[_0x1cb696(0x366)](/<OTB TURN ORDER FACE:[ ](.*),[ ](\d+)>/i))return Number(RegExp['$2']);return Window_OTB_TurnOrder['Settings'][_0x1cb696(0x1f5)];},Game_Enemy[_0x55ad4d(0x159)]['createTurnOrderOTBGraphicIconIndex']=function(){const _0x4f76cd=_0x55ad4d,_0x298560=this['enemy']()['note'];if(_0x298560[_0x4f76cd(0x366)](/<OTB TURN ORDER ICON:[ ](\d+)>/i)){if('ojDxT'===_0x4f76cd(0x259)){if(_0x5ea8ce[_0x4f76cd(0x2c6)])_0x2a5959-=0x1;}else return Number(RegExp['$1']);}return Window_OTB_TurnOrder[_0x4f76cd(0x2bd)]['EnemyBattlerIcon'];},VisuMZ[_0x55ad4d(0x153)]['Game_Party_addActor']=Game_Party[_0x55ad4d(0x159)][_0x55ad4d(0x248)],Game_Party[_0x55ad4d(0x159)][_0x55ad4d(0x248)]=function(_0x12a7d8){const _0x5b69f6=_0x55ad4d;VisuMZ['BattleSystemOTB'][_0x5b69f6(0x25f)][_0x5b69f6(0x175)](this,_0x12a7d8);if(Imported['VisuMZ_2_PartySystem'])return;SceneManager[_0x5b69f6(0x219)]()&&BattleManager[_0x5b69f6(0x1c9)]()&&(BattleManager[_0x5b69f6(0x277)](),BattleManager[_0x5b69f6(0x26c)]($gameActors[_0x5b69f6(0x1d9)](_0x12a7d8)));},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x1db)]=Game_Party['prototype'][_0x55ad4d(0x336)],Game_Party['prototype'][_0x55ad4d(0x336)]=function(_0x555520){const _0x1a7cea=_0x55ad4d;VisuMZ[_0x1a7cea(0x153)][_0x1a7cea(0x1db)]['call'](this,_0x555520);if(SceneManager[_0x1a7cea(0x219)]()&&BattleManager['isOTB']()){if(_0x1a7cea(0x209)===_0x1a7cea(0x209))BattleManager[_0x1a7cea(0x277)]();else{if(!this['isOTB']())return;const _0x114211=_0x58ebf8[_0x1a7cea(0x22c)]['_otbTurnOrderWindow'];while(_0x594dfe--){_0x51da54[_0x1a7cea(0x20d)](_0x579f2b),_0x114211&&_0x114211['addBattlerToTurnOrderAtEnd'](_0x2b3969,_0x534fca);}}}},VisuMZ['BattleSystemOTB']['Scene_Battle_createActorCommandWindow']=Scene_Battle['prototype'][_0x55ad4d(0x358)],Scene_Battle['prototype']['createActorCommandWindow']=function(){const _0x264078=_0x55ad4d;VisuMZ['BattleSystemOTB'][_0x264078(0x136)][_0x264078(0x175)](this),BattleManager['isOTB']()&&this[_0x264078(0x30a)]();},Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x30a)]=function(){const _0x218699=_0x55ad4d,_0x2d6220=this[_0x218699(0x35c)];this[_0x218699(0x292)]()&&delete _0x2d6220[_0x218699(0x201)]['cancel'];},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x1be)]=Scene_Battle[_0x55ad4d(0x159)]['commandCancel'],Scene_Battle['prototype'][_0x55ad4d(0x357)]=function(){const _0x2c2186=_0x55ad4d;if(BattleManager[_0x2c2186(0x1c9)]()){if(_0x2c2186(0x189)===_0x2c2186(0x31c))return _0xb773ee[_0x2c2186(0x153)][_0x2c2186(0x11f)][_0x2c2186(0x175)](this);else this[_0x2c2186(0x29c)]();}else VisuMZ[_0x2c2186(0x153)][_0x2c2186(0x1be)]['call'](this);},Scene_Battle['prototype'][_0x55ad4d(0x29c)]=function(){const _0x2d68c4=_0x55ad4d;BattleManager[_0x2d68c4(0x1bc)](),this['_partyCommandWindow'][_0x2d68c4(0x192)](),this[_0x2d68c4(0x35c)]['close']();},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x19a)]=Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x309)],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x309)]=function(){const _0x470462=_0x55ad4d;BattleManager[_0x470462(0x1c9)]()?this[_0x470462(0x371)]():VisuMZ['BattleSystemOTB']['Scene_Battle_commandFight'][_0x470462(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x1b3)]=Scene_Battle[_0x55ad4d(0x159)]['createAllWindows'],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x19e)]=function(){const _0x2c3da5=_0x55ad4d;VisuMZ['BattleSystemOTB'][_0x2c3da5(0x1b3)][_0x2c3da5(0x175)](this),this[_0x2c3da5(0x160)]();},Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x160)]=function(){const _0x20328d=_0x55ad4d;if(!BattleManager[_0x20328d(0x1c9)]())return;this['_otbTurnOrderWindow']=new Window_OTB_TurnOrder();const _0x53c656=this[_0x20328d(0x1a5)](this[_0x20328d(0x190)]);this[_0x20328d(0x21a)](this[_0x20328d(0x13e)],_0x53c656),this[_0x20328d(0x129)](),SceneManager['isPreviousSceneBattleTransitionable']()&&this[_0x20328d(0x13e)][_0x20328d(0x18e)]();},Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x129)]=function(){const _0x7b64fd=_0x55ad4d,_0x53c355=Window_OTB_TurnOrder[_0x7b64fd(0x2bd)];if(_0x53c355[_0x7b64fd(0x173)]!==_0x7b64fd(0x17c))return;if(!_0x53c355[_0x7b64fd(0x164)])return;if(!this[_0x7b64fd(0x13d)])return;const _0x12100d=this[_0x7b64fd(0x13e)]['y']-Math['round']((Graphics[_0x7b64fd(0x200)]-Graphics[_0x7b64fd(0x38c)])/0x2),_0x563550=_0x12100d+this[_0x7b64fd(0x13e)][_0x7b64fd(0x200)];this[_0x7b64fd(0x13d)]['y']=_0x563550+(_0x53c355['LogWindowOffsetY']||0x0);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x14e)]=Scene_Battle['prototype'][_0x55ad4d(0x118)],Scene_Battle['prototype']['commandAttack']=function(){const _0x486a61=_0x55ad4d;BattleManager[_0x486a61(0x1bc)](),VisuMZ[_0x486a61(0x153)][_0x486a61(0x14e)]['call'](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x34e)]=Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x381)],Scene_Battle['prototype'][_0x55ad4d(0x381)]=function(){const _0x134e2e=_0x55ad4d;BattleManager[_0x134e2e(0x1bc)](),VisuMZ[_0x134e2e(0x153)][_0x134e2e(0x34e)][_0x134e2e(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2e6)]=Scene_Battle[_0x55ad4d(0x159)]['onActorOk'],Scene_Battle['prototype'][_0x55ad4d(0x33c)]=function(){const _0x892de6=_0x55ad4d;BattleManager[_0x892de6(0x1bc)](),VisuMZ[_0x892de6(0x153)]['Scene_Battle_onActorOk'][_0x892de6(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x305)]=Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x1f4)],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x1f4)]=function(){const _0x556f03=_0x55ad4d;BattleManager[_0x556f03(0x1bc)](),VisuMZ[_0x556f03(0x153)][_0x556f03(0x305)][_0x556f03(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x106)]=Scene_Battle[_0x55ad4d(0x159)]['onEnemyOk'],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x1b1)]=function(){const _0x185aff=_0x55ad4d;BattleManager[_0x185aff(0x1bc)](),VisuMZ[_0x185aff(0x153)][_0x185aff(0x106)][_0x185aff(0x175)](this);},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x1d1)]=Scene_Battle['prototype']['onEnemyCancel'],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x38b)]=function(){const _0x25aa1f=_0x55ad4d;BattleManager[_0x25aa1f(0x1bc)](),VisuMZ[_0x25aa1f(0x153)][_0x25aa1f(0x1d1)][_0x25aa1f(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x29f)]=Scene_Battle['prototype'][_0x55ad4d(0x278)],Scene_Battle['prototype'][_0x55ad4d(0x278)]=function(){const _0xa5f086=_0x55ad4d;BattleManager[_0xa5f086(0x1bc)](),VisuMZ[_0xa5f086(0x153)]['Scene_Battle_onSkillOk'][_0xa5f086(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x24b)]=Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x341)],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x341)]=function(){const _0x571711=_0x55ad4d;BattleManager[_0x571711(0x1bc)](),VisuMZ[_0x571711(0x153)][_0x571711(0x24b)]['call'](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x2e4)]=Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x227)],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x227)]=function(){const _0x13db68=_0x55ad4d;BattleManager[_0x13db68(0x1bc)](),VisuMZ['BattleSystemOTB'][_0x13db68(0x2e4)]['call'](this);},VisuMZ['BattleSystemOTB'][_0x55ad4d(0x349)]=Scene_Battle[_0x55ad4d(0x159)]['onItemCancel'],Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x23f)]=function(){const _0x3f4311=_0x55ad4d;BattleManager[_0x3f4311(0x1bc)](),VisuMZ[_0x3f4311(0x153)][_0x3f4311(0x349)][_0x3f4311(0x175)](this);},VisuMZ[_0x55ad4d(0x153)][_0x55ad4d(0x299)]=Scene_Battle[_0x55ad4d(0x159)][_0x55ad4d(0x113)],Scene_Battle[_0x55ad4d(0x159)]['actorCommandSingleSkill']=function(){const _0x3d0b05=_0x55ad4d;BattleManager[_0x3d0b05(0x1bc)](),VisuMZ[_0x3d0b05(0x153)]['Scene_Battle_actorCommandSingleSkill'][_0x3d0b05(0x175)](this);};function _0xbb96(_0x618556,_0x813b0e){return _0xbb96=function(_0x30f4e4,_0xbb9667){_0x30f4e4=_0x30f4e4-0xfc;let _0x2e6f73=_0x30f4[_0x30f4e4];return _0x2e6f73;},_0xbb96(_0x618556,_0x813b0e);}function Sprite_OTB_TurnOrder_Battler(){const _0x1206d7=_0x55ad4d;this[_0x1206d7(0x191)](...arguments);}Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]=Object['create'](Sprite_Clickable['prototype']),Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x1ca)]=Sprite_OTB_TurnOrder_Battler,Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x191)]=function(_0x19f04c,_0x1b48cb,_0x49b2b1){const _0x4f2f88=_0x55ad4d;this[_0x4f2f88(0x32b)](_0x19f04c,_0x1b48cb,_0x49b2b1),Sprite_Clickable[_0x4f2f88(0x159)][_0x4f2f88(0x191)][_0x4f2f88(0x175)](this),this[_0x4f2f88(0x1e2)]=0x0,this[_0x4f2f88(0x264)](),this[_0x4f2f88(0x2f2)]();},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x32b)]=function(_0x8a01a,_0x12c751,_0xb5dbcf){const _0x54db41=_0x55ad4d;this['_unit']=_0x8a01a[_0x54db41(0x1af)]()?$gameParty:$gameTroop,this[_0x54db41(0x339)]=_0x8a01a[_0x54db41(0x367)](),this[_0x54db41(0x204)]=_0x12c751,this[_0x54db41(0x101)]=_0xb5dbcf;const _0x3d80eb=Window_OTB_TurnOrder[_0x54db41(0x2bd)],_0x536474=this['isHorz']();this[_0x54db41(0x2de)]=0x0,this[_0x54db41(0x238)]=_0x3d80eb[_0x54db41(0xfe)]?-_0x3d80eb['SpriteThin']:this[_0x54db41(0x206)]()['width'],this['_positionTargetY']=0x0,this['_fadeDuration']=0x0,this[_0x54db41(0x1fa)]=0xff,this[_0x54db41(0x1e6)]=![],this[_0x54db41(0x287)]=![],this[_0x54db41(0x17d)]=0x0,this[_0x54db41(0x1b8)]=0x0;},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]['createChildren']=function(){const _0x27d3b0=_0x55ad4d;this[_0x27d3b0(0x16e)](),this[_0x27d3b0(0x316)](),this[_0x27d3b0(0x154)](),this['createBorderSprite'](),this[_0x27d3b0(0x30c)]();},Sprite_OTB_TurnOrder_Battler['prototype'][_0x55ad4d(0x16e)]=function(){const _0x5b6ec7=_0x55ad4d;this['x']=this[_0x5b6ec7(0x238)],this['y']=this[_0x5b6ec7(0x37c)];},Sprite_OTB_TurnOrder_Battler['prototype'][_0x55ad4d(0x1ac)]=function(){return!![];},Sprite_OTB_TurnOrder_Battler['prototype'][_0x55ad4d(0x23a)]=function(){const _0x1d777c=_0x55ad4d,_0x34df4b=Window_OTB_TurnOrder[_0x1d777c(0x2bd)];return _0x34df4b[_0x1d777c(0x1f0)];},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x2ff)]=function(){const _0x5671d2=_0x55ad4d,_0x633869=Window_OTB_TurnOrder[_0x5671d2(0x2bd)];return _0x633869[_0x5671d2(0x2ed)];},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x377)]=function(){const _0x7331e7=_0x55ad4d;return this[_0x7331e7(0x359)]===$gameParty?'Actor':_0x7331e7(0x1e5);},Sprite_OTB_TurnOrder_Battler['prototype']['createBackgroundSprite']=function(){const _0x26397b=_0x55ad4d;if(!Window_OTB_TurnOrder['Settings']['ShowMarkerBg'])return;const _0x171844=Window_OTB_TurnOrder[_0x26397b(0x2bd)],_0xe2cabb=this[_0x26397b(0x377)](),_0x5bd0ec=_0x26397b(0x194)[_0x26397b(0x362)](_0xe2cabb),_0x4e058e=new Sprite();_0x4e058e[_0x26397b(0x137)]['x']=this[_0x26397b(0x137)]['x'],_0x4e058e[_0x26397b(0x137)]['y']=this[_0x26397b(0x137)]['y'];if(_0x171844[_0x5bd0ec])_0x4e058e[_0x26397b(0x2bb)]=ImageManager[_0x26397b(0x19c)](_0x171844[_0x5bd0ec]);else{const _0x311cde=this[_0x26397b(0x23a)](),_0x6f29a=this[_0x26397b(0x2ff)]();_0x4e058e[_0x26397b(0x2bb)]=new Bitmap(_0x311cde,_0x6f29a);const _0x4a5a47=ColorManager[_0x26397b(0x1c2)](_0x171844['%1BgColor1'['format'](_0xe2cabb)]),_0x1f75ae=ColorManager[_0x26397b(0x1c2)](_0x171844[_0x26397b(0x2d6)[_0x26397b(0x362)](_0xe2cabb)]);_0x4e058e[_0x26397b(0x2bb)][_0x26397b(0x298)](0x0,0x0,_0x311cde,_0x6f29a,_0x4a5a47,_0x1f75ae,!![]);}this[_0x26397b(0x285)]=_0x4e058e,this[_0x26397b(0x24e)](this[_0x26397b(0x285)]),this[_0x26397b(0x10d)]=this[_0x26397b(0x285)]['width'],this[_0x26397b(0x200)]=this[_0x26397b(0x285)]['height'];},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x154)]=function(){const _0x7dd30c=_0x55ad4d,_0x440d2f=new Sprite();_0x440d2f[_0x7dd30c(0x137)]['x']=this['anchor']['x'],_0x440d2f[_0x7dd30c(0x137)]['y']=this[_0x7dd30c(0x137)]['y'],this['_graphicSprite']=_0x440d2f,this[_0x7dd30c(0x24e)](this[_0x7dd30c(0x32d)]),this[_0x7dd30c(0x34b)]();},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]['createBorderSprite']=function(){const _0x1823c0=_0x55ad4d;if(!Window_OTB_TurnOrder[_0x1823c0(0x2bd)]['ShowMarkerBorder'])return;const _0x188961=Window_OTB_TurnOrder[_0x1823c0(0x2bd)],_0x45cc51=this['getUnitSideSide'](),_0x88f674='%1SystemBorder'[_0x1823c0(0x362)](_0x45cc51),_0x151f4e=new Sprite();_0x151f4e[_0x1823c0(0x137)]['x']=this[_0x1823c0(0x137)]['x'],_0x151f4e[_0x1823c0(0x137)]['y']=this[_0x1823c0(0x137)]['y'];if(_0x188961[_0x88f674])_0x151f4e['bitmap']=ImageManager[_0x1823c0(0x19c)](_0x188961[_0x88f674]);else{let _0x3bb9ec=this[_0x1823c0(0x23a)](),_0x1c8e3d=this[_0x1823c0(0x2ff)](),_0x1dbc66=this[_0x1823c0(0x282)]();_0x151f4e[_0x1823c0(0x2bb)]=new Bitmap(_0x3bb9ec,_0x1c8e3d);const _0x41fea8=_0x1823c0(0x388),_0x5ac88d=ColorManager[_0x1823c0(0x1c2)](_0x188961[_0x1823c0(0x2b7)['format'](_0x45cc51)]);_0x151f4e[_0x1823c0(0x2bb)]['fillRect'](0x0,0x0,_0x3bb9ec,_0x1c8e3d,_0x41fea8),_0x3bb9ec-=0x2,_0x1c8e3d-=0x2,_0x151f4e['bitmap'][_0x1823c0(0x270)](0x1,0x1,_0x3bb9ec,_0x1c8e3d,_0x5ac88d),_0x3bb9ec-=_0x1dbc66*0x2,_0x1c8e3d-=_0x1dbc66*0x2,_0x151f4e[_0x1823c0(0x2bb)][_0x1823c0(0x270)](0x1+_0x1dbc66,0x1+_0x1dbc66,_0x3bb9ec,_0x1c8e3d,_0x41fea8),_0x3bb9ec-=0x2,_0x1c8e3d-=0x2,_0x1dbc66+=0x1,_0x151f4e[_0x1823c0(0x2bb)][_0x1823c0(0x384)](0x1+_0x1dbc66,0x1+_0x1dbc66,_0x3bb9ec,_0x1c8e3d);}this[_0x1823c0(0x285)]=_0x151f4e,this['addChild'](this[_0x1823c0(0x285)]);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x282)]=function(){const _0x3934fc=_0x55ad4d,_0x14af06=Window_OTB_TurnOrder[_0x3934fc(0x2bd)];return _0x14af06[_0x3934fc(0x21c)];},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x30c)]=function(){const _0x2bff23=_0x55ad4d,_0x1a357f=Window_OTB_TurnOrder[_0x2bff23(0x2bd)];if(!_0x1a357f[_0x2bff23(0x31b)])return;if(this[_0x2bff23(0x359)]===$gameParty)return;const _0x1099f1=this[_0x2bff23(0x23a)](),_0x2b272c=this[_0x2bff23(0x2ff)](),_0xa4ea79=new Sprite();_0xa4ea79[_0x2bff23(0x137)]['x']=this[_0x2bff23(0x137)]['x'],_0xa4ea79['anchor']['y']=this[_0x2bff23(0x137)]['y'],_0xa4ea79[_0x2bff23(0x2bb)]=new Bitmap(_0x1099f1,_0x2b272c),this[_0x2bff23(0x134)]=_0xa4ea79,this['addChild'](this[_0x2bff23(0x134)]);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x1e8)]=function(){const _0x10ba40=_0x55ad4d;return this[_0x10ba40(0x359)]?this[_0x10ba40(0x359)][_0x10ba40(0x12f)]()[this[_0x10ba40(0x339)]]:null;},Sprite_OTB_TurnOrder_Battler['prototype'][_0x55ad4d(0x32f)]=function(){const _0x2d9045=_0x55ad4d;Sprite_Clickable[_0x2d9045(0x159)][_0x2d9045(0x32f)][_0x2d9045(0x175)](this),this['updatePosition'](),this['checkOpacity'](),this['updateOpacity'](),this['updateGraphic'](),this[_0x2d9045(0x114)](),this['updateLetter'](),this['updateSelectionEffect']();},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x207)]=function(_0x87a1cd,_0x3ea758){const _0x21af40=_0x55ad4d,_0x327322=Window_OTB_TurnOrder[_0x21af40(0x2bd)];this[_0x21af40(0x2de)]=_0x327322[_0x21af40(0xff)],this['_positionTargetX']=_0x87a1cd,this[_0x21af40(0x37c)]=_0x3ea758;},Sprite_OTB_TurnOrder_Battler['prototype'][_0x55ad4d(0x380)]=function(){const _0x3b5231=_0x55ad4d;if(this['_positionDuration']>0x0){if(_0x3b5231(0x36b)!==_0x3b5231(0x2da)){const _0x4725b5=this[_0x3b5231(0x2de)];this['x']=(this['x']*(_0x4725b5-0x1)+this['_positionTargetX'])/_0x4725b5,this['y']=(this['y']*(_0x4725b5-0x1)+this[_0x3b5231(0x37c)])/_0x4725b5,this['_positionDuration']--;}else this['_subject']['makeActions']();}if(this['_positionDuration']<=0x0){if(_0x3b5231(0x385)!==_0x3b5231(0x385))_0x4aa3c9[_0x3b5231(0x1c9)]()?this[_0x3b5231(0x29c)]():_0x3a56c4[_0x3b5231(0x153)][_0x3b5231(0x1be)][_0x3b5231(0x175)](this);else{this['x']=this[_0x3b5231(0x238)],this['y']=this[_0x3b5231(0x37c)];if(this['opacity']<0xff&&!this[_0x3b5231(0x272)]&&this[_0x3b5231(0x2b4)]<=0x0){const _0x4c7ff5=this['battler']();_0x4c7ff5&&(this['_fadeTarget']=_0x4c7ff5[_0x3b5231(0x13c)]()&&_0x4c7ff5[_0x3b5231(0x2d0)]()?0xff:0x0);}}}},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x241)]=function(){return 0x1;},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x206)]=function(){const _0x256442=_0x55ad4d;return SceneManager[_0x256442(0x22c)][_0x256442(0x13e)];},Sprite_OTB_TurnOrder_Battler['prototype'][_0x55ad4d(0x32a)]=function(){const _0x59863b=_0x55ad4d,_0x17c09d=this[_0x59863b(0x1e8)]();if(!_0x17c09d)return this[_0x59863b(0x241)]();if(_0x17c09d===BattleManager[_0x59863b(0x15d)])return 0x0;if(BattleManager[_0x59863b(0x1b9)][_0x59863b(0x2bc)](_0x17c09d)){const _0x4b3dd9=BattleManager[_0x59863b(0x1b9)]['indexOf'](_0x17c09d)+0x1;return _0x4b3dd9;}return this[_0x59863b(0x241)]();},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x258)]=function(_0x1d2414){const _0x3b5109=_0x55ad4d,_0x5f4e7a=Window_OTB_TurnOrder['Settings'];this[_0x3b5109(0x2b4)]=_0x5f4e7a[_0x3b5109(0xff)],this[_0x3b5109(0x1fa)]=_0x1d2414;},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]['checkOpacity']=function(){const _0x2fd3c8=_0x55ad4d,_0xa3ddd5=this[_0x2fd3c8(0x1e8)]();if(!_0xa3ddd5)return;if(this[_0x2fd3c8(0x1e6)]===_0xa3ddd5['isAlive']()&&this[_0x2fd3c8(0x287)]===_0xa3ddd5['isAppeared']())return;this[_0x2fd3c8(0x1e6)]=_0xa3ddd5[_0x2fd3c8(0x13c)](),this[_0x2fd3c8(0x287)]=_0xa3ddd5[_0x2fd3c8(0x2d0)]();let _0x296803=this['_isAlive']&&this[_0x2fd3c8(0x287)]?0xff:0x0;this[_0x2fd3c8(0x258)](_0x296803);},Sprite_OTB_TurnOrder_Battler['prototype'][_0x55ad4d(0x235)]=function(){const _0x1d7cc1=_0x55ad4d;if(this[_0x1d7cc1(0x2b4)]>0x0){if('sANms'===_0x1d7cc1(0x370)){const _0x302491=this[_0x1d7cc1(0x2b4)];this['opacity']=(this[_0x1d7cc1(0x1e2)]*(_0x302491-0x1)+this[_0x1d7cc1(0x1fa)])/_0x302491,this[_0x1d7cc1(0x2b4)]--,this[_0x1d7cc1(0x2b4)]<=0x0&&(_0x1d7cc1(0x37e)!==_0x1d7cc1(0x232)?this[_0x1d7cc1(0x1e2)]=this[_0x1d7cc1(0x1fa)]:this[_0x1d7cc1(0x290)]());}else _0x49a6bb[_0x1d7cc1(0x1bc)](),_0x1e00ed[_0x1d7cc1(0x153)][_0x1d7cc1(0x34e)]['call'](this);}if(this['_isBattleOver'])return;BattleManager[_0x1d7cc1(0x185)]===_0x1d7cc1(0x103)&&(this[_0x1d7cc1(0x272)]=!![],this[_0x1d7cc1(0x258)](0x0));},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]['updateGraphic']=function(){const _0x50608b=_0x55ad4d,_0x270b4b=this[_0x50608b(0x1e8)]();if(!_0x270b4b)return;const _0x323cae=Window_OTB_TurnOrder[_0x50608b(0x2bd)],_0x34bcab=this[_0x50608b(0x359)]===$gameParty?'Actor':'Enemy';let _0x5435b1=_0x270b4b['TurnOrderOTBGraphicType']();if(_0x270b4b[_0x50608b(0x1af)]()&&_0x5435b1===_0x50608b(0x1ed))_0x5435b1=_0x50608b(0x36f);else _0x270b4b[_0x50608b(0x22d)]()&&_0x5435b1==='svactor'&&(_0x5435b1=_0x50608b(0x1ed));if(this[_0x50608b(0x223)]!==_0x5435b1)return this[_0x50608b(0x34b)]();switch(this[_0x50608b(0x223)]){case'face':if(this[_0x50608b(0x257)]!==_0x270b4b[_0x50608b(0x16b)]())return'eYacY'===_0x50608b(0x2aa)?(this[_0x50608b(0x356)]===_0x4f9650&&(this[_0x50608b(0x356)]=this['createTurnOrderOTBGraphicFaceIndex']()),this['_otbTurnOrderFaceIndex']):this['processUpdateGraphic']();if(this['_graphicFaceIndex']!==_0x270b4b[_0x50608b(0x28b)]()){if(_0x50608b(0x1d5)==='ehZtR')this[_0x50608b(0x1c9)]()?this[_0x50608b(0x222)]():_0x1d0937[_0x50608b(0x153)][_0x50608b(0x363)]['call'](this);else return this[_0x50608b(0x34b)]();}break;case _0x50608b(0x293):if(this[_0x50608b(0x119)]!==_0x270b4b[_0x50608b(0x27b)]()){if(_0x50608b(0x145)==='RItju')_0x5a2faf['otbAddBattlerToTurnOrderAtEnd'](this,_0x3f4917,_0x233e2a[_0x50608b(0x1b9)]),_0x5c1ab7[_0x50608b(0x250)](this,_0xe60d41,_0x4ff406[_0x50608b(0x2b3)]);else return this[_0x50608b(0x34b)]();}break;case _0x50608b(0x1ed):if(_0x270b4b[_0x50608b(0x12b)]()){if(this[_0x50608b(0x2ab)]!==_0x270b4b[_0x50608b(0x144)]())return this[_0x50608b(0x34b)]();}else{if(this['_graphicEnemy']!==_0x270b4b[_0x50608b(0x108)]())return this['processUpdateGraphic']();}break;case _0x50608b(0x211):if(_0x270b4b['isActor']()){if(_0x50608b(0x311)===_0x50608b(0x311)){if(this['_graphicSv']!==_0x270b4b[_0x50608b(0x108)]())return this['processUpdateGraphic']();}else{const _0xa73115=_0x1e56a7[_0x50608b(0x2bd)],_0x2a6642=this[_0x50608b(0x23a)](),_0x3f23fc=this['bitmapHeight'](),_0x50a6b7=_0x1736fb['min'](_0x2a6642,_0x3f23fc);this[_0x50608b(0x32d)]['bitmap']=new _0x166155(_0x2a6642,_0x3f23fc);const _0x2962ed=this[_0x50608b(0x32d)][_0x50608b(0x2bb)],_0x2296de=_0x386ea2['min'](0x1,_0x50a6b7/_0xdd4733[_0x50608b(0x10d)],_0x50a6b7/_0x24e2c1[_0x50608b(0x200)]),_0x4c48ed=_0x3e5b05[_0x50608b(0x10d)]*_0x2296de,_0x9952e6=_0x3d4cd6['height']*_0x2296de,_0x1bf10f=_0x2f24ba['round']((_0x2a6642-_0x4c48ed)/0x2),_0x11e306=_0x2c9866['round']((_0x3f23fc-_0x9952e6)/0x2);_0x2962ed[_0x50608b(0x214)](_0x333f2a,0x0,0x0,_0x624f07['width'],_0x5a4092[_0x50608b(0x200)],_0x1bf10f,_0x11e306,_0x4c48ed,_0x9952e6);}}else{if(this[_0x50608b(0x150)]!==_0x270b4b['battlerName']()){if(_0x50608b(0x181)!==_0x50608b(0x36a))return this[_0x50608b(0x34b)]();else _0x2d48ee=(_0x3aaccd?_0x449ed5[_0x50608b(0x256)]:_0x2007a0[_0x50608b(0x1ff)])+_0x201727,_0x3e0243-=_0x3deb7b*_0x534e15;}}break;}},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]['processUpdateGraphic']=function(){const _0x1204d9=_0x55ad4d,_0x2eef5b=this[_0x1204d9(0x1e8)]();if(!_0x2eef5b)return;this[_0x1204d9(0x223)]=_0x2eef5b[_0x1204d9(0x2d4)]();if(_0x2eef5b[_0x1204d9(0x1af)]()&&this[_0x1204d9(0x223)]===_0x1204d9(0x1ed))this[_0x1204d9(0x223)]=_0x1204d9(0x36f);else _0x2eef5b[_0x1204d9(0x22d)]()&&this[_0x1204d9(0x223)]===_0x1204d9(0x211)&&(this['_graphicType']=_0x1204d9(0x1ed));let _0x20f4da;switch(this['_graphicType']){case _0x1204d9(0x36f):this[_0x1204d9(0x257)]=_0x2eef5b[_0x1204d9(0x16b)](),this['_graphicFaceIndex']=_0x2eef5b[_0x1204d9(0x28b)](),_0x20f4da=ImageManager['loadFace'](this[_0x1204d9(0x257)]),_0x20f4da[_0x1204d9(0x116)](this[_0x1204d9(0x115)][_0x1204d9(0x26b)](this,_0x20f4da));break;case'icon':this[_0x1204d9(0x119)]=_0x2eef5b['createTurnOrderOTBGraphicIconIndex'](),_0x20f4da=ImageManager[_0x1204d9(0x19c)](_0x1204d9(0x328)),_0x20f4da['addLoadListener'](this[_0x1204d9(0x332)][_0x1204d9(0x26b)](this,_0x20f4da));break;case _0x1204d9(0x1ed):if(_0x2eef5b[_0x1204d9(0x12b)]())this['_graphicSv']=_0x2eef5b[_0x1204d9(0x144)](),_0x20f4da=ImageManager['loadSvActor'](this['_graphicSv']),_0x20f4da['addLoadListener'](this[_0x1204d9(0x172)][_0x1204d9(0x26b)](this,_0x20f4da));else $gameSystem[_0x1204d9(0x29b)]()?(this[_0x1204d9(0x150)]=_0x2eef5b[_0x1204d9(0x108)](),_0x20f4da=ImageManager[_0x1204d9(0x26a)](this[_0x1204d9(0x150)]),_0x20f4da[_0x1204d9(0x116)](this['changeEnemyGraphicBitmap'][_0x1204d9(0x26b)](this,_0x20f4da))):(this[_0x1204d9(0x150)]=_0x2eef5b[_0x1204d9(0x108)](),_0x20f4da=ImageManager[_0x1204d9(0x320)](this['_graphicEnemy']),_0x20f4da[_0x1204d9(0x116)](this[_0x1204d9(0x1a8)][_0x1204d9(0x26b)](this,_0x20f4da)));break;case _0x1204d9(0x211):this['_graphicSv']=_0x2eef5b[_0x1204d9(0x108)](),_0x20f4da=ImageManager[_0x1204d9(0x16a)](this[_0x1204d9(0x2ab)]),_0x20f4da[_0x1204d9(0x116)](this['changeSvActorGraphicBitmap'][_0x1204d9(0x26b)](this,_0x20f4da));break;}},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x115)]=function(_0x3957da){const _0x3c347b=_0x55ad4d,_0x1dec67=this[_0x3c347b(0x1e3)],_0x26094f=this[_0x3c347b(0x23a)](),_0x2056ab=this[_0x3c347b(0x2ff)](),_0xdb6afe=Math[_0x3c347b(0x2f6)](_0x26094f,_0x2056ab);this[_0x3c347b(0x32d)]['bitmap']=new Bitmap(_0x26094f,_0x2056ab);const _0x29cb81=this[_0x3c347b(0x32d)][_0x3c347b(0x2bb)],_0x52d5c3=ImageManager[_0x3c347b(0x203)],_0x32c042=ImageManager[_0x3c347b(0x276)],_0x512d46=_0xdb6afe/Math[_0x3c347b(0x2f6)](_0x52d5c3,_0x32c042),_0x51f639=ImageManager[_0x3c347b(0x203)],_0x401f5f=ImageManager[_0x3c347b(0x276)],_0x2592f1=_0x1dec67%0x4*_0x52d5c3+(_0x52d5c3-_0x51f639)/0x2,_0x28d981=Math[_0x3c347b(0x2fd)](_0x1dec67/0x4)*_0x32c042+(_0x32c042-_0x401f5f)/0x2,_0x316615=(_0x26094f-_0x52d5c3*_0x512d46)/0x2,_0x1b9e6b=(_0x2056ab-_0x32c042*_0x512d46)/0x2;_0x29cb81[_0x3c347b(0x214)](_0x3957da,_0x2592f1,_0x28d981,_0x51f639,_0x401f5f,_0x316615,_0x1b9e6b,_0xdb6afe,_0xdb6afe);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x332)]=function(_0x47e3ca){const _0x5bd671=_0x55ad4d,_0x2bee4d=this['_graphicIconIndex'],_0x1e741c=this['bitmapWidth'](),_0x2428ea=this['bitmapHeight']();this[_0x5bd671(0x32d)][_0x5bd671(0x2bb)]=new Bitmap(_0x1e741c,_0x2428ea);const _0x4ced21=this[_0x5bd671(0x32d)][_0x5bd671(0x2bb)],_0x5d68c0=ImageManager['iconWidth'],_0x3007a6=ImageManager[_0x5bd671(0x20b)],_0x5a7d26=Math[_0x5bd671(0x21b)](_0x5d68c0,_0x3007a6,_0x1e741c,_0x2428ea),_0x5712a6=_0x2bee4d%0x10*_0x5d68c0,_0x2481ed=Math[_0x5bd671(0x2fd)](_0x2bee4d/0x10)*_0x3007a6,_0x28e66a=Math[_0x5bd671(0x2fd)](Math['max'](_0x1e741c-_0x5a7d26,0x0)/0x2),_0x5c4756=Math[_0x5bd671(0x2fd)](Math[_0x5bd671(0x2f6)](_0x2428ea-_0x5a7d26,0x0)/0x2);_0x4ced21[_0x5bd671(0x214)](_0x47e3ca,_0x5712a6,_0x2481ed,_0x5d68c0,_0x3007a6,_0x28e66a,_0x5c4756,_0x5a7d26,_0x5a7d26);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x172)]=function(_0x13fbb6){const _0x40cad9=_0x55ad4d,_0x568874=this[_0x40cad9(0x23a)](),_0x45c7ce=this['bitmapHeight'](),_0x4e5f32=Math['min'](_0x568874,_0x45c7ce);this[_0x40cad9(0x32d)][_0x40cad9(0x2bb)]=new Bitmap(_0x568874,_0x45c7ce);const _0x1a82ef=this['_graphicSprite']['bitmap'],_0x45c703=0x9,_0x238e8e=0x6,_0x2fec92=_0x13fbb6['width']/_0x45c703,_0x2ec592=_0x13fbb6[_0x40cad9(0x200)]/_0x238e8e,_0x1d1298=Math[_0x40cad9(0x21b)](0x1,_0x4e5f32/_0x2fec92,_0x4e5f32/_0x2ec592),_0x453adc=_0x2fec92*_0x1d1298,_0x33190a=_0x2ec592*_0x1d1298,_0x3198ac=Math[_0x40cad9(0x182)]((_0x568874-_0x453adc)/0x2),_0x5687ea=Math[_0x40cad9(0x182)]((_0x45c7ce-_0x33190a)/0x2);_0x1a82ef[_0x40cad9(0x214)](_0x13fbb6,0x0,0x0,_0x2fec92,_0x2ec592,_0x3198ac,_0x5687ea,_0x453adc,_0x33190a);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]['changeEnemyGraphicBitmap']=function(_0x35f43d){const _0x46eb5f=_0x55ad4d,_0x4f6eec=Window_OTB_TurnOrder['Settings'],_0x49051c=this['bitmapWidth'](),_0x213b0a=this[_0x46eb5f(0x2ff)](),_0x4874bc=Math['min'](_0x49051c,_0x213b0a);this[_0x46eb5f(0x32d)][_0x46eb5f(0x2bb)]=new Bitmap(_0x49051c,_0x213b0a);const _0x3174d5=this[_0x46eb5f(0x32d)][_0x46eb5f(0x2bb)],_0x3f8167=Math[_0x46eb5f(0x21b)](0x1,_0x4874bc/_0x35f43d['width'],_0x4874bc/_0x35f43d[_0x46eb5f(0x200)]),_0xfac03d=_0x35f43d['width']*_0x3f8167,_0x39ceb4=_0x35f43d[_0x46eb5f(0x200)]*_0x3f8167,_0x1d201e=Math[_0x46eb5f(0x182)]((_0x49051c-_0xfac03d)/0x2),_0x8fa97b=Math[_0x46eb5f(0x182)]((_0x213b0a-_0x39ceb4)/0x2);_0x3174d5[_0x46eb5f(0x214)](_0x35f43d,0x0,0x0,_0x35f43d['width'],_0x35f43d[_0x46eb5f(0x200)],_0x1d201e,_0x8fa97b,_0xfac03d,_0x39ceb4);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x114)]=function(){const _0x3619d0=_0x55ad4d,_0x5f3ce6=this['battler']();if(!_0x5f3ce6)return;if(!_0x5f3ce6[_0x3619d0(0x22d)]())return;if(this[_0x3619d0(0x318)]===_0x5f3ce6[_0x3619d0(0x1e0)]())return;this[_0x3619d0(0x318)]=_0x5f3ce6[_0x3619d0(0x1e0)]();if(_0x5f3ce6[_0x3619d0(0x12b)]())this['_graphicHue']=0x0;this[_0x3619d0(0x32d)][_0x3619d0(0x107)](this[_0x3619d0(0x318)]);},Sprite_OTB_TurnOrder_Battler['prototype']['updateLetter']=function(){const _0x43b97b=_0x55ad4d;if(!this[_0x43b97b(0x134)])return;const _0xae7ae1=this[_0x43b97b(0x1e8)]();if(!_0xae7ae1)return;if(this[_0x43b97b(0x26f)]===_0xae7ae1[_0x43b97b(0x26f)]&&this[_0x43b97b(0x327)]===_0xae7ae1[_0x43b97b(0x327)])return;this['_letter']=_0xae7ae1[_0x43b97b(0x26f)],this['_plural']=_0xae7ae1['_plural'];const _0x5e4acf=Window_OTB_TurnOrder[_0x43b97b(0x2bd)],_0x3d0a44=this[_0x43b97b(0x23a)](),_0x286ce0=this[_0x43b97b(0x2ff)](),_0x1774d2=this['_letterSprite']['bitmap'];_0x1774d2[_0x43b97b(0x2ba)]();if(!this[_0x43b97b(0x327)])return;_0x1774d2['fontFace']=_0x5e4acf[_0x43b97b(0x1ba)]||$gameSystem[_0x43b97b(0x31e)](),_0x1774d2['fontSize']=_0x5e4acf['EnemyBattlerFontSize']||0x10;if(_0x5e4acf['OrderDirection']){if(_0x43b97b(0x286)!==_0x43b97b(0x286))return _0x3a210c(_0x1fb7db['$1']);else _0x1774d2['drawText'](this[_0x43b97b(0x26f)]['trim'](),_0x3d0a44*0x1/0x8,_0x286ce0/0x2,_0x3d0a44,_0x286ce0/0x2,_0x43b97b(0x205));}else _0x1774d2[_0x43b97b(0x33a)](this[_0x43b97b(0x26f)][_0x43b97b(0x110)](),0x0,_0x286ce0/0x2,_0x3d0a44*0x7/0x8,_0x286ce0/0x2,'right');},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]['updateSelectionEffect']=function(){const _0x4acc24=_0x55ad4d,_0xe90d66=this[_0x4acc24(0x1e8)]();if(!_0xe90d66)return;const _0x1c6822=_0xe90d66['battler']();if(!_0x1c6822)return;const _0x25342c=_0x1c6822[_0x4acc24(0x245)]();if(!_0x25342c)return;this['setBlendColor'](_0x25342c['_blendColor']);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x1bb)]=function(){return null;},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x340)]=function(_0x3177c6){const _0x2dc16f=_0x55ad4d;this['_sourceArray']=_0x3177c6,this[_0x2dc16f(0x330)](),this[_0x2dc16f(0x101)]===null&&(_0x2dc16f(0x249)==='MTxds'?this[_0x2dc16f(0x204)]=-0x1:this[_0x2dc16f(0x217)](this[_0x2dc16f(0x15d)]));},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x330)]=function(){const _0xe198dc=_0x55ad4d,_0x1cc1b5=this[_0xe198dc(0x206)]();if(!_0x1cc1b5)return;const _0xee7edd=Window_OTB_TurnOrder[_0xe198dc(0x2bd)],_0x5f1179=_0xee7edd[_0xe198dc(0xfe)],_0x5aaf29=this[_0xe198dc(0x101)]===_0x1cc1b5[_0xe198dc(0x2c4)]?!![]:![],_0x47dfc4=this[_0xe198dc(0x204)]===-0x1&&BattleManager['_subject']===this['battler'](),_0x321d12=_0x1cc1b5[_0xe198dc(0x167)]-_0xee7edd[_0xe198dc(0x1f0)];let _0x4ebfb2=Math['ceil'](_0x321d12/(this[_0xe198dc(0x101)][_0xe198dc(0x2b2)]-0x1||0x1));_0x4ebfb2=Math[_0xe198dc(0x21b)](_0xee7edd[_0xe198dc(0x1f0)],_0x4ebfb2);let _0x25e35a=0x0,_0x35c27d=0x0,_0xbde9fe=_0x47dfc4?-0x1:this[_0xe198dc(0x101)][_0xe198dc(0x1b4)](this);!_0x47dfc4&&(_0xe198dc(0x147)!==_0xe198dc(0x10b)?_0xbde9fe=this[_0xe198dc(0x279)]():_0x54e848[_0xe198dc(0x153)]['Window_Help_setItem'][_0xe198dc(0x175)](this,_0x509ff1));if(_0x47dfc4)_0x25e35a=_0x1cc1b5[_0xe198dc(0x389)];else{if(_0x5f1179){if('DngLR'!==_0xe198dc(0x267))_0x25e35a=(_0x5aaf29?_0x1cc1b5[_0xe198dc(0x256)]:_0x1cc1b5[_0xe198dc(0x1ff)])+_0x321d12,_0x25e35a-=_0xbde9fe*_0x4ebfb2;else{if(!_0x20d8d3[_0xe198dc(0x219)]())return;if(!_0x5c7247[_0xe198dc(0x1c9)]())return;if(!this[_0xe198dc(0x2d3)]())return;if(!this[_0xe198dc(0x231)]())return;const _0x5be56b=_0x4b588a[_0xe198dc(0x153)][_0xe198dc(0x2f9)],_0x5d5c25=this[_0xe198dc(0x2d3)]()[_0xe198dc(0x350)];_0x5d5c25[_0xe198dc(0x366)](_0x5be56b[_0xe198dc(0x1ab)])&&this[_0xe198dc(0x231)]()[_0xe198dc(0x32c)](0x1);let _0x318575=this['otbCalcUserCurrentOrderChange'](),_0x17aac7=this['otbCalcUserNextOrderChange']();_0x318575!==0x0&&_0x3539db[_0xe198dc(0x195)](this['subject'](),-_0x318575,![]),_0x17aac7!==0x0&&_0x46d8bf[_0xe198dc(0x195)](this['subject'](),-_0x17aac7,!![]);}}else _0x25e35a=_0x5aaf29?_0x1cc1b5[_0xe198dc(0x256)]:_0x1cc1b5[_0xe198dc(0x1ff)],_0x25e35a+=_0xbde9fe*_0x4ebfb2;}_0x25e35a+=this['additionalTargetXAdjustments'](_0xbde9fe,_0xee7edd['SpriteThin']-_0x4ebfb2),!_0x47dfc4&&_0xbde9fe<0x0&&(_0x25e35a=this['x'],_0x35c27d=this['y'],this['startFade'](0x0)),this[_0xe198dc(0x207)](_0x25e35a,_0x35c27d);},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x251)]=function(_0x2a8a86,_0x459f97){return 0x0;},Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)][_0x55ad4d(0x279)]=function(){const _0x261e72=_0x55ad4d,_0x1d49c9=this[_0x261e72(0x206)]();if(!_0x1d49c9)return 0x0;const _0x10f917=this['_sourceArray']===_0x1d49c9[_0x261e72(0x2c4)]?!![]:![],_0x332827=_0x10f917?BattleManager[_0x261e72(0x2b3)]:BattleManager['_actionBattlers'],_0x3c4dc0=this[_0x261e72(0x1e8)](),_0xfc5710=VisuMZ['BattleSystemOTB'][_0x261e72(0x296)](_0x3c4dc0,_0x332827);return _0xfc5710[this[_0x261e72(0x204)]]??_0xfc5710[_0xfc5710['length']-0x1]??-0x1;};function Sprite_OTB_TurnOrder_Preview(){this['initialize'](...arguments);}Sprite_OTB_TurnOrder_Preview['prototype']=Object['create'](Sprite_OTB_TurnOrder_Battler[_0x55ad4d(0x159)]),Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)][_0x55ad4d(0x1ca)]=Sprite_OTB_TurnOrder_Preview,Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)][_0x55ad4d(0x191)]=function(_0x2f66f7,_0x5bbe73,_0x22725f,_0x246c63){const _0x499e87=_0x55ad4d;this[_0x499e87(0x2fa)]=_0x246c63,Sprite_OTB_TurnOrder_Battler[_0x499e87(0x159)][_0x499e87(0x191)][_0x499e87(0x175)](this,_0x2f66f7,_0x5bbe73,_0x22725f),this[_0x499e87(0x14f)]();},Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)][_0x55ad4d(0x14f)]=function(){const _0xc4401c=_0x55ad4d,_0x5ffcb6=Window_OTB_TurnOrder[_0xc4401c(0x2bd)];this[_0xc4401c(0x15c)]['x']=this['scale']['y']=_0x5ffcb6['PreviewScale'];},Sprite_OTB_TurnOrder_Preview['prototype'][_0x55ad4d(0x377)]=function(){const _0x3e392c=_0x55ad4d;return this['_unit']===$gameParty?_0x3e392c(0x28c):'PreviewEnemy';},Sprite_OTB_TurnOrder_Preview['prototype']['getBorderThickness']=function(){const _0x1650e7=_0x55ad4d,_0x10a405=Window_OTB_TurnOrder[_0x1650e7(0x2bd)];return Math[_0x1650e7(0x28f)](_0x10a405[_0x1650e7(0x21c)]/(_0x10a405['PreviewScale']||0.01));},Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)][_0x55ad4d(0x207)]=function(_0x1f5a26,_0x2b66f4){const _0x4f1ca8=_0x55ad4d;Sprite_OTB_TurnOrder_Battler[_0x4f1ca8(0x159)][_0x4f1ca8(0x207)]['call'](this,_0x1f5a26,_0x2b66f4),this['x']=this[_0x4f1ca8(0x238)],this['y']=this[_0x4f1ca8(0x37c)];},Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)][_0x55ad4d(0x258)]=function(_0x470551){const _0x43b78a=_0x55ad4d;Sprite_OTB_TurnOrder_Battler[_0x43b78a(0x159)][_0x43b78a(0x258)][_0x43b78a(0x175)](this,_0x470551),_0x470551>0x0?this[_0x43b78a(0x2b4)]=0x1:(this[_0x43b78a(0x2b4)]/=0x2,this[_0x43b78a(0x2b4)]=Math[_0x43b78a(0x2fd)](this['_fadeDuration']));},Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)]['additionalTargetXAdjustments']=function(_0x587e9d,_0x501149){const _0x2f2cab=_0x55ad4d,_0x43db7e=Window_OTB_TurnOrder[_0x2f2cab(0x2bd)];if(_0x587e9d>0x0){if(this[_0x2f2cab(0x2fa)]>0x0)return _0x43db7e[_0x2f2cab(0xfe)]?-_0x43db7e[_0x2f2cab(0x1f0)]:_0x43db7e[_0x2f2cab(0x1f0)];else{if(this[_0x2f2cab(0x2fa)]<0x0){if(_0x2f2cab(0x10a)!==_0x2f2cab(0x2d8)){if(_0x43db7e['OrderDirection'])return-_0x501149;else{if(_0x2f2cab(0xfc)===_0x2f2cab(0xfc))return _0x501149;else _0x103844[_0x2f2cab(0x101)][_0x2f2cab(0x368)](_0x193519);}}else _0x528715(_0x2f2cab(0x275)[_0x2f2cab(0x362)](_0x3be157,_0x49427c,_0x57c9cd)),_0x4f0d5f['exit']();}}}return 0x0;},Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)][_0x55ad4d(0x279)]=function(){const _0x3de100=_0x55ad4d,_0x28ef82=this[_0x3de100(0x206)](),_0x1028d7=this[_0x3de100(0x101)]===_0x28ef82[_0x3de100(0x2c4)]?!![]:![],_0x2ab29b=_0x1028d7?BattleManager[_0x3de100(0x2b3)]:BattleManager[_0x3de100(0x1b9)];let _0x30431e=0x0,_0x5045a3=_0x2ab29b['length']-0x1;if(_0x1028d7){if('ILiLw'===_0x3de100(0x12c))_0x30431e=Math[_0x3de100(0x2f6)](0x0,VisuMZ['BattleSystemOTB'][_0x3de100(0x302)](_0x2ab29b));else{if(!this['isOTB']())return;const _0x1cb324=_0x13e897[_0x3de100(0x22c)]['_otbTurnOrderWindow'];if(!_0x1cb324)return;_0x1cb324[_0x3de100(0x132)]();}}let _0x174e27=Sprite_OTB_TurnOrder_Battler[_0x3de100(0x159)]['calculateTargetIndex'][_0x3de100(0x175)](this);return _0x174e27+=this[_0x3de100(0x2fa)],_0x174e27[_0x3de100(0x1da)](_0x30431e,_0x5045a3);},Sprite_OTB_TurnOrder_Preview[_0x55ad4d(0x159)][_0x55ad4d(0x121)]=function(){},Window_Selectable['prototype'][_0x55ad4d(0x1ee)]=function(){return![];},VisuMZ[_0x55ad4d(0x153)]['Window_Selectable_select']=Window_Selectable[_0x55ad4d(0x159)][_0x55ad4d(0x1cf)],Window_Selectable[_0x55ad4d(0x159)][_0x55ad4d(0x1cf)]=function(_0x16a775){const _0x32fbf1=_0x55ad4d;VisuMZ['BattleSystemOTB'][_0x32fbf1(0x291)][_0x32fbf1(0x175)](this,_0x16a775);if(this[_0x32fbf1(0x1ee)]()&&this[_0x32fbf1(0x1c1)]){if(_0x32fbf1(0x202)===_0x32fbf1(0x202))this[_0x32fbf1(0x30e)]();else{if(!this['isOTB']())return;const _0x309d55=_0x59474d[_0x32fbf1(0x22c)][_0x32fbf1(0x13e)];if(!_0x309d55)return;_0x309d55['previewOrderByAction'](null);}}},Window_Selectable[_0x55ad4d(0x159)][_0x55ad4d(0x30e)]=function(){const _0x3949eb=_0x55ad4d;BattleManager[_0x3949eb(0x10c)]();},VisuMZ[_0x55ad4d(0x153)]['Window_Help_setItem']=Window_Help['prototype']['setItem'],Window_Help[_0x55ad4d(0x159)][_0x55ad4d(0x1cb)]=function(_0x41fa77){const _0x380b24=_0x55ad4d;BattleManager[_0x380b24(0x1c9)]()&&_0x41fa77&&_0x41fa77[_0x380b24(0x350)]&&_0x41fa77['note'][_0x380b24(0x366)](/<(?:OTB) HELP>\s*([\s\S]*)\s*<\/(?:OTB) HELP>/i)?_0x380b24(0x197)!==_0x380b24(0x197)?this[_0x380b24(0x13e)]['resumeTurnOrderSprites']():this[_0x380b24(0x314)](String(RegExp['$1'])):_0x380b24(0x252)!==_0x380b24(0x252)?_0x2acba7=_0x380b24(0x1ed):VisuMZ[_0x380b24(0x153)][_0x380b24(0x364)]['call'](this,_0x41fa77);},Window_ActorCommand[_0x55ad4d(0x159)]['isBattleItemWindowOTB']=function(){return BattleManager['isOTB']();},Window_ActorCommand[_0x55ad4d(0x159)][_0x55ad4d(0x30e)]=function(){const _0x1e8623=_0x55ad4d,_0x53736c=BattleManager[_0x1e8623(0x2e7)]();if(_0x53736c){const _0x22e33b=this[_0x1e8623(0x11c)]();switch(_0x22e33b){case _0x1e8623(0x2cd):_0x53736c[_0x1e8623(0x27e)]();break;case _0x1e8623(0x2e5):_0x53736c[_0x1e8623(0x1f3)]();break;case _0x1e8623(0x369):_0x53736c[_0x1e8623(0x2be)](this[_0x1e8623(0x372)]());break;default:_0x53736c[_0x1e8623(0x2be)](null);break;}}Window_Command[_0x1e8623(0x159)][_0x1e8623(0x30e)][_0x1e8623(0x175)](this);},Window_BattleSkill[_0x55ad4d(0x159)][_0x55ad4d(0x1ee)]=function(){const _0x1324c6=_0x55ad4d;return BattleManager[_0x1324c6(0x1c9)]();},Window_BattleSkill[_0x55ad4d(0x159)][_0x55ad4d(0x30e)]=function(){const _0x39061a=_0x55ad4d,_0x5a83e2=this[_0x39061a(0x2d3)](),_0x26dbac=BattleManager[_0x39061a(0x2e7)]();if(_0x26dbac)_0x26dbac[_0x39061a(0x2be)](_0x5a83e2?_0x5a83e2['id']:null);Window_SkillList['prototype'][_0x39061a(0x30e)][_0x39061a(0x175)](this);},Window_BattleItem['prototype'][_0x55ad4d(0x1ee)]=function(){return BattleManager['isOTB']();},Window_BattleItem[_0x55ad4d(0x159)][_0x55ad4d(0x30e)]=function(){const _0x2f3fdc=_0x55ad4d,_0x202948=this[_0x2f3fdc(0x2d3)](),_0x2f095e=BattleManager[_0x2f3fdc(0x2e7)]();if(_0x2f095e)_0x2f095e[_0x2f3fdc(0x1cb)](_0x202948?_0x202948['id']:null);Window_ItemList[_0x2f3fdc(0x159)][_0x2f3fdc(0x30e)]['call'](this);},Window_BattleActor[_0x55ad4d(0x159)][_0x55ad4d(0x1ee)]=function(){const _0xff44c2=_0x55ad4d;return BattleManager[_0xff44c2(0x1c9)]();},Window_BattleEnemy['prototype'][_0x55ad4d(0x1ee)]=function(){return BattleManager['isOTB']();};function Window_OTB_TurnOrder(){const _0x33cb4e=_0x55ad4d;this[_0x33cb4e(0x191)](...arguments);}Window_OTB_TurnOrder[_0x55ad4d(0x159)]=Object['create'](Window_Base['prototype']),Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x1ca)]=Window_OTB_TurnOrder,Window_OTB_TurnOrder[_0x55ad4d(0x2bd)]=VisuMZ[_0x55ad4d(0x153)]['Settings'][_0x55ad4d(0x325)],Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x191)]=function(){const _0x1e6b23=_0x55ad4d,_0x4be7f2=this[_0x1e6b23(0x33e)]();this['initHomePositions'](_0x4be7f2),Window_Base[_0x1e6b23(0x159)][_0x1e6b23(0x191)]['call'](this,_0x4be7f2),this[_0x1e6b23(0x1e2)]=0x0,this[_0x1e6b23(0x2dc)](),this[_0x1e6b23(0x2a9)](),this[_0x1e6b23(0x303)](),this[_0x1e6b23(0x1ea)]();},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x33e)]=function(){const _0x3cc3d9=_0x55ad4d,_0x210a1c=Window_OTB_TurnOrder['Settings'],_0x37744c=SceneManager[_0x3cc3d9(0x22c)][_0x3cc3d9(0x25b)][_0x3cc3d9(0x200)];let _0x2588b7=Graphics[_0x3cc3d9(0x10d)]-_0x210a1c[_0x3cc3d9(0x1a1)]*0x2,_0x512631=_0x210a1c['SpriteLength']+this[_0x3cc3d9(0x105)](),_0x55539c=_0x210a1c[_0x3cc3d9(0x1a1)],_0x959509=0x0;switch(_0x210a1c[_0x3cc3d9(0x173)]){case'bottom':_0x959509=Graphics[_0x3cc3d9(0x200)]-_0x37744c-_0x210a1c[_0x3cc3d9(0x1a1)]-_0x512631;break;default:_0x959509=_0x210a1c['ScreenBuffer'];break;}if(Imported[_0x3cc3d9(0x351)]&&BattleManager[_0x3cc3d9(0x234)]()){if(_0x3cc3d9(0x25c)!=='gkUKG'){const _0x3b7cc3=VisuMZ[_0x3cc3d9(0x32e)][_0x3cc3d9(0x2bd)]['StatusWindow'];_0x2588b7-=_0x3b7cc3['WidthBase']+_0x3b7cc3[_0x3cc3d9(0x2d1)],_0x2588b7-=_0x210a1c[_0x3cc3d9(0x1a1)];}else this[_0x3cc3d9(0x1f9)]=!![];}return _0x55539c+=_0x210a1c[_0x3cc3d9(0x187)]||0x0,_0x959509+=_0x210a1c[_0x3cc3d9(0x17f)]||0x0,new Rectangle(_0x55539c,_0x959509,_0x2588b7,_0x512631);},Window_OTB_TurnOrder['prototype']['initHomePositions']=function(_0x57a667){const _0x292e5d=_0x55ad4d;this[_0x292e5d(0x13a)]=this[_0x292e5d(0x255)]=_0x57a667['x'],this[_0x292e5d(0x24a)]=this[_0x292e5d(0x268)]=_0x57a667['y'],this[_0x292e5d(0x27a)]=0x0;const _0x3e0055=Window_OTB_TurnOrder['Settings'];this[_0x292e5d(0x167)]=Math[_0x292e5d(0x28f)]((_0x57a667[_0x292e5d(0x10d)]-_0x3e0055[_0x292e5d(0x1f0)]-_0x3e0055[_0x292e5d(0x213)]*0x2)/0x2);if(_0x3e0055[_0x292e5d(0xfe)]){if('oKnCm'===_0x292e5d(0x260))this[_0x292e5d(0x389)]=_0x57a667['width']-_0x3e0055['SpriteThin'],this[_0x292e5d(0x1ff)]=this[_0x292e5d(0x167)]+_0x3e0055[_0x292e5d(0x213)],this[_0x292e5d(0x256)]=0x0;else return 0x0;}else this['_subjectX']=0x0,this[_0x292e5d(0x1ff)]=_0x3e0055[_0x292e5d(0x1f0)]+_0x3e0055[_0x292e5d(0x213)],this[_0x292e5d(0x256)]=this['_currentX']+_0x3e0055['SubjectDistance']+this[_0x292e5d(0x167)];},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x142)]=function(){const _0xcae20d=_0x55ad4d;this[_0xcae20d(0x220)]=0x0;},Window_OTB_TurnOrder['prototype'][_0x55ad4d(0x2dc)]=function(){const _0x31aa39=_0x55ad4d,_0x54e482=Window_OTB_TurnOrder[_0x31aa39(0x2bd)];if(_0x54e482[_0x31aa39(0x13b)]===_0x31aa39(0x23d))return;if(_0x54e482['BgDimStyle']==='image'&&_0x54e482[_0x31aa39(0x2ac)]!==''){const _0x3a910b=ImageManager[_0x31aa39(0x19c)](_0x54e482[_0x31aa39(0x2ac)]);_0x3a910b['addLoadListener'](this[_0x31aa39(0x2e8)][_0x31aa39(0x26b)](this,_0x3a910b));return;};const _0x36dbaa=this['contentsBack'],_0x1b1183=ColorManager[_0x31aa39(0x2eb)](),_0x4eaa54=ColorManager[_0x31aa39(0x313)](),_0xefbd3=this[_0x31aa39(0x389)],_0x330cca=_0x54e482[_0x31aa39(0x1f0)],_0x4f06a0=0x0,_0x1b38d9=_0x54e482['SpriteLength'],_0x2f7e1a=this[_0x31aa39(0x1ff)],_0x949bd9=this['_nextX'],_0x5964b2=this[_0x31aa39(0x167)];switch(_0x54e482['BgDimStyle']){case _0x31aa39(0x2dd):if(_0x54e482[_0x31aa39(0xfe)]){if(_0x31aa39(0x1c8)!==_0x31aa39(0x170))_0x36dbaa[_0x31aa39(0x298)](_0xefbd3,_0x4f06a0,_0x330cca/0x2,_0x1b38d9,_0x4eaa54,_0x1b1183,![]),_0x36dbaa['fillRect'](_0xefbd3+_0x330cca/0x2,_0x4f06a0,_0x330cca/0x2,_0x1b38d9,_0x1b1183),_0x36dbaa[_0x31aa39(0x298)](_0x2f7e1a,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x4eaa54,_0x1b1183,![]),_0x36dbaa[_0x31aa39(0x270)](_0x2f7e1a+_0x5964b2/0x2,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x1b1183),_0x36dbaa[_0x31aa39(0x298)](_0x949bd9,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x4eaa54,_0x1b1183,![]),_0x36dbaa[_0x31aa39(0x270)](_0x949bd9+_0x5964b2/0x2,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x1b1183);else{if(_0x2de7f5[_0x28c5a2]===_0xe1fdee)_0x3a8534['push'](_0x1096c0);}}else{if(_0x31aa39(0x2a5)===_0x31aa39(0x18a)){if(!_0x1b4e92['Settings'][_0x31aa39(0x2a8)])return;const _0x5022d2=_0x4bad46[_0x31aa39(0x2bd)],_0x3c3826=this[_0x31aa39(0x377)](),_0x40cea9=_0x31aa39(0x324)[_0x31aa39(0x362)](_0x3c3826),_0x29a4e5=new _0x4d3bd9();_0x29a4e5['anchor']['x']=this[_0x31aa39(0x137)]['x'],_0x29a4e5[_0x31aa39(0x137)]['y']=this[_0x31aa39(0x137)]['y'];if(_0x5022d2[_0x40cea9])_0x29a4e5[_0x31aa39(0x2bb)]=_0x19035b[_0x31aa39(0x19c)](_0x5022d2[_0x40cea9]);else{let _0x470ee7=this[_0x31aa39(0x23a)](),_0x464f76=this[_0x31aa39(0x2ff)](),_0x117910=this[_0x31aa39(0x282)]();_0x29a4e5[_0x31aa39(0x2bb)]=new _0x171150(_0x470ee7,_0x464f76);const _0xcf6c41=_0x31aa39(0x388),_0x3e77f6=_0x53759f[_0x31aa39(0x1c2)](_0x5022d2[_0x31aa39(0x2b7)['format'](_0x3c3826)]);_0x29a4e5[_0x31aa39(0x2bb)]['fillRect'](0x0,0x0,_0x470ee7,_0x464f76,_0xcf6c41),_0x470ee7-=0x2,_0x464f76-=0x2,_0x29a4e5[_0x31aa39(0x2bb)][_0x31aa39(0x270)](0x1,0x1,_0x470ee7,_0x464f76,_0x3e77f6),_0x470ee7-=_0x117910*0x2,_0x464f76-=_0x117910*0x2,_0x29a4e5['bitmap']['fillRect'](0x1+_0x117910,0x1+_0x117910,_0x470ee7,_0x464f76,_0xcf6c41),_0x470ee7-=0x2,_0x464f76-=0x2,_0x117910+=0x1,_0x29a4e5[_0x31aa39(0x2bb)][_0x31aa39(0x384)](0x1+_0x117910,0x1+_0x117910,_0x470ee7,_0x464f76);}this[_0x31aa39(0x285)]=_0x29a4e5,this[_0x31aa39(0x24e)](this['_backgroundSprite']);}else _0x36dbaa['fillRect'](_0xefbd3,_0x4f06a0,_0x330cca/0x2,_0x1b38d9,_0x1b1183),_0x36dbaa[_0x31aa39(0x298)](_0xefbd3+_0x330cca/0x2,_0x4f06a0,_0x330cca/0x2,_0x1b38d9,_0x1b1183,_0x4eaa54,![]),_0x36dbaa['fillRect'](_0x2f7e1a,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x1b1183),_0x36dbaa[_0x31aa39(0x298)](_0x2f7e1a+_0x5964b2/0x2,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x1b1183,_0x4eaa54,![]),_0x36dbaa[_0x31aa39(0x270)](_0x949bd9,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x1b1183),_0x36dbaa['gradientFillRect'](_0x949bd9+_0x5964b2/0x2,_0x4f06a0,_0x5964b2/0x2,_0x1b38d9,_0x1b1183,_0x4eaa54,![]);}break;default:_0x36dbaa[_0x31aa39(0x270)](_0xefbd3,_0x4f06a0,_0x330cca,_0x1b38d9,_0x1b1183),_0x36dbaa[_0x31aa39(0x270)](_0x2f7e1a,_0x4f06a0,_0x5964b2,_0x1b38d9,_0x1b1183),_0x36dbaa[_0x31aa39(0x270)](_0x949bd9,_0x4f06a0,_0x5964b2,_0x1b38d9,_0x1b1183);break;}},Window_OTB_TurnOrder['prototype'][_0x55ad4d(0x2e8)]=function(_0x54587d){const _0x3423a7=_0x55ad4d;this[_0x3423a7(0x124)]=new Sprite(),this[_0x3423a7(0x124)][_0x3423a7(0x2bb)]=_0x54587d,this[_0x3423a7(0x34c)](this[_0x3423a7(0x124)]);const _0x229954=Window_OTB_TurnOrder['Settings'];this['_bgImageSprite']['x']=_0x229954[_0x3423a7(0x239)],this[_0x3423a7(0x124)]['y']=_0x229954['BgImageOffsetY'];},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x2a9)]=function(){const _0x53b6a3=_0x55ad4d;this[_0x53b6a3(0x23b)][_0x53b6a3(0x2ba)](),this[_0x53b6a3(0x177)]();const _0x972b05=Window_OTB_TurnOrder[_0x53b6a3(0x2bd)];this[_0x53b6a3(0x23b)][_0x53b6a3(0x300)]=_0x972b05[_0x53b6a3(0x14d)];let _0x401a08=_0x972b05['UiAlignment'];if(_0x401a08===_0x53b6a3(0x2f3)){if(_0x53b6a3(0x2b9)!==_0x53b6a3(0x344))_0x401a08=_0x972b05[_0x53b6a3(0xfe)]?_0x53b6a3(0x2f7):_0x53b6a3(0x205);else{let _0x4cad3c=this['_otb_createdFirstTurnOrders']?0x1:0x2;while(_0x4cad3c--){this[_0x53b6a3(0x37f)]();}const _0x194af7=!this[_0x53b6a3(0x102)];this[_0x53b6a3(0x102)]=!![];}}let _0x41bfe2=_0x972b05[_0x53b6a3(0x2ed)];if(_0x972b05[_0x53b6a3(0x21d)]!==''){const _0x43506c=this['_subjectX']+_0x972b05[_0x53b6a3(0x20c)],_0x57dbaa=_0x41bfe2+_0x972b05['UiSubjectOffsetY'],_0x305343=_0x972b05['SpriteThin'];this[_0x53b6a3(0x33a)](_0x972b05[_0x53b6a3(0x21d)],_0x43506c,_0x57dbaa,_0x305343,_0x53b6a3(0x236));}if(_0x972b05[_0x53b6a3(0x1c3)]!==''){const _0x3823ae=this[_0x53b6a3(0x1ff)]+_0x972b05[_0x53b6a3(0x112)],_0xbc476a=_0x41bfe2+_0x972b05[_0x53b6a3(0x310)],_0xebd5fb=this[_0x53b6a3(0x167)];this['drawText'](_0x972b05[_0x53b6a3(0x1c3)],_0x3823ae,_0xbc476a,_0xebd5fb,_0x401a08);}if(_0x972b05['UiNextText']!==''){if('UVgbj'!=='RYwAY'){const _0x1601b4=this[_0x53b6a3(0x256)]+_0x972b05[_0x53b6a3(0x1c7)],_0x4585f0=_0x41bfe2+_0x972b05[_0x53b6a3(0x18b)],_0xa227=this[_0x53b6a3(0x167)];this['drawText'](_0x972b05[_0x53b6a3(0x1eb)],_0x1601b4,_0x4585f0,_0xa227,_0x401a08);}else this[_0x53b6a3(0x224)](_0x8f01b9[_0x53b6a3(0x1b9)],!![]),this[_0x53b6a3(0x224)](_0x5a5ccf[_0x53b6a3(0x2b3)],!![]),this['shiftTurnOrderForSubject'](_0x4aeda5[_0x53b6a3(0x15d)],!![]),this[_0x53b6a3(0x2ae)]();}},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x303)]=function(){const _0x2e4360=_0x55ad4d,_0x5e11ad=Window_OTB_TurnOrder[_0x2e4360(0x2bd)];this['_spriteContainer']=new Sprite(),this['addChild'](this['_spriteContainer']),this['_subject']=null,this[_0x2e4360(0x109)]=[],this[_0x2e4360(0x2c4)]=[],this[_0x2e4360(0x319)]=new Sprite(),this[_0x2e4360(0x319)]['x']=_0x5e11ad['PreviewOffsetX'],this[_0x2e4360(0x319)]['y']=_0x5e11ad[_0x2e4360(0x130)],this[_0x2e4360(0x319)]['x']-=Math[_0x2e4360(0x28f)](_0x5e11ad[_0x2e4360(0x1f0)]*0.5*_0x5e11ad['PreviewScale']);if(_0x5e11ad[_0x2e4360(0xfe)]){if(_0x2e4360(0x33b)!==_0x2e4360(0x16d))this['_previewContainer']['x']+=_0x5e11ad[_0x2e4360(0x1f0)];else return'face';}this[_0x2e4360(0x319)]['y']-=Math[_0x2e4360(0x28f)](_0x5e11ad[_0x2e4360(0x2ed)]*0.5*_0x5e11ad['PreviewScale']),this[_0x2e4360(0x24e)](this[_0x2e4360(0x319)]),this[_0x2e4360(0x156)]=[],this[_0x2e4360(0x355)]=[];},Window_OTB_TurnOrder[_0x55ad4d(0x159)]['update']=function(){const _0x2509ce=_0x55ad4d;Window_Base['prototype']['update'][_0x2509ce(0x175)](this),this[_0x2509ce(0x2e1)](),this[_0x2509ce(0x380)](),this[_0x2509ce(0x1ea)](),this[_0x2509ce(0x2ae)]();},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x1f6)]=function(){const _0xb3225e=_0x55ad4d;this[_0xb3225e(0x30d)]=!![];},Window_OTB_TurnOrder[_0x55ad4d(0x159)]['updateTurnOrders']=function(){const _0x29b8b7=_0x55ad4d;if(!this[_0x29b8b7(0x30d)])return;this['_requestTurnOrderUpdate']=![];for(const _0x4ee01c of this[_0x29b8b7(0x109)]){if(!_0x4ee01c)continue;_0x4ee01c['calculateTargetPositions']();}for(const _0x1ebdee of this[_0x29b8b7(0x2c4)]){if(!_0x1ebdee)continue;_0x1ebdee[_0x29b8b7(0x330)]();}},Window_OTB_TurnOrder[_0x55ad4d(0x159)]['updatePosition']=function(){const _0x56b186=_0x55ad4d,_0x2af0fd=Window_OTB_TurnOrder[_0x56b186(0x2bd)];if(_0x2af0fd[_0x56b186(0x173)]!==_0x56b186(0x17c))return;if(!_0x2af0fd[_0x56b186(0x383)])return;const _0x3ab9a3=SceneManager[_0x56b186(0x22c)]['_helpWindow'];if(!_0x3ab9a3)return;if(_0x3ab9a3[_0x56b186(0x15f)]){if('FuAzu'===_0x56b186(0x2d7)){const _0x3bbdec=_0x174434['Settings'];this[_0x56b186(0x15c)]['x']=this[_0x56b186(0x15c)]['y']=_0x3bbdec[_0x56b186(0x1e7)];}else this['x']=this['_homeX']+(_0x2af0fd[_0x56b186(0x117)]||0x0),this['y']=this[_0x56b186(0x268)]+(_0x2af0fd[_0x56b186(0x1bf)]||0x0);}else this['x']=this['_homeX'],this['y']=this['_homeY'];const _0x250f45=SceneManager['_scene'][_0x56b186(0x190)];if(this[_0x56b186(0x297)]===undefined){if(_0x56b186(0x240)===_0x56b186(0x240))this[_0x56b186(0x297)]=Math[_0x56b186(0x182)]((Graphics['width']-Math[_0x56b186(0x21b)](Graphics[_0x56b186(0x1a7)],_0x250f45[_0x56b186(0x10d)]))/0x2),this['_ogWindowLayerY']=Math[_0x56b186(0x182)]((Graphics[_0x56b186(0x200)]-Math['min'](Graphics[_0x56b186(0x38c)],_0x250f45[_0x56b186(0x200)]))/0x2);else{if(!this[_0x56b186(0x15d)])return;this[_0x56b186(0x14b)](this[_0x56b186(0x15d)]);}}this['x']+=_0x250f45['x']-this[_0x56b186(0x297)],this['y']+=_0x250f45['y']-this[_0x56b186(0x169)];},Window_OTB_TurnOrder['prototype'][_0x55ad4d(0x1ea)]=function(){const _0x109043=_0x55ad4d;this[_0x109043(0x15f)]=$gameSystem[_0x109043(0x24d)]();if(BattleManager[_0x109043(0x185)]===_0x109043(0x103)){if(_0x109043(0x27f)!=='UkrWi'){if(!this[_0x109043(0x253)]){const _0x50bdf0=Window_OTB_TurnOrder[_0x109043(0x2bd)];this['_fadeSpeed']=Math[_0x109043(0x28f)](0xff/(_0x50bdf0[_0x109043(0xff)]||0x1));}this[_0x109043(0x1e2)]-=this[_0x109043(0x253)],this[_0x109043(0x2db)]-=this['_fadeSpeed'],this[_0x109043(0x15a)][_0x109043(0x1e2)]-=this['_fadeSpeed'];}else this[_0x109043(0x220)]=0x0;}},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x2ae)]=function(){const _0x5de659=_0x55ad4d;if(!this[_0x5de659(0x1ae)])return;const _0x1bb052=Window_OTB_TurnOrder[_0x5de659(0x2bd)],_0x40a56c=_0x1bb052['OrderDirection'];_0x40a56c?this[_0x5de659(0x1ae)][_0x5de659(0x36e)][_0x5de659(0x2ea)]((_0x2f0684,_0x39f70b)=>_0x2f0684['x']-_0x39f70b['x']):_0x5de659(0x2fe)===_0x5de659(0x2fe)?this['_spriteContainer']['children'][_0x5de659(0x2ea)]((_0x2482c3,_0x364e1b)=>_0x364e1b['x']-_0x2482c3['x']):(this['x']=this[_0x5de659(0x255)],this['y']=this[_0x5de659(0x268)]);},Window_OTB_TurnOrder['prototype']['removeSprite']=function(_0x4057ed){const _0x5df0af=_0x55ad4d;if(!_0x4057ed)return;_0x4057ed[_0x5df0af(0x101)]&&_0x4057ed[_0x5df0af(0x101)][_0x5df0af(0x368)](_0x4057ed);const _0x186fe5=Window_OTB_TurnOrder[_0x5df0af(0x2bd)],_0x3d2923=0x3e8/0x3c*_0x186fe5['UpdateFrames']+0x1f4;_0x4057ed[_0x5df0af(0x258)](0x0),setTimeout(this[_0x5df0af(0x1c5)][_0x5df0af(0x26b)](this,_0x4057ed),_0x3d2923);},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x1c5)]=function(_0x2b1b9f){const _0x584869=_0x55ad4d;_0x2b1b9f[_0x584869(0x101)]&&_0x2b1b9f['_sourceArray'][_0x584869(0x368)](_0x2b1b9f),this[_0x584869(0x1ae)]['removeChild'](_0x2b1b9f),this['_previewContainer'][_0x584869(0x168)](_0x2b1b9f);},Window_OTB_TurnOrder['prototype'][_0x55ad4d(0x246)]=function(){const _0x15043f=_0x55ad4d;if(!this['_subject'])return;this[_0x15043f(0x14b)](this[_0x15043f(0x15d)]);},Window_OTB_TurnOrder['prototype'][_0x55ad4d(0x1d4)]=function(){const _0x357de9=_0x55ad4d;while(this[_0x357de9(0x109)]['length']){const _0x1d0709=this['_currentTurn']['shift']();_0x1d0709[_0x357de9(0x258)](0x0);}while(this[_0x357de9(0x2c4)]['length']){const _0x139e07=this[_0x357de9(0x2c4)][_0x357de9(0x1f2)]();if(!_0x139e07)continue;this['_currentTurn'][_0x357de9(0x20d)](_0x139e07);}for(const _0x3e3e17 of this[_0x357de9(0x109)]){if(_0x357de9(0x352)!==_0x357de9(0x2a0)){if(!_0x3e3e17)continue;_0x3e3e17[_0x357de9(0x340)](this[_0x357de9(0x109)]);}else _0x249a7d['otbPreviewOrderClear'](),_0x282f9c[_0x357de9(0x153)][_0x357de9(0x349)]['call'](this);}},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x224)]=function(_0x159672,_0x24fe13){const _0x488811=_0x55ad4d,_0x4028c6=_0x159672===BattleManager[_0x488811(0x1b9)]?this[_0x488811(0x109)]:this[_0x488811(0x2c4)],_0x24e0dc={};for(const _0x31ffe7 of _0x159672){const _0x1d18c9=_0x488811(0x152)[_0x488811(0x362)](_0x31ffe7[_0x488811(0x1af)]()?_0x488811(0x1d9):_0x488811(0x1ed),_0x31ffe7[_0x488811(0x367)]());_0x24e0dc[_0x1d18c9]=_0x24e0dc[_0x1d18c9]||0x0;const _0x27b883=_0x24e0dc[_0x1d18c9]++,_0x3c0972=new Sprite_OTB_TurnOrder_Battler(_0x31ffe7,_0x27b883,_0x4028c6);this['_spriteContainer'][_0x488811(0x24e)](_0x3c0972),_0x4028c6[_0x488811(0x20d)](_0x3c0972);}for(const _0x17d48e of _0x4028c6){if(_0x488811(0x1b6)!=='ZdWLo'){if(!_0x17d48e)continue;_0x17d48e[_0x488811(0x258)](0xff),_0x17d48e['calculateTargetPositions'](),_0x24fe13&&(_0x488811(0x20a)===_0x488811(0x1e4)?this[_0x488811(0x127)]():(_0x17d48e[_0x488811(0x1e2)]=0xff,_0x17d48e['x']=_0x17d48e[_0x488811(0x238)],_0x17d48e[_0x488811(0x2de)]=0x0));}else this[_0x488811(0x2b4)]/=0x2,this[_0x488811(0x2b4)]=_0x5b4c69[_0x488811(0x2fd)](this[_0x488811(0x2b4)]);}},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x2ef)]=function(){const _0x5366ec=_0x55ad4d,_0x557423=BattleManager[_0x5366ec(0x2b3)];this[_0x5366ec(0x224)](_0x557423);},Window_OTB_TurnOrder[_0x55ad4d(0x159)]['shiftTurnOrderForSubject']=function(_0x5ace0f,_0x590c63){const _0x1146b6=_0x55ad4d;this[_0x1146b6(0x246)]();for(const _0x52e234 of this['_currentTurn']){if('MAYzQ'!==_0x1146b6(0x199)){const _0xda0bba=_0x4e82fb[_0x1146b6(0x2bd)];if(_0xda0bba[_0x1146b6(0x173)]!==_0x1146b6(0x17c))return;if(!_0xda0bba[_0x1146b6(0x164)])return;if(!this['_logWindow'])return;const _0x516986=this[_0x1146b6(0x13e)]['y']-_0xa89ccd[_0x1146b6(0x182)]((_0x4b928f[_0x1146b6(0x200)]-_0x167a0d['boxHeight'])/0x2),_0x435df7=_0x516986+this[_0x1146b6(0x13e)][_0x1146b6(0x200)];this[_0x1146b6(0x13d)]['y']=_0x435df7+(_0xda0bba['LogWindowOffsetY']||0x0);}else{if(!_0x52e234)continue;_0x52e234[_0x1146b6(0x1e8)]()===_0x5ace0f&&(_0x52e234[_0x1146b6(0x204)]=_0x52e234[_0x1146b6(0x204)]||0x0,_0x52e234[_0x1146b6(0x204)]--);}}const _0x3fa915=this[_0x1146b6(0x109)][_0x1146b6(0x361)](_0x44ba65=>_0x44ba65[_0x1146b6(0x1e8)]()===_0x5ace0f);if(this[_0x1146b6(0x109)][_0x3fa915])this['_subject']=this[_0x1146b6(0x109)][_0x3fa915],this[_0x1146b6(0x109)][_0x3fa915][_0x1146b6(0x330)](),this[_0x1146b6(0x109)][_0x1146b6(0x22e)](_0x3fa915,0x1);else{const _0x4bbc4e=new Sprite_OTB_TurnOrder_Battler(_0x5ace0f,-0x1,null);this[_0x1146b6(0x1ae)]['addChild'](_0x4bbc4e),this[_0x1146b6(0x15d)]=_0x4bbc4e,_0x4bbc4e['startFade'](0xff),_0x4bbc4e[_0x1146b6(0x2de)]=0x258,_0x4bbc4e['x']=this[_0x1146b6(0x389)],_0x4bbc4e[_0x1146b6(0x238)]=this[_0x1146b6(0x389)];if(_0x590c63){if(_0x1146b6(0x242)!==_0x1146b6(0x242)){if(!this['isOTB']())return;this[_0x1146b6(0x277)]();this[_0x1146b6(0x15d)]&&(this[_0x1146b6(0x2e9)](this[_0x1146b6(0x15d)]),this[_0x1146b6(0x15d)]=null);this['_forcedBattlers'][_0x1146b6(0x2b2)]>0x0&&(this[_0x1146b6(0x15d)]=this[_0x1146b6(0x37b)]());;}else _0x4bbc4e[_0x1146b6(0x1e2)]=0xff;}}for(const _0x43a233 of this[_0x1146b6(0x109)]){if(_0x1146b6(0x18c)!==_0x1146b6(0x1c4)){if(!_0x43a233)continue;_0x43a233['calculateTargetPositions']();}else{if(this[_0x1146b6(0x1e8)]())this['battler']()[_0x1146b6(0x123)]();return![];}}},Window_OTB_TurnOrder[_0x55ad4d(0x159)]['removeUnableTurnOrderSprites']=function(){const _0x35916d=_0x55ad4d;for(const _0x3f6470 of this[_0x35916d(0x109)]){if(!_0x3f6470)continue;const _0x47e697=_0x3f6470['battler']();if(BattleManager[_0x35916d(0x1b9)][_0x35916d(0x2bc)](_0x47e697))continue;this[_0x35916d(0x14b)](_0x3f6470);}for(const _0x22b9d9 of this[_0x35916d(0x2c4)]){if('VnbYk'==='VnbYk'){if(!_0x22b9d9)continue;const _0x548035=_0x22b9d9['battler']();if(BattleManager[_0x35916d(0x2b3)]['includes'](_0x548035))continue;this[_0x35916d(0x14b)](_0x22b9d9);}else this[_0x35916d(0x2e9)](this[_0x35916d(0x15d)]),this['_subject']=null;}},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x1c6)]=function(_0x5ca130,_0x3e02e6){const _0x90811f=_0x55ad4d,_0x548c46=_0x3e02e6===BattleManager[_0x90811f(0x1b9)]?this[_0x90811f(0x109)]:this[_0x90811f(0x2c4)];if(!_0x548c46)return;const _0x2ac7b1=VisuMZ[_0x90811f(0x153)][_0x90811f(0x296)](_0x5ca130,_0x3e02e6),_0x33d9eb=_0x2ac7b1[_0x90811f(0x2b2)]-0x1,_0x206394=new Sprite_OTB_TurnOrder_Battler(_0x5ca130,_0x33d9eb,_0x548c46);this['_spriteContainer']['addChild'](_0x206394),_0x548c46[_0x90811f(0x20d)](_0x206394),_0x206394['startFade'](0xff),this['requestUpdateTurnOrders']();},Window_OTB_TurnOrder[_0x55ad4d(0x159)]['addBattlerToTurnOrderAtStart']=function(_0x2a8ae4,_0x4ab017){const _0x1870ba=_0x55ad4d,_0x33524c=_0x4ab017===BattleManager[_0x1870ba(0x1b9)]?this[_0x1870ba(0x109)]:this[_0x1870ba(0x2c4)];if(!_0x33524c)return;for(const _0x555ad9 of _0x33524c){if('GhDtM'===_0x1870ba(0x2b8))return this['_otbTurnOrderGraphicType']===_0x47a948&&(this[_0x1870ba(0x338)]=this[_0x1870ba(0x261)]()),this[_0x1870ba(0x338)];else{if(!_0x555ad9)continue;_0x555ad9['battler']()===_0x2a8ae4&&(_0x1870ba(0x25a)!=='cZVKn'?(_0x555ad9[_0x1870ba(0x204)]=_0x555ad9[_0x1870ba(0x204)]||0x0,_0x555ad9[_0x1870ba(0x204)]++):(_0x251cf0['unshift'](_0xf1348),_0xb73f14&&_0x398ffb[_0x1870ba(0x237)](_0x5401cc,_0x421e8e)));}}const _0x18d4e9=0x0,_0x392986=new Sprite_OTB_TurnOrder_Battler(_0x2a8ae4,_0x18d4e9,_0x33524c);this[_0x1870ba(0x1ae)][_0x1870ba(0x24e)](_0x392986),_0x33524c['push'](_0x392986),_0x392986[_0x1870ba(0x258)](0xff),_0x392986[_0x1870ba(0x2de)]=0x258,_0x392986['x']=this[_0x1870ba(0x389)],this[_0x1870ba(0x1f6)]();},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x18e)]=function(){const _0x14f15b=_0x55ad4d;this[_0x14f15b(0x224)](BattleManager[_0x14f15b(0x1b9)],!![]),this[_0x14f15b(0x224)](BattleManager[_0x14f15b(0x2b3)],!![]),this['shiftTurnOrderForSubject'](BattleManager[_0x14f15b(0x15d)],!![]),this[_0x14f15b(0x2ae)]();},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x162)]=function(_0x3f5c6e){const _0x540f62=_0x55ad4d;this['clearOrderPreview'](),_0x3f5c6e&&_0x3f5c6e[_0x540f62(0x2d3)]()!==null&&(_0x540f62(0x11d)!==_0x540f62(0x2c2)?this[_0x540f62(0x2c0)](_0x3f5c6e):this[_0x540f62(0x295)](_0x22e134,!![],_0x7f38a));},Window_OTB_TurnOrder[_0x55ad4d(0x159)]['clearOrderPreview']=function(){const _0x529cbc=_0x55ad4d;for(const _0x603745 of this[_0x529cbc(0x319)][_0x529cbc(0x36e)]){if(!_0x603745)continue;this[_0x529cbc(0x14b)](_0x603745);}},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x2c0)]=function(_0x47576b){const _0x4f56a3=_0x55ad4d,_0x4f3fff=_0x47576b['subject'](),_0x5c15d6=_0x47576b[_0x4f56a3(0x176)](),_0x401370=_0x47576b[_0x4f56a3(0x317)]();_0x5c15d6!==0x0&&(_0x4f56a3(0x348)!==_0x4f56a3(0x2f5)?this['createOrderPreviewSprite'](_0x4f3fff,![],_0x5c15d6):this[_0x4f56a3(0x356)]=this[_0x4f56a3(0x365)]());_0x401370!==0x0&&('ZiQBo'!==_0x4f56a3(0x1c0)?(_0x50719a[_0x4f56a3(0x1bc)](),_0x7b8d8c['BattleSystemOTB'][_0x4f56a3(0x299)][_0x4f56a3(0x175)](this)):this['createOrderPreviewSprite'](_0x4f3fff,!![],_0x401370));if(!_0x47576b[_0x4f56a3(0x33f)]())return;const _0x39f9fc=SceneManager['_scene'][_0x4f56a3(0x2c7)],_0x1c2603=SceneManager['_scene'][_0x4f56a3(0x2af)];let _0x35e3cc=null;if(_0x39f9fc&&_0x39f9fc[_0x4f56a3(0x1c1)])_0x35e3cc=_0x39f9fc[_0x4f56a3(0x1d9)](_0x39f9fc['index']());else _0x1c2603&&_0x1c2603[_0x4f56a3(0x1c1)]&&(_0x35e3cc=_0x1c2603[_0x4f56a3(0x1ed)]());if(!_0x35e3cc)return;const _0x1de50e=_0x47576b[_0x4f56a3(0x14c)](_0x35e3cc),_0x1b0a96=_0x47576b[_0x4f56a3(0x22a)](_0x35e3cc);_0x1de50e!==0x0&&(_0x4f56a3(0x158)!==_0x4f56a3(0x158)?this[_0x4f56a3(0x26d)]():this[_0x4f56a3(0x295)](_0x35e3cc,![],_0x1de50e)),_0x1b0a96!==0x0&&this[_0x4f56a3(0x295)](_0x35e3cc,!![],_0x1b0a96);},Window_OTB_TurnOrder[_0x55ad4d(0x159)][_0x55ad4d(0x295)]=function(_0x26c6e,_0x4aa781,_0x3a3780){const _0x30c435=_0x55ad4d;if(!_0x26c6e)return;if(_0x3a3780===0x0)return;const _0x5c2f76=_0x4aa781?BattleManager['_otb_actionBattlersNext']:BattleManager[_0x30c435(0x1b9)],_0x5b8969=VisuMZ[_0x30c435(0x153)][_0x30c435(0x296)](_0x26c6e,_0x5c2f76),_0x45a220=_0x4aa781?this[_0x30c435(0x2c4)]:this[_0x30c435(0x109)],_0x3a420b=_0x4aa781?this['_previewNext']:this['_previewCurrent'];if(_0x5b8969['length']<=0x0)return;for(let _0x1d751d=0x0;_0x1d751d<_0x5b8969['length'];_0x1d751d++){const _0x59b167=new Sprite_OTB_TurnOrder_Preview(_0x26c6e,_0x1d751d,_0x45a220,_0x3a3780);this[_0x30c435(0x319)]['addChild'](_0x59b167),_0x3a420b[_0x30c435(0x20d)](_0x59b167),_0x59b167[_0x30c435(0x330)](),_0x59b167['startFade'](0xff);}};