//=============================================================================
// VisuStella MZ - Class Change System
// VisuMZ_2_ClassChangeSystem.js
//=============================================================================

var Imported = Imported || {};
Imported.VisuMZ_2_ClassChangeSystem = true;

var VisuMZ = VisuMZ || {};
VisuMZ.ClassChangeSystem = VisuMZ.ClassChangeSystem || {};
VisuMZ.ClassChangeSystem.version = 1.07;

//=============================================================================
 /*:
 * @target MZ
 * @plugindesc [RPG Maker MZ] [Tier 2] [Version 1.07] [ClassChangeSystem]
 * @author VisuStella
 * @url http://www.yanfly.moe/wiki/Class_Change_System_VisuStella_MZ
 * @orderAfter VisuMZ_0_CoreEngine
 *
 * @help
 * ============================================================================
 * Introduction
 * ============================================================================
 *
 * This plugin adds the ability for your player to freely change the classes of
 * actors outside of battle from a menu. When changing into different classes,
 * players adjust the game's actors to a different playstyle with different
 * skills, equipment, and traits to make them behave differently.
 * 
 * Multiclassing is also possible. Actors can possess one class to many, from
 * two to ten to as many as you've set up in the Plugin Parameters. Adjust the
 * rulings for how multiclasses behave in your game. Let actors inherit a small
 * percentage of parameters from the multiclasses, skills, equipment access,
 * and more!
 *
 * Features include all (but not limited to) the following:
 * 
 * * A custom scene to let actors change their classes inside of.
 * * When class changing, determine if levels are maintained across all classes
 *   or if each class has their own levels to raise.
 * * Multiclasses allow actors to have more than one class at a time.
 * * Determine the rulings for each multiclass tier through the Plugin
 *   Parameters to gain control over how they influence your game.
 * * Restrict certain multiclass tiers from being able to change classes.
 * * Allow only some classes to be equippable to specific multiclass tiers.
 * * Unlock new classes automatically by reaching certain class levels or when
 *   certain resources have reached certain thresholds.
 * * These resources the new Class Points and Job Points.
 * * Class Points and Job Points are brand new resources added through this
 *   plugin which can be acquired through a variety a means ranging from
 *   participating in battle, defeating enemies, and/or leveling up.
 * * Also unlock classes through Plugin Commands!
 * * Actors can have class specific graphics depending on their primary class.
 *   Appearance changes range from faces, map sprites, battlers, and portraits
 *   used by other VisuStella MZ plugins.
 * * Play an animation on the actor inside the Class Change scene when changing
 *   classes.
 *
 * ============================================================================
 * Requirements
 * ============================================================================
 *
 * This plugin is made for RPG Maker MZ. This will not work in other iterations
 * of RPG Maker.
 *
 * ------ Tier 2 ------
 *
 * This plugin is a Tier 2 plugin. Place it under other plugins of lower tier
 * value on your Plugin Manager list (ie: 0, 1, 2, 3, 4, 5). This is to ensure
 * that your plugins will have the best compatibility with the rest of the
 * VisuStella MZ library.
 *
 * ============================================================================
 * Major Changes
 * ============================================================================
 *
 * This plugin adds some new hard-coded features to RPG Maker MZ's functions.
 * The following is a list of them.
 *
 * ---
 *
 * Class Specific Graphics
 * 
 * If an actor has class specific graphics, they will overwrite the face
 * graphic, map character sprite graphic, battler graphic, and any portraits
 * that have been added through the VisuStella MZ plugins. The class specific
 * graphics will take priority over the default graphics.
 * 
 * ---
 * 
 * Change Actor Images Event Command
 * 
 * When changing an actor's graphics through the "Change Actor Images" event
 * command, these changes will take priority over the Class Specific Graphics.
 * If you want to remove these priority graphics, set the "Change Actor Images"
 * images to "(None)".
 * 
 * Keep in mind that this means you cannot make an "invisible" graphic through
 * the "(None)" selection anymore. Instead, you need to make a work around by
 * making a custom graphic image that is fully transparent.
 *
 * ---
 *
 * ============================================================================
 * Extra Features
 * ============================================================================
 *
 * There are some extra features found if other VisuStella MZ plugins are found
 * present in the Plugin Manager list.
 *
 * ---
 *
 * VisuMZ_3_VictoryAftermath
 *
 * If VisuStella MZ's Victory Aftermath plugin is installed, the amount of
 * Job Points and Class Points earned can be visibly shown in the rewards
 * window.
 *
 * ---
 *
 * VisuMZ_1_BattleCore
 * 
 * VisuMZ_1_MainMenuCore
 *
 * If the Battle Core and/or the Main Menu Core is installed, the Class Change
 * System also gives access to notetags that alter their battle portraits
 * and/or menu portraits based on whatever class an actor is.
 *
 * ---
 *
 * ============================================================================
 * Clarification
 * ============================================================================
 *
 * This section is to add clarification on some questions you may have
 * regarding the Class Change System.
 *
 * ---
 *
 * Q. Why do my actors have access to random skill(s) of x class(es)?
 * 
 * A. Are those classes a part of the classes that have already been unlocked?
 * Are the skills learned at level 1 for those classes? And are those classes
 * sharing a particular Skill Type? Then that's your answer.
 * 
 * When classes are unlocked, they are unlocked at level 1. When unlocked at
 * level 1, all of the skills at level 1 are also learned by that actor. And if
 * the classes all share a Skill Type, those skills will also become available
 * to that Skill Type.
 * 
 * If you don't want your classes to have access to all of the skills of the
 * same Skill Type, then give them different Skill Types unique to each class
 * and change the Skill Types of the skills taught for those classes to that
 * class's unique Skill Type.
 *
 * ---
 * 
 * Q. Why does the <Passive State: x> notetag from Skills and States Core apply
 * even if my actor does not have access to the parent skill?
 * 
 * A. Skills with the <Passive State: x> notetag only have a requirement of the
 * skills needing to be learned. It does not have a requirement of the skills
 * needing to be accessible through the Skill Types.
 * 
 * Even without the Class Change System, if you teach an actor a skill that
 * has a Skill Type the actor does not have access to, that actor will still
 * benefit from the <Passive State: x> notetag.
 * 
 * To make it apply only when a certain class is present, you will need to
 * utilize the Passive Condition notetags found in the Skills and States Core.
 * 
 * ---
 * 
 * Q. How do I get the data on which classes and multiclasses an actor has?
 * 
 * A. You would have to use the following code to acquire their data:
 * 
 *   actor.multiclasses()
 *   - This returns an array of all of the multiclasses an actor has.
 *   - This includes the actor's primary class.
 * 
 *   actor.multiclass(x)
 *   - This returns the class data (not ID) of whatever class the actor has
 *     in x multiclass slot.
 *   - An x value of 1 would yield the primary class.
 * 
 *   actor.multiclassId(x)
 *   - This returns the class ID (not data) of whatever class the actor has
 *     in x multiclass slot.
 *   - An x value of 1 would yield the primary class's ID.
 * 
 * ---
 * 
 * Q. How come my subclasses don't gain levels or EXP when I use event commands
 *    on my actors?
 * 
 * A. EXP Reward Rates for subclasses only apply to battle rewards. The event
 *    commands do not affect class settings in case the game dev wishes to fine
 *    tune the amount of EXP each class.
 * 
 * ---
 * 
 * Q. How come subclasses do not appear in the Skill Learn System?
 * 
 * A. That's because class-based resources and requirements are different
 *    depending on the primary class and how they're set up. To avoid
 *    conflicting with subclass resources and requirements, the Skill Learn
 *    System only makes it available for the primary class to learn skills from
 *    at a time. To learn skills from a subclass through the Skill Learn System
 *    the player would have to change to the subclass' class as the primary and
 *    then learn from it.
 * 
 * ---
 *
 * ============================================================================
 * Notetags
 * ============================================================================
 *
 * The following are notetags that have been added through this plugin. These
 * notetags will not work with your game if this plugin is OFF or not present.
 *
 * ---
 * 
 * === Class Basics-Related Notetags ===
 * 
 * ---
 *
 * <Icon: x>
 *
 * - Used for: Class Notetags
 * - Assigns an icon index for the class to 'x'.
 * - Replace 'x' with a number representing the index value on the IconSet
 *   image in the img/system/ folder for the icon you want to assign.
 * - If this notetag is not used, the icon index will default to the setting
 *   found in the Class Change System's Plugin Parameters.
 *
 * ---
 *
 * <Help Description>
 *  text
 *  text
 * </Help Description>
 *
 * - Used for: Class Notetags
 * - Assigns a help description to the class.
 * - Replace 'text' with text you want displayed when this class is selected
 *   in the Class Change scene's class list.
 * - If this notetag is not used, the help description will default to the
 *   setting found in the Class Change System's Plugin Parameters.
 *
 * ---
 *
 * <Class Change Animation: x>
 *
 * - Used for: Class Notetags
 * - Assigns an animation for the class when the actor changes to that class.
 * - Replace 'x' with a number representing the ID of the animation found in
 *   the database to play when the actor changes to that class.
 * - If this notetag is not used, the animation will default to the setting
 *   found in the Class Change System's Plugin Parameters.
 *
 * ---
 * 
 * <Class Change Picture: filename>
 * <Picture: filename>
 * 
 * - Used for: Class Notetags
 * - Uses a picture from your project's /img/pictures/ folder instead of the
 *   class's icon during for the Class Change scene.
 * - Replace 'filename' with the filename of the image.
 *   - Do not include the file extension.
 * - Scaling will not apply to the picture.
 * - Use the <Picture: filename> version for any other plugins that may be
 *   using this as an image outside of class changing, too.
 * - The size used for the image will vary based on your game's resolution.
 * 
 * ---
 * 
 * === Class Specific Graphics-Related Notetags ===
 * 
 * ---
 *
 * <Class id Face: filename, index>
 * 
 * <Class name Face: filename, index>
 *
 * - Used for: Actor Notetags
 * - Gives this actor a class specific face graphic.
 * - For 'id' variant, replace 'id' with a number representing class's ID.
 * - For 'name' variant, replace 'name' with the class's name.
 * - Replace 'filename' with the filename of the graphic found inside the
 *   img/faces/ folder. Do not include the file extension.
 * - Replace 'index' with the index of the graphic. Index values start at 0.
 * 
 * Examples: 
 * 
 *   <Class 1 Face: Actor2, 0>
 * 
 *   <Class Swordsman Face: Actor2, 0>
 *
 * ---
 *
 * <Class id Character: filename, index>
 * 
 * <Class name Character: filename, index>
 *
 * - Used for: Actor Notetags
 * - Gives this actor a class specific map character sprite graphic.
 * - For 'id' variant, replace 'id' with a number representing class's ID.
 * - For 'name' variant, replace 'name' with the class's name.
 * - Replace 'filename' with the filename of the graphic found inside the
 *   img/characters/ folder. Do not include the file extension.
 * - Replace 'index' with the index of the graphic. Index values start at 0.
 * 
 * Examples: 
 * 
 *   <Class 1 Character: Actor2, 0>
 * 
 *   <Class Swordsman Character: Actor2, 0>
 *
 * ---
 *
 * <Class id Battler: filename>
 * 
 * <Class name Battler: filename>
 *
 * - Used for: Actor Notetags
 * - Gives this actor a class specific sideview battler graphic.
 * - For 'id' variant, replace 'id' with a number representing class's ID.
 * - For 'name' variant, replace 'name' with the class's name.
 * - Replace 'filename' with the filename of the graphic found inside the
 *   img/sv_actors/ folder. Do not include the file extension.
 * 
 * Examples: 
 * 
 *   <Class 1 Battler: Actor2_1>
 * 
 *   <Class Swordsman Battler: Actor2_1>
 *
 * ---
 *
 * <Class id Menu Portrait: filename>
 * 
 * <Class name Menu Portrait: filename>
 *
 * - Used for: Actor Notetags
 * - Requires VisuMZ_1_MainMenuCore!
 * - Gives this actor a class specific menu portrait graphic.
 * - For 'id' variant, replace 'id' with a number representing class's ID.
 * - For 'name' variant, replace 'name' with the class's name.
 * - Replace 'filename' with the filename of the graphic found inside the
 *   img/pictures/ folder. Do not include the file extension.
 * 
 * Examples: 
 * 
 *   <Class 1 Menu Portrait: Actor2_1>
 * 
 *   <Class Swordsman Menu Portrait: Actor2_1>
 *
 * ---
 *
 * <Class id Battle Portrait: filename>
 * 
 * <Class name Battle Portrait: filename>
 *
 * - Used for: Actor Notetags
 * - Requires VisuMZ_1_BattleCore!
 * - Gives this actor a class specific battle portrait graphic.
 * - For 'id' variant, replace 'id' with a number representing class's ID.
 * - For 'name' variant, replace 'name' with the class's name.
 * - Replace 'filename' with the filename of the graphic found inside the
 *   img/pictures/ folder. Do not include the file extension.
 * 
 * Examples: 
 * 
 *   <Class 1 Battle Portrait: Actor2_1>
 * 
 *   <Class Swordsman Battle Portrait: Actor2_1>
 *
 * ---
 * 
 * === Class Unlocking-Related Notetags ===
 * 
 * ---
 *
 * <Unlocked Classes: id>
 * <Unlocked Classes: id, id, id>
 * 
 * <Unlocked Classes: name>
 * <Unlocked Classes: name, name, name>
 *
 * - Used for: Actor Notetags
 * - Allows this actor to start with certain classes unlocked. These classes
 *   are unlocked in addition to the ones found in the Plugin Parameters.
 * - For 'id' variant, replace 'id' with a number representing class's ID.
 * - For 'name' variant, replace 'name' with the class's name.
 * - Insert multiple data entries to unlock more classes.
 *
 * ---
 *
 * <Auto Unlock Requirements>
 *  Class id: Level x
 *  Class name: Level x
 * 
 *  Class id: x AP
 *  Class name: x AP
 * 
 *  Class id: x CP
 *  Class name: x CP
 * 
 *  Class id: x JP
 *  Class name: x JP
 * 
 *  Class id: x SP
 *  Class name: x SP
 * 
 *  AP: x
 *  CP: x
 *  JP: x
 *  SP: x
 * </Auto Unlock Requirements>
 *
 * - Used for: Class Notetags
 * - Have this class unlock automatically whenever all of the conditions have
 *   been met after a battle is over or upon entering the Class Change scene.
 * - Insert/delete any number of copies of the middle conditions as needed.
 * - For 'id' conditions, replace 'id' with a number representing class's ID.
 * - For 'name' conditions, replace 'name' with the class's name.
 * - For 'AP', 'CP', 'JP', 'SP' conditions that have class markers, they
 *   require that many of the resource as the 'x' value for that class.
 *   These are best used with resource types that are class specific.
 * - For 'AP', 'CP', 'JP', 'SP' conditions that have class markers, they
 *   require that many of the resource as the 'x' value for the current class.
 *   These are best used with resource types that are shared.
 * - 'AP' and 'SP' conditions require VisuMZ_2_SkillLearnSystem.
 * 
 * Examples:
 * 
 * <Auto Unlock Requirements>
 *  Class 4: Level 20
 *  Class 6: Level 15
 * </Auto Unlock Requirements>
 * 
 * <Auto Unlock Requirements>
 *  Class Knight: Level 20
 *  Class Spellblade: Level 15
 * </Auto Unlock Requirements>
 * 
 * <Auto Unlock Requirements>
 *  Class Knight: 200 JP
 *  Class Spellblade: 100 JP
 * </Auto Unlock Requirements>
 * 
 * <Auto Unlock Requirements>
 *  Class Knight: 200 JP
 *  CP: 500
 * </Auto Unlock Requirements>
 *
 * ---
 * 
 * === Category-Related Notetags ===
 * 
 * ---
 *
 * <Starting Multiclasses: x>
 *
 * - Used for: Actor Notetags
 * - Lets the actor start with 'x' amount of class slots to assign.
 * - Replace 'x' with a number value representing the number of slots the
 *   actor can assign classes to.
 * - If this notetag is not used, the slot values will default to the setting
 *   found in the Class Change System's Plugin Parameters.
 * - Slot values cannot go under 1 or exceed the maximum number of layers found
 *   in the "Multiclass Settings" Plugin Parameters.
 *
 * ---
 *
 * <Starting Tier x Class: id>
 * 
 * <Starting Tier x Class: name>
 *
 * - Used for: Actor Notetags
 * - If an actor has multiclass slots, determine which subclasses are assigned
 *   to them at the start.
 * - Replace 'x' with a number value representing the multiclass slot to assign
 *   to. '1' is the primary slot. '2' is the second slot.
 * - For 'id' conditions, replace 'id' with a number representing class's ID.
 * - For 'name' conditions, replace 'name' with the class's name.
 * - Insert multiple copies of this notetag to assign multiple classes to
 *   different slots.
 * 
 * Example:
 * 
 * <Starting Tier 2 Class: Sorcerer>
 * 
 * <Starting Tier 3 Class: Priest>
 *
 * ---
 *
 * <Restrict Class Change Tier: x>
 * <Restrict Class Change Tiers: x, x, x>
 *
 * - Used for: Actor Notetags
 * - This makes an actor unable to change the class found in any of the listed
 *   tier slots unless this effect is cancelled by Plugin Commands.
 * - Replace 'x' with a number representing the tier slot(s) to restrict.
 *
 * ---
 *
 * <Class Change Tier Only: x>
 * <Class Change Tiers Only: x, x, x>
 *
 * - Used for: Class Notetags
 * - This makes the specific class only assignable to specific class tiers.
 * - Replace 'x' with a number representing the tier slot(s) that this class
 *   can be assigned and equipped to.
 *
 * ---
 * 
 * === Class Points-Related Notetags ===
 * 
 * ---
 *
 * <Starting CP: x>
 *
 * - Used for: Actor Notetags
 * - Determines the amount of Class Points the actor starts with in his/her
 *   starting class.
 * - Replace 'x' with a numeric value representing the amount of Class Points
 *   to start out with.
 *
 * ---
 *
 * <Class id Starting CP: x>
 * <Class name Starting CP: x>
 *
 * - Used for: Actor Notetags
 * - Determines the amount of Class Points the actor starts with in a specific
 *   class if Class Points aren't shared across all classes.
 * - Replace 'x' with a numeric value representing the amount of Class Points
 *   to start out with.
 * - Replace 'id' with the ID of the class to set starting Class Points for.
 * - Replace 'name' with the name of the class to set starting Class Points
 *   for.
 *
 * ---
 *
 * <CP Gain: x>
 * <User CP Gain: x>
 *
 * - Used for: Skill, Item Notetags
 * - When this skill/item is used in battle, the user will acquire 'x' amount
 *   of Class Points.
 * - Replace 'x' with a number representing the amount of Class Points for the
 *   user to earn upon usage.
 * - This effect will trigger each time per "hit".
 * - This effect will take over the "Per Action Hit" Class Points gain from the
 *   Plugin Parameters.
 *
 * ---
 *
 * <Target CP Gain: x>
 *
 * - Used for: Skill, Item Notetags
 * - When this skill/item is used in battle, the target will acquire 'x' amount
 *   of Class Points.
 * - Replace 'x' with a number representing the amount of Class Points for the
 *   target to earn upon usage.
 * - This effect will trigger each time per "hit".
 *
 * ---
 *
 * <CP: x>
 *
 * - Used for: Enemy Notetags
 * - Determines the amount of Class Points the enemy will give the player's
 *   party upon being defeated.
 * - Replace 'x' with a number representing the amount of Class Points to grant
 *   the player's party each.
 * - This effect will take over the "Per Enemy" Class Points gain from the
 *   Plugin Parameters.
 *
 * ---
 *
 * <Class Points Rate: x%>
 *
 * - Used for: Actor, Class, Weapon, Armor, State Notetags
 * - Increases the amount of Class Points the affected battler will gain by a
 *   percentile value.
 * - Replace 'x' with a percentage number representing the amount of Class
 *   Points that will be acquired.
 * - This stacks multiplicatively with each other.
 * - This does not apply when Class Points are directly added, lost, or set.
 *
 * ---
 * 
 * === Job Points-Related Notetags ===
 * 
 * ---
 *
 * <Starting JP: x>
 *
 * - Used for: Actor Notetags
 * - Determines the amount of Job Points the actor starts with in his/her
 *   starting class.
 * - Replace 'x' with a numeric value representing the amount of Job Points to
 *   start out with.
 *
 * ---
 *
 * <Class id Starting JP: x>
 * <Class name Starting JP: x>
 *
 * - Used for: Actor Notetags
 * - Determines the amount of Job Points the actor starts with in a specific
 *   class if Job Points aren't shared across all classes.
 * - Replace 'x' with a numeric value representing the amount of Job Points to
 *   start out with.
 * - Replace 'id' with the ID of the class to set starting Job Points for.
 * - Replace 'name' with the name of the class to set starting Job Points for.
 *
 * ---
 *
 * <JP Gain: x>
 * <User JP Gain: x>
 *
 * - Used for: Skill, Item Notetags
 * - When this skill/item is used in battle, the user will acquire 'x' amount
 *   of Job Points.
 * - Replace 'x' with a number representing the amount of Job Points for the
 *   user to earn upon usage.
 * - This effect will trigger each time per "hit".
 * - This effect will take over the "Per Action Hit" Job Points gain from the
 *   Plugin Parameters.
 *
 * ---
 *
 * <Target JP Gain: x>
 *
 * - Used for: Skill, Item Notetags
 * - When this skill/item is used in battle, the target will acquire 'x' amount
 *   of Job Points.
 * - Replace 'x' with a number representing the amount of Job Points for the
 *   target to earn upon usage.
 * - This effect will trigger each time per "hit".
 *
 * ---
 *
 * <JP: x>
 *
 * - Used for: Enemy Notetags
 * - Determines the amount of Job Points the enemy will give the player's party
 *   upon being defeated.
 * - Replace 'x' with a number representing the amount of Job Points to grant
 *   the player's party each.
 * - This effect will take over the "Per Enemy" Job Points gain from the
 *   Plugin Parameters.
 *
 * ---
 *
 * <Job Points Rate: x%>
 *
 * - Used for: Actor, Class, Weapon, Armor, State Notetags
 * - Increases the amount of Job Points the affected battler will gain by a
 *   percentile value.
 * - Replace 'x' with a percentage number representing the amount of Job Points
 *   that will be acquired.
 * - This stacks multiplicatively with each other.
 * - This does not apply when Job Points are directly added, lost, or set.
 *
 * ---
 *
 * ============================================================================
 * Plugin Commands
 * ============================================================================
 *
 * The following are Plugin Commands that come with this plugin. They can be
 * accessed through the Plugin Command event command.
 *
 * ---
 * 
 * === Unlock Class Plugin Commands ===
 * 
 * ---
 *
 * Unlock Class: Add For Actor(s)
 * - Unlock class(es) for target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to unlock class(es) for.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to be unlocked.
 *
 * ---
 *
 * Unlock Class: Add For Global
 * - Unlock class(es) for all party members.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to be unlocked.
 *
 * ---
 *
 * Unlock Class: Remove From Actor(s)
 * - Remove unlock class(es) for target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to remove an unlocked class(es) for.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to be removed from the unlocked status.
 *
 * ---
 *
 * Unlock Class: Remove From Global
 * - Remove unlock class(es) for all party members.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to be removed from the unlocked status.
 *
 * ---
 * 
 * === Change Restriction Plugin Commands ===
 * 
 * ---
 *
 * Change Restriction: Add Tier Restriction
 * - Add restrictions to prevent class changing specific tier(s) to
 *   target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to restrict class tier(s) for.
 *
 *   Tiers(s):
 *   - Select which class tier(s) to restrict changing for.
 *
 * ---
 *
 * Change Restriction: Remove Tier Restriction
 * - Remove restrictions to allow class changing specific tier(s) for
 *   target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to remove class tier(s) restrictions for.
 *
 *   Tiers(s):
 *   - Select which class tier(s) to remove restrictions for.
 *
 * ---
 * 
 * === Multiclass Plugin Commands ===
 * 
 * ---
 *
 * Multiclass: Change Actor(s) Multiclass
 * - Changes a specific multiclass for target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to change the multiclass limit to.
 *
 *   Tier:
 *   - Which multiclass tier to change for the target actor(s)?
 *
 *   Class ID:
 *   - Which class should go into this multiclass tier slot?
 *
 * ---
 *
 * Multiclass: Raise Limit for Actor(s)
 * - Raise the multiclass limit for target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to change the multiclass limit to.
 *
 *   Raise Limit By:
 *   - Raise the multiclass limit for target actor(s) by this much.
 *
 * ---
 *
 * Multiclass: Lower Limit for Actor(s)
 * - Lower the multiclass limit for target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to change the multiclass limit to.
 *
 *   Reduce Limit By:
 *   - Lower the multiclass limit for target actor(s) by this much.
 *
 * ---
 *
 * Multiclass: Set Limit for Actor(s)
 * - Set multiclass limit for target actor(s).
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to change the multiclass limit to.
 *
 *   Set Limit To:
 *   - Set multiclass limit for target actor(s) to this much.
 *
 * ---
 * 
 * === Class Points Plugin Commands ===
 * 
 * ---
 *
 * Class Points: Gain
 * - The target actor(s) gains Class Points.
 * - Gained amounts are affected by Class Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to gain Class Points for.
 *   - Use "0" for the current class.
 *
 *   Class Points:
 *   - Determine how many Class Points will be gained.
 *   - You may use code.
 *
 * ---
 *
 * Class Points: Add
 * - The target actor(s) receives Class Points.
 * - Received amounts are NOT affected by Class Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to receive Class Points for.
 *   - Use "0" for the current class.
 *
 *   Class Points:
 *   - Determine how many Class Points will be added.
 *   - You may use code.
 *
 * ---
 *
 * Class Points: Lose
 * - The target actor(s) loses Class Points.
 * - Lost amounts are NOT affected by Class Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to lose Class Points for.
 *   - Use "0" for the current class.
 *
 *   Class Points:
 *   - Determine how many Class Points will be lost.
 *   - You may use code.
 *
 * ---
 *
 * Class Points: Set
 * - Changes the exact Class Points for the target actor(s).
 * - Changed amounts are NOT affected by Class Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to change Class Points for.
 *   - Use "0" for the current class.
 *
 *   Class Points:
 *   - Determine how many Class Points will be set exactly to.
 *   - You may use code.
 *
 * ---
 * 
 * === Job Points Plugin Commands ===
 * 
 * ---
 *
 * Job Points: Gain
 * - The target actor(s) gains Job Points.
 * - Gained amounts are affected by Job Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to gain Job Points for.
 *   - Use "0" for the current class.
 *
 *   Job Points:
 *   - Determine how many Job Points will be gained.
 *   - You may use code.
 *
 * ---
 *
 * Job Points: Add
 * - The target actor(s) receives Job Points.
 * - Received amounts are NOT affected by Job Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to receive Job Points for.
 *   - Use "0" for the current class.
 *
 *   Job Points:
 *   - Determine how many Job Points will be added.
 *   - You may use code.
 *
 * ---
 *
 * Job Points: Lose
 * - The target actor(s) loses Job Points.
 * - Lost amounts are NOT affected by Job Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to lose Job Points for.
 *   - Use "0" for the current class.
 *
 *   Job Points:
 *   - Determine how many Job Points will be lost.
 *   - You may use code.
 *
 * ---
 *
 * Job Points: Set
 * - Changes the exact Job Points for the target actor(s).
 * - Changed amounts are NOT affected by Job Point bonus rates.
 *
 *   Actor ID(s):
 *   - Select which Actor ID(s) to affect.
 *
 *   Class ID(s):
 *   - Select which Class ID(s) to change Job Points for.
 *   - Use "0" for the current class.
 *
 *   Job Points:
 *   - Determine how many Job Points will be set exactly to.
 *   - You may use code.
 *
 * ---
 * 
 * === System Plugin Commands ===
 * 
 * ---
 *
 * System: Enable Class Change in Menu?
 * - Enables/disables Class Change inside the main menu.
 *
 *   Enable/Disable?:
 *   - Enables/disables Class Change inside the main menu.
 *
 * ---
 *
 * System: Show Class Change in Menu?
 * - Shows/hides Class Change inside the main menu.
 *
 *   Show/Hide?:
 *   - Shows/hides Class Change inside the main menu.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: General Settings
 * ============================================================================
 *
 * General settings for Class Change System.
 *
 * ---
 *
 * Basics
 * 
 *   Default Help:
 *   - Default help description for all classes.
 *   - %1 - Class Name
 * 
 *   Default Icon:
 *   - Default icon used for all classes.
 * 
 *   Maintain Levels?:
 *   - Make each class have the same level or make each class have
 *     their own level?
 * 
 *   Change-Adjust HP/MP:
 *   - Adjust HP/MP differences after changing classes with MaxHP/MaxMP values.
 *
 * ---
 *
 * Class Unlocking
 * 
 *   Always Unlocked:
 *   - Which classes are always unlocked and available?
 * 
 *   Starting Multiclasses:
 *   - How many classes can actors use at the start by default?
 *   - Use 1 for just the primary class.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Background Settings
 * ============================================================================
 *
 * Background settings for Scene_ClassChange.
 *
 * ---
 *
 * Background Settings
 * 
 *   Snapshop Opacity:
 *   - Snapshot opacity for the scene.
 * 
 *   Background 1:
 *   - Filename used for the bottom background image.
 *   - Leave empty if you don't wish to use one.
 * 
 *   Background 2:
 *   - Filename used for the upper background image.
 *   - Leave empty if you don't wish to use one.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Class Change Sound Settings
 * ============================================================================
 *
 * Sound effect played when changing classes through Scene_ClassChange.
 *
 * ---
 *
 * Class Change Sound Settings
 * 
 *   Filename:
 *   - Filename of the sound effect played.
 * 
 *   Volume:
 *   - Volume of the sound effect played.
 * 
 *   Pitch:
 *   - Pitch of the sound effect played.
 * 
 *   Pan:
 *   - Pan of the sound effect played.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Main Access Settings
 * ============================================================================
 *
 * Menu Access settings for Class Change.
 *
 * ---
 *
 * Main Menu Settings
 * 
 *   Command Name:
 *   - Name of the 'ClassChangeSystem' option in the Main Menu.
 * 
 *   Show in Main Menu?:
 *   - Add the 'ClassChangeSystem' option to the Main Menu by default?
 * 
 *   Enable in Main Menu?:
 *   - Enable the 'ClassChangeSystem' option to the Main Menu by default?
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Multiclass Settings
 * ============================================================================
 *
 * Multiclass settings for this plugin. Each tier allows you to have separate
 * settings. The order the tiers are inserted will represent the settings that
 * will be applied to those tiers when classes are assigned in those slots.
 * 
 * The majority of these settings do not apply to Tier 1 because Tier 1 is the
 * primary class. However, Tier 1 must exist in these Plugin Parameters to
 * provide settings for the Class Change scene.
 *
 * ---
 *
 * General
 * 
 *   Class Tier Name:
 *   - Name of this class tier.
 * 
 *   Text Color:
 *   - Use #rrggbb for custom colors or regular numbers for text colors from
 *     the Window Skin.
 * 
 *   Help Description:
 *   - Help description when this multiclass slot is picked.
 *
 * ---
 *
 * Base Parameter Bonuses
 * 
 *   MaxHP:
 *   MaxMP:
 *   ATK:
 *   DEF:
 *   MAT:
 *   MDF:
 *   AGI:
 *   LUK:
 *   - How little of this class tier's parameter should be added to base stats?
 *   - Does not apply to Tier 1.
 *
 * ---
 *
 * Reward Rates
 * 
 *   EXP:
 *   - How much EXP does a class in this tier earn?
 *   - Does not apply to Tier 1. Only for Battle Rewards.
 * 
 *   Resources:
 *   - Resource rate (ie. CP, JP) earned for this tier.
 *   - Does not apply to Tier 1. Only for Battle Rewards.
 *
 * ---
 *
 * Inherit Traits > Rates
 * 
 *   Element Rates?:
 *   - Inherit the element rates from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   Debuff Rates?:
 *   - Inherit the debuff rates from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   State Rates?:
 *   - Inherit the state rates from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   State Resistance?:
 *   - Inherit the state resistances from this class tier?
 *   - Does not apply to Tier 1.
 *
 * ---
 *
 * Inherit Traits > Param Rates
 * 
 *   Base-Param Rates?:
 *   - Inherit Base Parameter rates from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   X-Param Rates?:
 *   - Inherit X-Parameter rates from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   S-Param Rates?:
 *   - Inherit S-Parameter rates from this class tier?
 *   - Does not apply to Tier 1.
 *
 * ---
 *
 * Inherit Traits > Attack
 * 
 *   Attack Elements?:
 *   - Inherit the attack elements from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   Attack States?:
 *   - Inherit the attack states from this class tier?
 *   - Does not apply to Tier 1.
 *
 * ---
 *
 * Inherit Traits > Skills
 * 
 *   Added STypes?:
 *   - Inherit the added STypes from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   Added Skills?:
 *   - Inherit the added skills from this class tier?
 *   - Does not apply to Tier 1.
 *
 * ---
 *
 * Inherit Traits > Equipment
 * 
 *   Equippable Weapons?:
 *   - Inherit the equippable weapons from this class tier?
 *   - Does not apply to Tier 1.
 * 
 *   Equippable Armors?:
 *   - Inherit the equippable armors from this class tier?
 *   - Does not apply to Tier 1.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Window Settings
 * ============================================================================
 *
 * Window settings for Scene_ClassChange. These adjust the overall layout of
 * the scene as well as how some of the content inside of the windows look. Not
 * all aspects of the scene are fully customizable due to mechanical limits.
 *
 * ---
 *
 * Scene_ClassChange
 * 
 *   Recommended Layout?:
 *   - Use the recommended Menu Layout provided by this plugin?
 * 
 *   Layout Style:
 *   - If using an updated layout, how do you want to style the menu
 *     scene layout?
 * 
 *   Displayed Resources:
 *   - Select which resources to display in Scene_Class's class lists.
 *   - Non-shared resources appear in the lists up to a limit of 2.
 * 
 *   Confirm Animation ID:
 *   - Play this animation when a class change has been made.
 * 
 *     Primary Offset X:
 *     Primary Offset Y:
 *     Subclass Offset X:
 *     Subclass Offset Y:
 *     - Adjust the offsets for the class change animation.
 *
 * ---
 *
 * Window_ClassStatus
 * 
 *   Background Type:
 *   - Select background type for this window.
 * 
 *   Param Font Size:
 *   - The font size used for parameter values.
 * 
 *   Show Menu Portraits?:
 *   - If Main Menu Core is installed, display the Menu Portraits instead of
 *     the actor's face in the status window?
 * 
 *   Show Back Rectangles?:
 *   - Show back rectangles of darker colors to display information better?
 * 
 *   Back Rectangle Color:
 *   - Use #rrggbb for custom colors or regular numbers for text colors from
 *     the Window Skin.
 * 
 *   JS: X, Y, W, H:
 *   - Code used to determine the dimensions for this window.
 * 
 *   JS: Portrait Upper:
 *   - If Menu Portraits are available, this is code used to draw the upper
 *     data like this in the Status Window.
 * 
 *   JS: Face Upper:
 *   - If faces used used, this is code used to draw the upper data like this
 *     in the Status Window.
 * 
 *   JS: Parameter Lower:
 *   - Code to determine how parameters are drawn in the Status Window.
 *
 * ---
 *
 * Window_ClassTier
 * 
 *   Background Type:
 *   - Select background type for this window.
 * 
 *   No Class Assigned:
 *   - Text used when no class is assigned to the slot.
 * 
 *   Use SHIFT Shortcut?:
 *   - Add the "Shift" button as a shortcut key to removing classes?
 * 
 *   Button Assist Text:
 *   - Text used for the Button Assist Window
 * 
 *   JS: Extra Data:
 *   - Code used to draw extra data if there is enough room.
 * 
 *   JS: X, Y, W, H:
 *   - Code used to determine the dimensions for this window.
 *
 * ---
 *
 * Window_ClassList
 * 
 *   Background Type:
 *   - Select background type for this window.
 * 
 *   Unassign Class:
 *   - Text used for an empty class slot.
 * 
 *     Help Description:
 *     - Help description for unassigning a class.
 * 
 *   JS: X, Y, W, H:
 *   - Code used to determine the dimensions for this window.
 *
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Class Points Settings
 * ============================================================================
 *
 * Class Points are an actor-only resource used as a currency for this plugin.
 * You can determine how they appear in-game, how they're earned, and what kind
 * of mechanics are involved with them. Class Points can also be used in other
 * VisuStella plugins.
 *
 * ---
 *
 * Mechanics
 * 
 *   Shared Class Points:
 *   - Do you want Class Points to be shared across all classes?
 *   - Or do you want all classes to have their own?
 * 
 *   Maximum:
 *   - What's the maximum amount of Class Points an actor can have?
 *   - Use 0 for unlimited Class Points.
 *
 * ---
 *
 * Visual
 * 
 *   Show In Menus?:
 *   - Do you wish to show Class Points in menus that allow them?
 * 
 *   Icon:
 *   - What is the icon you want to use to represent Class Points?
 *
 * ---
 *
 * Vocabulary
 * 
 *   Full Text:
 *   - The full text of how Class Points appears in-game.
 * 
 *   Abbreviated Text:
 *   - The abbreviation of how Class Points appears in-game.
 * 
 *   Menu Text Format:
 *   - What is the text format for it to be displayed in windows.
 *   - %1 - Value, %2 - Abbr, %3 - Icon, %4 - Full Text
 *
 * ---
 *
 * Gain
 * 
 *   Per Action Hit:
 *   - How many Class Points should an actor gain per action?
 *   - You may use code.
 * 
 *   Per Level Up:
 *   - How many Class Points should an actor gain per level up?
 *   - You may use code.
 * 
 *   Per Enemy Defeated:
 *   - How many Class Points should an actor gain per enemy?
 *   - You may use code.
 * 
 *     Alive Actors?:
 *     - Do actors have to be alive to receive Class Points from
 *       defeated enemies?
 *
 * ---
 *
 * Victory
 * 
 *   Show During Victory?:
 *   - Show how much CP an actor has earned in battle during the victory phase?
 * 
 *   Victory Text:
 *   - For no Victory Aftermath, this is the text that appears.
 *   - %1 - Actor, %2 - Earned, %3 - Abbr, %4 - Full Text
 * 
 *   Aftermath Display?:
 *   - Requires VisuMZ_3_VictoryAftermath. 
 *   - Show Class Points as the main acquired resource in the actor windows?
 * 
 *   Aftermath Text:
 *   - For no Victory Aftermath, this is the text that appears.
 *   - %1 - Earned, %2 - Abbr, %3 - Full Text
 *
 * ---
 * 
 * For those who wish to display how many Class Points an actor has for a
 * specific class, you can use the following JavaScript code inside of a
 * window object.
 * 
 *   this.drawClassPoints(value, x, y, width, align);
 *   - The 'value' variable refers to the number you wish to display.
 *   - The 'x' variable refers to the x coordinate to draw at.
 *   - The 'y' variable refers to the y coordinate to draw at.
 *   - The 'width' variable refers to the width of the data area.
 *   - Replace 'align' with a string saying 'left', 'center', or 'right' to
 *     determine how you want the data visibly aligned.
 * 
 *   this.drawActorClassPoints(actor, classID, x, y, width, align);
 *   - The 'actor' variable references the actor to get data from.
 *   - The 'classID' variable is the class to get data from.
 *     - Use 0 if Class Points aren't shared or if you want the Class
 *       Points from the actor's current class.
 *   - The 'x' variable refers to the x coordinate to draw at.
 *   - The 'y' variable refers to the y coordinate to draw at.
 *   - The 'width' variable refers to the width of the data area.
 *   - Replace 'align' with a string saying 'left', 'center', or 'right' to
 *     determine how you want the data visibly aligned.
 * 
 * ---
 *
 * ============================================================================
 * Plugin Parameters: Job Points Settings
 * ============================================================================
 *
 * Job Points are an actor-only resource used as a currency for this plugin.
 * You can determine how they appear in-game, how they're earned, and what kind
 * of mechanics are involved with them. Job Points can also be used in other
 * VisuStella plugins.
 *
 * ---
 *
 * Mechanics
 * 
 *   Shared Job Points:
 *   - Do you want Job Points to be shared across all classes?
 *   - Or do you want all classes to have their own?
 * 
 *   Maximum:
 *   - What's the maximum amount of Job Points an actor can have?
 *   - Use 0 for unlimited Job Points.
 *
 * ---
 *
 * Visual
 * 
 *   Show In Menus?:
 *   - Do you wish to show Job Points in menus that allow them?
 * 
 *   Icon:
 *   - What is the icon you want to use to represent Job Points?
 *
 * ---
 *
 * Vocabulary
 * 
 *   Full Text:
 *   - The full text of how Job Points appears in-game.
 * 
 *   Abbreviated Text:
 *   - The abbreviation of how Job Points appears in-game.
 * 
 *   Menu Text Format:
 *   - What is the text format for it to be displayed in windows.
 *   - %1 - Value, %2 - Abbr, %3 - Icon, %4 - Full Text
 *
 * ---
 *
 * Gain
 * 
 *   Per Action Hit:
 *   - How many Job Points should an actor gain per action?
 *   - You may use code.
 * 
 *   Per Level Up:
 *   - How many Job Points should an actor gain per level up?
 *   - You may use code.
 * 
 *   Per Enemy Defeated:
 *   - How many Job Points should an actor gain per enemy?
 *   - You may use code.
 * 
 *     Alive Actors?:
 *     - Do actors have to be alive to receive Job Points from
 *       defeated enemies?
 *
 * ---
 *
 * Victory
 * 
 *   Show During Victory?:
 *   - Show how much JP an actor has earned in battle during the victory phase?
 * 
 *   Victory Text:
 *   - For no Victory Aftermath, this is the text that appears.
 *   - %1 - Actor, %2 - Earned, %3 - Abbr, %4 - Full Text
 * 
 *   Aftermath Display?:
 *   - Requires VisuMZ_3_VictoryAftermath. 
 *   - Show Job Points as the main acquired resource in the actor windows?
 * 
 *   Aftermath Text:
 *   - For no Victory Aftermath, this is the text that appears.
 *   - %1 - Earned, %2 - Abbr, %3 - Full Text
 *
 * ---
 * 
 * For those who wish to display how many Job Points an actor has for a
 * specific class, you can use the following JavaScript code inside of a
 * window object.
 * 
 *   this.drawJobPoints(value, x, y, width, align);
 *   - The 'value' variable refers to the number you wish to display.
 *   - The 'x' variable refers to the x coordinate to draw at.
 *   - The 'y' variable refers to the y coordinate to draw at.
 *   - The 'width' variable refers to the width of the data area.
 *   - Replace 'align' with a string saying 'left', 'center', or 'right' to
 *     determine how you want the data visibly aligned.
 * 
 *   this.drawActorJobPoints(actor, classID, x, y, width, align);
 *   - The 'actor' variable references the actor to get data from.
 *   - The 'classID' variable is the class to get data from.
 *     - Use 0 if Job Points aren't shared or if you want the Job
 *       Points from the actor's current class.
 *   - The 'x' variable refers to the x coordinate to draw at.
 *   - The 'y' variable refers to the y coordinate to draw at.
 *   - The 'width' variable refers to the width of the data area.
 *   - Replace 'align' with a string saying 'left', 'center', or 'right' to
 *     determine how you want the data visibly aligned.
 * 
 * ---
 *
 * ============================================================================
 * Terms of Use
 * ============================================================================
 *
 * 1. These plugins may be used in free or commercial games provided that they
 * have been acquired through legitimate means at VisuStella.com and/or any
 * other official approved VisuStella sources. Exceptions and special
 * circumstances that may prohibit usage will be listed on VisuStella.com.
 * 
 * 2. All of the listed coders found in the Credits section of this plugin must
 * be given credit in your games or credited as a collective under the name:
 * "VisuStella".
 * 
 * 3. You may edit the source code to suit your needs, so long as you do not
 * claim the source code belongs to you. VisuStella also does not take
 * responsibility for the plugin if any changes have been made to the plugin's
 * code, nor does VisuStella take responsibility for user-provided custom code
 * used for custom control effects including advanced JavaScript notetags
 * and/or plugin parameters that allow custom JavaScript code.
 * 
 * 4. You may NOT redistribute these plugins nor take code from this plugin to
 * use as your own. These plugins and their code are only to be downloaded from
 * VisuStella.com and other official/approved VisuStella sources. A list of
 * official/approved sources can also be found on VisuStella.com.
 *
 * 5. VisuStella is not responsible for problems found in your game due to
 * unintended usage, incompatibility problems with plugins outside of the
 * VisuStella MZ library, plugin versions that aren't up to date, nor
 * responsible for the proper working of compatibility patches made by any
 * third parties. VisuStella is not responsible for errors caused by any
 * user-provided custom code used for custom control effects including advanced
 * JavaScript notetags and/or plugin parameters that allow JavaScript code.
 *
 * 6. If a compatibility patch needs to be made through a third party that is
 * unaffiliated with VisuStella that involves using code from the VisuStella MZ
 * library, contact must be made with a member from VisuStella and have it
 * approved. The patch would be placed on VisuStella.com as a free download
 * to the public. Such patches cannot be sold for monetary gain, including
 * commissions, crowdfunding, and/or donations.
 * 
 * 7. If this VisuStella MZ plugin is a paid product, all project team members
 * must purchase their own individual copies of the paid product if they are to
 * use it. Usage includes working on related game mechanics, managing related
 * code, and/or using related Plugin Commands and features. Redistribution of
 * the plugin and/or its code to other members of the team is NOT allowed
 * unless they own the plugin itself as that conflicts with Article 4.
 * 
 * 8. Any extensions and/or addendums made to this plugin's Terms of Use can be
 * found on VisuStella.com and must be followed.
 *
 * ============================================================================
 * Credits
 * ============================================================================
 * 
 * If you are using this plugin, credit the following people in your game:
 * 
 * Team VisuStella
 * * Yanfly
 * * Arisu
 * * Olivia
 * * Irina
 *
 * ============================================================================
 * Changelog
 * ============================================================================
 * 
 * Version 1.07: April 30, 2021
 * * Bug Fixes!
 * ** Multiclasses with Adjust HP/MP settings should now properly adjust
 *    without the Core Engine installed. Fix made by Arisu.
 * ** Those without Victory Aftermath should no longer experience crashes when
 *    gaining Class Points or Job Points after battle. Fix made by Olivia.
 * ** With the Maintained Levels setting enabled, all unlocked multiclasses
 *    will also acquire skills upon leveling up and not just when switching to
 *    the classes manually. Fix made by Olivia.
 * * Feature Update!
 * ** During battle, equipment types belonging multiclasses will not be
 *    unequipped to prevent odd happenings. Update change by Arisu.
 * 
 * Version 1.06: April 16, 2021
 * * Bug Fixes!
 * ** Map based character sprite changes should now be reflected instantly.
 *    Fix made by Olivia.
 * * Documentation Update!
 * ** Added two more entries to the Clarification section. Updated by Arisu.
 * 
 * Version 1.05: February 12, 2021
 * * Bug Fixes!
 * ** Param bonuses for subclasses are no longer based on the current level but
 *    instead, the level for the subclass. Fix made by Irina.
 * * Optimization Update!
 * ** Plugin should run more optimized.
 * 
 * Version 1.04: January 8, 2021
 * * Bug Fixes!
 * ** Leveling up should now automatically cache the current class level.
 *    Fix made by Irina.
 * 
 * Version 1.03: January 1, 2021
 * * Bug Fixes!
 * ** General Settings should now have default values when added. If you are
 *    still getting an error when starting a new game, please open up the
 *    General Settings in the Plugin Parameters and hit OK. Fix made by Yanfly.
 * 
 * Version 1.02: December 25, 2020
 * * Bug Fixes!
 * ** Added a refresh after setting up new actors to recalculate any cached
 *    parameter values, skills, and passive states. Fix made by Yanfly.
 * ** Equipment duplication glitch should no longer occur.
 * * Documentation Update!
 * ** Help file updated for new features.
 * * New Features!
 * ** New notetag added by Yanfly.
 * *** <Class Picture: filename> and <Picture: filename>
 * **** Uses a picture from your project's /img/pictures/ folder instead of the
 *      class icon for the Class Change scene.
 * ** New Plugin Parameters added by Yanfly.
 * *** Window Settings > Scene_ClassChange > Confirm Animation ID > Offset X
 * *** Window Settings > Scene_ClassChange > Confirm Animation ID > Offset Y
 * **** Offsets have been added to let you adjust where the animation occurs
 *      for primary and subclass changing.
 * 
 * Version 1.01: December 18, 2020
 * * Bug Fixes!
 * ** Class specific character graphics no longer default to index 0 when no
 *    index is found or declared by notetags. Fix made by Yanfly.
 * * Documentation Update!
 * ** Added "Clarification" section to the documentation to explain some things
 *    that users might not understand correctly.
 * * Feature Update!
 * ** The button assist text for the "SHIFT" removal is now offset towards the
 *    left a bit for more room. Update made by Yanfly.
 *
 * Version 1.00 Official Release Date: January 11, 2021
 * * Finished Plugin!
 *
 * ============================================================================
 * End of Helpfile
 * ============================================================================
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassUnlockForActor
 * @text Unlock Class: Add For Actor(s)
 * @desc Unlock class(es) for target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to unlock class(es) for.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to be unlocked.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassUnlockForGlobal
 * @text Unlock Class: Add For Global
 * @desc Unlock class(es) for all party members.
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to be unlocked.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassUnlockRemoveActor
 * @text Unlock Class: Remove From Actor(s)
 * @desc Remove unlock class(es) for target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to remove an unlocked class(es) for.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to be removed from the unlocked status.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassUnlockRemoveGlobal
 * @text Unlock Class: Remove From Global
 * @desc Remove unlock class(es) for all party members.
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to be removed from the unlocked status.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassChangeAddRestrictTier
 * @text Change Restriction: Add Tier Restriction
 * @desc Add restrictions to prevent class changing specific tier(s)
 * to target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to restrict class tier(s) for.
 * @default ["1"]
 *
 * @arg Tiers:arraynum
 * @text Tiers(s)
 * @type number[]
 * @desc Select which class tier(s) to restrict changing for.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassChangeRemoveRestrictTier
 * @text Change Restriction: Remove Tier Restriction
 * @desc Remove restrictions to allow class changing specific tier(s)
 * for target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to remove class tier(s) restrictions for.
 * @default ["1"]
 *
 * @arg Tiers:arraynum
 * @text Tiers(s)
 * @type number[]
 * @desc Select which class tier(s) to remove restrictions for.
 * @default ["1"]
 *
 * @ --------------------------------------------------------------------------
 *
 * @command MulticlassChangeActorClass
 * @text Multiclass: Change Actor(s) Multiclass
 * @desc Changes a specific multiclass for target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to change the multiclass limit to.
 * @default ["1"]
 *
 * @arg Tier:num
 * @text Tier
 * @type number
 * @min 1
 * @desc Which multiclass tier to change for the target actor(s)?
 * @default 2
 *
 * @arg ClassID:num
 * @text Class ID
 * @type class
 * @desc Which class should go into this multiclass tier slot?
 * @default 1
 *
 * @ --------------------------------------------------------------------------
 *
 * @command MulticlassRaiseLimit
 * @text Multiclass: Raise Limit for Actor(s)
 * @desc Raise the multiclass limit for target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to change the multiclass limit to.
 * @default ["1"]
 *
 * @arg Limit:num
 * @text Raise Limit By
 * @type number
 * @min 1
 * @desc Raise the multiclass limit for target actor(s) by this much.
 * @default 1
 *
 * @ --------------------------------------------------------------------------
 *
 * @command MulticlassLowerLimit
 * @text Multiclass: Lower Limit for Actor(s)
 * @desc Lower the multiclass limit for target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to change the multiclass limit to.
 * @default ["1"]
 *
 * @arg Limit:num
 * @text Reduce Limit By
 * @type number
 * @min 1
 * @desc Lower the multiclass limit for target actor(s) by this much.
 * @default 1
 *
 * @ --------------------------------------------------------------------------
 *
 * @command MulticlassSetLimit
 * @text Multiclass: Set Limit for Actor(s)
 * @desc Set multiclass limit for target actor(s).
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to change the multiclass limit to.
 * @default ["1"]
 *
 * @arg Limit:num
 * @text Set Limit To
 * @type number
 * @min 1
 * @desc Set multiclass limit for target actor(s) to this much.
 * @default 2
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassPointsGain
 * @text Class Points: Gain
 * @desc The target actor(s) gains Class Points.
 * Gained amounts are affected by Class Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to gain Class Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Class Points
 * @desc Determine how many Class Points will be gained.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassPointsAdd
 * @text Class Points: Add
 * @desc The target actor(s) receives Class Points.
 * Received amounts are NOT affected by Class Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to receive Class Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Class Points
 * @desc Determine how many Class Points will be added.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassPointsLose
 * @text Class Points: Lose
 * @desc The target actor(s) loses Class Points.
 * Lost amounts are NOT affected by Class Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to lose Class Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Class Points
 * @desc Determine how many Class Points will be lost.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command ClassPointsSet
 * @text Class Points: Set
 * @desc Changes the exact Class Points for the target actor(s).
 * Changed amounts are NOT affected by Class Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to change Class Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Class Points
 * @desc Determine how many Class Points will be set exactly to.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command JobPointsGain
 * @text Job Points: Gain
 * @desc The target actor(s) gains Job Points.
 * Gained amounts are affected by Job Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to gain Job Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Job Points
 * @desc Determine how many Job Points will be gained.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command JobPointsAdd
 * @text Job Points: Add
 * @desc The target actor(s) receives Job Points.
 * Received amounts are NOT affected by Job Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to receive Job Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Job Points
 * @desc Determine how many Job Points will be added.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command JobPointsLose
 * @text Job Points: Lose
 * @desc The target actor(s) loses Job Points.
 * Lost amounts are NOT affected by Job Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to lose Job Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Job Points
 * @desc Determine how many Job Points will be lost.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command JobPointsSet
 * @text Job Points: Set
 * @desc Changes the exact Job Points for the target actor(s).
 * Changed amounts are NOT affected by Job Point bonus rates.
 *
 * @arg Actors:arraynum
 * @text Actor ID(s)
 * @type actor[]
 * @desc Select which Actor ID(s) to affect.
 * @default ["1"]
 *
 * @arg Classes:arraynum
 * @text Class ID(s)
 * @type class[]
 * @desc Select which Class ID(s) to change Job Points for.
 * Use "0" for the current class.
 * @default ["0"]
 *
 * @arg Points:eval
 * @text Job Points
 * @desc Determine how many Job Points will be set exactly to.
 * You may use code.
 * @default 100
 *
 * @ --------------------------------------------------------------------------
 *
 * @command SystemEnableClassChangeSystemMenu
 * @text System: Enable Class Change in Menu?
 * @desc Enables/disables Class Change inside the main menu.
 *
 * @arg Enable:eval
 * @text Enable/Disable?
 * @type boolean
 * @on Enable
 * @off Disable
 * @desc Enables/disables Class Change inside the main menu.
 * @default true
 *
 * @ --------------------------------------------------------------------------
 *
 * @command SystemShowClassChangeSystemMenu
 * @text System: Show Class Change in Menu?
 * @desc Shows/hides Class Change inside the main menu.
 *
 * @arg Show:eval
 * @text Show/Hide?
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Shows/hides Class Change inside the main menu.
 * @default true
 *
 * @ --------------------------------------------------------------------------
 *
 * @ ==========================================================================
 * @ Plugin Parameters
 * @ ==========================================================================
 *
 * @param BreakHead
 * @text --------------------------
 * @default ----------------------------------
 *
 * @param ClassChangeSystem
 * @default Plugin Parameters
 *
 * @param ATTENTION
 * @default READ THE HELP FILE
 *
 * @param BreakSettings
 * @text --------------------------
 * @default ----------------------------------
 * 
 * @param ClassChange
 * @text Class Change
 *
 * @param General:struct
 * @text General Settings
 * @parent ClassChange
 * @type struct<General>
 * @desc General settings for Class Change System.
 * @default {"Basics":"","HelpDescription:json":"\"The %1 class.\"","Icon:num":"96","MaintainLevels:eval":"false","ChangeAdjusHpMp:eval":"true","Unlock":"","AlwaysUnlocked:arraynum":"[\"1\",\"2\",\"3\",\"4\"]","StartingMulticlasses:num":"2"}
 *
 * @param BgSettings:struct
 * @text Background Settings
 * @parent ClassChange
 * @type struct<BgSettings>
 * @desc Background settings for Scene_ClassChange.
 * @default {"SnapshotOpacity:num":"192","BgFilename1:str":"","BgFilename2:str":""}
 *
 * @param ChangeClassSound:struct
 * @text Change Class Sound
 * @parent ClassChange
 * @type struct<Sound>
 * @desc Sound effect played when changing classes through Scene_ClassChange.
 * @default {"name:str":"Equip2","volume:num":"90","pitch:num":"100","pan:num":"0"}
 *
 * @param MainMenu:struct
 * @text Menu Access Settings
 * @parent ClassChange
 * @type struct<MenuAccess>
 * @desc Menu Access settings for Class Change.
 * @default {"Name:str":"Class","ShowMainMenu:eval":"true","EnableMainMenu:eval":"true"}
 *
 * @param Multiclass:arraystruct
 * @text Multiclass Settings
 * @parent ClassChange
 * @type struct<Multiclass>[]
 * @desc Multiclass settings for this plugin. Each tier allows you to have separate settings.
 * @default ["{\"Name:str\":\"Primary\",\"TextColor:str\":\"6\",\"HelpDescription:json\":\"\\\"Units gain all the benefits of its primary class.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"1.00\",\"paramRate1:num\":\"1.00\",\"paramRate2:num\":\"1.00\",\"paramRate3:num\":\"1.00\",\"paramRate4:num\":\"1.00\",\"paramRate5:num\":\"1.00\",\"paramRate6:num\":\"1.00\",\"paramRate7:num\":\"1.00\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"true\",\"DebuffRates:eval\":\"true\",\"StateRates:eval\":\"true\",\"StateResistance:eval\":\"true\",\"Param\":\"\",\"ParamRates:eval\":\"true\",\"XParamRates:eval\":\"true\",\"SParamRates:eval\":\"true\",\"Attack\":\"\",\"AttackElements:eval\":\"true\",\"AttackStates:eval\":\"true\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"Subclass\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"3rd Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"4th Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"5th Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"6th Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"7th Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"8th Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"9th Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}","{\"Name:str\":\"10th Class\",\"TextColor:str\":\"4\",\"HelpDescription:json\":\"\\\"Assign a class to this slot.\\\"\",\"BaseParameters\":\"\",\"paramRate0:num\":\"0.10\",\"paramRate1:num\":\"0.10\",\"paramRate2:num\":\"0.10\",\"paramRate3:num\":\"0.10\",\"paramRate4:num\":\"0.10\",\"paramRate5:num\":\"0.10\",\"paramRate6:num\":\"0.10\",\"paramRate7:num\":\"0.10\",\"Rewards\":\"\",\"expRate:num\":\"0.25\",\"resourceRate:num\":\"0.25\",\"Traits\":\"\",\"Rates\":\"\",\"ElementRates:eval\":\"false\",\"DebuffRates:eval\":\"false\",\"StateRates:eval\":\"false\",\"StateResistance:eval\":\"false\",\"Param\":\"\",\"ParamRates:eval\":\"false\",\"XParamRates:eval\":\"false\",\"SParamRates:eval\":\"false\",\"Attack\":\"\",\"AttackElements:eval\":\"false\",\"AttackStates:eval\":\"false\",\"Skills\":\"\",\"AddedStypes:eval\":\"true\",\"AddedSkills:eval\":\"true\",\"Equip\":\"\",\"EquipWeapons:eval\":\"true\",\"EquipArmors:eval\":\"true\"}"]
 *
 * @param Window:struct
 * @text Window Settings
 * @parent ClassChange
 * @type struct<Window>
 * @desc Window settings for Scene_ClassChange.
 * @default {"Scene":"","EnableLayout:eval":"true","LayoutStyle:str":"upper/right","DisplayedResources:arraystr":"[\"AP\",\"CP\",\"JP\",\"SP\"]","ConfirmAnimationID:num":"120","ConfirmAniPrimaryOffsetX:num":"0","ConfirmAniPrimaryOffsetY:num":"0","ConfirmAniSubclassOffsetX:num":"0","ConfirmAniSubclassOffsetY:num":"0","Window_ClassStatus":"","Window_ClassStatus_BgType:num":"0","ParamValueFontSize:num":"22","MenuPortraits:eval":"true","DrawBackRect:eval":"true","BackRectColor:str":"19","Window_ClassStatus_RectJS:func":"\"const ww = Math.floor(Graphics.boxWidth / 2);\\nconst wh = this.mainAreaHeight();\\nconst wx = this.isRightInputMode() ? 0 : ww;\\nconst wy = this.mainAreaTop();\\nreturn new Rectangle(wx, wy, ww, wh);\"","DrawPortraitJS:func":"\"// Declare Variables\\nconst lineHeight = this.lineHeight();\\nconst padding = this.itemPadding();\\nconst x1 = padding;\\nconst x2 = this.innerWidth / 2;\\n\\n// Draw Menu Image\\nthis.drawItemActorMenuImage(this._actor, 0, 0, this.innerWidth, this.innerHeight);\\n\\n// Draw Data\\nthis.drawActorName(this._actor, x1, lineHeight * 0);\\nthis.drawActorLevel(this._actor, x1, lineHeight * 1);\\nthis.placeBasicGauges(this._actor, x1, lineHeight * 2);\\nthis.drawActorResources(x2, lineHeight * 0, this.innerWidth / 2);\"","DrawFaceJS:func":"\"// Declare Variables\\nconst lineHeight = this.lineHeight();\\nconst gaugeLineHeight = this.gaugeLineHeight();\\nconst x = Math.floor(this.innerWidth / 2);\\nconst limitHeight = this.innerHeight - (this.actorParams().length * lineHeight);\\nconst actorX = Math.floor((x - ImageManager.faceWidth) / 2);\\nconst actorY = Math.floor((limitHeight - ImageManager.faceHeight) / 2);\\nlet dataHeight = lineHeight * 3;\\ndataHeight += gaugeLineHeight * ($dataSystem.optDisplayTp ? 3 : 2);\\nconst dataY = Math.floor((limitHeight - dataHeight) / 2);\\n\\n// Draw Data\\nthis.drawActorFace(this._actor, actorX, actorY, ImageManager.faceWidth, ImageManager.faceHeight);\\nthis.drawActorName(this._actor, x, dataY + lineHeight * 0);\\nthis.drawActorResources(x, dataY + this.lineHeight() * 1, ImageManager.faceWidth);\"","DrawParamJS:func":"\"// Declare variables\\nconst params = this.actorParams();\\nconst lineHeight = this.lineHeight();\\nconst padding = this.itemPadding();\\nconst baseX = 0;\\nconst baseY = this.innerHeight - params.length * lineHeight;\\nconst baseWidth = this.innerWidth;\\nconst valueFontSize = this.paramValueFontSize();\\n\\n// Calculate Widths\\nlet paramNameWidth = Math.max(...params.map(param => this.textWidth(TextManager.param(param))));\\nparamNameWidth += padding * 2;\\nif (this.isUseParamNamesWithIcons()) {\\n    paramNameWidth += ImageManager.iconWidth + 4;\\n}\\nlet arrowWidth = this.rightArrowWidth();\\nconst totalDivides = this.innerWidth >= 500 ? 3 : 2;\\nlet paramValueWidth = Math.floor((baseWidth - paramNameWidth - arrowWidth) / totalDivides);\\nparamNameWidth = baseWidth - (paramValueWidth * totalDivides) - arrowWidth;\\n\\n// Draw Parameters\\nlet x = baseX;\\nlet y = baseY;\\nlet value = 0;\\nlet diffValue = 0;\\nlet alter = 2;\\nfor (const paramId of params) {\\n    // Reset\\n    this.resetFontSettings();\\n\\n    // Draw Param Name\\n    this.drawItemDarkRect(x, y, paramNameWidth, lineHeight, alter);\\n    this.drawUpdatedParamName(paramId, x, y, paramNameWidth);\\n    this.resetFontSettings();\\n    x += paramNameWidth;\\n\\n    // Draw Param Before\\n    this.contents.fontSize = valueFontSize;\\n    this.drawItemDarkRect(x, y, paramValueWidth, lineHeight, alter);\\n    this.drawUpdatedBeforeParamValue(paramId, x, y, paramValueWidth);\\n    this.resetFontSettings();\\n    x += paramValueWidth;\\n\\n    // Draw Arrow\\n    this.drawItemDarkRect(x, y, arrowWidth, lineHeight, alter);\\n    this.drawRightArrow(x, y);\\n    x += arrowWidth;\\n\\n    // Draw Param After\\n    this.contents.fontSize = valueFontSize;\\n    this.drawItemDarkRect(x, y, paramValueWidth, lineHeight, alter);\\n    this.drawUpdatedAfterParamValue(paramId, x, y, paramValueWidth);\\n    x += paramValueWidth;\\n\\n    // Draw Param Change\\n    if (totalDivides > 2) {\\n        this.drawItemDarkRect(x, y, paramValueWidth, lineHeight, alter);\\n        this.drawUpdatedParamValueDiff(paramId, x, y, paramValueWidth);\\n    }\\n\\n    // Prepare Next Parameter\\n    x = baseX;\\n    y += lineHeight;\\n    alter = alter === 2 ? 1 : 2;\\n}\"","Window_ClassTier":"","Window_ClassTier_BgType:num":"0","VocabNoClassAssigned:str":"No Class Assigned","ShiftShortcutKey:eval":"true","ShiftButtonAssistText:str":"Unassign","Window_ClassTier_ExtraJS:func":"\"// Declare Arguments\\nconst classID = arguments[0];\\nconst tier = arguments[1];\\nconst settings = arguments[2];\\nconst rect = arguments[3];\\nconst targetClass = $dataClasses[classID];\\nconst wordWrap = Imported.VisuMZ_1_MessageCore;\\nconst removeIcons = true;\\nconst fontSize = 22;\\n\\n// Create Coordinates\\nlet x = rect.x + (this.itemPadding() * 4);\\nlet y = rect.y + (this.lineHeight() * 3.25);\\nlet width = rect.width - (this.itemPadding() * 8);\\n\\n// Skill Type Access\\nif (settings.AddedStypes && ((y + this.lineHeight()) <= (rect.y + rect.height))) {\\n    let stypes = targetClass.traits.\\n        filter(trait => trait.code === Game_BattlerBase.TRAIT_STYPE_ADD).\\n        map(trait => $dataSystem.skillTypes[trait.dataId]).\\n        join(', ');\\n    let text = '\\\\\\\\C[16]%1:\\\\\\\\C[0] \\\\\\\\FS[%3]%2'.format(TextManager.skill, stypes, fontSize || 22);\\n    if (removeIcons) text = text.replace(/\\\\\\\\I\\\\[(\\\\d+)\\\\]/gi, '');\\n    if (wordWrap) text = '<WordWrap>' + text;\\n    this.drawTextEx(text, x, y, width);\\n    y += this.lineHeight();\\n}\\n\\n// Weapon Access\\nif (settings.EquipWeapons && ((y + this.lineHeight()) <= (rect.y + rect.height))) {\\n    let stypes = targetClass.traits.\\n        filter(trait => trait.code === Game_BattlerBase.TRAIT_EQUIP_WTYPE).\\n        map(trait => $dataSystem.weaponTypes[trait.dataId]).\\n        join(', ');\\n    let text = '\\\\\\\\C[16]%1:\\\\\\\\C[0] \\\\\\\\FS[%3]%2'.format(TextManager.weapon, stypes, fontSize || 22);\\n    if (removeIcons) text = text.replace(/\\\\\\\\I\\\\[(\\\\d+)\\\\]/gi, '');\\n    if (wordWrap) text = '<WordWrap>' + text;\\n    this.drawTextEx(text, x, y, width);\\n    y += this.lineHeight();\\n}\"","Window_ClassTier_RectJS:func":"\"const ww = Graphics.boxWidth - this._statusWindow.width;\\nconst wh = this.mainAreaHeight();\\nconst wx = this.isRightInputMode() ? ww : 0;\\nconst wy = this.mainAreaTop();\\nreturn new Rectangle(wx, wy, ww, wh);\"","Window_ClassList":"","Window_ClassList_BgType:num":"0","VocabUnassignClass:str":"Unassign Class","UnassignHelpDescription:json":"\"Remove any classes for this slot.\"","Window_ClassList_RectJS:func":"\"const ww = Graphics.boxWidth - this._statusWindow.width;\\nconst wh = this.mainAreaHeight();\\nconst wx = this.isRightInputMode() ? ww : 0;\\nconst wy = this.mainAreaTop();\\nreturn new Rectangle(wx, wy, ww, wh);\""}
 * 
 * @param Resources
 *
 * @param ClassPoints:struct
 * @text Class Points Settings
 * @parent Resources
 * @type struct<ClassPoints>
 * @desc Settings for Class Points and how they work in-game.
 * @default {"Mechanics":"","SharedResource:eval":"true","MaxResource:num":"0","Visual":"","ShowInMenus:eval":"true","Icon:num":"87","Vocabulary":"","FullText:str":"Class Points","AbbrText:str":"CP","TextFmt:str":"%1 \\c[5]%2\\c[0]%3","Gain":"","PerAction:str":"0","PerLevelUp:str":"100","PerEnemy:str":"0","AliveActors:eval":"true","Victory":"","ShowVictory:eval":"false","VictoryText:str":"%1 gains %2 %3!","AftermathActorDisplay:eval":"false","AftermathText:str":"+%1 %2"}
 *
 * @param JobPoints:struct
 * @text Job Points Settings
 * @parent Resources
 * @type struct<JobPoints>
 * @desc Settings for Job Points and how they work in-game.
 * @default {"Mechanics":"","SharedResource:eval":"false","MaxResource:num":"0","Visual":"","ShowInMenus:eval":"true","Icon:num":"188","Vocabulary":"","FullText:str":"Job Points","AbbrText:str":"JP","TextFmt:str":"%1 \\c[5]%2\\c[0]%3","Gain":"","PerAction:str":"10 + Math.randomInt(10)","PerLevelUp:str":"0","PerEnemy:str":"50 + Math.randomInt(50)","AliveActors:eval":"true","Victory":"","ShowVictory:eval":"true","VictoryText:str":"%1 gains %2 %3!","AftermathActorDisplay:eval":"true","AftermathText:str":"+%1 %2"}
 *
 * @param BreakEnd1
 * @text --------------------------
 * @default ----------------------------------
 *
 * @param End Of
 * @default Plugin Parameters
 *
 * @param BreakEnd2
 * @text --------------------------
 * @default ----------------------------------
 *
 */
/* ----------------------------------------------------------------------------
 * General Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~General:
 *
 * @param Basics
 *
 * @param HelpDescription:json
 * @text Default Help
 * @parent Basics
 * @type note
 * @desc Default help description for all classes.
 * %1 - Class Name
 * @default "The %1 class."
 *
 * @param Icon:num
 * @text Default Icon
 * @parent Basics
 * @desc Default icon used for all classes.
 * @default 96
 *
 * @param MaintainLevels:eval
 * @text Maintain Levels?
 * @parent Basics
 * @type boolean
 * @on Each Class Same Level
 * @off Each Class Separate
 * @desc Make each class have the same level or
 * make each class have their own level?
 * @default false
 *
 * @param ChangeAdjusHpMp:eval
 * @text Change-Adjust HP/MP
 * @parent Basics
 * @type boolean
 * @on Adjust
 * @off Don't
 * @desc Adjust HP/MP differences after changing classes with MaxHP/MaxMP values.
 * @default true
 * 
 * @param Unlock
 * @text Class Unlocking
 *
 * @param AlwaysUnlocked:arraynum
 * @text Always Unlocked
 * @parent Unlock
 * @type class[]
 * @desc Which classes are always unlocked and available?
 * @default ["1","2","3","4"]
 *
 * @param StartingMulticlasses:num
 * @text Starting Multiclasses
 * @parent Unlock
 * @type number
 * @min 1
 * @desc How many classes can actors use at the start by default?
 * Use 1 for just the primary class.
 * @default 2
 *
 */
/* ----------------------------------------------------------------------------
 * Background Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~BgSettings:
 *
 * @param SnapshotOpacity:num
 * @text Snapshop Opacity
 * @type number
 * @min 0
 * @max 255
 * @desc Snapshot opacity for the scene.
 * @default 192
 *
 * @param BgFilename1:str
 * @text Background 1
 * @type file
 * @dir img/titles1/
 * @desc Filename used for the bottom background image.
 * Leave empty if you don't wish to use one.
 * @default 
 *
 * @param BgFilename2:str
 * @text Background 2
 * @type file
 * @dir img/titles2/
 * @desc Filename used for the upper background image.
 * Leave empty if you don't wish to use one.
 * @default 
 *
 */
/* ----------------------------------------------------------------------------
 * Menu Access Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~MenuAccess:
 *
 * @param Name:str
 * @text Command Name
 * @parent Options
 * @desc Name of the 'Template' option in the Main Menu.
 * @default Class
 *
 * @param ShowMainMenu:eval
 * @text Show in Main Menu?
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Add the 'Template' option to the Main Menu by default?
 * @default true
 *
 * @param EnableMainMenu:eval
 * @text Enable in Main Menu?
 * @type boolean
 * @on Enable
 * @off Disable
 * @desc Enable the 'Template' option to the Main Menu by default?
 * @default true
 *
 */
/* ----------------------------------------------------------------------------
 * Multiclass Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~Multiclass:
 *
 * @param Name:str
 * @text Class Tier Name
 * @desc Name of this class tier.
 * @default Untitled
 * 
 * @param TextColor:str
 * @text Text Color
 * @parent Name:str
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 4
 *
 * @param HelpDescription:json
 * @text Help Description
 * @parent Name:str
 * @type note
 * @desc Help description when this multiclass slot is picked.
 * @default "Assign a class to this slot."
 * 
 * @param BaseParameters
 * @text Base Parameter Bonuses
 * 
 * @param paramRate0:num
 * @text MaxHP
 * @parent BaseParameters
 * @desc How little of this class tier's MaxHP should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param paramRate1:num
 * @text MaxMP
 * @parent BaseParameters
 * @desc How little of this class tier's MaxMP should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param paramRate2:num
 * @text ATK
 * @parent BaseParameters
 * @desc How little of this class tier's ATK should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param paramRate3:num
 * @text DEF
 * @parent BaseParameters
 * @desc How little of this class tier's DEF should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param paramRate4:num
 * @text MAT
 * @parent BaseParameters
 * @desc How little of this class tier's MAT should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param paramRate5:num
 * @text MDF
 * @parent BaseParameters
 * @desc How little of this class tier's MDF should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param paramRate6:num
 * @text AGI
 * @parent BaseParameters
 * @desc How little of this class tier's AGI should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param paramRate7:num
 * @text LUK
 * @parent BaseParameters
 * @desc How little of this class tier's LUK should be added
 * to base stats? Does not apply to Tier 1.
 * @default 0.10
 * 
 * @param Rewards
 * @text Reward Rates
 * 
 * @param expRate:num
 * @text EXP
 * @parent Rewards
 * @desc How much EXP does a class in this tier earn?
 * Does not apply to Tier 1. Only for Battle Rewards.
 * @default 0.25
 * 
 * @param resourceRate:num
 * @text Resources
 * @parent Rewards
 * @desc Resource rate (ie. CP, JP) earned for this tier.
 * Does not apply to Tier 1. Only for Battle Rewards.
 * @default 0.25
 * 
 * @param Traits
 * @text Inherit Traits
 * 
 * @param Rates
 * @parent Traits
 *
 * @param ElementRates:eval
 * @text Element Rates?
 * @parent Rates
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the element rates from this class tier?
 * Does not apply to Tier 1.
 * @default false
 *
 * @param DebuffRates:eval
 * @text Debuff Rates?
 * @parent Rates
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the debuff rates from this class tier?
 * Does not apply to Tier 1.
 * @default false
 *
 * @param StateRates:eval
 * @text State Rates?
 * @parent Rates
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the state rates from this class tier?
 * Does not apply to Tier 1.
 * @default false
 *
 * @param StateResistance:eval
 * @text State Resistance?
 * @parent Rates
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the state resistances from this class tier?
 * Does not apply to Tier 1.
 * @default false
 * 
 * @param Param
 * @text Param Rates
 * @parent Traits
 *
 * @param ParamRates:eval
 * @text Base-Param Rates?
 * @parent Param
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit Base Parameter rates from this class tier?
 * Does not apply to Tier 1.
 * @default false
 *
 * @param XParamRates:eval
 * @text X-Param Rates?
 * @parent Param
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit X-Parameter rates from this class tier?
 * Does not apply to Tier 1.
 * @default false
 *
 * @param SParamRates:eval
 * @text S-Param Rates?
 * @parent Param
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit S-Parameter rates from this class tier?
 * Does not apply to Tier 1.
 * @default false
 * 
 * @param Attack
 * @parent Traits
 *
 * @param AttackElements:eval
 * @text Attack Elements?
 * @parent Attack
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the attack elements from this class tier?
 * Does not apply to Tier 1.
 * @default false
 *
 * @param AttackStates:eval
 * @text Attack States?
 * @parent Attack
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the attack states from this class tier?
 * Does not apply to Tier 1.
 * @default false
 * 
 * @param Skills
 * @parent Traits
 *
 * @param AddedStypes:eval
 * @text Added STypes?
 * @parent Skills
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the added STypes from this class tier?
 * Does not apply to Tier 1.
 * @default true
 *
 * @param AddedSkills:eval
 * @text Added Skills?
 * @parent Skills
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the added skills from this class tier?
 * Does not apply to Tier 1.
 * @default true
 * 
 * @param Equip
 * @text Equipment
 * @parent Traits
 *
 * @param EquipWeapons:eval
 * @text Equippable Weapons?
 * @parent Equip
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the equippable weapons from this class tier?
 * Does not apply to Tier 1.
 * @default true
 *
 * @param EquipArmors:eval
 * @text Equippable Armors?
 * @parent Equip
 * @type boolean
 * @on Inherit
 * @off Don't
 * @desc Inherit the equippable armors from this class tier?
 * Does not apply to Tier 1.
 * @default true
 *
 */
/* ----------------------------------------------------------------------------
 * Sound Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~Sound:
 *
 * @param name:str
 * @text Filename
 * @type file
 * @dir audio/se/
 * @desc Filename of the sound effect played.
 * @default Equip2
 *
 * @param volume:num
 * @text Volume
 * @type number
 * @max 100
 * @desc Volume of the sound effect played.
 * @default 90
 *
 * @param pitch:num
 * @text Pitch
 * @type number
 * @max 100
 * @desc Pitch of the sound effect played.
 * @default 100
 *
 * @param pan:num
 * @text Pan
 * @desc Pan of the sound effect played.
 * @default 0
 *
 */
/* ----------------------------------------------------------------------------
 * Window Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~Window:
 * 
 * @param Scene
 * @text Scene_ClassChange
 *
 * @param EnableLayout:eval
 * @text Recommended Layout?
 * @parent Scene
 * @type boolean
 * @on Use
 * @off Don't Use
 * @desc Use the recommended Menu Layout provided by this plugin?
 * @default true
 *
 * @param LayoutStyle:str
 * @text Layout Style
 * @parent Scene
 * @type select
 * @option Upper Help, Left Input
 * @value upper/left
 * @option Upper Help, Right Input
 * @value upper/right
 * @option Lower Help, Left Input
 * @value lower/left
 * @option Lower Help, Right Input
 * @value lower/right
 * @desc If using an updated layout, how do you want to style
 * the menu scene layout?
 * @default upper/right
 * 
 * @param DisplayedResources:arraystr
 * @text Displayed Resources
 * @parent Scene
 * @type select[]
 * @option AP - Ability Points (Requires VisuMZ_2_SkillLearnSystem)
 * @value AP
 * @option CP - Class Points
 * @value CP
 * @option JP - Job Points
 * @value JP
 * @option SP - Skill Points (Requires VisuMZ_2_SkillLearnSystem)
 * @value SP
 * @desc Select which resources to display in Scene_Class's class
 * lists. Non-shared (limit: 2) resources appear in the lists.
 * @default ["AP","CP","JP","SP"]
 *
 * @param ConfirmAnimationID:num
 * @text Confirm Animation ID
 * @parent Scene
 * @type animation
 * @desc Play this animation when a class change has been made.
 * @default 120
 *
 * @param ConfirmAniPrimaryOffsetX:num
 * @text Primary Offset X
 * @parent ConfirmAnimationID:num
 * @desc Adjust the offset X of primary class animations.
 * Negative for left. Positive for right.
 * @default 0
 *
 * @param ConfirmAniPrimaryOffsetY:num
 * @text Primary Offset Y
 * @parent ConfirmAnimationID:num
 * @desc Adjust the offset Y of primary class animations.
 * Negative for up. Positive for down.
 * @default 0
 *
 * @param ConfirmAniSubclassOffsetX:num
 * @text Subclass Offset X
 * @parent ConfirmAnimationID:num
 * @desc Adjust the offset X of subclass animations.
 * Negative for left. Positive for right.
 * @default 0
 *
 * @param ConfirmAniSubclassOffsetY:num
 * @text Subclass Offset Y
 * @parent ConfirmAnimationID:num
 * @desc Adjust the offset Y of subclass animations.
 * Negative for up. Positive for down.
 * @default 0
 *
 * @param Window_ClassStatus
 *
 * @param Window_ClassStatus_BgType:num
 * @text Background Type
 * @parent Window_ClassStatus
 * @type select
 * @option 0 - Window
 * @value 0
 * @option 1 - Dim
 * @value 1
 * @option 2 - Transparent
 * @value 2
 * @desc Select background type for this window.
 * @default 0
 *
 * @param ParamValueFontSize:num
 * @text Param Font Size
 * @parent Window_ClassStatus
 * @desc The font size used for parameter values.
 * @default 22
 *
 * @param MenuPortraits:eval
 * @text Show Menu Portraits?
 * @parent Window_ClassStatus
 * @type boolean
 * @on Use Portraits
 * @off Use Faces
 * @desc If Main Menu Core is installed, display the Menu Portraits
 * instead of the actor's face in the status window?
 * @default true
 *
 * @param DrawBackRect:eval
 * @text Show Back Rectangles?
 * @parent Window_ClassStatus
 * @type boolean
 * @on Draw
 * @off Don't Draw
 * @desc Show back rectangles of darker colors to display information better?
 * @default true
 *
 * @param BackRectColor:str
 * @text Back Rectangle Color
 * @parent DrawBackRect:eval
 * @desc Use #rrggbb for custom colors or regular numbers
 * for text colors from the Window Skin.
 * @default 19
 *
 * @param Window_ClassStatus_RectJS:func
 * @text JS: X, Y, W, H
 * @parent Window_ClassStatus
 * @type note
 * @desc Code used to determine the dimensions for this window.
 * @default "const ww = Math.floor(Graphics.boxWidth / 2);\nconst wh = this.mainAreaHeight();\nconst wx = this.isRightInputMode() ? 0 : ww;\nconst wy = this.mainAreaTop();\nreturn new Rectangle(wx, wy, ww, wh);"
 *
 * @param DrawPortraitJS:func
 * @text JS: Portrait Upper
 * @parent Window_ClassStatus
 * @type note
 * @desc If Menu Portraits are available, this is code used to draw
 * the upper data like this in the Status Window.
 * @default "// Declare Variables\nconst lineHeight = this.lineHeight();\nconst padding = this.itemPadding();\nconst x1 = padding;\nconst x2 = this.innerWidth / 2;\n\n// Draw Menu Image\nthis.drawItemActorMenuImage(this._actor, 0, 0, this.innerWidth, this.innerHeight);\n\n// Draw Data\nthis.drawActorName(this._actor, x1, lineHeight * 0);\nthis.drawActorLevel(this._actor, x1, lineHeight * 1);\nthis.placeBasicGauges(this._actor, x1, lineHeight * 2);\nthis.drawActorResources(x2, lineHeight * 0, this.innerWidth / 2);"
 *
 * @param DrawFaceJS:func
 * @text JS: Face Upper
 * @parent Window_ClassStatus
 * @type note
 * @desc If faces used used, this is code used to draw the upper
 * data like this in the Status Window.
 * @default "// Declare Variables\nconst lineHeight = this.lineHeight();\nconst gaugeLineHeight = this.gaugeLineHeight();\nconst x = Math.floor(this.innerWidth / 2);\nconst limitHeight = this.innerHeight - (this.actorParams().length * lineHeight);\nconst actorX = Math.floor((x - ImageManager.faceWidth) / 2);\nconst actorY = Math.floor((limitHeight - ImageManager.faceHeight) / 2);\nlet dataHeight = lineHeight * 3;\ndataHeight += gaugeLineHeight * ($dataSystem.optDisplayTp ? 3 : 2);\nconst dataY = Math.floor((limitHeight - dataHeight) / 2);\n\n// Draw Data\nthis.drawActorFace(this._actor, actorX, actorY, ImageManager.faceWidth, ImageManager.faceHeight);\nthis.drawActorName(this._actor, x, dataY + lineHeight * 0);\nthis.drawActorResources(x, dataY + this.lineHeight() * 1, ImageManager.faceWidth);"
 *
 * @param DrawParamJS:func
 * @text JS: Parameter Lower
 * @parent Window_ClassStatus
 * @type note
 * @desc Code to determine how parameters are drawn in the
 * Status Window.
 * @default "// Declare variables\nconst params = this.actorParams();\nconst lineHeight = this.lineHeight();\nconst padding = this.itemPadding();\nconst baseX = 0;\nconst baseY = this.innerHeight - params.length * lineHeight;\nconst baseWidth = this.innerWidth;\nconst valueFontSize = this.paramValueFontSize();\n\n// Calculate Widths\nlet paramNameWidth = Math.max(...params.map(param => this.textWidth(TextManager.param(param))));\nparamNameWidth += padding * 2;\nif (this.isUseParamNamesWithIcons()) {\n    paramNameWidth += ImageManager.iconWidth + 4;\n}\nlet arrowWidth = this.rightArrowWidth();\nconst totalDivides = this.innerWidth >= 500 ? 3 : 2;\nlet paramValueWidth = Math.floor((baseWidth - paramNameWidth - arrowWidth) / totalDivides);\nparamNameWidth = baseWidth - (paramValueWidth * totalDivides) - arrowWidth;\n\n// Draw Parameters\nlet x = baseX;\nlet y = baseY;\nlet value = 0;\nlet diffValue = 0;\nlet alter = 2;\nfor (const paramId of params) {\n    // Reset\n    this.resetFontSettings();\n\n    // Draw Param Name\n    this.drawItemDarkRect(x, y, paramNameWidth, lineHeight, alter);\n    this.drawUpdatedParamName(paramId, x, y, paramNameWidth);\n    this.resetFontSettings();\n    x += paramNameWidth;\n\n    // Draw Param Before\n    this.contents.fontSize = valueFontSize;\n    this.drawItemDarkRect(x, y, paramValueWidth, lineHeight, alter);\n    this.drawUpdatedBeforeParamValue(paramId, x, y, paramValueWidth);\n    this.resetFontSettings();\n    x += paramValueWidth;\n\n    // Draw Arrow\n    this.drawItemDarkRect(x, y, arrowWidth, lineHeight, alter);\n    this.drawRightArrow(x, y);\n    x += arrowWidth;\n\n    // Draw Param After\n    this.contents.fontSize = valueFontSize;\n    this.drawItemDarkRect(x, y, paramValueWidth, lineHeight, alter);\n    this.drawUpdatedAfterParamValue(paramId, x, y, paramValueWidth);\n    x += paramValueWidth;\n\n    // Draw Param Change\n    if (totalDivides > 2) {\n        this.drawItemDarkRect(x, y, paramValueWidth, lineHeight, alter);\n        this.drawUpdatedParamValueDiff(paramId, x, y, paramValueWidth);\n    }\n\n    // Prepare Next Parameter\n    x = baseX;\n    y += lineHeight;\n    alter = alter === 2 ? 1 : 2;\n}"
 *
 * @param Window_ClassTier
 *
 * @param Window_ClassTier_BgType:num
 * @text Background Type
 * @parent Window_ClassTier
 * @type select
 * @option 0 - Window
 * @value 0
 * @option 1 - Dim
 * @value 1
 * @option 2 - Transparent
 * @value 2
 * @desc Select background type for this window.
 * @default 0
 *
 * @param VocabNoClassAssigned:str
 * @text No Class Assigned
 * @parent Window_ClassTier
 * @desc Text used when no class is assigned to the slot.
 * @default No Class Assigned
 *
 * @param ShiftShortcutKey:eval
 * @text Use SHIFT Shortcut?
 * @parent Window_ClassTier
 * @type boolean
 * @on Use
 * @off Don't
 * @desc Add the "Shift" button as a shortcut key to removing classes?
 * @default true
 *
 * @param ShiftButtonAssistText:str
 * @text Button Assist Text
 * @parent ShiftShortcutKey:eval
 * @desc Text used for the Button Assist Window
 * @default Unassign
 *
 * @param Window_ClassTier_ExtraJS:func
 * @text JS: Extra Data
 * @parent Window_ClassTier
 * @type note
 * @desc Code used to draw extra data if there is enough room.
 * @default "// Declare Arguments\nconst classID = arguments[0];\nconst tier = arguments[1];\nconst settings = arguments[2];\nconst rect = arguments[3];\nconst targetClass = $dataClasses[classID];\nconst wordWrap = Imported.VisuMZ_1_MessageCore;\nconst removeIcons = true;\nconst fontSize = 22;\n\n// Create Coordinates\nlet x = rect.x + (this.itemPadding() * 4);\nlet y = rect.y + (this.lineHeight() * 3.25);\nlet width = rect.width - (this.itemPadding() * 8);\n\n// Skill Type Access\nif (settings.AddedStypes && ((y + this.lineHeight()) <= (rect.y + rect.height))) {\n    let stypes = targetClass.traits.\n        filter(trait => trait.code === Game_BattlerBase.TRAIT_STYPE_ADD).\n        map(trait => $dataSystem.skillTypes[trait.dataId]).\n        join(', ');\n    let text = '\\\\C[16]%1:\\\\C[0] \\\\FS[%3]%2'.format(TextManager.skill, stypes, fontSize || 22);\n    if (removeIcons) text = text.replace(/\\\\I\\[(\\d+)\\]/gi, '');\n    if (wordWrap) text = '<WordWrap>' + text;\n    this.drawTextEx(text, x, y, width);\n    y += this.lineHeight();\n}\n\n// Weapon Access\nif (settings.EquipWeapons && ((y + this.lineHeight()) <= (rect.y + rect.height))) {\n    let stypes = targetClass.traits.\n        filter(trait => trait.code === Game_BattlerBase.TRAIT_EQUIP_WTYPE).\n        map(trait => $dataSystem.weaponTypes[trait.dataId]).\n        join(', ');\n    let text = '\\\\C[16]%1:\\\\C[0] \\\\FS[%3]%2'.format(TextManager.weapon, stypes, fontSize || 22);\n    if (removeIcons) text = text.replace(/\\\\I\\[(\\d+)\\]/gi, '');\n    if (wordWrap) text = '<WordWrap>' + text;\n    this.drawTextEx(text, x, y, width);\n    y += this.lineHeight();\n}"
 *
 * @param Window_ClassTier_RectJS:func
 * @text JS: X, Y, W, H
 * @parent Window_ClassTier
 * @type note
 * @desc Code used to determine the dimensions for this window.
 * @default "const ww = Graphics.boxWidth - this._statusWindow.width;\nconst wh = this.mainAreaHeight();\nconst wx = this.isRightInputMode() ? ww : 0;\nconst wy = this.mainAreaTop();\nreturn new Rectangle(wx, wy, ww, wh);"
 *
 * @param Window_ClassList
 *
 * @param Window_ClassList_BgType:num
 * @text Background Type
 * @parent Window_ClassList
 * @type select
 * @option 0 - Window
 * @value 0
 * @option 1 - Dim
 * @value 1
 * @option 2 - Transparent
 * @value 2
 * @desc Select background type for this window.
 * @default 0
 *
 * @param VocabUnassignClass:str
 * @text Unassign Class
 * @parent Window_ClassList
 * @desc Text used for an empty class slot.
 * @default Unassign Class
 *
 * @param UnassignHelpDescription:json
 * @text Help Description
 * @parent VocabUnassignClass:str
 * @type note
 * @desc Help description for unassigning a class.
 * @default "Remove any classes for this slot."
 *
 * @param Window_ClassList_RectJS:func
 * @text JS: X, Y, W, H
 * @parent Window_ClassList
 * @type note
 * @desc Code used to determine the dimensions for this window.
 * @default "const ww = Graphics.boxWidth - this._statusWindow.width;\nconst wh = this.mainAreaHeight();\nconst wx = this.isRightInputMode() ? ww : 0;\nconst wy = this.mainAreaTop();\nreturn new Rectangle(wx, wy, ww, wh);"
 *
 */
/* ----------------------------------------------------------------------------
 * Class Points Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~ClassPoints:
 *
 * @param Mechanics
 *
 * @param SharedResource:eval
 * @text Shared Class Points
 * @parent Mechanics
 * @type boolean
 * @on Shared Across Classes
 * @off Classes Separate
 * @desc Do you want Class Points to be shared across all classes?
 * Or do you want all classes to have their own?
 * @default false
 *
 * @param MaxResource:num
 * @text Maximum
 * @parent Mechanics
 * @type Number
 * @desc What's the maximum amount of Class Points an actor can have?
 * Use 0 for unlimited Class Points.
 * @default 0
 *
 * @param Visual
 *
 * @param ShowInMenus:eval
 * @text Show In Menus?
 * @parent Visual
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Do you wish to show Class Points in menus that allow them?
 * @default true
 *
 * @param Icon:num
 * @text Icon
 * @parent Visual
 * @desc What is the icon you want to use to represent Class Points?
 * @default 87
 *
 * @param Vocabulary
 *
 * @param FullText:str
 * @text Full Text
 * @parent Vocabulary
 * @desc The full text of how Class Points appears in-game.
 * @default Class Points
 *
 * @param AbbrText:str
 * @text Abbreviated Text
 * @parent Vocabulary
 * @desc The abbreviation of how Class Points appears in-game.
 * @default CP
 *
 * @param TextFmt:str
 * @text Menu Text Format
 * @parent Vocabulary
 * @desc What is the text format for it to be displayed in windows.
 * %1 - Value, %2 - Abbr, %3 - Icon, %4 - Full Text
 * @default %1 \c[5]%2\c[0]%3
 *
 * @param Gain
 *
 * @param PerAction:str
 * @text Per Action Hit
 * @parent Gain
 * @desc How many Class Points should an actor gain per action?
 * You may use code.
 * @default 0
 *
 * @param PerLevelUp:str
 * @text Per Level Up
 * @parent Gain
 * @desc How many Class Points should an actor gain per level up?
 * You may use code.
 * @default 100
 *
 * @param PerEnemy:str
 * @text Per Enemy Defeated
 * @parent Gain
 * @desc How many Class Points should an actor gain per enemy?
 * You may use code.
 * @default 0
 *
 * @param AliveActors:eval
 * @text Alive Actors?
 * @parent PerEnemy:str
 * @type boolean
 * @on Alive Requirement
 * @off No Requirement
 * @desc Do actors have to be alive to receive Class Points from
 * defeated enemies?
 * @default true
 *
 * @param Victory
 *
 * @param ShowVictory:eval
 * @text Show During Victory?
 * @parent Victory
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Show how much CP an actor has earned in battle during the
 * victory phase?
 * @default true
 *
 * @param VictoryText:str
 * @text Victory Text
 * @parent Victory
 * @desc For no Victory Aftermath, this is the text that appears.
 * %1 - Actor, %2 - Earned, %3 - Abbr, %4 - Full Text
 * @default %1 gains %2 %3!
 *
 * @param AftermathActorDisplay:eval
 * @text Aftermath Display?
 * @parent Victory
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Requires VisuMZ_3_VictoryAftermath. Show Class Points as
 * the main acquired resource in the actor windows?
 * @default true
 *
 * @param AftermathText:str
 * @text Aftermath Text
 * @parent Victory
 * @desc For no Victory Aftermath, this is the text that appears.
 * %1 - Earned, %2 - Abbr, %3 - Full Text
 * @default +%1 %2
 *
 */
/* ----------------------------------------------------------------------------
 * Job Points Settings
 * ----------------------------------------------------------------------------
 */
/*~struct~JobPoints:
 *
 * @param Mechanics
 *
 * @param SharedResource:eval
 * @text Shared Job Points
 * @parent Mechanics
 * @type boolean
 * @on Shared Across Classes
 * @off Classes Separate
 * @desc Do you want Job Points to be shared across all classes?
 * Or do you want all classes to have their own?
 * @default false
 *
 * @param MaxResource:num
 * @text Maximum
 * @parent Mechanics
 * @type Number
 * @desc What's the maximum amount of Job Points an actor can have?
 * Use 0 for unlimited Job Points.
 * @default 0
 *
 * @param Visual
 *
 * @param ShowInMenus:eval
 * @text Show In Menus?
 * @parent Visual
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Do you wish to show Job Points in menus that allow them?
 * @default true
 *
 * @param Icon:num
 * @text Icon
 * @parent Visual
 * @desc What is the icon you want to use to represent Job Points?
 * @default 188
 *
 * @param Vocabulary
 *
 * @param FullText:str
 * @text Full Text
 * @parent Vocabulary
 * @desc The full text of how Job Points appears in-game.
 * @default Job Points
 *
 * @param AbbrText:str
 * @text Abbreviated Text
 * @parent Vocabulary
 * @desc The abbreviation of how Job Points appears in-game.
 * @default JP
 *
 * @param TextFmt:str
 * @text Menu Text Format
 * @parent Vocabulary
 * @desc What is the text format for it to be displayed in windows.
 * %1 - Value, %2 - Abbr, %3 - Icon, %4 - Full Text
 * @default %1 \c[5]%2\c[0]%3
 *
 * @param Gain
 *
 * @param PerAction:str
 * @text Per Action Hit
 * @parent Gain
 * @desc How many Job Points should an actor gain per action?
 * You may use code.
 * @default 10 + Math.randomInt(10)
 *
 * @param PerLevelUp:str
 * @text Per Level Up
 * @parent Gain
 * @desc How many Job Points should an actor gain per level up?
 * You may use code.
 * @default 0
 *
 * @param PerEnemy:str
 * @text Per Enemy Defeated
 * @parent Gain
 * @desc How many Job Points should an actor gain per enemy?
 * You may use code.
 * @default 50 + Math.randomInt(50)
 *
 * @param AliveActors:eval
 * @text Alive Actors?
 * @parent PerEnemy:str
 * @type boolean
 * @on Alive Requirement
 * @off No Requirement
 * @desc Do actors have to be alive to receive Job Points from
 * defeated enemies?
 * @default true
 *
 * @param Victory
 *
 * @param ShowVictory:eval
 * @text Show During Victory?
 * @parent Victory
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Show how much JP an actor has earned in battle during the
 * victory phase?
 * @default true
 *
 * @param VictoryText:str
 * @text Victory Text
 * @parent Victory
 * @desc For no Victory Aftermath, this is the text that appears.
 * %1 - Actor, %2 - Earned, %3 - Abbr, %4 - Full Text
 * @default %1 gains %2 %3!
 *
 * @param AftermathActorDisplay:eval
 * @text Aftermath Display?
 * @parent Victory
 * @type boolean
 * @on Show
 * @off Hide
 * @desc Requires VisuMZ_3_VictoryAftermath. Show Job Points as
 * the main acquired resource in the actor windows?
 * @default true
 *
 * @param AftermathText:str
 * @text Aftermath Text
 * @parent Victory
 * @desc For no Victory Aftermath, this is the text that appears.
 * %1 - Earned, %2 - Abbr, %3 - Full Text
 * @default +%1 %2
 *
 */
//=============================================================================

const _0x3478=['description','AttackStates','expGaugeColor2','OvaEY','ParseClassNotetags','AxdfS','vFnza','isEnabled','learnings','_priorityFaceIndex','ParamRates','XjUCl','needsPageButtons','ClassMenuPortrait','iyaGC','_priorityCharacterName','StartClassClassPoints','General','StartingMulticlasses','getActorClassCharacterName','GOsWn','findMulticlassTier','isBattleMember','HIjDF','addClassPoints','allMembers','ZLlgg','UnassignHelpDescription','expForClassLevel','classChange','item','aRPMl','getActorClassMenuPortrait','createCommandWindow','_actor','iconIndex','commandPersonal','Game_Actor_faceName','MulticlassChangeActorClass','SParamRates','FYwSb','multiclassId','_backSprite1','addClassChangeSystemCommandAutomatically','Classes','tlqum','\x5cI[%1]','removeClassChangeTierRestriction','JobPointsSet','concat','EmsmU','VFKeo','jDQHy','WBSuT','STR','ClassID','mhp','Scene_Menu_createCommandWindow','map','709ZYpeVm','Window_ClassTier_ExtraJS','DTCVC','EnemyClassPoints','_classChangeTierRestrictions','from','RIXcI','Game_BattlerBase_isEquipAtypeOk','index','HelpDescription','Game_BattlerBase_attackElements','drawItemActorMenuImage','isMainMenuClassChangeSystemEnabled','levelUpGainJobPoints','FRMxS','paramBaseForClass','_helpWindow','drawItemActorSprite','ARfOw','idpNa','lineHeight','toUpperCase','addClassChangeSystemCommand','initClassLevels','updatedLayoutStyle','shown','AddedStypes','JobPointsRate','clamp','JcsWB','Icon','MQSGg','floor','setStatusWindow','createAnimationDummySprite','ClassUnlockRemoveGlobal','TextFmt','rEBYp','characterIndex','(%1)','length','ipIga','Settings','characterName','PrSdj','DrawPortraitJS','Actor-%1-%2','ARRAYFUNC','ConfirmAniPrimaryOffsetX','TCUES','_backSprite2','Game_Actor_setCharacterImage','tbdxL','ClassUnlockForGlobal','classChange_multiclass_ShiftHelp','paramchangeTextColor','actorClassCharacterIndex','highestTier','isUseParamNamesWithIcons','qqjNa','getActorClassBattlePortrait','drawExtraContents','AxemT','isActor','_updateClassLearnedSkills','DrawFaceJS','Window_ClassStatus_RectJS','textColor','initMulticlass','Game_BattlerBase_elementRate','579rprsSG','yLRPj','nlgvf','push','ClassChangeSystem','setCharacterImage','ClassChangeAnimation','hYOZB','expParams','drawActorFace','DrawParamJS','NtJfn','isRightInputMode','YUuXL','loseClassPoints','xXhTD','faceName','return\x200','pow','actorClassFaceIndex','centerSprite','StateResistance','AliveActors','36908yCJjmu','uiMenuStyle','ngrGq','Game_BattlerBase_attackStates','gainClassPointsForMulticlasses','drawItemDarkRect','isMainMenuCoreMenuImageOptionAvailable','Points','gainClassPoints','ARRAYEVAL','ParseActorNotetags','CLASS_CHANGE_ADJUST_HP_MP','(+%1)','SkillLearnSystem','learnSkill','%1\x20is\x20missing\x20a\x20required\x20plugin.\x0aPlease\x20install\x20%2\x20into\x20the\x20Plugin\x20Manager.','ConfirmAnimationID','jobPointsFull','pagedown','setBattlerImage','refresh','registerCommand','exAhg','Game_Actor_faceIndex','dweuc','bitmap','VisuMZ_1_MessageCore','VisuMZ_2_ClassChangeSystem','dimColor2','displayRewardsClassPoints','format','Item-%1-%2','KLevo','CoreEngine','kmhdV','UIgel','WmDhH','_classLevel','drawParameterList','125735FgImff','isPlaying','getJobPoints','cancel','name','min','applyMulticlassObjects','_classListWindow','BwSrv','processCursorMove','removeUnlockedClass','getInputButtonString','Limit','Game_BattlerBase_isEquipWtypeOk','createClassChangeAnimation','includes','Game_Actor_setup','displayRewards','battleMembers','resourceRate','updateClassLevel','buttonAssistOffset3','bDvxF','_targets','jVocY','xparam','ClassChangeAddRestrictTier','BgFilename1','Game_BattlerBase_stateRate','ClassUnlockForActor','prototype','getMulticlasses','Job','pageup','buttonAssistSlotWindowShift','NTUxj','Class-%1-%2','tIhhJ','dimColor1','FSIeW','JRLIH','setup','padding','RegExp','exp','drawUpdatedBeforeParamValue','Parse_Notetags_Basic','level','Window_MenuCommand_addOriginalCommands','drawActorResources','addJobPoints','1985234TgXpGP','createStatusWindow','boxWidth','parameters','WLBSj','_scene','ChangeAdjusHpMp','forceSelect','fTFge','_earnedClassPoints','EnemyJobPoints','inBattle','newPage','Game_System_initialize','getClassChangeAnimationID','ShiftShortcutKey','loadPicture','classChange_multiclass_remove_help','gainMulticlassRewardPoints','Game_Actor_getBattlePortraitFilename','createCustomBackgroundImages','call','Game_BattlerBase_xparam','svbattler','optExtraExp','uwihl','Name','12802VjeDWL','AHMvN','status','Game_Party_initialize','nextClassLevelExp','weaponTypes','_list','AttackElements','updateStatusWindow','changeTextColor','makeRewards','actor','shift','ABasJ','DOIxO','Game_Actor_tradeItemWithParty','iconWidth','actorClassBattlePortrait','qEQEZ','deadMembers','elementRate','dDoOD','clearParamPlus','_rewards','actorClassFaceName','onPersonalOk','setHp','DgtLH','add','isClassChangeCommandVisible','debuffRate','IconSet','attackStates','armorTypes','drawActorAbilityPoints','pFFtS','initClassChangeUnlocks','paramRate','playOkSound','okFBn','unlockClass','ClassPointsRate','_priorityBattlerName','drawBigItemIcon','drawClassLevel','processShiftRemoveShortcut','getMenuImage','isShiftRemoveShortcutEnabled','test','MyfQW','xfuZv','CeWOr','isRecommendedLayout','addLoadListener','Window_ClassList_BgType','join','initClassChangeSystemMainMenu','ClassPointsAdd','checkMulticlasses','BgSettings','RTLbY','uHoBx','Window_ClassStatus_BgType','classPointsFmt','isBottomHelpMode','changeExp','classChangeMenuCommand','Game_Actor_setBattlerImage','getClassPoints','maxLvGaugeColor1','changePaintOpacity','MainMenu','removeChild','DisplayedResources','EnableMainMenu','resetFontSettings','Skill-%1-%2','getActorClassFaceIndex','actorClassBattlerName','ConvertParams','VisuMZ_1_MainMenuCore','Game_Actor_levelUp','expGaugeColor1','statusWindowRect','xJFCd','deactivate','gain%1Points','setHandler','playBuzzerSound','_priorityFaceName','STRUCT','Game_Actor_traitObjects','RestrictClassChangeTier','currentSymbol','battlerName','addMulticlassTiers','earnedJobPoints','VisuMZ_0_CoreEngine','QqHfI','JobPointsAdd','Game_BattlerBase_addedSkillTypes','gradientFillRect','naturalUnlockClass','UGEpF','ARRAYSTRUCT','UUlVi','height','FjLcF','subject','VisuMZ_2_SkillLearnSystem','onClassListOk','LsOOv','UKLYi','onljE','JobPointsGain','uAjMn','Param','active','AlwaysUnlocked','azhJG','mainAreaHeight','jobPoints','drawItemActorSvBattler','maxLvGaugeColor2','TextColor','AOOpg','multiclass','MenuPortraits','jTmlm','classPointsFull','TRAIT_STYPE_ADD','onMulticlassOk','_tier','Window_ClassList_RectJS','Game_Battler_onBattleStart','StartingClassPoints','enemy','drawBigItemImage','process_VisuMZ_ClassChangeSystem_Notetags','indexOf','jobPointsRate','nextActor','xnHBE','nOjFM','setMainMenuClassChangeSystemVisible','right','paintOpacity','gLQCI','checkShiftRemoveShortcut','StartingClassTier','bigPicture','JZrWQ','isClassAutoUnlockRequirementsMet','Enemy-%1-%2','buttonAssistText3','isEquipAtypeOk','currentExt','%1\x20is\x20incorrectly\x20placed\x20on\x20the\x20plugin\x20list.\x0aIt\x20is\x20a\x20Tier\x20%2\x20plugin\x20placed\x20over\x20other\x20Tier\x20%3\x20plugins.\x0aPlease\x20reorder\x20the\x20plugin\x20list\x20from\x20smallest\x20to\x20largest\x20tier\x20numbers.','teDra','startClassChangeAnimation','GjNzU','skill','ExtDisplayedParams','vCvKF','JobPointsLose','ConfirmAniSubclassOffsetY','stateResistSet','classAdjustHpMp','forceRemoveClassChangeAnimations','setText','Game_BattlerBase_addedSkills','btoZu','setMp','bind','_animations','classLevel','getBackgroundOpacity','releaseUnequippableItems','Japia','StartClassJobPoints','AKNkV','show','ChangeClassSound','SharedResource','version','smoothSelect','applyJobPoints','initClassChangeSystem','sctls','makeRewardsJobPoints','BattleManager_makeRewards','TargetGainJobPoints','registerActorClassImages','levelUpGainSkillPoints','getSkillPoints','attackStatesRate','frames','drawActorJobPoints','<WordWrap>','maintainLevels','_classId','jRzpT','UkJnS','isAlive','getBattlePortraitFilename','ClassBattlePortrait','SnDMx','visibleResources','drawUpdatedAfterParamValue','NUM','DSQNN','ClassBattlerName','Game_Actor_releaseUnequippableItems','_priorityMenuImage','classTierWindowRect','createJS','mvIVr','Game_Action_applyItemUserEffect','MaintainLevels','trim','isUseSkillsStatesCoreUpdatedLayout','onDatabaseLoaded','paramRate%1','isSceneBattle','prepareRefreshItemsEquipsCoreLayout','_classIDs','drawUpdatedParamName','traits','gainStartingClassPoints','SkFEA','applyClassPoints','Game_BattlerBase_attackStatesRate','ajtfv','QGspp','EIKST','drawText','classListWindowRect','maxLevel','AbbrText','QrCzS','Actors','Window_ClassTier_RectJS','itemRectWithPadding','jobPointsFmt','mpRate','ARRAYNUM','popScene','pop','onMenuImageLoad','adjustSprite','gainJobPointsForMulticlasses','onClassListCancel','tradeItemWithParty','_cache','getClassIdWithName','ynyTo','sUWfq','azGak','txxlZ','JSON','MaxResource','cDCBR','jobPointsIcon','_context','levelUpGainClassPoints','levelA','drawRightArrow','weapon','ClassUnlockRemoveActor','EOWpI','update','classPointsAbbr','JobPoints','%1\x27s\x20version\x20does\x20not\x20match\x20plugin\x27s.\x20Please\x20update\x20it\x20in\x20the\x20Plugin\x20Manager.','create','earnedClassPoints','actorParams','DrawBackRect','Game_Actor_characterIndex','mmp','gainStartingJobPoints','hjpZx','addClassChangeTierRestriction','split','NPuqQ','setMulticlassTiers','Scene_Boot_onDatabaseLoaded','BQYNr','AutoUnlockRequirements','ElementRates','classPointsIcon','_priorityCharacterIndex','jobPointsAbbr','highestMulticlassTier','IRqwq','setMainMenuClassChangeSystemEnabled','hpRate','replace','classExpRate','setActor','sFgPI','onBattleStart','setFaceImage','GHXhl','_unlockedClasses','Tiers','Mcift','SkillPoints','isEquipWtypeOk','_priorityBattlePortrait','log','SystemEnableClassChangeSystemMenu','_wordWrap','itemPadding','amOVm','changeMulticlass','actorId','addWindow','paramValueByName','scaleSprite','EquipWeapons','Game_Actor_battlerName','setBackgroundOpacity','setJobPoints','itemRect','_ClassChangeSystem_MainMenu','activate','ShowMainMenu','isMainMenuClassChangeSystemVisible','#%1','MoCao','UserGainClassPoints','callUpdateHelp','actorClassMenuPortrait','122bmXPWJ','setTier','expRate','ClassCharaName','getClassChangeBackColor2','applyItemUserEffect','setHelpWindow','finalExpRate','exit','loseJobPoints','_multiclassCheck','multiclasses','SMaGh','createBackground','armor','isMVAnimation','gainExp','Window','drawParamText','faceIndex','Game_BattlerBase_stateResistSet','SfIrI','Weapon-%1-%2','iFNDg','drawItem','Actor-%1-Class-%2','initClassChangeRestrictions','playClassChange','drawClassPoints','drawTextEx','classIcon','State-%1-%2','itofq','updateHelp','setClassPoints','_highestTier','drawJobPoints','IYvMP','itemHeight','isClassExpGaugeDrawn','Game_Actor_getMenuImage','HmNlr','BattleManager_gainExp','_classTierWindow','displayRewardsJobPoints','applyClassChangeSystemUserEffect','HDjrw','classPointsTotal','_exp','setMenuImage','_multiclasses','setupClassChangeSystem','Class','isClassChangeCommandEnabled','classLevelUp','previousActor','BattleManager_displayRewards','actorClassCharacterName','hideAdditionalSprites','FWNrl','match','paramValueFontSize','dNAgS','addedSkills','isClassChangeTierRestricted','TRAIT_EQUIP_WTYPE','resetTextColor','ndhJJ','param','ShiftButtonAssistText','ConfirmAniPrimaryOffsetY','classPoints','gainMulticlassExp','1605203Tsrsyt','MYvAI','drawActorClassPoints','mainAreaTop','hide','getActorUnlockedClasses','blt','BgFilename2','unMfM','getColor','params','colSpacing','initJobPoints','PerAction','stateRate','Scene_Menu_onPersonalOk','Game_Actor_changeClass','FullText','ARRAYSTR','EibWf','gainRewardsClassPoints','uiInputPosition','filter','endBattle','LvExpGauge','jobPointsTotal','code','sImOQ','StateRates','ClassPoints','YXMKO','drawPicture','pfdTO','currentClassLevelExp','Game_BattlerBase_paramRate','VictoryText','totalMulticlass','KScOI','TrPQB','PerEnemy','NalgS','618247OnOkIK','getUnlockedClasses','refreshNoMenuImage','parse','PeRuy','addOriginalCommands','jobPointsVisible','Game_BattlerBase_sparam','QfsCs','imageSmoothingEnabled','classPicture','refreshActorPortrait','setTempActor','addedSkillTypes','classPointsVisible','_windowLayer','Game_Actor_characterName','ext','loseMulticlassTiers','\x5cC[16]%1:\x5cC[0]\x20\x5cFS[%3]%2','DrawIcons','Armor-%1-%2','max','hewmn','LayoutStyle','currentClass','Show','initialize','levelUp','makeCommandList','DdWMu','applyItemClassChangeSystemUserEffect','getAbilityPoints','initClassPoints','ActorUnlockedClasses','Multiclass','gainRewardsJobPoints','setBackgroundType','JgvIb','4mfwJnp','recoverAll','changeClass','BattleManager_endBattle','sparam','ClassPointsSet','width','getActorClassFaceName','center','bdACh','Game_Actor_paramBase','createClassTierWindow','_classPoints','EnableLayout','isWordWrapEnabled','pvAPG','loadTitle2','CMGuD','makeDeepCopy','isLearnedSkill','wYjOx','left','DSWFn','rightArrowWidth','Tier','getMulticlassAtTier','createClassListWindow','ClassFaceName','createKeyJS','addCommand','updateClassLearnedSkills','_earnedJobPoints','changeClassExp','getActorClassBattlerName','systemColor','CiXlv','_jobPoints','_multiclassTiers','Game_Actor_setBattlePortrait','checkForAutoClassUnlocks','_buttonAssistWindow','gainJobPoints','canShiftRemoveClass','drawClassResources','getClassChangeTiersOnly','apply','\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20//\x20Declare\x20Variables\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20const\x20user\x20=\x20arguments[0];\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20const\x20target\x20=\x20arguments[1];\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20let\x20value\x20=\x200;\x0a\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20//\x20Process\x20Code\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20try\x20{\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20%1\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20}\x20catch\x20(e)\x20{\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20if\x20($gameTemp.isPlaytest())\x20console.log(e);\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20}\x0a\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20//\x20Return\x20Value\x0a\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20return\x20value;\x0a\x20\x20\x20\x20\x20\x20\x20\x20','BackRectColor','dataId','_commandWindow','EquipArmors','makeRewardsClassPoints','refreshActor','PerLevelUp','drawClassExpGauge','Enable','classPointsRate','isPlaytest','sort','OfScJ','ParseAllNotetags','textSizeEx','Game_BattlerBase_debuffRate','LnGHC','ShowVictory','loadSystem','_statusWindow','round','attackElements','remove','MAX_SAFE_INTEGER','szvCC','determineActiveWindow','constructor','graphicType','traitObjects','createHelpWindow','dYMQT','gqxgu','_tempActor','buttonAssistKey3','Window_ClassTier_BgType','_ClassChangeSystem_preventLevelUpGain','enabled','SjufF','UWEfJ','members','ParamArrow','MulticlassLowerLimit','classChange_multiclass_noClass','reduce','arePageButtonsEnabled','drawFadedItemBackground','contents','tier','playStaticSe','classChange_multiclass_remove','process_VisuMZ_ClassChangeSystem','note','addChild','onActorChange','aKCmb'];const _0x45bccd=_0x2d2d;(function(_0x41bd3d,_0xb0797b){const _0x2f3ecc=_0x2d2d;while(!![]){try{const _0x5a9a44=-parseInt(_0x2f3ecc(0x29a))+-parseInt(_0x2f3ecc(0x3e6))+-parseInt(_0x2f3ecc(0x419))+-parseInt(_0x2f3ecc(0x434))*-parseInt(_0x2f3ecc(0x228))+-parseInt(_0x2f3ecc(0x2c1))*-parseInt(_0x2f3ecc(0x3bf))+parseInt(_0x2f3ecc(0x271))+parseInt(_0x2f3ecc(0x3a8))*parseInt(_0x2f3ecc(0x362));if(_0x5a9a44===_0xb0797b)break;else _0x41bd3d['push'](_0x41bd3d['shift']());}catch(_0x1ada2a){_0x41bd3d['push'](_0x41bd3d['shift']());}}}(_0x3478,0xf3286));var label=_0x45bccd(0x3ac),tier=tier||0x0,dependencies=[],pluginData=$plugins[_0x45bccd(0x287)](function(_0x1e2f4a){const _0x25b85=_0x45bccd;return _0x1e2f4a[_0x25b85(0x436)]&&_0x1e2f4a[_0x25b85(0x327)][_0x25b85(0x3f5)]('['+label+']');})[0x0];VisuMZ[label][_0x45bccd(0x38c)]=VisuMZ[label][_0x45bccd(0x38c)]||{},VisuMZ[_0x45bccd(0x483)]=function(_0x925aab,_0x496598){const _0x47421a=_0x45bccd;for(const _0x8ac0ad in _0x496598){if(_0x47421a(0x354)!==_0x47421a(0x4ab)){if(_0x8ac0ad['match'](/(.*):(.*)/i)){if(_0x47421a(0x3d7)===_0x47421a(0x3d7)){const _0x58e4c9=String(RegExp['$1']),_0x49f581=String(RegExp['$2'])[_0x47421a(0x377)]()[_0x47421a(0x1b5)]();let _0xb5894f,_0x5a7988,_0x37e3d9;switch(_0x49f581){case _0x47421a(0x1ab):_0xb5894f=_0x496598[_0x8ac0ad]!==''?Number(_0x496598[_0x8ac0ad]):0x0;break;case _0x47421a(0x1cf):_0x5a7988=_0x496598[_0x8ac0ad]!==''?JSON[_0x47421a(0x29d)](_0x496598[_0x8ac0ad]):[],_0xb5894f=_0x5a7988[_0x47421a(0x361)](_0xcbe0f0=>Number(_0xcbe0f0));break;case'EVAL':_0xb5894f=_0x496598[_0x8ac0ad]!==''?eval(_0x496598[_0x8ac0ad]):null;break;case _0x47421a(0x3c8):_0x5a7988=_0x496598[_0x8ac0ad]!==''?JSON[_0x47421a(0x29d)](_0x496598[_0x8ac0ad]):[],_0xb5894f=_0x5a7988[_0x47421a(0x361)](_0x3d8104=>eval(_0x3d8104));break;case _0x47421a(0x1dd):_0xb5894f=_0x496598[_0x8ac0ad]!==''?JSON[_0x47421a(0x29d)](_0x496598[_0x8ac0ad]):'';break;case'ARRAYJSON':_0x5a7988=_0x496598[_0x8ac0ad]!==''?JSON[_0x47421a(0x29d)](_0x496598[_0x8ac0ad]):[],_0xb5894f=_0x5a7988['map'](_0x5ef41d=>JSON[_0x47421a(0x29d)](_0x5ef41d));break;case'FUNC':_0xb5894f=_0x496598[_0x8ac0ad]!==''?new Function(JSON[_0x47421a(0x29d)](_0x496598[_0x8ac0ad])):new Function(_0x47421a(0x3b9));break;case _0x47421a(0x391):_0x5a7988=_0x496598[_0x8ac0ad]!==''?JSON['parse'](_0x496598[_0x8ac0ad]):[],_0xb5894f=_0x5a7988['map'](_0x3a2532=>new Function(JSON[_0x47421a(0x29d)](_0x3a2532)));break;case _0x47421a(0x35d):_0xb5894f=_0x496598[_0x8ac0ad]!==''?String(_0x496598[_0x8ac0ad]):'';break;case _0x47421a(0x283):_0x5a7988=_0x496598[_0x8ac0ad]!==''?JSON['parse'](_0x496598[_0x8ac0ad]):[],_0xb5894f=_0x5a7988[_0x47421a(0x361)](_0x344438=>String(_0x344438));break;case _0x47421a(0x48e):_0x37e3d9=_0x496598[_0x8ac0ad]!==''?JSON[_0x47421a(0x29d)](_0x496598[_0x8ac0ad]):{},_0xb5894f=VisuMZ[_0x47421a(0x483)]({},_0x37e3d9);break;case _0x47421a(0x49c):_0x5a7988=_0x496598[_0x8ac0ad]!==''?JSON[_0x47421a(0x29d)](_0x496598[_0x8ac0ad]):[],_0xb5894f=_0x5a7988['map'](_0x38d847=>VisuMZ[_0x47421a(0x483)]({},JSON[_0x47421a(0x29d)](_0x38d847)));break;default:continue;}_0x925aab[_0x58e4c9]=_0xb5894f;}else{function _0x4eea6b(){const _0x3df756=_0x47421a;this[_0x3df756(0x458)](),this['initMulticlass'](),this[_0x3df756(0x379)](),this[_0x3df756(0x242)](),this[_0x3df756(0x2df)](),this[_0x3df756(0x3d3)](),this[_0x3df756(0x44a)](),this[_0x3df756(0x2c2)]();}}}}else{function _0x176c50(){const _0x4aba67=_0x47421a;this[_0x4aba67(0x2b5)](...arguments);}}}return _0x925aab;},(_0x151ca2=>{const _0x19125f=_0x45bccd,_0xfe859=_0x151ca2[_0x19125f(0x3ea)];for(const _0x516f64 of dependencies){if(_0x19125f(0x30f)!==_0x19125f(0x30f)){function _0x392062(){const _0xee20dd=_0x19125f,_0x3ff128=this[_0xee20dd(0x213)]();let _0x466252=0x0;_0x2d4fa3['VisuMZ_0_CoreEngine']?_0x466252=this[_0xee20dd(0x349)]['paramValueByName'](_0x4d33b4,!![]):_0x466252=this[_0xee20dd(0x349)][_0xee20dd(0x26c)](_0x3c6e92);const _0x3d3a5e=_0x466252;this[_0xee20dd(0x1c5)](_0x466252,_0x2ea824,_0x551de8,_0x5e07c0-_0x3ff128,_0xee20dd(0x16b)),this[_0xee20dd(0x26a)]();}}else{if(!Imported[_0x516f64]){alert(_0x19125f(0x3ce)[_0x19125f(0x3dd)](_0xfe859,_0x516f64)),SceneManager['exit']();break;}}}const _0x6a0e5=_0x151ca2[_0x19125f(0x327)];if(_0x6a0e5[_0x19125f(0x264)](/\[Version[ ](.*?)\]/i)){const _0xfacb5f=Number(RegExp['$1']);_0xfacb5f!==VisuMZ[label][_0x19125f(0x192)]&&(alert(_0x19125f(0x1eb)[_0x19125f(0x3dd)](_0xfe859,_0xfacb5f)),SceneManager[_0x19125f(0x230)]());}if(_0x6a0e5[_0x19125f(0x264)](/\[Tier[ ](\d+)\]/i)){const _0x41037f=Number(RegExp['$1']);_0x41037f<tier?(alert(_0x19125f(0x177)[_0x19125f(0x3dd)](_0xfe859,_0x41037f,tier)),SceneManager['exit']()):tier=Math['max'](_0x41037f,tier);}VisuMZ[_0x19125f(0x483)](VisuMZ[label][_0x19125f(0x38c)],_0x151ca2[_0x19125f(0x41c)]);})(pluginData),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x403),_0xe078a8=>{const _0x353502=_0x45bccd;VisuMZ[_0x353502(0x483)](_0xe078a8,_0xe078a8);const _0x8aa50d=_0xe078a8[_0x353502(0x1ca)][_0x353502(0x361)](_0x44af5e=>$gameActors[_0x353502(0x43f)](_0x44af5e)),_0x47f0d2=_0xe078a8['Classes'];for(const _0x476d51 of _0x8aa50d){if(_0x353502(0x1a3)===_0x353502(0x1a3)){if(!_0x476d51)continue;for(const _0x4f7c45 of _0x47f0d2){_0x476d51[_0x353502(0x45c)](_0x4f7c45);}}else{function _0x15fcba(){const _0x21f5c0=_0x353502;return this[_0x21f5c0(0x343)](_0xa5b2d8,_0x56c8b3+0x1);}}}}),PluginManager[_0x45bccd(0x3d4)](pluginData['name'],_0x45bccd(0x397),_0x5ae522=>{const _0x8cf689=_0x45bccd;VisuMZ[_0x8cf689(0x483)](_0x5ae522,_0x5ae522);const _0xf12a46=_0x5ae522[_0x8cf689(0x353)];for(const _0x3c2e27 of _0xf12a46){$gameParty['unlockClass'](_0x3c2e27);}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x1e6),_0x3a6241=>{const _0x7f3cc8=_0x45bccd;VisuMZ[_0x7f3cc8(0x483)](_0x3a6241,_0x3a6241);const _0x5600cb=_0x3a6241[_0x7f3cc8(0x1ca)][_0x7f3cc8(0x361)](_0x5717f3=>$gameActors[_0x7f3cc8(0x43f)](_0x5717f3)),_0x241e8d=_0x3a6241[_0x7f3cc8(0x353)];for(const _0x1104e7 of _0x5600cb){if('uDUxV'==='uDUxV'){if(!_0x1104e7)continue;for(const _0x1feaf5 of _0x241e8d){_0x1104e7[_0x7f3cc8(0x3f0)](_0x1feaf5);}}else{function _0x3ea083(){const _0x3ff27f=_0x7f3cc8;if(this[_0x3ff27f(0x20a)]===_0x58bd02)this['initClassChangeUnlocks']();return this[_0x3ff27f(0x20a)];}}}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x385),_0x166247=>{const _0x4624f1=_0x45bccd;VisuMZ[_0x4624f1(0x483)](_0x166247,_0x166247);const _0x295c28=_0x166247['Classes'];for(const _0x2eae76 of _0x295c28){if(_0x4624f1(0x1b2)!==_0x4624f1(0x1b2)){function _0x57c216(){const _0x269671=_0x4624f1;return!!_0x4f98fa[_0x269671(0x19e)];}}else $gameParty[_0x4624f1(0x3f0)](_0x2eae76);}}),PluginManager['registerCommand'](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x400),_0x254dd0=>{const _0x23e94c=_0x45bccd;VisuMZ[_0x23e94c(0x483)](_0x254dd0,_0x254dd0);const _0xc27614=_0x254dd0[_0x23e94c(0x1ca)]['map'](_0x1e2243=>$gameActors[_0x23e94c(0x43f)](_0x1e2243)),_0xfd96e7=_0x254dd0[_0x23e94c(0x20b)];for(const _0x353c5d of _0xc27614){if(!_0x353c5d)continue;for(const _0x1969f8 of _0xfd96e7){if(_0x23e94c(0x17d)==='hhNBf'){function _0x2c2747(){const _0x5143cf=_0x23e94c;_0x2f1e8d[_0x5143cf(0x3ac)]['Settings'][_0x5143cf(0x239)]['DrawPortraitJS'][_0x5143cf(0x42e)](this),this[_0x5143cf(0x3e5)]();}}else _0x353c5d['addClassChangeTierRestriction'](_0x1969f8);}}}),PluginManager['registerCommand'](pluginData['name'],'ClassChangeRemoveRestrictTier',_0x42f686=>{const _0x13f2fb=_0x45bccd;VisuMZ[_0x13f2fb(0x483)](_0x42f686,_0x42f686);const _0x2c5b99=_0x42f686[_0x13f2fb(0x1ca)][_0x13f2fb(0x361)](_0x3a9212=>$gameActors['actor'](_0x3a9212)),_0x433505=_0x42f686[_0x13f2fb(0x20b)];for(const _0x246cb9 of _0x2c5b99){if('tbdxL'!==_0x13f2fb(0x396)){function _0x2c1dee(){const _0x36329d=_0x13f2fb;_0x3f2a82[_0x36329d(0x483)](_0x5e5ace,_0x4a3ba7);const _0x10383a=_0x2ae76c[_0x36329d(0x353)];for(const _0x165e34 of _0x10383a){_0x39e8fc[_0x36329d(0x45c)](_0x165e34);}}}else{if(!_0x246cb9)continue;for(const _0x1fea3c of _0x433505){if(_0x13f2fb(0x341)===_0x13f2fb(0x196)){function _0x4f1e39(){const _0xa7d845=_0x13f2fb;_0xaf9b6[_0xa7d845(0x215)](0x0,this[_0xa7d845(0x4b8)]);}}else _0x246cb9[_0x13f2fb(0x356)](_0x1fea3c);}}}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x34d),_0x1438a6=>{const _0x25d46f=_0x45bccd;VisuMZ[_0x25d46f(0x483)](_0x1438a6,_0x1438a6);const _0x12abe8=_0x1438a6[_0x25d46f(0x1ca)][_0x25d46f(0x361)](_0x57e8c0=>$gameActors[_0x25d46f(0x43f)](_0x57e8c0)),_0x4d35c7=_0x1438a6[_0x25d46f(0x2d9)],_0x35bf75=_0x1438a6[_0x25d46f(0x35e)];for(const _0x410287 of _0x12abe8){if(!_0x410287)continue;_0x410287[_0x25d46f(0x215)](_0x35bf75,_0x4d35c7);}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],'MulticlassRaiseLimit',_0x429781=>{const _0x5159c2=_0x45bccd;VisuMZ['ConvertParams'](_0x429781,_0x429781);const _0x297532=_0x429781['Actors'][_0x5159c2(0x361)](_0x2d4c5f=>$gameActors[_0x5159c2(0x43f)](_0x2d4c5f)),_0x5f45f3=_0x429781[_0x5159c2(0x3f2)];for(const _0x4155d5 of _0x297532){if(!_0x4155d5)continue;_0x4155d5[_0x5159c2(0x493)](_0x5f45f3);}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x319),_0x3cf5ea=>{const _0x249269=_0x45bccd;VisuMZ['ConvertParams'](_0x3cf5ea,_0x3cf5ea);const _0x42d186=_0x3cf5ea[_0x249269(0x1ca)][_0x249269(0x361)](_0x35407f=>$gameActors[_0x249269(0x43f)](_0x35407f)),_0x2846d1=_0x3cf5ea[_0x249269(0x3f2)];for(const _0xef1b92 of _0x42d186){if('LnGHC'===_0x249269(0x300)){if(!_0xef1b92)continue;_0xef1b92['loseMulticlassTiers'](_0x2846d1);}else{function _0x27d67a(){const _0x3e0b75=_0xce8b0d(_0x5e1f12['$1']);this['subject']()['gainClassPoints'](_0x3e0b75);}}}}),PluginManager[_0x45bccd(0x3d4)](pluginData['name'],'MulticlassSetLimit',_0x469cca=>{const _0x19a47c=_0x45bccd;VisuMZ['ConvertParams'](_0x469cca,_0x469cca);const _0x2728fd=_0x469cca['Actors'][_0x19a47c(0x361)](_0x51a5af=>$gameActors[_0x19a47c(0x43f)](_0x51a5af)),_0x359dd1=_0x469cca[_0x19a47c(0x3f2)];for(const _0x1c7cfd of _0x2728fd){if(!_0x1c7cfd)continue;_0x1c7cfd[_0x19a47c(0x1f7)](_0x359dd1);}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],'ClassPointsGain',_0x165f0d=>{const _0x462818=_0x45bccd;VisuMZ[_0x462818(0x483)](_0x165f0d,_0x165f0d);const _0x582a45=_0x165f0d[_0x462818(0x1ca)]['map'](_0x588c72=>$gameActors['actor'](_0x588c72)),_0x48d4ac=_0x165f0d[_0x462818(0x353)],_0x4aa4f9=_0x165f0d[_0x462818(0x3c6)];for(const _0x55c997 of _0x582a45){if(!_0x55c997)continue;for(const _0x1b50b9 of _0x48d4ac){_0x55c997[_0x462818(0x3c7)](_0x4aa4f9,_0x1b50b9);}}}),PluginManager[_0x45bccd(0x3d4)](pluginData['name'],_0x45bccd(0x46d),_0x5efc92=>{const _0x571818=_0x45bccd;VisuMZ['ConvertParams'](_0x5efc92,_0x5efc92);const _0x336b22=_0x5efc92['Actors'][_0x571818(0x361)](_0x3a010e=>$gameActors['actor'](_0x3a010e)),_0x471b7d=_0x5efc92['Classes'],_0x179097=_0x5efc92[_0x571818(0x3c6)];for(const _0x4023db of _0x336b22){if(!_0x4023db)continue;for(const _0x2e587c of _0x471b7d){_0x4023db[_0x571818(0x33f)](_0x179097,_0x2e587c);}}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],'ClassPointsLose',_0xbb3683=>{const _0x51fc53=_0x45bccd;VisuMZ[_0x51fc53(0x483)](_0xbb3683,_0xbb3683);const _0x4c076f=_0xbb3683[_0x51fc53(0x1ca)]['map'](_0x591f3a=>$gameActors[_0x51fc53(0x43f)](_0x591f3a)),_0x21a6c9=_0xbb3683[_0x51fc53(0x353)],_0x53fd63=_0xbb3683[_0x51fc53(0x3c6)];for(const _0x2d6063 of _0x4c076f){if(!_0x2d6063)continue;for(const _0x15238d of _0x21a6c9){if('FjLcF'!==_0x51fc53(0x49f)){function _0x3c7e3c(){const _0x2e9fd9=_0x51fc53;this[_0x2e9fd9(0x29c)]();}}else _0x2d6063[_0x51fc53(0x3b6)](_0x53fd63,_0x15238d);}}}),PluginManager['registerCommand'](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x2c6),_0x2817bc=>{const _0x36d348=_0x45bccd;VisuMZ[_0x36d348(0x483)](_0x2817bc,_0x2817bc);const _0x45c7e8=_0x2817bc[_0x36d348(0x1ca)]['map'](_0x2b9394=>$gameActors[_0x36d348(0x43f)](_0x2b9394)),_0x4b54a3=_0x2817bc['Classes'],_0x335ad6=_0x2817bc[_0x36d348(0x3c6)];for(const _0x2c41d4 of _0x45c7e8){if(!_0x2c41d4)continue;for(const _0x1ea0ee of _0x4b54a3){if(_0x36d348(0x3df)===_0x36d348(0x28c)){function _0x723018(){const _0x5e4752=_0x36d348;this[_0x5e4752(0x336)]=_0x54a151,this[_0x5e4752(0x1fd)]=_0x1540a6;}}else _0x2c41d4['setClassPoints'](_0x335ad6,_0x1ea0ee);}}}),PluginManager[_0x45bccd(0x3d4)](pluginData['name'],_0x45bccd(0x4a6),_0xaab02c=>{const _0x3f54f3=_0x45bccd;VisuMZ['ConvertParams'](_0xaab02c,_0xaab02c);const _0xb81ecf=_0xaab02c[_0x3f54f3(0x1ca)][_0x3f54f3(0x361)](_0x4884f9=>$gameActors[_0x3f54f3(0x43f)](_0x4884f9)),_0x2d2095=_0xaab02c['Classes'],_0xeeb8e5=_0xaab02c[_0x3f54f3(0x3c6)];for(const _0x54ebde of _0xb81ecf){if(!_0x54ebde)continue;for(const _0x147c85 of _0x2d2095){if(_0x3f54f3(0x23f)!=='teVpb')_0x54ebde[_0x3f54f3(0x2ea)](_0xeeb8e5,_0x147c85);else{function _0x9984be(){const _0x23f013=_0x3f54f3;_0x16fb2c=_0x10e5e1[_0x23f013(0x1d8)](_0x3f6aef['$1']);}}}}}),PluginManager['registerCommand'](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x497),_0x7a20f=>{const _0x56ac97=_0x45bccd;VisuMZ[_0x56ac97(0x483)](_0x7a20f,_0x7a20f);const _0xc09daa=_0x7a20f[_0x56ac97(0x1ca)][_0x56ac97(0x361)](_0x5f1731=>$gameActors[_0x56ac97(0x43f)](_0x5f1731)),_0x44bed3=_0x7a20f[_0x56ac97(0x353)],_0x996cd5=_0x7a20f[_0x56ac97(0x3c6)];for(const _0x310298 of _0xc09daa){if(_0x56ac97(0x446)==='XrIKw'){function _0x2351c2(){const _0x48ae23=_0x56ac97;let _0x551649=_0x542903[_0x48ae23(0x1bd)][_0x48ae23(0x287)](_0x1d8d15=>_0x1d8d15[_0x48ae23(0x28b)]===_0x25eea2[_0x48ae23(0x269)])[_0x48ae23(0x361)](_0x494228=>_0x285fe5[_0x48ae23(0x439)][_0x494228[_0x48ae23(0x2f1)]])[_0x48ae23(0x46b)](',\x20'),_0x252bf2='\x5cC[16]%1:\x5cC[0]\x20\x5cFS[%3]%2'['format'](_0x16c101[_0x48ae23(0x1e5)],_0x551649,_0x586f06||0x16);if(_0x36236c)_0x252bf2=_0x252bf2[_0x48ae23(0x203)](/\\I\[(\d+)\]/gi,'');if(_0x3cc8c2)_0x252bf2=_0x48ae23(0x1a0)+_0x252bf2;this[_0x48ae23(0x245)](_0x252bf2,_0x1d2ade,_0x20db54,_0x49a640),_0x9722f0+=this[_0x48ae23(0x376)]();}}else{if(!_0x310298)continue;for(const _0x38f1d1 of _0x44bed3){if(_0x56ac97(0x2a2)===_0x56ac97(0x1dc)){function _0x23310b(){const _0x3f8a69=_0x56ac97;if(!_0x2bedd0)return'';const _0x1cfb8f=_0x3f8a69(0x241)['format'](_0xc1d625[_0x3f8a69(0x216)](),_0x5d11a3[_0x3f8a69(0x2b3)]()['id']);return _0x16baaf['actorClassBattlerName'][_0x1cfb8f]||'';}}else _0x310298[_0x56ac97(0x418)](_0x996cd5,_0x38f1d1);}}}}),PluginManager['registerCommand'](pluginData['name'],_0x45bccd(0x17e),_0x4cdf44=>{const _0x20f99e=_0x45bccd;VisuMZ[_0x20f99e(0x483)](_0x4cdf44,_0x4cdf44);const _0x2403fe=_0x4cdf44[_0x20f99e(0x1ca)]['map'](_0x2c4efe=>$gameActors['actor'](_0x2c4efe)),_0x3fcffa=_0x4cdf44[_0x20f99e(0x353)],_0x30e2ef=_0x4cdf44['Points'];for(const _0x4f5102 of _0x2403fe){if(!_0x4f5102)continue;for(const _0x3e1895 of _0x3fcffa){if(_0x20f99e(0x2e4)==='FcyKg'){function _0x4816b3(){const _0x58bd2e=_0x20f99e;if(!this[_0x58bd2e(0x352)]())return;if(!this['isClassChangeCommandVisible']())return;const _0x543e68=_0x83a124[_0x58bd2e(0x476)],_0x130fae=this['isClassChangeCommandEnabled']();this['addCommand'](_0x543e68,'ClassChangeSystem',_0x130fae);}}else _0x4f5102[_0x20f99e(0x231)](_0x30e2ef,_0x3e1895);}}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x357),_0x4b0355=>{const _0x2e3203=_0x45bccd;VisuMZ[_0x2e3203(0x483)](_0x4b0355,_0x4b0355);const _0xee56a6=_0x4b0355[_0x2e3203(0x1ca)][_0x2e3203(0x361)](_0xac3e94=>$gameActors['actor'](_0xac3e94)),_0x433481=_0x4b0355[_0x2e3203(0x353)],_0x18de9e=_0x4b0355[_0x2e3203(0x3c6)];for(const _0x57f536 of _0xee56a6){if(!_0x57f536)continue;for(const _0x1739fc of _0x433481){_0x57f536['setJobPoints'](_0x18de9e,_0x1739fc);}}}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],_0x45bccd(0x211),_0x3d2e2a=>{const _0x45cbaa=_0x45bccd;VisuMZ[_0x45cbaa(0x483)](_0x3d2e2a,_0x3d2e2a),$gameSystem[_0x45cbaa(0x201)](_0x3d2e2a[_0x45cbaa(0x2f8)]);}),PluginManager[_0x45bccd(0x3d4)](pluginData[_0x45bccd(0x3ea)],'SystemShowClassChangeSystemMenu',_0x184ae1=>{const _0x39e2c1=_0x45bccd;VisuMZ[_0x39e2c1(0x483)](_0x184ae1,_0x184ae1),$gameSystem[_0x39e2c1(0x16a)](_0x184ae1[_0x39e2c1(0x2b4)]);}),VisuMZ[_0x45bccd(0x3ac)]['functionName']=function(){const _0x3ba195=_0x45bccd;try{}catch(_0x4d9fd1){if(_0x3ba195(0x335)===_0x3ba195(0x335)){if($gameTemp[_0x3ba195(0x2fa)]())console[_0x3ba195(0x210)](_0x4d9fd1);}else{function _0x23e429(){const _0x2fcb7b=_0x3ba195;if(!this[_0x2fcb7b(0x349)])return;const _0x3d7660=this[_0x2fcb7b(0x1cc)](_0x4a5982),_0x26db62=this[_0x2fcb7b(0x43a)][_0x1c23b7][_0x2fcb7b(0x2ab)]||0x1,_0x3bccfb=this[_0x2fcb7b(0x349)][_0x2fcb7b(0x2da)](_0x26db62),_0x45c4ed=_0x3bccfb?_0x3bccfb['id']:0x0,_0x14ca18=_0x42ec6b['ClassChangeSystem']['Settings']['Multiclass'];if(!_0x14ca18)return;const _0x159493=_0x14ca18[_0x26db62-0x1];if(!_0x159493)return;let _0x23d031=_0x3d7660['x'],_0xb6ff81=_0x3d7660['y'],_0x3d9dd2=_0x3d7660[_0x2fcb7b(0x2c7)]-this['itemPadding']()*0x2,_0x2dfea8=_0x3d7660['height'],_0x395864=_0x1dc4fc[_0x2fcb7b(0x3eb)](_0x3d9dd2,_0x2dfea8,this[_0x2fcb7b(0x376)]()*0x3);_0x395864=_0x4fa458['floor'](_0x395864/_0x485ffe['iconWidth'])*_0x1a83d7[_0x2fcb7b(0x444)],_0x23d031+=_0x395864+this[_0x2fcb7b(0x213)]()*0x4,this[_0x2fcb7b(0x47f)](),this[_0x2fcb7b(0x26a)](),this[_0x2fcb7b(0x31d)](_0x3d7660),this[_0x2fcb7b(0x47a)](this[_0x2fcb7b(0x32e)](_0x26db62)),this['drawBigItemImage'](_0x3f9c8f,_0x3bccfb,_0x3d7660),this[_0x2fcb7b(0x43d)](_0x3ae065['getColor'](_0x159493[_0x2fcb7b(0x4b0)])),this['drawText'](_0x159493[_0x2fcb7b(0x433)],_0x3d7660['x'],_0x3d7660['y'],_0x3d7660[_0x2fcb7b(0x2c7)],_0x2fcb7b(0x2c9)),this[_0x2fcb7b(0x26a)]();if(!_0x3bccfb){this[_0x2fcb7b(0x47a)](![]);const _0x486bf8=_0x258894[_0x2fcb7b(0x304)](_0x3d7660['y']+this[_0x2fcb7b(0x376)]()+(_0x3d7660[_0x2fcb7b(0x49e)]-this[_0x2fcb7b(0x376)]()*0x2)/0x2);this[_0x2fcb7b(0x1c5)](_0x5af392[_0x2fcb7b(0x31a)],_0x3d7660['x'],_0x486bf8,_0x3d7660[_0x2fcb7b(0x2c7)],_0x2fcb7b(0x2c9));return;}_0xb6ff81+=this[_0x2fcb7b(0x376)](),this[_0x2fcb7b(0x1c5)](_0x3bccfb[_0x2fcb7b(0x3ea)],_0x23d031,_0xb6ff81,_0x3d7660['width']-_0x23d031),_0xb6ff81+=this['lineHeight'](),this[_0x2fcb7b(0x460)](this[_0x2fcb7b(0x349)],_0x45c4ed,_0x23d031,_0xb6ff81-0x4),_0xb6ff81+=this[_0x2fcb7b(0x376)](),this[_0x2fcb7b(0x2ec)](_0x45c4ed,_0x3d7660),this['drawExtraContents'](_0x45c4ed,_0x26db62,_0x159493,_0x3d7660);}}}},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x1f8)]=Scene_Boot[_0x45bccd(0x404)][_0x45bccd(0x1b7)],Scene_Boot[_0x45bccd(0x404)][_0x45bccd(0x1b7)]=function(){const _0x156452=_0x45bccd;VisuMZ['ClassChangeSystem'][_0x156452(0x1f8)][_0x156452(0x42e)](this),this[_0x156452(0x322)]();},Scene_Boot[_0x45bccd(0x404)]['process_VisuMZ_ClassChangeSystem']=function(){const _0x32955d=_0x45bccd;this[_0x32955d(0x4be)]();},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x411)]={'StartingClassPoints':/<STARTING (?:CLASS POINTS|JP):[ ](.*)>/i,'StartClassClassPoints':/<CLASS (.*) STARTING (?:CLASS POINTS|JP):[ ](.*)>/gi,'UserGainClassPoints':/<(?:CLASS POINTS|JP|USER CLASS POINTS|USER JP) GAIN:[ ](.*)>/i,'TargetGainClassPoints':/<TARGET (?:CLASS POINTS|JP) GAIN:[ ](.*)>/i,'EnemyClassPoints':/<(?:CLASS POINTS|JP):[ ](.*)>/i,'ClassPointsRate':/<(?:CLASS POINTS|JP) RATE:[ ](\d+)([%％])>/i,'StartingJobPoints':/<STARTING (?:JOB POINTS|JP):[ ](.*)>/i,'StartClassJobPoints':/<CLASS (.*) STARTING (?:JOB POINTS|JP):[ ](.*)>/gi,'UserGainJobPoints':/<(?:JOB POINTS|JP|USER JOB POINTS|USER JP) GAIN:[ ](.*)>/i,'TargetGainJobPoints':/<TARGET (?:JOB POINTS|JP) GAIN:[ ](.*)>/i,'EnemyJobPoints':/<(?:JOB POINTS|JP):[ ](.*)>/i,'JobPointsRate':/<(?:JOB POINTS|JP) RATE:[ ](\d+)([%％])>/i,'ClassDescription':/<(?:HELP|DESCRIPTION|HELP DESCRIPTION)>\s*([\s\S]*)\s*<\/(?:HELP|DESCRIPTION|HELP DESCRIPTION)>/i,'ClassIcon':/<(?:ICON|ICON INDEX):[ ](\d+)>/i,'classPicture':/<(?:CLASS|CLASS CHANGE) (?:PICTURE|FILENAME):[ ](.*)>/i,'bigPicture':/<PICTURE:[ ](.*)>/i,'ClassFaceName':/<(.*)[ ]FACE:[ ](.*),[ ](\d+)>/gi,'ClassCharaName':/<(.*)[ ](?:CHARACTER|CHARA|SPRITE):[ ](.*),[ ](\d+)>/gi,'ClassBattlerName':/<(.*)[ ](?:BATTLER|SV_ACTOR|SV ACTOR|SVACTOR):[ ](.*)>/gi,'ClassMenuPortrait':/<(.*)[ ]MENU (?:PORTRAIT|IMAGE):[ ](.*)>/gi,'ClassBattlePortrait':/<(.*)[ ]BATTLE (?:PORTRAIT|IMAGE):[ ](.*)>/gi,'ActorUnlockedClasses':/<(?:UNLOCK|UNLOCKED) (?:CLASS|CLASSES):[ ](.*)>/gi,'AutoUnlockRequirements':/<(?:AUTO|AUTOMATIC) UNLOCK REQUIREMENTS>\s*([\s\S]*)\s*<\/(?:AUTO|AUTOMATIC) UNLOCK REQUIREMENTS>/i,'StartingMulticlasses':/<STARTING MULTICLASSES:[ ](\d+)>/i,'StartingClassTier':/<STARTING TIER[ ](\d+)[ ]CLASS:[ ](.*)>/gi,'RestrictClassChangeTier':/<RESTRICT CLASS CHANGE (?:TIER|TIERS):[ ](.*)>/gi,'TierOnlyClass':/<CLASS CHANGE (?:TIER|TIERS) ONLY:[ ](.*)>/gi,'ClassChangeAnimation':/<CLASS CHANGE ANIMATION:[ ](\d+)>/i},Scene_Boot[_0x45bccd(0x404)][_0x45bccd(0x4be)]=function(){const _0x27b2d5=_0x45bccd;if(VisuMZ[_0x27b2d5(0x2fd)])return;for(const _0x37c930 of $dataActors){if(!_0x37c930)continue;ImageManager[_0x27b2d5(0x19a)](_0x37c930);}for(const _0x40a91c of $dataClasses){if(!_0x40a91c)continue;VisuMZ['ClassChangeSystem'][_0x27b2d5(0x414)](_0x40a91c);}},VisuMZ[_0x45bccd(0x3ac)]['JS']={},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x1b1)]=function(_0x3b580e,_0x1e75c1,_0x467c7d){const _0x308177=_0x45bccd,_0xd37260=_0x3b580e['note'];if(_0xd37260[_0x308177(0x264)](_0x467c7d)){const _0x25411c=String(RegExp['$1']),_0x51d081=_0x308177(0x2ef)[_0x308177(0x3dd)](_0x25411c),_0x52919d=VisuMZ[_0x308177(0x3ac)][_0x308177(0x2dd)](_0x3b580e,_0x1e75c1);VisuMZ[_0x308177(0x3ac)]['JS'][_0x52919d]=new Function(_0x51d081);}},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x2dd)]=function(_0x10d01f,_0x33ba27){const _0x58cc80=_0x45bccd;let _0x3026b5='';if($dataActors[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5=_0x58cc80(0x390)[_0x58cc80(0x3dd)](_0x10d01f['id'],_0x33ba27);if($dataClasses[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5='Class-%1-%2'[_0x58cc80(0x3dd)](_0x10d01f['id'],_0x33ba27);if($dataSkills[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5=_0x58cc80(0x480)['format'](_0x10d01f['id'],_0x33ba27);if($dataItems[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5='Item-%1-%2'[_0x58cc80(0x3dd)](_0x10d01f['id'],_0x33ba27);if($dataWeapons[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5=_0x58cc80(0x23e)['format'](_0x10d01f['id'],_0x33ba27);if($dataArmors[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5=_0x58cc80(0x2af)['format'](_0x10d01f['id'],_0x33ba27);if($dataEnemies[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5=_0x58cc80(0x173)[_0x58cc80(0x3dd)](_0x10d01f['id'],_0x33ba27);if($dataStates[_0x58cc80(0x3f5)](_0x10d01f))_0x3026b5='State-%1-%2'[_0x58cc80(0x3dd)](_0x10d01f['id'],_0x33ba27);return _0x3026b5;},VisuMZ['ClassChangeSystem'][_0x45bccd(0x3c9)]=VisuMZ['ParseActorNotetags'],VisuMZ['ParseActorNotetags']=function(_0x597f40){const _0x523f67=_0x45bccd;VisuMZ[_0x523f67(0x3ac)][_0x523f67(0x3c9)][_0x523f67(0x42e)](this,_0x597f40),ImageManager[_0x523f67(0x19a)](_0x597f40);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x32b)]=VisuMZ[_0x45bccd(0x32b)],VisuMZ['ParseClassNotetags']=function(_0x16f5ca){const _0x412e8b=_0x45bccd;VisuMZ['ClassChangeSystem'][_0x412e8b(0x32b)]['call'](this,_0x16f5ca),VisuMZ[_0x412e8b(0x3ac)]['Parse_Notetags_Basic'](_0x16f5ca);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x414)]=function(_0x2b0806){const _0x10fe7a=_0x45bccd;_0x2b0806['iconIndex']=ImageManager['classIcon']||0x0,_0x2b0806[_0x10fe7a(0x327)]=TextManager['classDescription'][_0x10fe7a(0x3dd)](_0x2b0806[_0x10fe7a(0x3ea)]||'');const _0x1edbb7=VisuMZ[_0x10fe7a(0x3ac)][_0x10fe7a(0x411)],_0x265b5b=_0x2b0806[_0x10fe7a(0x323)];_0x265b5b['match'](_0x1edbb7['ClassIcon'])&&(_0x2b0806[_0x10fe7a(0x34a)]=Number(RegExp['$1']));if(_0x265b5b['match'](_0x1edbb7['ClassDescription'])){if(_0x10fe7a(0x393)==='fOVqd'){function _0x1e67fc(){const _0x2dd4b0=_0x10fe7a;return this[_0x2dd4b0(0x422)]=this['_earnedClassPoints']||0x0,this['getClassPoints']()-this[_0x2dd4b0(0x422)];}}else _0x2b0806['description']=String(RegExp['$1']);}},DataManager[_0x45bccd(0x276)]=function(_0x3232c1){const _0x357b52=_0x45bccd;if(!_0x3232c1)return[];let _0x546686=[];return _0x546686=_0x546686[_0x357b52(0x358)](_0x3232c1[_0x357b52(0x405)]()[_0x357b52(0x361)](_0x5b3605=>_0x5b3605['id'])),_0x546686=_0x546686['concat'](_0x3232c1[_0x357b52(0x29b)]()),_0x546686=_0x546686[_0x357b52(0x358)]($gameParty['getUnlockedClasses']()),_0x546686=_0x546686[_0x357b52(0x358)](VisuMZ[_0x357b52(0x3ac)][_0x357b52(0x38c)][_0x357b52(0x338)][_0x357b52(0x4aa)]),_0x546686=_0x546686[_0x357b52(0x287)]((_0x1b26c9,_0x4b6406,_0x41ecae)=>_0x41ecae[_0x357b52(0x165)](_0x1b26c9)===_0x4b6406),_0x546686[_0x357b52(0x2fb)](function(_0x22b856,_0x5c529e){return _0x22b856-_0x5c529e;}),_0x546686[_0x357b52(0x361)](_0x4a601e=>$dataClasses[_0x4a601e])[_0x357b52(0x306)](null);},DataManager['checkForNewUnlockedClasses']=function(_0x508557){const _0x33918f=_0x45bccd,_0x46e8ce=[],_0x340f6d=DataManager['getActorUnlockedClasses'](_0x508557);for(const _0x14f33d of $dataClasses){if(!_0x14f33d)continue;if(_0x340f6d[_0x33918f(0x3f5)](_0x14f33d))continue;this['isClassAutoUnlockRequirementsMet'](_0x508557,_0x14f33d)&&_0x46e8ce[_0x33918f(0x3ab)](_0x14f33d['id']);}return _0x46e8ce;},DataManager[_0x45bccd(0x172)]=function(_0x501a89,_0x4bf6a0){const _0x2e24b6=_0x45bccd;if(!_0x501a89)return![];if(!_0x4bf6a0)return![];const _0x4a7e25=VisuMZ[_0x2e24b6(0x3ac)][_0x2e24b6(0x411)],_0x173d73=_0x4bf6a0[_0x2e24b6(0x323)];if(_0x173d73[_0x2e24b6(0x264)](_0x4a7e25[_0x2e24b6(0x1fa)])){if(_0x2e24b6(0x297)==='aXrMO'){function _0x32236d(){const _0x47bd3e=_0x2e24b6;this[_0x47bd3e(0x45e)]=_0xb39d28;}}else{const _0x5e7e4d=String(RegExp['$1'])[_0x2e24b6(0x1f5)](/[\r\n]+/);for(const _0x5da916 of _0x5e7e4d){if('xnHBE'===_0x2e24b6(0x168)){let _0x67450b=0x0;if(_0x5da916[_0x2e24b6(0x264)](/(.*):[ ](.*)/i)){const _0x2ba1f6=String(RegExp['$1']),_0x932591=String(RegExp['$2']);if(_0x2ba1f6[_0x2e24b6(0x264)](/CLASS[ ](\d+)/i))_0x67450b=Number(RegExp['$1']);else{if(_0x2ba1f6['match'](/CLASS[ ](.*)/i))_0x67450b=this[_0x2e24b6(0x1d8)](RegExp['$1']);else{if(_0x2ba1f6[_0x2e24b6(0x264)](/\b(?:AP|CP|JP|SP)\b/i)){const _0x116efd=_0x2ba1f6[_0x2e24b6(0x377)]()[_0x2e24b6(0x1b5)](),_0x5b01b8=Number(_0x932591)||0x0;if(Imported[_0x2e24b6(0x4a1)]){if(_0x2e24b6(0x40b)!==_0x2e24b6(0x40b)){function _0x59db10(){const _0x17172a=_0x2e24b6,_0x21a618=_0x5ae4dd[_0x17172a(0x3e0)]['Settings']['UI'][_0x17172a(0x318)];this[_0x17172a(0x1c5)](_0x21a618,_0x3d0ccd,_0x3b79a8,_0x1f0114,'center');}}else{if(_0x116efd==='AP'){const _0x58d1d4=_0x501a89['getAbilityPoints']();if(_0x58d1d4<_0x5b01b8)return![];}else{if(_0x116efd==='SP'){const _0x29f7f8=_0x501a89[_0x2e24b6(0x19c)]();if(_0x29f7f8<_0x5b01b8)return![];}}}}if(Imported[_0x2e24b6(0x3da)]){if(_0x2e24b6(0x32a)===_0x2e24b6(0x32a)){if(_0x116efd==='CP'){if('jNkpO'===_0x2e24b6(0x1db)){function _0x3eaf8e(){_0x80e738=_0x27d0de['getClassIdWithName'](_0x353c20);}}else{const _0x48e38c=_0x501a89[_0x2e24b6(0x478)]();if(_0x48e38c<_0x5b01b8)return![];}}else{if(_0x116efd==='JP'){const _0x5af075=_0x501a89[_0x2e24b6(0x3e8)]();if(_0x5af075<_0x5b01b8)return![];}}}else{function _0x258565(){const _0x258ee0=_0x2e24b6,_0x50adcf=this[_0x258ee0(0x213)]();_0x170184[_0x258ee0(0x495)]?this[_0x258ee0(0x23a)](_0x3c37ba+_0x50adcf,_0x13e77d,_0x2aa5a3,_0xb47bc2,![]):(this['changeTextColor'](_0x398923[_0x258ee0(0x2e3)]()),this['drawText'](_0x274e1b[_0x258ee0(0x26c)](_0x43270a),_0x5a5531+_0x50adcf,_0x5ab021,_0x1144cc),this['resetTextColor']());}}}}}}if(_0x932591[_0x2e24b6(0x264)](/LEVEL[ ](\d+)/i)){if(_0x2e24b6(0x266)!==_0x2e24b6(0x266)){function _0x772515(){this['initClassChangeRestrictions']();}}else{const _0x40d827=Number(RegExp['$1']);if(_0x501a89[_0x2e24b6(0x189)](_0x67450b)<_0x40d827)return![];}}else{if(_0x932591['match'](/(\d+)[ ]CP/i)){const _0xbd98c2=Number(RegExp['$1']);if(_0x501a89['getClassPoints'](_0x67450b)<_0xbd98c2)return![];}else{if(_0x932591['match'](/(\d+)[ ]JP/i)){const _0xc17625=Number(RegExp['$1']);if(_0x501a89[_0x2e24b6(0x3e8)](_0x67450b)<_0xc17625)return![];}else{if(_0x932591[_0x2e24b6(0x264)](/(\d+)[ ]AP/i)){if(!Imported[_0x2e24b6(0x4a1)])continue;const _0x2a135a=Number(RegExp['$1']);if(_0x501a89['getAbilityPoints'](_0x67450b)<_0x2a135a)return![];}else{if(_0x932591[_0x2e24b6(0x264)](/(\d+)[ ]SP/i)){const _0x3c5977=Number(RegExp['$1']);if(_0x501a89['getSkillPoints'](_0x67450b)<_0x3c5977)return![];}}}}}}}else{function _0x4090a6(){const _0x881268=_0x2e24b6;return this['deadMembers']()['reduce']((_0x5c8110,_0x535004)=>_0x5c8110+_0x535004[_0x881268(0x4ad)](),0x0);}}}return!![];}}return![];},DataManager[_0x45bccd(0x2ed)]=function(_0x2edb69){const _0x20e928=_0x45bccd;if(!_0x2edb69)return[];const _0x3e527f=VisuMZ['ClassChangeSystem']['RegExp'],_0x40373a=_0x2edb69[_0x20e928(0x323)];let _0x556f3d=[];const _0x3eb669=_0x40373a[_0x20e928(0x264)](_0x3e527f['TierOnlyClass']);if(_0x3eb669){if(_0x20e928(0x449)!=='NKExb'){for(const _0x4aa361 of _0x3eb669){if(_0x20e928(0x3a0)===_0x20e928(0x3a0)){if(!_0x4aa361)continue;_0x4aa361[_0x20e928(0x264)](_0x3e527f['TierOnlyClass']);const _0x4b3334=String(RegExp['$1'])[_0x20e928(0x1f5)](',')[_0x20e928(0x361)](_0x57ab68=>Number(_0x57ab68))[_0x20e928(0x306)](null)[_0x20e928(0x306)](undefined)[_0x20e928(0x306)](NaN);_0x556f3d=_0x556f3d[_0x20e928(0x358)](_0x4b3334);}else{function _0x89e6b6(){const _0x50e719=_0x20e928;this[_0x50e719(0x2c3)](_0x259bcd);return;}}}return _0x556f3d;}else{function _0x78b806(){const _0x513002=_0x20e928;_0x7da391=_0xced25b||0x1,this[_0x513002(0x47a)](![]);const _0x69bfcd=_0x18b409[_0x513002(0x40c)](),_0x26551b=_0x50038e[_0x513002(0x3db)](),_0x1ca41f=_0x1e248a['width']/0x2,_0x3203ae=this['lineHeight']();while(_0x282202--){this['contents'][_0x513002(0x499)](_0x581f6c['x'],_0x1d500f['y'],_0x1ca41f,_0x3203ae,_0x26551b,_0x69bfcd),this[_0x513002(0x31e)][_0x513002(0x499)](_0x57fb8a['x']+_0x1ca41f,_0x461046['y'],_0x1ca41f,_0x3203ae,_0x69bfcd,_0x26551b);}this[_0x513002(0x47a)](!![]);}}}else{const _0x49543b=VisuMZ['ClassChangeSystem']['Settings'][_0x20e928(0x2bd)][_0x20e928(0x38a)];return Array[_0x20e928(0x367)]({'length':_0x49543b},(_0x3556a4,_0x2179fd)=>_0x2179fd+0x1);}},DataManager['getClassIdWithName']=function(_0x9770c4){const _0x3eadc0=_0x45bccd;_0x9770c4=_0x9770c4[_0x3eadc0(0x377)]()[_0x3eadc0(0x1b5)](),this['_classIDs']=this[_0x3eadc0(0x1bb)]||{};if(this['_classIDs'][_0x9770c4])return this[_0x3eadc0(0x1bb)][_0x9770c4];for(const _0x1d57d7 of $dataClasses){if('xnchM'==='xdRYr'){function _0x8f853c(){const _0x598590=_0x3eadc0;this[_0x598590(0x48c)]();}}else{if(!_0x1d57d7)continue;let _0xa08549=_0x1d57d7['name'];_0xa08549=_0xa08549[_0x3eadc0(0x203)](/\x1I\[(\d+)\]/gi,''),_0xa08549=_0xa08549[_0x3eadc0(0x203)](/\\I\[(\d+)\]/gi,''),this['_classIDs'][_0xa08549[_0x3eadc0(0x377)]()[_0x3eadc0(0x1b5)]()]=_0x1d57d7['id'];}}return this[_0x3eadc0(0x1bb)][_0x9770c4]||0x0;},ImageManager[_0x45bccd(0x1fc)]=VisuMZ[_0x45bccd(0x3ac)]['Settings'][_0x45bccd(0x28e)][_0x45bccd(0x380)],ImageManager[_0x45bccd(0x1e0)]=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)]['JobPoints'][_0x45bccd(0x380)],ImageManager[_0x45bccd(0x246)]=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)]['General'][_0x45bccd(0x380)],ImageManager[_0x45bccd(0x44c)]={},ImageManager[_0x45bccd(0x3bb)]={},ImageManager[_0x45bccd(0x261)]={},ImageManager[_0x45bccd(0x39a)]={},ImageManager[_0x45bccd(0x482)]={},ImageManager['actorClassMenuPortrait']={},ImageManager[_0x45bccd(0x445)]={},ImageManager[_0x45bccd(0x19a)]=function(_0xe3506e){const _0x4b56ef=_0x45bccd;if(!_0xe3506e)return;const _0x586eb4=VisuMZ[_0x4b56ef(0x3ac)][_0x4b56ef(0x411)],_0x19e2d1=_0xe3506e[_0x4b56ef(0x323)],_0x4a120f=_0xe3506e['id'],_0xc61316=_0x19e2d1[_0x4b56ef(0x264)](_0x586eb4[_0x4b56ef(0x2dc)]);if(_0xc61316){if(_0x4b56ef(0x33b)!==_0x4b56ef(0x33b)){function _0x2860e9(){_0x471efc=_0x234d52(_0x9c364c['$1']);}}else for(const _0x1df036 of _0xc61316){if(_0x4b56ef(0x2d7)===_0x4b56ef(0x2d7)){if(!_0x1df036)continue;_0x1df036[_0x4b56ef(0x264)](_0x586eb4[_0x4b56ef(0x2dc)]);const _0x3f1a95=String(RegExp['$1']),_0x4001cb=String(RegExp['$2'])[_0x4b56ef(0x1b5)](),_0x31ca4d=Number(RegExp['$3']);let _0x5906f1=0x0;if(_0x3f1a95[_0x4b56ef(0x264)](/CLASS[ ](\d+)/i))_0x5906f1=Number(RegExp['$1']);else{if(_0x3f1a95[_0x4b56ef(0x264)](/CLASS[ ](.*)/i)){if(_0x4b56ef(0x169)===_0x4b56ef(0x169))_0x5906f1=DataManager['getClassIdWithName'](RegExp['$1']);else{function _0x43853f(){const _0x5bfdac=_0x4b56ef,_0x100364=this['_cache']||{};this[_0x5bfdac(0x3cd)](_0x1f1153['skillId']),this[_0x5bfdac(0x1d7)]=_0x100364;}}}else{if(_0x4b56ef(0x1bf)===_0x4b56ef(0x1bf))_0x5906f1=DataManager[_0x4b56ef(0x1d8)](_0x3f1a95);else{function _0x582580(){const _0x45f283=_0x4b56ef;return _0x2405cd(_0xc10785[_0x45f283(0x298)]);}}}}if(_0x5906f1>0x0){if(_0x4b56ef(0x18c)!==_0x4b56ef(0x18c)){function _0x49c22a(){const _0x26a705=_0x4b56ef;if(!_0x258207)return;if(!_0x5a1036[_0x26a705(0x3a1)]())return;const _0x4be7dd=0x80,_0x1bcd16=_0x1a5741[_0x26a705(0x204)](_0x57c7d5);let _0x581765=_0x4be6bd[_0x26a705(0x486)](),_0x5cd470=_0x523393[_0x26a705(0x329)]();_0x1bcd16>=0x1&&(_0x581765=_0x5c0658[_0x26a705(0x479)](),_0x5cd470=_0x26b21d['maxLvGaugeColor2']()),this['drawGauge'](_0x3c1341,_0x286fbe,_0x4be7dd,_0x1bcd16,_0x581765,_0x5cd470);}}else{const _0x3b7372=_0x4b56ef(0x241)[_0x4b56ef(0x3dd)](_0x4a120f,_0x5906f1);ImageManager[_0x4b56ef(0x44c)][_0x3b7372]=_0x4001cb,ImageManager[_0x4b56ef(0x3bb)][_0x3b7372]=_0x31ca4d;}}}else{function _0x507b8c(){const _0x23c4f5=_0x4b56ef;this[_0x23c4f5(0x43d)](_0x17db74['paramchangeTextColor'](_0x27545a)),_0x2346d3=(_0x133948>0x0?'(+%1)':_0x23c4f5(0x389))['format'](_0x493854),this[_0x23c4f5(0x1c5)](_0x32ca18,_0x4284b2+_0x5f2b74,_0x175658,_0x36f17e,_0x23c4f5(0x2d6));}}}}const _0x4cdd5d=_0x19e2d1[_0x4b56ef(0x264)](_0x586eb4[_0x4b56ef(0x22b)]);if(_0x4cdd5d){if(_0x4b56ef(0x442)===_0x4b56ef(0x284)){function _0x1b1af1(){return _0x359175*(_0xd449a0(_0x2b0de8['$1'])*0.01);}}else for(const _0x29d5c0 of _0x4cdd5d){if(!_0x29d5c0)continue;_0x29d5c0[_0x4b56ef(0x264)](_0x586eb4[_0x4b56ef(0x22b)]);const _0x3d6514=String(RegExp['$1']),_0x5e1905=String(RegExp['$2'])[_0x4b56ef(0x1b5)](),_0xb7a1ad=Number(RegExp['$3']);let _0x3ba089=0x0;if(_0x3d6514['match'](/CLASS[ ](\d+)/i)){if(_0x4b56ef(0x256)===_0x4b56ef(0x1c2)){function _0x230993(){const _0x20cbe4=_0x4b56ef;this[_0x20cbe4(0x262)](),this[_0x20cbe4(0x47f)]();if(this[_0x20cbe4(0x349)])this[_0x20cbe4(0x349)]['refresh']();this[_0x20cbe4(0x1ba)]();}}else _0x3ba089=Number(RegExp['$1']);}else{if(_0x3d6514[_0x4b56ef(0x264)](/CLASS[ ](.*)/i)){if(_0x4b56ef(0x2b1)===_0x4b56ef(0x2b1))_0x3ba089=DataManager[_0x4b56ef(0x1d8)](RegExp['$1']);else{function _0x4bddc8(){const _0x276080=_0x4b56ef;if(!_0x208ba5)return'';const _0x4e9848=_0x276080(0x241)[_0x276080(0x3dd)](_0x21ab9f[_0x276080(0x216)](),_0x34b2b0[_0x276080(0x2b3)]()['id']);return _0x5c8683['actorClassFaceName'][_0x4e9848]||'';}}}else _0x3ba089=DataManager['getClassIdWithName'](_0x3d6514);}if(_0x3ba089>0x0){const _0x318a8b='Actor-%1-Class-%2'[_0x4b56ef(0x3dd)](_0x4a120f,_0x3ba089);ImageManager['actorClassCharacterName'][_0x318a8b]=_0x5e1905,ImageManager[_0x4b56ef(0x39a)][_0x318a8b]=_0xb7a1ad;}}}const _0x427700=_0x19e2d1['match'](_0x586eb4[_0x4b56ef(0x1ad)]);if(_0x427700)for(const _0x3f059a of _0x427700){if(!_0x3f059a)continue;_0x3f059a[_0x4b56ef(0x264)](_0x586eb4[_0x4b56ef(0x1ad)]);const _0x3d9633=String(RegExp['$1']),_0x1385fd=String(RegExp['$2'])[_0x4b56ef(0x1b5)]();let _0x5e1562=0x0;if(_0x3d9633[_0x4b56ef(0x264)](/CLASS[ ](\d+)/i))_0x5e1562=Number(RegExp['$1']);else{if(_0x3d9633[_0x4b56ef(0x264)](/CLASS[ ](.*)/i))_0x5e1562=DataManager[_0x4b56ef(0x1d8)](RegExp['$1']);else{if('ewoPx'===_0x4b56ef(0x291)){function _0x293b69(){const _0x4d9e7f=_0x4b56ef;_0x44c47f[_0x4d9e7f(0x3ac)][_0x4d9e7f(0x1b3)][_0x4d9e7f(0x42e)](this,_0x2f57be),this['applyClassChangeSystemUserEffect'](_0x2efff3);}}else _0x5e1562=DataManager[_0x4b56ef(0x1d8)](_0x3d9633);}}if(_0x5e1562>0x0){const _0x35af83=_0x4b56ef(0x241)['format'](_0x4a120f,_0x5e1562);ImageManager[_0x4b56ef(0x482)][_0x35af83]=_0x1385fd;}}const _0x19e02a=_0x19e2d1[_0x4b56ef(0x264)](_0x586eb4['ClassMenuPortrait']);if(_0x19e02a)for(const _0x324e75 of _0x19e02a){if('KFUcM'==='KFUcM'){if(!_0x324e75)continue;_0x324e75[_0x4b56ef(0x264)](_0x586eb4[_0x4b56ef(0x334)]);const _0x59c5c6=String(RegExp['$1']),_0x7e8147=String(RegExp['$2'])[_0x4b56ef(0x1b5)]();let _0x4cbf44=0x0;if(_0x59c5c6[_0x4b56ef(0x264)](/CLASS[ ](\d+)/i))_0x4cbf44=Number(RegExp['$1']);else _0x59c5c6['match'](/CLASS[ ](.*)/i)?_0x4cbf44=DataManager[_0x4b56ef(0x1d8)](RegExp['$1']):_0x4cbf44=DataManager[_0x4b56ef(0x1d8)](_0x59c5c6);if(_0x4cbf44>0x0){const _0x239c53=_0x4b56ef(0x241)[_0x4b56ef(0x3dd)](_0x4a120f,_0x4cbf44);ImageManager['actorClassMenuPortrait'][_0x239c53]=_0x7e8147;}}else{function _0x3cd2a5(){const _0x210a68=_0x4b56ef;if(this[_0x210a68(0x1af)]!==_0x5ba37e)return this[_0x210a68(0x1af)];return _0x7e6d73[_0x210a68(0x347)](this)||_0x17b3aa[_0x210a68(0x3ac)][_0x210a68(0x250)]['call'](this);;}}}const _0x57af49=_0x19e2d1[_0x4b56ef(0x264)](_0x586eb4['ClassBattlePortrait']);if(_0x57af49)for(const _0xeeacb7 of _0x57af49){if(_0x4b56ef(0x272)===_0x4b56ef(0x41d)){function _0xcc12be(){return _0xde1452;}}else{if(!_0xeeacb7)continue;_0xeeacb7['match'](_0x586eb4[_0x4b56ef(0x1a7)]);const _0x243141=String(RegExp['$1']),_0x1d00d3=String(RegExp['$2'])[_0x4b56ef(0x1b5)]();let _0x574e9a=0x0;if(_0x243141['match'](/CLASS[ ](\d+)/i))_0x574e9a=Number(RegExp['$1']);else _0x243141[_0x4b56ef(0x264)](/CLASS[ ](.*)/i)?_0x574e9a=DataManager['getClassIdWithName'](RegExp['$1']):_0x574e9a=DataManager[_0x4b56ef(0x1d8)](_0x243141);if(_0x574e9a>0x0){if(_0x4b56ef(0x200)!==_0x4b56ef(0x200)){function _0x5d2f73(){const _0x370750=_0x4b56ef;_0x3b5cff!==''?this[_0x370750(0x20f)]=_0x1cc9dd:this[_0x370750(0x20f)]=_0x5e144e;if(_0x42bf9c[_0x370750(0x1b9)]()&&_0xa03d3e[_0x370750(0x3f8)]()[_0x370750(0x3f5)](this)){const _0x2ca1c1=_0x156b8b[_0x370750(0x41e)][_0x370750(0x303)];if(_0x2ca1c1)_0x2ca1c1[_0x370750(0x2a5)](this);}}}else{const _0x33a4f1=_0x4b56ef(0x241)[_0x4b56ef(0x3dd)](_0x4a120f,_0x574e9a);ImageManager[_0x4b56ef(0x445)][_0x33a4f1]=_0x1d00d3;}}}}},ImageManager[_0x45bccd(0x2c8)]=function(_0x203c20){const _0x30d4c1=_0x45bccd;if(!_0x203c20)return'';const _0x5dfadc=_0x30d4c1(0x241)[_0x30d4c1(0x3dd)](_0x203c20[_0x30d4c1(0x216)](),_0x203c20[_0x30d4c1(0x2b3)]()['id']);return ImageManager[_0x30d4c1(0x44c)][_0x5dfadc]||'';},ImageManager[_0x45bccd(0x481)]=function(_0x2c89fa){const _0x126f63=_0x45bccd;if(!_0x2c89fa)return undefined;const _0xd43946=_0x126f63(0x241)[_0x126f63(0x3dd)](_0x2c89fa['actorId'](),_0x2c89fa[_0x126f63(0x2b3)]()['id']);return ImageManager[_0x126f63(0x3bb)][_0xd43946]||undefined;},ImageManager[_0x45bccd(0x33a)]=function(_0x405bf2){const _0x32c7ff=_0x45bccd;if(!_0x405bf2)return'';const _0x5566db=_0x32c7ff(0x241)['format'](_0x405bf2[_0x32c7ff(0x216)](),_0x405bf2[_0x32c7ff(0x2b3)]()['id']);return ImageManager[_0x32c7ff(0x261)][_0x5566db]||'';},ImageManager['getActorClassCharacterIndex']=function(_0x34eb76){const _0x5c58b1=_0x45bccd;if(!_0x34eb76)return undefined;const _0x22c6b3=_0x5c58b1(0x241)['format'](_0x34eb76[_0x5c58b1(0x216)](),_0x34eb76[_0x5c58b1(0x2b3)]()['id']);return ImageManager['actorClassCharacterIndex'][_0x22c6b3]||undefined;},ImageManager['getActorClassBattlerName']=function(_0x310eeb){const _0x384fa2=_0x45bccd;if(!_0x310eeb)return'';const _0x4cd3b8=_0x384fa2(0x241)[_0x384fa2(0x3dd)](_0x310eeb['actorId'](),_0x310eeb[_0x384fa2(0x2b3)]()['id']);return ImageManager[_0x384fa2(0x482)][_0x4cd3b8]||'';},ImageManager[_0x45bccd(0x347)]=function(_0x17f5eb){const _0x35364b=_0x45bccd;if(!_0x17f5eb)return'';const _0x2907e0='Actor-%1-Class-%2'['format'](_0x17f5eb[_0x35364b(0x216)](),_0x17f5eb[_0x35364b(0x2b3)]()['id']);return ImageManager[_0x35364b(0x227)][_0x2907e0]||'';},ImageManager[_0x45bccd(0x39e)]=function(_0x4b6b8b){const _0x41a928=_0x45bccd;if(!_0x4b6b8b)return'';const _0x15a2ac='Actor-%1-Class-%2'[_0x41a928(0x3dd)](_0x4b6b8b[_0x41a928(0x216)](),_0x4b6b8b[_0x41a928(0x2b3)]()['id']);return ImageManager[_0x41a928(0x445)][_0x15a2ac]||'';},SoundManager[_0x45bccd(0x243)]=function(_0xcd3c9b){const _0x961bab=_0x45bccd;AudioManager[_0x961bab(0x320)](VisuMZ[_0x961bab(0x3ac)][_0x961bab(0x38c)][_0x961bab(0x190)]);},TextManager[_0x45bccd(0x476)]=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)][_0x45bccd(0x47b)][_0x45bccd(0x433)],TextManager[_0x45bccd(0x4b5)]=VisuMZ['ClassChangeSystem'][_0x45bccd(0x38c)]['ClassPoints'][_0x45bccd(0x282)],TextManager[_0x45bccd(0x1e9)]=VisuMZ[_0x45bccd(0x3ac)]['Settings'][_0x45bccd(0x28e)][_0x45bccd(0x1c8)],TextManager[_0x45bccd(0x473)]=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)][_0x45bccd(0x28e)][_0x45bccd(0x386)],TextManager[_0x45bccd(0x3d0)]=VisuMZ[_0x45bccd(0x3ac)]['Settings']['JobPoints'][_0x45bccd(0x282)],TextManager['jobPointsAbbr']=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)][_0x45bccd(0x1ea)]['AbbrText'],TextManager[_0x45bccd(0x1cd)]=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)][_0x45bccd(0x1ea)][_0x45bccd(0x386)],TextManager['classDescription']=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)][_0x45bccd(0x338)]['HelpDescription'],TextManager[_0x45bccd(0x31a)]=VisuMZ['ClassChangeSystem']['Settings'][_0x45bccd(0x239)]['VocabNoClassAssigned'],TextManager[_0x45bccd(0x398)]=VisuMZ['ClassChangeSystem'][_0x45bccd(0x38c)][_0x45bccd(0x239)][_0x45bccd(0x26d)],TextManager[_0x45bccd(0x321)]=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)]['Window']['VocabUnassignClass'],TextManager[_0x45bccd(0x42a)]=VisuMZ['ClassChangeSystem']['Settings'][_0x45bccd(0x239)][_0x45bccd(0x342)],ColorManager[_0x45bccd(0x27a)]=function(_0x4c5252){const _0x4c1c06=_0x45bccd;return _0x4c5252=String(_0x4c5252),_0x4c5252['match'](/#(.*)/i)?_0x4c1c06(0x223)[_0x4c1c06(0x3dd)](String(RegExp['$1'])):this[_0x4c1c06(0x3a5)](Number(_0x4c5252));},VisuMZ['ClassChangeSystem'][_0x45bccd(0x198)]=BattleManager['makeRewards'],BattleManager[_0x45bccd(0x43e)]=function(){const _0x2be147=_0x45bccd;VisuMZ['ClassChangeSystem'][_0x2be147(0x198)]['call'](this),this['makeRewardsClassPoints'](),this[_0x2be147(0x285)](),this[_0x2be147(0x197)](),this[_0x2be147(0x2be)]();},VisuMZ['ClassChangeSystem'][_0x45bccd(0x260)]=BattleManager[_0x45bccd(0x3f7)],BattleManager[_0x45bccd(0x3f7)]=function(){const _0x307b25=_0x45bccd;VisuMZ[_0x307b25(0x3ac)][_0x307b25(0x260)][_0x307b25(0x42e)](this),this[_0x307b25(0x3dc)](),this['displayRewardsJobPoints']();},VisuMZ[_0x45bccd(0x3ac)]['BattleManager_gainExp']=BattleManager[_0x45bccd(0x238)],BattleManager['gainExp']=function(){const _0x3e4e44=_0x45bccd;VisuMZ['ClassChangeSystem'][_0x3e4e44(0x252)][_0x3e4e44(0x42e)](this);const _0x160bd0=this[_0x3e4e44(0x44b)][_0x3e4e44(0x412)];for(const _0x149774 of $gameParty['allMembers']()){_0x149774[_0x3e4e44(0x270)](_0x160bd0);}},VisuMZ[_0x45bccd(0x3ac)]['BattleManager_endBattle']=BattleManager[_0x45bccd(0x288)],BattleManager[_0x45bccd(0x288)]=function(_0x555a83){const _0x49cab2=_0x45bccd;VisuMZ[_0x49cab2(0x3ac)]['BattleManager_endBattle']['call'](this,_0x555a83);for(const _0x93ca5e of $gameParty['allMembers']()){_0x93ca5e['checkForAutoClassUnlocks']();}},BattleManager[_0x45bccd(0x2f4)]=function(){const _0x1fe7e3=_0x45bccd;this[_0x1fe7e3(0x44b)]['classPoints']=$gameTroop['classPointsTotal']();},BattleManager[_0x45bccd(0x3dc)]=function(){const _0x4cfb8f=_0x45bccd;if(!this[_0x4cfb8f(0x2a8)]())return;$gameMessage[_0x4cfb8f(0x425)]();const _0x59026e=$gameParty['members'](),_0x3157bf=VisuMZ[_0x4cfb8f(0x3ac)][_0x4cfb8f(0x38c)][_0x4cfb8f(0x28e)],_0x36b17a=_0x3157bf[_0x4cfb8f(0x294)];for(const _0x14baa0 of _0x59026e){if(!_0x14baa0)continue;const _0x4c2722=_0x36b17a[_0x4cfb8f(0x3dd)](_0x14baa0['name'](),_0x14baa0['earnedClassPoints'](),TextManager[_0x4cfb8f(0x1e9)],TextManager['classPointsFmt']);$gameMessage[_0x4cfb8f(0x450)]('\x5c.'+_0x4c2722);}},BattleManager[_0x45bccd(0x285)]=function(){const _0x3695bb=_0x45bccd;this['_rewards'][_0x3695bb(0x26f)]=this['_rewards'][_0x3695bb(0x26f)]||0x0;let _0x3aec30=$gameParty[_0x3695bb(0x340)]();VisuMZ[_0x3695bb(0x3ac)][_0x3695bb(0x38c)]['ClassPoints'][_0x3695bb(0x3be)]&&(_0x3aec30=_0x3aec30['filter'](_0x52c4c8=>_0x52c4c8[_0x3695bb(0x1a5)]()));for(const _0x33bc3b of _0x3aec30){if(!_0x33bc3b)continue;if(!$dataSystem['optExtraExp']&&!_0x33bc3b['isBattleMember']())continue;_0x33bc3b['gainClassPoints'](this[_0x3695bb(0x44b)][_0x3695bb(0x26f)]),_0x33bc3b[_0x3695bb(0x3c3)](this[_0x3695bb(0x44b)][_0x3695bb(0x26f)]);}},BattleManager['classPointsVisible']=function(){const _0x54fb43=_0x45bccd;return VisuMZ[_0x54fb43(0x3ac)][_0x54fb43(0x38c)][_0x54fb43(0x28e)][_0x54fb43(0x301)];},BattleManager[_0x45bccd(0x197)]=function(){const _0xf6914d=_0x45bccd;this[_0xf6914d(0x44b)]['jobPoints']=$gameTroop['jobPointsTotal']();},BattleManager[_0x45bccd(0x254)]=function(){const _0x21192d=_0x45bccd;if(!this[_0x21192d(0x2a0)]())return;$gameMessage[_0x21192d(0x425)]();const _0x39137d=$gameParty[_0x21192d(0x317)](),_0x32ae7e=VisuMZ[_0x21192d(0x3ac)][_0x21192d(0x38c)][_0x21192d(0x1ea)],_0x2de44e=_0x32ae7e['VictoryText'];for(const _0x11232a of _0x39137d){if(_0x21192d(0x387)!==_0x21192d(0x32c)){if(!_0x11232a)continue;const _0x4e2421=_0x2de44e['format'](_0x11232a[_0x21192d(0x3ea)](),_0x11232a[_0x21192d(0x494)](),TextManager[_0x21192d(0x1fe)],TextManager[_0x21192d(0x1cd)]);$gameMessage[_0x21192d(0x450)]('\x5c.'+_0x4e2421);}else{function _0x49e0ce(){_0x567bca+=0x0;}}}},BattleManager['gainRewardsJobPoints']=function(){const _0x1bf8ae=_0x45bccd;this[_0x1bf8ae(0x44b)][_0x1bf8ae(0x4ad)]=this['_rewards']['jobPoints']||0x0;let _0x2bd564=$gameParty[_0x1bf8ae(0x340)]();VisuMZ[_0x1bf8ae(0x3ac)][_0x1bf8ae(0x38c)]['JobPoints'][_0x1bf8ae(0x3be)]&&(_0x2bd564=_0x2bd564['filter'](_0x400415=>_0x400415['isAlive']()));for(const _0x38208c of _0x2bd564){if(_0x1bf8ae(0x209)!==_0x1bf8ae(0x326)){if(!_0x38208c)continue;if(!$dataSystem[_0x1bf8ae(0x431)]&&!_0x38208c[_0x1bf8ae(0x33d)]())continue;_0x38208c[_0x1bf8ae(0x2ea)](this[_0x1bf8ae(0x44b)][_0x1bf8ae(0x4ad)]),_0x38208c[_0x1bf8ae(0x1d4)](this[_0x1bf8ae(0x44b)][_0x1bf8ae(0x4ad)]);}else{function _0x3a070c(){const _0x484ae7=_0x1bf8ae;_0x1ffe5d[_0x484ae7(0x3ac)][_0x484ae7(0x426)][_0x484ae7(0x42e)](this),this[_0x484ae7(0x46c)]();}}}},BattleManager[_0x45bccd(0x2a0)]=function(){const _0x732424=_0x45bccd;return VisuMZ[_0x732424(0x3ac)][_0x732424(0x38c)][_0x732424(0x1ea)][_0x732424(0x301)];},VisuMZ['ClassChangeSystem'][_0x45bccd(0x426)]=Game_System[_0x45bccd(0x404)][_0x45bccd(0x2b5)],Game_System[_0x45bccd(0x404)][_0x45bccd(0x2b5)]=function(){const _0x3196f0=_0x45bccd;VisuMZ[_0x3196f0(0x3ac)]['Game_System_initialize'][_0x3196f0(0x42e)](this),this['initClassChangeSystemMainMenu']();},Game_System['prototype'][_0x45bccd(0x46c)]=function(){const _0x45c436=_0x45bccd;this[_0x45c436(0x21f)]={'shown':VisuMZ[_0x45c436(0x3ac)][_0x45c436(0x38c)][_0x45c436(0x47b)][_0x45c436(0x221)],'enabled':VisuMZ['ClassChangeSystem'][_0x45c436(0x38c)][_0x45c436(0x47b)][_0x45c436(0x47e)]};},Game_System[_0x45bccd(0x404)][_0x45bccd(0x222)]=function(){const _0x1b2e13=_0x45bccd;if(this[_0x1b2e13(0x21f)]===undefined)this[_0x1b2e13(0x195)]();return this[_0x1b2e13(0x21f)][_0x1b2e13(0x37b)];},Game_System[_0x45bccd(0x404)][_0x45bccd(0x16a)]=function(_0x5a0437){const _0x5550eb=_0x45bccd;if(this[_0x5550eb(0x21f)]===undefined)this[_0x5550eb(0x195)]();this[_0x5550eb(0x21f)][_0x5550eb(0x37b)]=_0x5a0437;},Game_System[_0x45bccd(0x404)][_0x45bccd(0x36e)]=function(){const _0x14b920=_0x45bccd;if(this[_0x14b920(0x21f)]===undefined)this[_0x14b920(0x195)]();return this['_ClassChangeSystem_MainMenu'][_0x14b920(0x314)];},Game_System[_0x45bccd(0x404)][_0x45bccd(0x201)]=function(_0x23238b){const _0x30627e=_0x45bccd;if(this['_ClassChangeSystem_MainMenu']===undefined)this[_0x30627e(0x195)]();this[_0x30627e(0x21f)]['enabled']=_0x23238b;},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x1b3)]=Game_Action[_0x45bccd(0x404)][_0x45bccd(0x22d)],Game_Action[_0x45bccd(0x404)][_0x45bccd(0x22d)]=function(_0x2b01fb){const _0x9a35c2=_0x45bccd;VisuMZ['ClassChangeSystem'][_0x9a35c2(0x1b3)][_0x9a35c2(0x42e)](this,_0x2b01fb),this[_0x9a35c2(0x255)](_0x2b01fb);},Game_Action[_0x45bccd(0x404)][_0x45bccd(0x255)]=function(_0x3a24ed){const _0x463763=_0x45bccd;if(this[_0x463763(0x345)]())this[_0x463763(0x2b9)](_0x3a24ed);},Game_Action['prototype'][_0x45bccd(0x2b9)]=function(_0x4c0776){const _0x5b49de=_0x45bccd,_0x1ba4be=VisuMZ[_0x5b49de(0x3ac)][_0x5b49de(0x411)],_0x514cb4=this[_0x5b49de(0x345)]()['note'];if($gameParty[_0x5b49de(0x424)]()){if(this['subject']()[_0x5b49de(0x3a1)]()&&_0x514cb4['match'](_0x1ba4be[_0x5b49de(0x225)])){const _0x44a883=eval(RegExp['$1']);this[_0x5b49de(0x4a0)]()[_0x5b49de(0x3c7)](_0x44a883);}else this[_0x5b49de(0x1c0)]();if(_0x4c0776[_0x5b49de(0x3a1)]()&&_0x514cb4['match'](_0x1ba4be['TargetGainClassPoints'])){const _0x3f37b4=eval(RegExp['$1']);_0x4c0776[_0x5b49de(0x3c7)](_0x3f37b4);}}if($gameParty[_0x5b49de(0x424)]()){if(this[_0x5b49de(0x4a0)]()[_0x5b49de(0x3a1)]()&&_0x514cb4[_0x5b49de(0x264)](_0x1ba4be['UserGainJobPoints'])){const _0xd3e8a3=eval(RegExp['$1']);this['subject']()[_0x5b49de(0x2ea)](_0xd3e8a3);}else this[_0x5b49de(0x194)]();if(_0x4c0776[_0x5b49de(0x3a1)]()&&_0x514cb4[_0x5b49de(0x264)](_0x1ba4be[_0x5b49de(0x199)])){if(_0x5b49de(0x4a4)!==_0x5b49de(0x4a4)){function _0x31f0d0(){const _0x2a50dd=_0x5b49de;_0x445069=_0x5e291d[_0x2a50dd(0x1d8)](_0x33cd42);}}else{const _0x5ac1d8=eval(RegExp['$1']);_0x4c0776[_0x5b49de(0x2ea)](_0x5ac1d8);}}}if(_0x514cb4[_0x5b49de(0x264)](/<NOTETAG>/i)){}},Game_Action[_0x45bccd(0x404)][_0x45bccd(0x1c0)]=function(){const _0x579979=_0x45bccd;if(!$gameParty['inBattle']())return;if(!this[_0x579979(0x4a0)]()[_0x579979(0x3a1)]())return;const _0x4a2d4a=VisuMZ['ClassChangeSystem']['Settings'][_0x579979(0x28e)];let _0x3deae7=0x0;try{_0x3deae7=eval(_0x4a2d4a[_0x579979(0x27e)]);}catch(_0x572618){if(_0x579979(0x441)!==_0x579979(0x1f6)){if($gameTemp[_0x579979(0x2fa)]())console['log'](_0x572618);}else{function _0x4df44a(){const _0x186421=_0x579979,_0x3c3802=_0x1d7fac[_0x37f7fb-0x1];_0x3c3802&&(this[_0x186421(0x43d)](_0x4a9d54[_0x186421(0x27a)](_0x3c3802[_0x186421(0x4b0)])),this['drawText'](_0x3c3802[_0x186421(0x433)],_0x26f168['x'],_0x3e1e0e['y'],_0x5055bd[_0x186421(0x2c7)],_0x186421(0x2c9)),this[_0x186421(0x26a)]());}}}this[_0x579979(0x4a0)]()['gainClassPoints'](_0x3deae7);},Game_Action['prototype'][_0x45bccd(0x194)]=function(){const _0x311206=_0x45bccd;if(!$gameParty[_0x311206(0x424)]())return;if(!this['subject']()[_0x311206(0x3a1)]())return;const _0x5b2f62=VisuMZ['ClassChangeSystem'][_0x311206(0x38c)][_0x311206(0x1ea)];let _0x2c2260=0x0;try{_0x2c2260=eval(_0x5b2f62['PerAction']);}catch(_0x495c74){if($gameTemp[_0x311206(0x2fa)]())console[_0x311206(0x210)](_0x495c74);}this['subject']()[_0x311206(0x2ea)](_0x2c2260);},VisuMZ[_0x45bccd(0x3ac)]['Game_Battler_onBattleStart']=Game_Battler['prototype'][_0x45bccd(0x207)],Game_Battler[_0x45bccd(0x404)][_0x45bccd(0x207)]=function(_0x38b5bb){const _0x2d7a2c=_0x45bccd;VisuMZ['ClassChangeSystem'][_0x2d7a2c(0x4ba)][_0x2d7a2c(0x42e)](this,_0x38b5bb);if(this[_0x2d7a2c(0x3a1)]()){if(_0x2d7a2c(0x40d)==='GXtxH'){function _0x123f3f(){const _0x471fe1=_0x2d7a2c;this[_0x471fe1(0x45c)](_0x3892b3);}}else this['_earnedClassPoints']=this['getClassPoints'](),this[_0x2d7a2c(0x2e0)]=this['getJobPoints']();}},Game_Actor['CLASS_CHANGE_ADJUST_HP_MP']=VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x38c)][_0x45bccd(0x338)][_0x45bccd(0x41f)],VisuMZ['ClassChangeSystem'][_0x45bccd(0x3f6)]=Game_Actor['prototype'][_0x45bccd(0x40f)],Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x40f)]=function(_0xe5543d){const _0x4c5c7c=_0x45bccd;VisuMZ['ClassChangeSystem']['Game_Actor_setup']['call'](this,_0xe5543d),this['initClassPoints'](),this['gainStartingClassPoints'](),this[_0x4c5c7c(0x27d)](),this[_0x4c5c7c(0x1f2)](),this[_0x4c5c7c(0x25b)]();},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x25b)]=function(){const _0x36df40=_0x45bccd;this[_0x36df40(0x458)](),this['initMulticlass'](),this['initClassLevels'](),this[_0x36df40(0x242)](),this[_0x36df40(0x2df)](),this[_0x36df40(0x3d3)](),this['clearParamPlus'](),this[_0x36df40(0x2c2)]();},VisuMZ['ClassChangeSystem'][_0x45bccd(0x281)]=Game_Actor[_0x45bccd(0x404)]['changeClass'],Game_Actor[_0x45bccd(0x404)]['changeClass']=function(_0x4bd7c6,_0x4d6449){const _0x22b868=_0x45bccd;_0x4d6449=this['maintainLevels']();_0x4d6449&&(this[_0x22b868(0x258)]=this[_0x22b868(0x258)]||{},this[_0x22b868(0x258)][_0x4bd7c6]=this[_0x22b868(0x258)][this[_0x22b868(0x1a2)]]||0x0,_0x4d6449=![]);this[_0x22b868(0x313)]=!![];const _0x20d5f3=JsonEx['makeDeepCopy'](this);_0x20d5f3[_0x22b868(0x310)]=!![],VisuMZ[_0x22b868(0x3ac)][_0x22b868(0x281)][_0x22b868(0x42e)](this,_0x4bd7c6,_0x4d6449),this[_0x22b868(0x181)](_0x20d5f3),this['checkMulticlasses'](),this[_0x22b868(0x49a)](_0x4bd7c6),this['_ClassChangeSystem_preventLevelUpGain']=undefined;if($gamePlayer)$gamePlayer[_0x22b868(0x3d3)]();},VisuMZ[_0x45bccd(0x3ac)]['Game_Actor_tradeItemWithParty']=Game_Actor[_0x45bccd(0x404)]['tradeItemWithParty'],Game_Actor['prototype'][_0x45bccd(0x1d6)]=function(_0x4ce22c,_0x3a5484){const _0x298dec=_0x45bccd;if(this[_0x298dec(0x310)])return![];return VisuMZ['ClassChangeSystem'][_0x298dec(0x443)][_0x298dec(0x42e)](this,_0x4ce22c,_0x3a5484);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x1ae)]=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x18b)],Game_Actor[_0x45bccd(0x404)]['releaseUnequippableItems']=function(_0x4f3130){const _0x32a007=_0x45bccd;if($gameParty[_0x32a007(0x424)]())return;VisuMZ[_0x32a007(0x3ac)]['Game_Actor_releaseUnequippableItems']['call'](this,_0x4f3130);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x485)]=Game_Actor['prototype'][_0x45bccd(0x2b6)],Game_Actor['prototype']['levelUp']=function(){const _0x413690=_0x45bccd;VisuMZ[_0x413690(0x3ac)][_0x413690(0x485)][_0x413690(0x42e)](this);const _0x58a49d=this[_0x413690(0x2b3)]()['id'];this['levelUpGainClassPoints'](_0x58a49d),this[_0x413690(0x36f)](_0x58a49d),this[_0x413690(0x3e4)]=this['_classLevel']||{},this[_0x413690(0x3e4)][_0x58a49d]=this[_0x413690(0x415)];if(this[_0x413690(0x1a1)]()){if(_0x413690(0x3e1)!==_0x413690(0x374))this[_0x413690(0x2df)]();else{function _0x184368(){this['_earnedClassPoints']=this['getClassPoints'](),this['_earnedJobPoints']=this['getJobPoints']();}}}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x181)]=function(_0x2d7643){const _0xd38f39=_0x45bccd;if(!Game_Actor[_0xd38f39(0x3ca)])return;const _0x270c8e=Math['round'](_0x2d7643[_0xd38f39(0x202)]()*this[_0xd38f39(0x35f)]),_0x2f523f=Math[_0xd38f39(0x304)](_0x2d7643[_0xd38f39(0x1ce)]()*this[_0xd38f39(0x1f1)]);if(this['hp']>0x0)this[_0xd38f39(0x44e)](_0x270c8e);if(this['mp']>0x0)this[_0xd38f39(0x186)](_0x2f523f);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2bb)]=function(){this['_classPoints']={};},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x1be)]=function(){const _0xce6946=_0x45bccd,_0x1a92cb=VisuMZ['ClassChangeSystem']['RegExp'],_0x1f0034=this[_0xce6946(0x43f)]()[_0xce6946(0x323)];if(_0x1f0034[_0xce6946(0x264)](_0x1a92cb[_0xce6946(0x4bb)])){const _0x5e367a=eval(RegExp['$1']);this[_0xce6946(0x3c7)](_0x5e367a);}const _0x3d87f2=VisuMZ[_0xce6946(0x3ac)][_0xce6946(0x38c)][_0xce6946(0x28e)];if(!_0x3d87f2[_0xce6946(0x191)])return;const _0x57da9c=_0x1f0034[_0xce6946(0x264)](_0x1a92cb[_0xce6946(0x337)]);if(_0x57da9c)for(const _0x108362 of _0x57da9c){if(_0xce6946(0x2d0)===_0xce6946(0x2d0)){if(!_0x108362)continue;_0x108362[_0xce6946(0x264)](_0x1a92cb[_0xce6946(0x337)]);const _0x97b0cc=String(RegExp['$1']),_0x5de372=eval(RegExp['$2']),_0x172ef3=/^\d+$/[_0xce6946(0x464)](_0x97b0cc);let _0x334de4=0x0;if(_0x172ef3)_0x334de4=Number(_0x97b0cc);else{if(_0xce6946(0x24d)!==_0xce6946(0x24d)){function _0x5b06bf(){const _0xabebd6=_0xce6946;if(_0xf606b8==='AP'){const _0xb8f005=_0x3f0457[_0xabebd6(0x2ba)]();if(_0xb8f005<_0x23bdb2)return![];}else{if(_0x5c5357==='SP'){const _0x1731f1=_0x4048d5[_0xabebd6(0x19c)]();if(_0x1731f1<_0x38ffed)return![];}}}}else _0x334de4=DataManager[_0xce6946(0x1d8)](_0x97b0cc);}this[_0xce6946(0x3c7)](_0x5de372,_0x334de4);}else{function _0x31ff38(){const _0xe2db6=_0xce6946;return this[_0xe2db6(0x253)]&&this[_0xe2db6(0x253)][_0xe2db6(0x4a9)];}}}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x478)]=function(_0x1bae0c){const _0x335b87=_0x45bccd;this['_classPoints']===undefined&&this[_0x335b87(0x2bb)]();const _0x5cf6b1=VisuMZ['ClassChangeSystem'][_0x335b87(0x38c)][_0x335b87(0x28e)];return _0x5cf6b1[_0x335b87(0x191)]?_0x1bae0c=0x0:_0x1bae0c=_0x1bae0c||this[_0x335b87(0x2b3)]()['id'],this[_0x335b87(0x2cd)][_0x1bae0c]=this[_0x335b87(0x2cd)][_0x1bae0c]||0x0,Math['round'](this[_0x335b87(0x2cd)][_0x1bae0c]);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x24a)]=function(_0x451936,_0x23fe7d){const _0x5ada6b=_0x45bccd;this[_0x5ada6b(0x2cd)]===undefined&&this[_0x5ada6b(0x2bb)]();const _0x48e8dc=VisuMZ['ClassChangeSystem'][_0x5ada6b(0x38c)][_0x5ada6b(0x28e)];if(_0x48e8dc['SharedResource']){if('iNass'!==_0x5ada6b(0x224))_0x23fe7d=0x0;else{function _0x40610a(){const _0x6a1bee=_0x5ada6b;_0x32ddf3[_0x6a1bee(0x48b)](_0x6a1bee(0x3d1),this['nextActor']['bind'](this)),_0x215b58[_0x6a1bee(0x48b)](_0x6a1bee(0x407),this[_0x6a1bee(0x25f)][_0x6a1bee(0x187)](this));}}}else _0x23fe7d=_0x23fe7d||this[_0x5ada6b(0x2b3)]()['id'];this[_0x5ada6b(0x2cd)][_0x23fe7d]=this[_0x5ada6b(0x2cd)][_0x23fe7d]||0x0,this[_0x5ada6b(0x2cd)][_0x23fe7d]=Math[_0x5ada6b(0x304)](_0x451936||0x0);const _0x1a4707=_0x48e8dc[_0x5ada6b(0x1de)]||Number[_0x5ada6b(0x307)];this['_classPoints'][_0x23fe7d]=this['_classPoints'][_0x23fe7d][_0x5ada6b(0x37e)](0x0,_0x1a4707);},Game_Actor['prototype']['gainClassPoints']=function(_0x10253d,_0x42ec5f){const _0x36ebbf=_0x45bccd;_0x10253d>0x0&&(_0x10253d*=this[_0x36ebbf(0x2f9)]()),this[_0x36ebbf(0x33f)](_0x10253d,_0x42ec5f);},Game_Actor['prototype'][_0x45bccd(0x3c3)]=function(_0x1b23f3){const _0x129197=_0x45bccd;if(!Imported['VisuMZ_2_ClassChangeSystem'])return;_0x1b23f3>0x0&&(_0x1b23f3*=this[_0x129197(0x2f9)]()),this[_0x129197(0x42b)](_0x1b23f3,_0x129197(0x25c));},Game_Actor[_0x45bccd(0x404)]['addClassPoints']=function(_0x1687e4,_0x42140c){const _0x273b22=_0x45bccd,_0x5eaf44=VisuMZ['ClassChangeSystem']['Settings'][_0x273b22(0x28e)];_0x5eaf44[_0x273b22(0x191)]?_0x42140c=0x0:_0x42140c=_0x42140c||this['currentClass']()['id'],_0x1687e4+=this[_0x273b22(0x478)](_0x42140c),this[_0x273b22(0x24a)](_0x1687e4,_0x42140c);},Game_Actor[_0x45bccd(0x404)]['loseClassPoints']=function(_0x1575e3,_0x3f22e7){const _0x5799ff=_0x45bccd;this[_0x5799ff(0x33f)](-_0x1575e3,_0x3f22e7);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2f9)]=function(){const _0x6a913c=_0x45bccd;return this[_0x6a913c(0x30c)]()['reduce']((_0x34f455,_0x4c680a)=>{const _0x93f657=_0x6a913c;if(_0x4c680a&&_0x4c680a[_0x93f657(0x323)][_0x93f657(0x264)](VisuMZ[_0x93f657(0x3ac)][_0x93f657(0x411)][_0x93f657(0x45d)])){if(_0x93f657(0x26b)===_0x93f657(0x26b))return _0x34f455*(Number(RegExp['$1'])*0.01);else{function _0x260cad(){const _0x15ce4e=_0x93f657;_0x22f844[_0x15ce4e(0x404)]['create']['call'](this),this['createHelpWindow'](),this[_0x15ce4e(0x41a)](),this[_0x15ce4e(0x2cc)](),this[_0x15ce4e(0x2db)](),this[_0x15ce4e(0x309)](),this['refreshActor']();}}}else{if(_0x93f657(0x346)!==_0x93f657(0x346)){function _0x19cb9c(){const _0x1f555e=_0x93f657;try{return _0x1517e3(_0x1670b8['$1']);}catch(_0x3a9111){if(_0x12656b[_0x1f555e(0x2fa)]())_0x23a5b8[_0x1f555e(0x210)](_0x3a9111);return 0x0;}}}else return _0x34f455;}},0x1);},Game_Actor['prototype']['levelUpGainClassPoints']=function(_0x3b9eda){const _0x321bc4=_0x45bccd;if(this[_0x321bc4(0x313)])return;const _0x37d0df=VisuMZ[_0x321bc4(0x3ac)]['Settings']['ClassPoints'];let _0x37cedf=0x0;try{_0x37cedf=eval(_0x37d0df[_0x321bc4(0x2f6)]);}catch(_0x49a412){if(_0x321bc4(0x39d)!==_0x321bc4(0x3fe)){if($gameTemp[_0x321bc4(0x2fa)]())console[_0x321bc4(0x210)](_0x49a412);}else{function _0x4d1fc5(){const _0x12dea1=_0x321bc4;this[_0x12dea1(0x25a)][_0x4ce869]===_0x2f7854&&(this['_multiclasses'][_0x5becdd]=0x0);}}}this[_0x321bc4(0x3c7)](_0x37cedf,_0x3b9eda);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x1ed)]=function(){const _0x15fe97=_0x45bccd;return this['_earnedClassPoints']=this['_earnedClassPoints']||0x0,this[_0x15fe97(0x478)]()-this['_earnedClassPoints'];},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x27d)]=function(){const _0x57b483=_0x45bccd;this[_0x57b483(0x2e5)]={};},Game_Actor['prototype']['gainStartingJobPoints']=function(){const _0x14c0de=_0x45bccd,_0x59b3cf=VisuMZ[_0x14c0de(0x3ac)][_0x14c0de(0x411)],_0x3c22dc=this[_0x14c0de(0x43f)]()[_0x14c0de(0x323)];if(_0x3c22dc[_0x14c0de(0x264)](_0x59b3cf['StartingJobPoints'])){if(_0x14c0de(0x30e)===_0x14c0de(0x30e)){const _0x15f863=eval(RegExp['$1']);this[_0x14c0de(0x2ea)](_0x15f863);}else{function _0x1c125e(){_0xe61d26=_0x97f082['getClassIdWithName'](_0x148871);}}}const _0x244f6=VisuMZ['ClassChangeSystem'][_0x14c0de(0x38c)][_0x14c0de(0x1ea)];if(!_0x244f6[_0x14c0de(0x191)])return;const _0x4ac1a5=_0x3c22dc[_0x14c0de(0x264)](_0x59b3cf[_0x14c0de(0x18d)]);if(_0x4ac1a5)for(const _0x4a6486 of _0x4ac1a5){if(!_0x4a6486)continue;_0x4a6486[_0x14c0de(0x264)](_0x59b3cf['StartClassJobPoints']);const _0xef0750=String(RegExp['$1']),_0x3db69e=eval(RegExp['$2']),_0x2db595=/^\d+$/[_0x14c0de(0x464)](_0xef0750);let _0x150565=0x0;if(_0x2db595)_0x150565=Number(_0xef0750);else{if(_0x14c0de(0x4a3)!==_0x14c0de(0x4a3)){function _0xaeaf3(){const _0x9e8ef3=_0xe7c506['getJobPoints']();if(_0x9e8ef3<_0x31564f)return![];}}else _0x150565=DataManager[_0x14c0de(0x1d8)](_0xef0750);}this['gainJobPoints'](_0x3db69e,_0x150565);}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3e8)]=function(_0x3818fc){const _0x3f5ddb=_0x45bccd;this[_0x3f5ddb(0x2e5)]===undefined&&this[_0x3f5ddb(0x27d)]();const _0x24a5a5=VisuMZ['ClassChangeSystem'][_0x3f5ddb(0x38c)][_0x3f5ddb(0x1ea)];return _0x24a5a5[_0x3f5ddb(0x191)]?_0x3818fc=0x0:_0x3818fc=_0x3818fc||this['currentClass']()['id'],this[_0x3f5ddb(0x2e5)][_0x3818fc]=this['_jobPoints'][_0x3818fc]||0x0,Math['round'](this[_0x3f5ddb(0x2e5)][_0x3818fc]);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x21d)]=function(_0x4cf2ea,_0x53e06e){const _0x10e80e=_0x45bccd;if(this[_0x10e80e(0x2e5)]===undefined){if(_0x10e80e(0x33e)===_0x10e80e(0x466)){function _0x320b50(){_0x410242+=_0x26a5f5['round']((_0x5e1285-_0x4999df)/0x2);}}else this[_0x10e80e(0x27d)]();}const _0x2283e5=VisuMZ[_0x10e80e(0x3ac)][_0x10e80e(0x38c)][_0x10e80e(0x1ea)];if(_0x2283e5['SharedResource']){if(_0x10e80e(0x171)!==_0x10e80e(0x171)){function _0x17e0bc(){const _0x1bd5fb=_0x10e80e;_0x5aef1a[_0x1bd5fb(0x483)](_0x1feb79,_0x154ed2);const _0x47864b=_0x2959aa[_0x1bd5fb(0x353)];for(const _0x16ab18 of _0x47864b){_0x49ba34[_0x1bd5fb(0x3f0)](_0x16ab18);}}}else _0x53e06e=0x0;}else _0x53e06e=_0x53e06e||this[_0x10e80e(0x2b3)]()['id'];this[_0x10e80e(0x2e5)][_0x53e06e]=this[_0x10e80e(0x2e5)][_0x53e06e]||0x0,this[_0x10e80e(0x2e5)][_0x53e06e]=Math[_0x10e80e(0x304)](_0x4cf2ea||0x0);const _0x164b57=_0x2283e5[_0x10e80e(0x1de)]||Number[_0x10e80e(0x307)];this[_0x10e80e(0x2e5)][_0x53e06e]=this[_0x10e80e(0x2e5)][_0x53e06e][_0x10e80e(0x37e)](0x0,_0x164b57);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2ea)]=function(_0x75a5b9,_0x58e1aa){const _0x54be8f=_0x45bccd;_0x75a5b9>0x0&&(_0x75a5b9*=this['jobPointsRate']()),this[_0x54be8f(0x418)](_0x75a5b9,_0x58e1aa);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x1d4)]=function(_0x4993c6){const _0x3f2f19=_0x45bccd;if(!Imported['VisuMZ_2_ClassChangeSystem'])return;_0x4993c6>0x0&&(_0x4993c6*=this[_0x3f2f19(0x166)]()),this['gainMulticlassRewardPoints'](_0x4993c6,_0x3f2f19(0x406));},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x418)]=function(_0x616d4f,_0x59d0b1){const _0x1fc0ff=_0x45bccd,_0x4a4490=VisuMZ[_0x1fc0ff(0x3ac)][_0x1fc0ff(0x38c)][_0x1fc0ff(0x1ea)];_0x4a4490[_0x1fc0ff(0x191)]?_0x59d0b1=0x0:_0x59d0b1=_0x59d0b1||this[_0x1fc0ff(0x2b3)]()['id'],_0x616d4f+=this[_0x1fc0ff(0x3e8)](_0x59d0b1),this['setJobPoints'](_0x616d4f,_0x59d0b1);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x231)]=function(_0x25c759,_0x4e835c){const _0x5dd63c=_0x45bccd;this[_0x5dd63c(0x418)](-_0x25c759,_0x4e835c);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x166)]=function(){const _0xbee050=_0x45bccd;return this['traitObjects']()[_0xbee050(0x31b)]((_0x25f1ae,_0xcf31a5)=>{const _0x6e126d=_0xbee050;if(_0xcf31a5&&_0xcf31a5[_0x6e126d(0x323)][_0x6e126d(0x264)](VisuMZ[_0x6e126d(0x3ac)][_0x6e126d(0x411)]['JobPointsRate'])){if(_0x6e126d(0x1da)!==_0x6e126d(0x1da)){function _0x14e149(){const _0x1f618e=_0x6e126d;this[_0x1f618e(0x232)]='EquipWeapons';}}else return _0x25f1ae*(Number(RegExp['$1'])*0.01);}else return _0x25f1ae;},0x1);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x36f)]=function(_0x165474){const _0x406733=_0x45bccd;if(this[_0x406733(0x313)])return;const _0x808ce6=VisuMZ[_0x406733(0x3ac)][_0x406733(0x38c)][_0x406733(0x1ea)];let _0x14d1af=0x0;try{if(_0x406733(0x2d5)!==_0x406733(0x1d9))_0x14d1af=eval(_0x808ce6[_0x406733(0x2f6)]);else{function _0x272c81(){return _0x3aec78(_0x4a5db5['$1']);}}}catch(_0x3e5007){if($gameTemp[_0x406733(0x2fa)]())console[_0x406733(0x210)](_0x3e5007);}this[_0x406733(0x2ea)](_0x14d1af,_0x165474);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x494)]=function(){const _0x3ae78d=_0x45bccd;return this['_earnedJobPoints']=this[_0x3ae78d(0x2e0)]||0x0,this[_0x3ae78d(0x3e8)]()-this[_0x3ae78d(0x2e0)];},VisuMZ[_0x45bccd(0x3ac)]['Game_Actor_setFaceImage']=Game_Actor[_0x45bccd(0x404)]['setFaceImage'],Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x208)]=function(_0x5a6f4f,_0x15e494){const _0x399560=_0x45bccd;_0x5a6f4f!==''?(this[_0x399560(0x48d)]=_0x5a6f4f,this[_0x399560(0x330)]=_0x15e494):(this[_0x399560(0x48d)]=undefined,this[_0x399560(0x330)]=undefined);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x34c)]=Game_Actor['prototype'][_0x45bccd(0x3b8)],Game_Actor[_0x45bccd(0x404)]['faceName']=function(){const _0x2d06f0=_0x45bccd;if(this['_priorityFaceName']!==undefined){if(_0x2d06f0(0x1a4)===_0x2d06f0(0x1a4))return this[_0x2d06f0(0x48d)];else{function _0xb0b210(){const _0x5875d2=_0x2d06f0;if(this[_0x5875d2(0x3a1)]())this['_multiclassCheck']=_0x5875d2(0x37c);return _0x6a6d82[_0x5875d2(0x3ac)]['Game_BattlerBase_addedSkillTypes'][_0x5875d2(0x42e)](this);}}}return ImageManager[_0x2d06f0(0x2c8)](this)||VisuMZ['ClassChangeSystem'][_0x2d06f0(0x34c)][_0x2d06f0(0x42e)](this);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x3d6)]=Game_Actor['prototype'][_0x45bccd(0x23b)],Game_Actor['prototype'][_0x45bccd(0x23b)]=function(){const _0x4e28d8=_0x45bccd;if(this['_priorityFaceIndex']!==undefined){if(_0x4e28d8(0x457)!=='rqVCB')return this[_0x4e28d8(0x330)];else{function _0x53123e(){const _0x46f5a0=_0x4e28d8,_0x4effc9=this[_0x46f5a0(0x43f)]();_0x4effc9[_0x46f5a0(0x2e8)](),this[_0x46f5a0(0x303)][_0x46f5a0(0x205)](_0x4effc9),this['_classTierWindow'][_0x46f5a0(0x205)](_0x4effc9),this[_0x46f5a0(0x3ed)][_0x46f5a0(0x205)](_0x4effc9);}}}const _0xa76e54=ImageManager[_0x4e28d8(0x481)](this);if(_0xa76e54!==undefined)return _0xa76e54;return VisuMZ[_0x4e28d8(0x3ac)][_0x4e28d8(0x3d6)][_0x4e28d8(0x42e)](this);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x395)]=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3ad)],Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3ad)]=function(_0x442d29,_0x3a6514){const _0x19cac2=_0x45bccd;if(_0x442d29!=='')this['_priorityCharacterName']=_0x442d29,this[_0x19cac2(0x1fd)]=_0x3a6514;else{if(_0x19cac2(0x470)!=='wqluX')this[_0x19cac2(0x336)]=undefined,this[_0x19cac2(0x1fd)]=undefined;else{function _0x581a20(){_0x18f69a['apply'](this,arguments);return;}}}},VisuMZ[_0x45bccd(0x3ac)]['Game_Actor_characterName']=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x38d)],Game_Actor['prototype'][_0x45bccd(0x38d)]=function(){const _0x5156bf=_0x45bccd;if(this[_0x5156bf(0x336)]!==undefined)return this[_0x5156bf(0x336)];return ImageManager[_0x5156bf(0x33a)](this)||VisuMZ[_0x5156bf(0x3ac)][_0x5156bf(0x2aa)][_0x5156bf(0x42e)](this);},VisuMZ['ClassChangeSystem']['Game_Actor_characterIndex']=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x388)],Game_Actor[_0x45bccd(0x404)]['characterIndex']=function(){const _0x5e4b06=_0x45bccd;if(this[_0x5e4b06(0x1fd)]!==undefined){if('jcZKC'!=='jcZKC'){function _0x5c80d5(){const _0x5a8817=_0x5e4b06;_0x6eb388['prototype'][_0x5a8817(0x3b1)][_0x5a8817(0x42e)](this,_0x12f7e5,_0x2d2dd5,_0x21eb63,_0x419903,_0xf12043);}}else return this[_0x5e4b06(0x1fd)];}const _0x45c328=ImageManager['getActorClassCharacterIndex'](this);if(_0x45c328!==undefined)return _0x45c328;return VisuMZ[_0x5e4b06(0x3ac)][_0x5e4b06(0x1f0)]['call'](this);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x477)]=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3d2)],Game_Actor['prototype'][_0x45bccd(0x3d2)]=function(_0x134def){const _0x4f1a3d=_0x45bccd;if(_0x134def!==''){if(_0x4f1a3d(0x248)===_0x4f1a3d(0x248))this[_0x4f1a3d(0x45e)]=_0x134def;else{function _0x2ffd42(){const _0x59270d=_0x4f1a3d;return this['deadMembers']()[_0x59270d(0x31b)]((_0x10994c,_0x9dcc57)=>_0x10994c+_0x9dcc57['classPoints'](),0x0);}}}else{if(_0x4f1a3d(0x1f9)==='xmPUu'){function _0x55c3c6(){const _0x3a6c57=_0x4f1a3d;return _0x3a6c57(0x223)[_0x3a6c57(0x3dd)](_0x1ad141(_0x1469d7['$1']));}}else this['_priorityBattlerName']=undefined;}},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x21b)]=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x492)],Game_Actor['prototype'][_0x45bccd(0x492)]=function(){const _0x2ef38a=_0x45bccd;if(this['_priorityBattlerName']!==undefined)return this[_0x2ef38a(0x45e)];return ImageManager[_0x2ef38a(0x2e2)](this)||VisuMZ[_0x2ef38a(0x3ac)][_0x2ef38a(0x21b)][_0x2ef38a(0x42e)](this);;},VisuMZ['ClassChangeSystem']['Game_Actor_setMenuImage']=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x259)],Game_Actor['prototype'][_0x45bccd(0x259)]=function(_0x4e9ce3){const _0x9e477e=_0x45bccd;_0x4e9ce3!==''?this[_0x9e477e(0x1af)]=_0x4e9ce3:this[_0x9e477e(0x1af)]=undefined;},VisuMZ[_0x45bccd(0x3ac)]['Game_Actor_getMenuImage']=Game_Actor[_0x45bccd(0x404)]['getMenuImage'],Game_Actor['prototype'][_0x45bccd(0x462)]=function(){const _0x4891f5=_0x45bccd;if(this[_0x4891f5(0x1af)]!==undefined){if(_0x4891f5(0x308)!=='SaXIn')return this[_0x4891f5(0x1af)];else{function _0x23aa13(){const _0x5e5508=_0x4891f5;this[_0x5e5508(0x242)]();}}}return ImageManager[_0x4891f5(0x347)](this)||VisuMZ[_0x4891f5(0x3ac)][_0x4891f5(0x250)][_0x4891f5(0x42e)](this);;},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x2e7)]=Game_Actor[_0x45bccd(0x404)]['setBattlePortrait'],Game_Actor[_0x45bccd(0x404)]['setBattlePortrait']=function(_0x45d068){const _0x281f16=_0x45bccd;_0x45d068!==''?this[_0x281f16(0x20f)]=_0x45d068:this[_0x281f16(0x20f)]=undefined;if(SceneManager[_0x281f16(0x1b9)]()&&$gameParty[_0x281f16(0x3f8)]()[_0x281f16(0x3f5)](this)){const _0x19938d=SceneManager[_0x281f16(0x41e)][_0x281f16(0x303)];if(_0x19938d)_0x19938d[_0x281f16(0x2a5)](this);}},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x42c)]=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x1a6)],Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x1a6)]=function(){const _0x3e70cd=_0x45bccd;if(this[_0x3e70cd(0x20f)]!==undefined)return this[_0x3e70cd(0x20f)];return ImageManager[_0x3e70cd(0x39e)](this)||VisuMZ['ClassChangeSystem'][_0x3e70cd(0x42c)]['call'](this);;},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x458)]=function(){const _0x22805f=_0x45bccd;this[_0x22805f(0x20a)]=[this['currentClass']()['id']];const _0x43fb3d=VisuMZ['ClassChangeSystem'][_0x22805f(0x411)],_0x473b54=this[_0x22805f(0x43f)]()[_0x22805f(0x323)],_0x5712f8=_0x473b54[_0x22805f(0x264)](_0x43fb3d['ActorUnlockedClasses']);if(_0x5712f8)for(const _0x328f07 of _0x5712f8){if(_0x22805f(0x44f)!==_0x22805f(0x3fc)){if(!_0x328f07)continue;_0x328f07[_0x22805f(0x264)](_0x43fb3d[_0x22805f(0x2bc)]);const _0x24ee92=String(RegExp['$1'])['split'](',');for(let _0x1748c3 of _0x24ee92){_0x1748c3=(String(_0x1748c3)||'')[_0x22805f(0x1b5)]();const _0x1980c9=/^\d+$/['test'](_0x1748c3);_0x1980c9?this[_0x22805f(0x20a)][_0x22805f(0x3ab)](Number(_0x1748c3)):this['_unlockedClasses'][_0x22805f(0x3ab)](DataManager[_0x22805f(0x1d8)](_0x1748c3));}}else{function _0x27b0af(){const _0x38e75b=_0x22805f;this[_0x38e75b(0x45f)](_0x14c87a,_0x3339e8);}}}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x29b)]=function(){if(this['_unlockedClasses']===undefined)this['initClassChangeUnlocks']();return this['_unlockedClasses'];},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x45c)]=function(_0x5c8638){const _0x565b64=_0x45bccd;if(this[_0x565b64(0x20a)]===undefined)this[_0x565b64(0x458)]();if(this['_unlockedClasses'][_0x565b64(0x3f5)](_0x5c8638))return;this[_0x565b64(0x20a)][_0x565b64(0x3ab)](_0x5c8638),this[_0x565b64(0x20a)][_0x565b64(0x306)](0x0),this['_unlockedClasses']['sort'](function(_0x43e96f,_0x636f14){return _0x43e96f-_0x636f14;});},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3f0)]=function(_0x21ce90){const _0xcb332f=_0x45bccd;if(this[_0xcb332f(0x20a)]===undefined)this[_0xcb332f(0x458)]();if(!this[_0xcb332f(0x20a)][_0xcb332f(0x3f5)](_0x21ce90))return;this[_0xcb332f(0x20a)][_0xcb332f(0x306)](_0x21ce90)['remove'](null),this[_0xcb332f(0x20a)][_0xcb332f(0x2fb)](function(_0x2bb5c2,_0x584383){const _0x463afb=_0xcb332f;if('oMYZn'===_0x463afb(0x29e)){function _0x2649e4(){const _0x34c992=_0x463afb;let _0x2e8916='';if(_0x42af2c[_0x34c992(0x3f5)](_0x3641fd))_0x2e8916=_0x34c992(0x390)[_0x34c992(0x3dd)](_0x44a60b['id'],_0x4ad1a2);if(_0x426db4[_0x34c992(0x3f5)](_0xab1c6))_0x2e8916=_0x34c992(0x40a)[_0x34c992(0x3dd)](_0x4619b6['id'],_0x5425a1);if(_0x955a83['includes'](_0x3bdbf3))_0x2e8916='Skill-%1-%2'[_0x34c992(0x3dd)](_0x359968['id'],_0xccd0cc);if(_0x33dd39['includes'](_0x4c554e))_0x2e8916=_0x34c992(0x3de)[_0x34c992(0x3dd)](_0xb6566f['id'],_0x228e9f);if(_0x446d53[_0x34c992(0x3f5)](_0x50f665))_0x2e8916=_0x34c992(0x23e)['format'](_0xc5a2e9['id'],_0x55e0a0);if(_0x5f5d79[_0x34c992(0x3f5)](_0x118962))_0x2e8916=_0x34c992(0x2af)['format'](_0x5b15bb['id'],_0x2b0df0);if(_0x38863d[_0x34c992(0x3f5)](_0x4e8eda))_0x2e8916='Enemy-%1-%2'[_0x34c992(0x3dd)](_0x1fabf7['id'],_0x554db7);if(_0x4097d9[_0x34c992(0x3f5)](_0x337a7c))_0x2e8916=_0x34c992(0x247)[_0x34c992(0x3dd)](_0x7b303d['id'],_0xbb57b7);return _0x2e8916;}}else return _0x2bb5c2-_0x584383;});},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x49a)]=function(_0x5744fa){const _0x1584d6=_0x45bccd;this[_0x1584d6(0x45c)](_0x5744fa);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3a6)]=function(){const _0xbd1a7b=_0x45bccd;this[_0xbd1a7b(0x2e6)]=VisuMZ[_0xbd1a7b(0x3ac)][_0xbd1a7b(0x38c)][_0xbd1a7b(0x338)][_0xbd1a7b(0x339)],this[_0xbd1a7b(0x25a)]=[this[_0xbd1a7b(0x1a2)]];const _0x4a8aa8=this[_0xbd1a7b(0x43f)]()[_0xbd1a7b(0x323)],_0x4de588=VisuMZ[_0xbd1a7b(0x3ac)]['RegExp'];_0x4a8aa8['match'](_0x4de588[_0xbd1a7b(0x339)])&&(this[_0xbd1a7b(0x2e6)]=Number(RegExp['$1']));const _0x367c6b=_0x4a8aa8[_0xbd1a7b(0x264)](_0x4de588[_0xbd1a7b(0x16f)]);if(_0x367c6b)for(const _0x3f91d2 of _0x367c6b){if(!_0x3f91d2)continue;_0x3f91d2[_0xbd1a7b(0x264)](_0x4de588['StartingClassTier']);const _0x31f70e=Number(RegExp['$1'])-0x1;if(_0x31f70e+0x1>this[_0xbd1a7b(0x2e6)])continue;let _0x330a28=(String(RegExp['$2'])||'')[_0xbd1a7b(0x1b5)]();const _0xd3a009=/^\d+$/['test'](_0x330a28);_0xd3a009?this[_0xbd1a7b(0x25a)][_0x31f70e]=Number(_0x330a28):this[_0xbd1a7b(0x25a)][_0x31f70e]=DataManager['getClassIdWithName'](_0x330a28);}this[_0xbd1a7b(0x46e)](),this['_multiclassTiers']=this[_0xbd1a7b(0x2e6)][_0xbd1a7b(0x37e)](0x1,VisuMZ[_0xbd1a7b(0x3ac)][_0xbd1a7b(0x38c)][_0xbd1a7b(0x2bd)][_0xbd1a7b(0x38a)]||0x1);for(const _0x54ca67 of this['_multiclasses']){if(_0xbd1a7b(0x40e)!==_0xbd1a7b(0x279))this['unlockClass'](_0x54ca67);else{function _0x175e8f(){const _0x3e90e5=_0xbd1a7b,_0x51a455=this[_0x3e90e5(0x1c6)](),_0x2188c3=new _0xd19cd3(_0x51a455);_0x2188c3[_0x3e90e5(0x22e)](this['_helpWindow']),_0x2188c3[_0x3e90e5(0x383)](this[_0x3e90e5(0x303)]),_0x2188c3[_0x3e90e5(0x2bf)](_0x109978[_0x3e90e5(0x3ac)]['Settings'][_0x3e90e5(0x239)][_0x3e90e5(0x46a)]),this['addWindow'](_0x2188c3),this[_0x3e90e5(0x3ed)]=_0x2188c3,_0x2188c3[_0x3e90e5(0x48b)]('cancel',this[_0x3e90e5(0x1d5)][_0x3e90e5(0x187)](this)),this[_0x3e90e5(0x39b)]()<=0x1&&(_0x2188c3[_0x3e90e5(0x48b)](_0x3e90e5(0x3d1),this['nextActor'][_0x3e90e5(0x187)](this)),_0x2188c3[_0x3e90e5(0x48b)](_0x3e90e5(0x407),this[_0x3e90e5(0x25f)][_0x3e90e5(0x187)](this))),_0x2188c3[_0x3e90e5(0x48b)]('classChange',this[_0x3e90e5(0x4a2)]['bind'](this));}}}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x405)]=function(){const _0x30728f=_0x45bccd;if(this[_0x30728f(0x25a)]===undefined)this['initMulticlass']();return this[_0x30728f(0x25a)][0x0]=this[_0x30728f(0x1a2)],this[_0x30728f(0x25a)][_0x30728f(0x287)](_0x4d9c0c=>!!$dataClasses[_0x4d9c0c])[_0x30728f(0x361)](_0x680ee=>$dataClasses[_0x680ee]);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x233)]=function(){const _0x239bdc=_0x45bccd;return this[_0x239bdc(0x405)]();},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2da)]=function(_0xd110ac){const _0x50de3a=_0x45bccd;if(this[_0x50de3a(0x25a)]===undefined)this['initMulticlass']();return _0xd110ac-=0x1,$dataClasses[this['_multiclasses'][_0xd110ac]]||null;},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x4b2)]=function(_0xde084){const _0x36bd90=_0x45bccd;return this[_0x36bd90(0x2da)](_0xde084);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x350)]=function(_0x2daf99){const _0x3f9886=_0x45bccd,_0xce7aa0=this[_0x3f9886(0x2da)](_0x2daf99);return _0xce7aa0?_0xce7aa0['id']:0x0;},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x295)]=function(){const _0x13fde8=_0x45bccd;if(this[_0x13fde8(0x2e6)]===undefined)this[_0x13fde8(0x3a6)]();return this[_0x13fde8(0x2e6)]=this['_multiclassTiers'][_0x13fde8(0x37e)](0x1,VisuMZ['ClassChangeSystem']['Settings'][_0x13fde8(0x2bd)]['length']||0x1),this[_0x13fde8(0x2e6)];},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x1f7)]=function(_0x48e051){const _0x45fda5=_0x45bccd;if(this[_0x45fda5(0x2e6)]===undefined)this['initMulticlass']();this['_multiclassTiers']=_0x48e051[_0x45fda5(0x37e)](0x1,VisuMZ[_0x45fda5(0x3ac)][_0x45fda5(0x38c)][_0x45fda5(0x2bd)][_0x45fda5(0x38a)]||0x1);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x493)]=function(_0x251e6d){const _0x1db7d8=_0x45bccd;_0x251e6d+=this[_0x1db7d8(0x295)](),this['setMulticlassTiers'](_0x251e6d);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2ac)]=function(_0x310062){const _0x4fe7d7=_0x45bccd;_0x310062=this[_0x4fe7d7(0x295)]()-_0x310062,this[_0x4fe7d7(0x1f7)](_0x310062);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x46e)]=function(){const _0xbb80cc=_0x45bccd;if(this[_0xbb80cc(0x25a)]===undefined)this[_0xbb80cc(0x3a6)]();let _0x282571=![];const _0x171bcf=this['totalMulticlass']();while(this['_multiclasses'][_0xbb80cc(0x38a)]>_0x171bcf){if(_0xbb80cc(0x370)!==_0xbb80cc(0x1c4))_0x282571=!![],this[_0xbb80cc(0x25a)][_0xbb80cc(0x1d1)]();else{function _0x48e823(){const _0x407ac3=_0xbb80cc;return this['_classListWindow']&&this[_0x407ac3(0x3ed)]['active'];}}}this[_0xbb80cc(0x25a)][0x0]=this[_0xbb80cc(0x2b3)]()['id'];const _0x44abf4=this[_0xbb80cc(0x25a)]['length'];for(let _0xd70fe5=0x1;_0xd70fe5<_0x44abf4;_0xd70fe5++){if('ndCiS'===_0xbb80cc(0x3b3)){function _0xd4dbaa(){const _0x409e79=_0xbb80cc;_0x2c58b8[_0x409e79(0x33f)](_0x44d333,_0xc02748);}}else{if(this[_0xbb80cc(0x25a)][_0xd70fe5]===this[_0xbb80cc(0x2b3)]()['id']){if('pGmkA'===_0xbb80cc(0x3e3)){function _0x1b927e(){_0x332462=_0xbf2e07(_0x23c14b);}}else this[_0xbb80cc(0x25a)][_0xd70fe5]=0x0,_0x282571=!![];}}}if(_0x282571)this[_0xbb80cc(0x3d3)]();},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x3a7)]=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x448)],Game_BattlerBase['prototype']['elementRate']=function(_0x1beb5b){const _0x520aa5=_0x45bccd;if(this[_0x520aa5(0x3a1)]())this['_multiclassCheck']=_0x520aa5(0x1fb);return VisuMZ[_0x520aa5(0x3ac)][_0x520aa5(0x3a7)]['call'](this,_0x1beb5b);},VisuMZ['ClassChangeSystem']['Game_BattlerBase_debuffRate']=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x452)],Game_BattlerBase[_0x45bccd(0x404)]['debuffRate']=function(_0x1f2af2){const _0x2219db=_0x45bccd;if(this[_0x2219db(0x3a1)]())this['_multiclassCheck']='DebuffRates';return VisuMZ[_0x2219db(0x3ac)][_0x2219db(0x2ff)][_0x2219db(0x42e)](this,_0x1f2af2);},VisuMZ[_0x45bccd(0x3ac)]['Game_BattlerBase_stateRate']=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x27f)],Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x27f)]=function(_0x2f9652){const _0x196202=_0x45bccd;if(this[_0x196202(0x3a1)]())this['_multiclassCheck']=_0x196202(0x28d);return VisuMZ[_0x196202(0x3ac)][_0x196202(0x402)][_0x196202(0x42e)](this,_0x2f9652);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x23c)]=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x180)],Game_BattlerBase['prototype'][_0x45bccd(0x180)]=function(){const _0x9e6ec8=_0x45bccd;if(this[_0x9e6ec8(0x3a1)]())this[_0x9e6ec8(0x232)]=_0x9e6ec8(0x3bd);return VisuMZ[_0x9e6ec8(0x3ac)][_0x9e6ec8(0x23c)][_0x9e6ec8(0x42e)](this);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x293)]=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x459)],Game_BattlerBase['prototype'][_0x45bccd(0x459)]=function(_0x23177a){const _0x29f8e8=_0x45bccd;if(this[_0x29f8e8(0x3a1)]())this[_0x29f8e8(0x232)]=_0x29f8e8(0x331);return VisuMZ[_0x29f8e8(0x3ac)][_0x29f8e8(0x293)][_0x29f8e8(0x42e)](this,_0x23177a);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x42f)]=Game_BattlerBase[_0x45bccd(0x404)]['xparam'],Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x3ff)]=function(_0x39519e){const _0x2b7a15=_0x45bccd;if(this[_0x2b7a15(0x3a1)]())this[_0x2b7a15(0x232)]='XParamRates';return VisuMZ['ClassChangeSystem'][_0x2b7a15(0x42f)]['call'](this,_0x39519e);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x2a1)]=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x2c5)],Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x2c5)]=function(_0x3b4bf5){const _0x361a7c=_0x45bccd;if(this['isActor']())this[_0x361a7c(0x232)]=_0x361a7c(0x34e);return VisuMZ[_0x361a7c(0x3ac)]['Game_BattlerBase_sparam']['call'](this,_0x3b4bf5);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x36c)]=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x305)],Game_BattlerBase[_0x45bccd(0x404)]['attackElements']=function(){const _0x575536=_0x45bccd;if(this[_0x575536(0x3a1)]())this[_0x575536(0x232)]=_0x575536(0x43b);return VisuMZ[_0x575536(0x3ac)][_0x575536(0x36c)]['call'](this);},VisuMZ['ClassChangeSystem']['Game_BattlerBase_attackStates']=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x454)],Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x454)]=function(){const _0x46a070=_0x45bccd;if(this['isActor']())this['_multiclassCheck']=_0x46a070(0x328);return VisuMZ['ClassChangeSystem'][_0x46a070(0x3c2)][_0x46a070(0x42e)](this);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x1c1)]=Game_BattlerBase['prototype'][_0x45bccd(0x19d)],Game_BattlerBase['prototype'][_0x45bccd(0x19d)]=function(_0x3eb72d){const _0x50d871=_0x45bccd;if(this[_0x50d871(0x3a1)]())this[_0x50d871(0x232)]=_0x50d871(0x328);return VisuMZ[_0x50d871(0x3ac)][_0x50d871(0x1c1)]['call'](this,_0x3eb72d);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x498)]=Game_BattlerBase[_0x45bccd(0x404)]['addedSkillTypes'],Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x2a7)]=function(){const _0xfbabce=_0x45bccd;if(this[_0xfbabce(0x3a1)]())this[_0xfbabce(0x232)]=_0xfbabce(0x37c);return VisuMZ[_0xfbabce(0x3ac)][_0xfbabce(0x498)]['call'](this);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x184)]=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x267)],Game_BattlerBase['prototype'][_0x45bccd(0x267)]=function(){const _0x2af605=_0x45bccd;if(this[_0x2af605(0x3a1)]())this['_multiclassCheck']='AddedSkills';return VisuMZ['ClassChangeSystem'][_0x2af605(0x184)]['call'](this);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x3f3)]=Game_BattlerBase[_0x45bccd(0x404)][_0x45bccd(0x20e)],Game_BattlerBase['prototype'][_0x45bccd(0x20e)]=function(_0x252ef4){const _0x43caf7=_0x45bccd;return this[_0x43caf7(0x3a1)]()&&(this[_0x43caf7(0x232)]=_0x43caf7(0x21a)),VisuMZ[_0x43caf7(0x3ac)]['Game_BattlerBase_isEquipWtypeOk'][_0x43caf7(0x42e)](this,_0x252ef4);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x369)]=Game_BattlerBase[_0x45bccd(0x404)]['isEquipAtypeOk'],Game_BattlerBase['prototype'][_0x45bccd(0x175)]=function(_0x286732){const _0x80e17=_0x45bccd;if(this['isActor']())this['_multiclassCheck']=_0x80e17(0x2f3);return VisuMZ[_0x80e17(0x3ac)][_0x80e17(0x369)][_0x80e17(0x42e)](this,_0x286732);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x48f)]=Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x30c)],Game_Actor['prototype']['traitObjects']=function(){const _0x1ceaa7=_0x45bccd;let _0xfe1ba2=VisuMZ['ClassChangeSystem']['Game_Actor_traitObjects'][_0x1ceaa7(0x42e)](this);return this[_0x1ceaa7(0x232)]&&(_0xfe1ba2=this[_0x1ceaa7(0x3ec)](_0xfe1ba2)),this['_multiclassCheck']=undefined,_0xfe1ba2;},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3ec)]=function(_0x209e97){const _0x5ee3be=_0x45bccd;if(this[_0x5ee3be(0x25a)]===undefined)this[_0x5ee3be(0x3a6)]();const _0x49acfe=this[_0x5ee3be(0x232)];let _0x383fe6=_0x209e97[_0x5ee3be(0x165)](this[_0x5ee3be(0x2b3)]());const _0x473e6c=VisuMZ['ClassChangeSystem'][_0x5ee3be(0x38c)][_0x5ee3be(0x2bd)],_0x1b04ff=_0x473e6c['length'];for(let _0x256727=0x1;_0x256727<_0x1b04ff;_0x256727++){let _0x5a4d06=$dataClasses[this[_0x5ee3be(0x25a)][_0x256727]||0x0];if(!_0x5a4d06)continue;if(_0x5a4d06===this[_0x5ee3be(0x2b3)]())continue;const _0x25fa92=_0x473e6c[_0x256727];if(!_0x25fa92)continue;if(_0x25fa92[this[_0x5ee3be(0x232)]]){if(_0x5ee3be(0x45b)!==_0x5ee3be(0x45b)){function _0x2e0388(){const _0x453152=_0x5ee3be;this['_classListWindow'][_0x453152(0x193)](_0x28d7bf);}}else _0x209e97['splice'](++_0x383fe6,0x0,_0x5a4d06);}}return _0x209e97;},Game_Actor['prototype'][_0x45bccd(0x42b)]=function(_0x4cb624,_0x2edae8){const _0x116ac=_0x45bccd;if(_0x4cb624<=0x0)return;if(!_0x2edae8)return;if(!$dataSystem['optExtraExp']&&!this['isBattleMember']())return;this[_0x116ac(0x405)]();const _0x235d40=VisuMZ[_0x116ac(0x3ac)][_0x116ac(0x38c)][_0x116ac(0x2bd)],_0x97898d=_0x235d40[_0x116ac(0x38a)];for(let _0x2cd35a=0x1;_0x2cd35a<_0x97898d;_0x2cd35a++){if(_0x116ac(0x1c3)!==_0x116ac(0x1c3)){function _0x5746e2(){const _0x14deae=_0x116ac;_0x3e2c68[_0x14deae(0x3ac)][_0x14deae(0x198)][_0x14deae(0x42e)](this),this[_0x14deae(0x2f4)](),this[_0x14deae(0x285)](),this[_0x14deae(0x197)](),this[_0x14deae(0x2be)]();}}else{let _0x2b4988=$dataClasses[this[_0x116ac(0x25a)][_0x2cd35a]||0x0];if(!_0x2b4988)continue;if(_0x2b4988===this[_0x116ac(0x2b3)]())continue;const _0x4bf645=_0x235d40[_0x2cd35a];if(!_0x4bf645)continue;if(this[_0x116ac(0x48a)['format'](_0x2edae8)]){if(_0x116ac(0x38b)===_0x116ac(0x38b)){const _0x35236e=_0x4bf645[_0x116ac(0x3f9)],_0x4aedd7=_0x35236e*_0x4cb624;this[_0x116ac(0x48a)['format'](_0x2edae8)](_0x4aedd7,this[_0x116ac(0x25a)][_0x2cd35a]);}else{function _0x1f9d55(){const _0x531dc3=_0x116ac;_0x31a4a7[_0x531dc3(0x404)][_0x531dc3(0x3b4)][_0x531dc3(0x42e)](this);}}}}}},Game_Actor[_0x45bccd(0x404)]['gainMulticlassExp']=function(_0x4c7bf5){const _0x13d23a=_0x45bccd;if(!_0x4c7bf5)return;if(this[_0x13d23a(0x1a1)]())return;this['getMulticlasses']();const _0x59703c=VisuMZ[_0x13d23a(0x3ac)][_0x13d23a(0x38c)][_0x13d23a(0x2bd)],_0x4ab9e0=_0x59703c[_0x13d23a(0x38a)];for(let _0xb2eb2b=0x1;_0xb2eb2b<_0x4ab9e0;_0xb2eb2b++){if(_0x13d23a(0x2d2)==='PTQyN'){function _0x1ea3ac(){if(_0xe38403['isPlaytest']())_0x2750ba['log'](_0x3ca4cb);return 0x0;}}else{let _0x16a6a8=$dataClasses[this[_0x13d23a(0x25a)][_0xb2eb2b]||0x0];if(!_0x16a6a8)continue;if(_0x16a6a8===this[_0x13d23a(0x2b3)]())continue;const _0x4d020b=_0x59703c[_0xb2eb2b];if(!_0x4d020b)continue;const _0x1cf960=_0x4d020b[_0x13d23a(0x22a)],_0x231bb5=Math[_0x13d23a(0x304)](_0x4c7bf5*_0x1cf960*this[_0x13d23a(0x22f)]()),_0x50c127=this[_0x13d23a(0x25a)][_0xb2eb2b];this[_0x13d23a(0x258)][_0x50c127]=this['_exp'][_0x50c127]||0x0;const _0x5252ad=this[_0x13d23a(0x258)][_0x50c127]+_0x231bb5;this['changeClassExp'](_0x5252ad,_0x50c127);}}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x215)]=function(_0x46e99e,_0x34adf7){const _0x491772=_0x45bccd;if(this[_0x491772(0x25a)]===undefined)this[_0x491772(0x3a6)]();_0x34adf7-=0x1;if(_0x46e99e<=0x0&&_0x34adf7<=0x0)return;this[_0x491772(0x45c)](_0x46e99e);const _0x59dfe6=this[_0x491772(0x25a)][_0x491772(0x38a)];for(let _0x30b9fb=0x0;_0x30b9fb<_0x59dfe6;_0x30b9fb++){if(this[_0x491772(0x25a)][_0x30b9fb]===_0x46e99e){if(_0x491772(0x4b4)===_0x491772(0x4b4))this[_0x491772(0x25a)][_0x30b9fb]=0x0;else{function _0x365207(){const _0x39e67f=_0x491772;this['_multiclasses'][_0x2e3da6]=_0x2ed765[_0x39e67f(0x1d8)](_0x4d3c95);}}}}this[_0x491772(0x25a)][0x0]=this[_0x491772(0x2b3)]()['id'];if(_0x34adf7<=0x0){this['changeClass'](_0x46e99e);return;}const _0x1d5bd9=JsonEx[_0x491772(0x2d3)](this);_0x1d5bd9[_0x491772(0x310)]=!![],this['_multiclasses'][_0x34adf7]=_0x46e99e,this[_0x491772(0x46e)](),this[_0x491772(0x3d3)](),this[_0x491772(0x181)](_0x1d5bd9),this[_0x491772(0x46e)]();},Game_Actor['prototype'][_0x45bccd(0x33c)]=function(_0x580475){const _0x2f2c1c=_0x45bccd;if(this[_0x2f2c1c(0x25a)]===undefined)this[_0x2f2c1c(0x3a6)]();return this[_0x2f2c1c(0x25a)][0x0]=this[_0x2f2c1c(0x2b3)]()['id'],this[_0x2f2c1c(0x25a)][_0x2f2c1c(0x165)](_0x580475)+0x1;},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x379)]=function(){const _0x1dd97d=_0x45bccd;this[_0x1dd97d(0x3e4)]={},this[_0x1dd97d(0x3e4)][this[_0x1dd97d(0x2b3)]()['id']]=this[_0x1dd97d(0x415)];},Game_Actor['prototype']['maintainLevels']=function(){const _0x434a7b=_0x45bccd;return VisuMZ[_0x434a7b(0x3ac)][_0x434a7b(0x38c)]['General'][_0x434a7b(0x1b4)];},Game_Actor['prototype'][_0x45bccd(0x189)]=function(_0x5041f1){const _0x18389a=_0x45bccd;if(this[_0x18389a(0x1a1)]())return this['level'];return this[_0x18389a(0x3fa)](_0x5041f1),this[_0x18389a(0x3e4)][_0x5041f1];},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2e1)]=function(_0x378caa,_0x5561bc){const _0x33ab45=_0x45bccd;if(this[_0x33ab45(0x1a1)]())return this[_0x33ab45(0x475)](_0x378caa);this[_0x33ab45(0x258)][_0x5561bc]=Math[_0x33ab45(0x2b0)](_0x378caa,0x0),this[_0x33ab45(0x3fa)](_0x5561bc);if(_0x5561bc===this[_0x33ab45(0x2b3)]()['id'])this['refresh']();},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x3fa)]=function(_0x469529){const _0x4f1627=_0x45bccd;if(this[_0x4f1627(0x1a1)]())return;this['_exp'][_0x469529]=this[_0x4f1627(0x258)][_0x469529]||0x0,this['_classLevel']=this[_0x4f1627(0x3e4)]||{},this[_0x4f1627(0x3e4)][_0x469529]=this[_0x4f1627(0x3e4)][_0x469529]||0x1;while(!(this[_0x4f1627(0x3e4)][_0x469529]>=this[_0x4f1627(0x1c7)]())&&this[_0x4f1627(0x258)][_0x469529]>=this[_0x4f1627(0x438)](_0x469529,this[_0x4f1627(0x3e4)][_0x469529])){this['_classLevel'][_0x469529]+=0x1,this[_0x4f1627(0x25e)](_0x469529);}while(this[_0x4f1627(0x258)][_0x469529]<this[_0x4f1627(0x292)](_0x469529,this[_0x4f1627(0x3e4)][_0x469529])){this['_classLevel'][_0x469529]-=0x1;}this[_0x4f1627(0x2df)]();},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x343)]=function(_0x53fc8f,_0x180652){const _0x536b02=_0x45bccd,_0x3527cd=$dataClasses[_0x53fc8f],_0x17002f=_0x3527cd[_0x536b02(0x3b0)][0x0],_0x1bce43=_0x3527cd[_0x536b02(0x3b0)][0x1],_0x4ece00=_0x3527cd['expParams'][0x2],_0x1bfa78=_0x3527cd[_0x536b02(0x3b0)][0x3];return Math['round'](_0x17002f*Math[_0x536b02(0x3ba)](_0x180652-0x1,0.9+_0x4ece00/0xfa)*_0x180652*(_0x180652+0x1)/(0x6+Math[_0x536b02(0x3ba)](_0x180652,0x2)/0x32/_0x1bfa78)+(_0x180652-0x1)*_0x1bce43);},Game_Actor[_0x45bccd(0x404)]['nextClassLevelExp']=function(_0x3c6a63,_0x2cf74c){return this['expForClassLevel'](_0x3c6a63,_0x2cf74c+0x1);},Game_Actor[_0x45bccd(0x404)]['currentClassLevelExp']=function(_0x472f98,_0xaf133c){return this['expForClassLevel'](_0x472f98,_0xaf133c);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x25e)]=function(_0xeb4394){const _0x583fe0=_0x45bccd;this[_0x583fe0(0x1e2)](_0xeb4394),this[_0x583fe0(0x36f)](_0xeb4394);if(Imported['VisuMZ_2_SkillLearnSystem']){if(_0x583fe0(0x467)!=='CeWOr'){function _0x45a0c5(){const _0x5c588d=_0x583fe0;if(this['_unlockedClasses']===_0x1f3c4e)this[_0x5c588d(0x458)]();return this['_unlockedClasses'];}}else this['levelUpGainAbilityPoints'](_0xeb4394),this[_0x583fe0(0x19b)](_0xeb4394);}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2df)]=function(){const _0x3e746a=_0x45bccd;if(this[_0x3e746a(0x3a2)])return;this[_0x3e746a(0x3a2)]=!![];const _0x4d170d=DataManager[_0x3e746a(0x276)](this);for(const _0x6b83ef of _0x4d170d){if(_0x3e746a(0x20c)!==_0x3e746a(0x3c1)){if(!_0x6b83ef)continue;const _0x62fb5d=_0x6b83ef[_0x3e746a(0x32f)];if(!_0x62fb5d)continue;for(const _0x59cc52 of _0x62fb5d){if(_0x3e746a(0x409)===_0x3e746a(0x409)){if(this[_0x3e746a(0x2d4)](_0x59cc52['skillId']))continue;if(this[_0x3e746a(0x189)](_0x6b83ef['id'])>=_0x59cc52[_0x3e746a(0x415)]){if(_0x3e746a(0x315)!==_0x3e746a(0x251)){const _0x12fde2=this[_0x3e746a(0x1d7)]||{};this['learnSkill'](_0x59cc52['skillId']),this[_0x3e746a(0x1d7)]=_0x12fde2;}else{function _0x1c5a95(){_0x493c40=0x0;}}}}else{function _0x4b174b(){const _0x3173e7=_0x3e746a,_0x78408f=_0x3173e7(0x241)[_0x3173e7(0x3dd)](_0x2dc42c,_0x56d73d);_0xaf5fbd[_0x3173e7(0x227)][_0x78408f]=_0x5287fc;}}}}else{function _0x22d8ad(){const _0x374282=_0x3e746a;_0x39d67c=this['maintainLevels']();_0x49767c&&(this[_0x374282(0x258)]=this['_exp']||{},this[_0x374282(0x258)][_0x10426b]=this[_0x374282(0x258)][this[_0x374282(0x1a2)]]||0x0,_0x3fa432=![]);this[_0x374282(0x313)]=!![];const _0x47b184=_0x687ca2[_0x374282(0x2d3)](this);_0x47b184[_0x374282(0x310)]=!![],_0x491f9d['ClassChangeSystem'][_0x374282(0x281)][_0x374282(0x42e)](this,_0x34e1ab,_0x18ef9e),this[_0x374282(0x181)](_0x47b184),this[_0x374282(0x46e)](),this[_0x374282(0x49a)](_0x957116),this[_0x374282(0x313)]=_0xb80547;if(_0x5bdb58)_0x1d99eb[_0x374282(0x3d3)]();}}}this[_0x3e746a(0x3a2)]=![];},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x2cb)]=Game_Actor[_0x45bccd(0x404)]['paramBase'],Game_Actor['prototype']['paramBase']=function(_0xc277bc){const _0x5d6cc6=_0x45bccd;let _0x42e444=VisuMZ[_0x5d6cc6(0x3ac)][_0x5d6cc6(0x2cb)][_0x5d6cc6(0x42e)](this,_0xc277bc);this['getMulticlasses']();const _0x3a6ccf=VisuMZ['ClassChangeSystem']['Settings'][_0x5d6cc6(0x2bd)],_0x1b5f58=_0x5d6cc6(0x1b8)[_0x5d6cc6(0x3dd)](_0xc277bc),_0x148852=_0x3a6ccf[_0x5d6cc6(0x38a)];for(let _0x9b4ebe=0x1;_0x9b4ebe<_0x148852;_0x9b4ebe++){if(_0x5d6cc6(0x263)===_0x5d6cc6(0x381)){function _0x51a042(){const _0x4bc1a6=_0x5d6cc6;_0x44a8df[_0x4bc1a6(0x404)][_0x4bc1a6(0x2b5)]['call'](this,_0x3f8984);}}else{let _0x26d34d=$dataClasses[this['_multiclasses'][_0x9b4ebe]||0x0];if(!_0x26d34d)continue;if(_0x26d34d===this['currentClass']())continue;const _0x2f8b4c=_0x3a6ccf[_0x9b4ebe];if(!_0x2f8b4c)continue;const _0x43e9db=_0x2f8b4c[_0x1b5f58];_0x42e444+=_0x43e9db*this[_0x5d6cc6(0x371)](this[_0x5d6cc6(0x25a)][_0x9b4ebe],_0xc277bc);}}return _0x42e444;},Game_Actor[_0x45bccd(0x404)]['paramBaseForClass']=function(_0x199349,_0x514282){const _0x5ed763=_0x45bccd,_0x2648c3=$dataClasses[_0x199349],_0x175885=this['classLevel'](_0x199349);if(_0x175885>0x63){if('QhpNh'!==_0x5ed763(0x465)){const _0x1fb5d2=_0x2648c3[_0x5ed763(0x27b)][_0x514282][0x63],_0x33d208=_0x2648c3[_0x5ed763(0x27b)][_0x514282][0x62];return _0x1fb5d2+(_0x1fb5d2-_0x33d208)*(_0x175885-0x63);}else{function _0x38ad90(){const _0x21b2c4=_0x5ed763;_0x3fd0b4=_0x1e347b(_0x102c25[_0x21b2c4(0x2f6)]);}}}else return _0x2648c3['params'][_0x514282][_0x175885];},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x204)]=function(_0xe6b6d2){const _0x53505b=_0x45bccd;if(this[_0x53505b(0x3e4)][_0xe6b6d2]>=this[_0x53505b(0x1c7)]())return 0x1;const _0xa2622a=this[_0x53505b(0x189)](_0xe6b6d2),_0x4f605f=this[_0x53505b(0x438)](_0xe6b6d2,_0xa2622a)-this['currentClassLevelExp'](_0xe6b6d2,_0xa2622a);this[_0x53505b(0x258)][_0xe6b6d2]=this[_0x53505b(0x258)][_0xe6b6d2]||0x0;const _0x3968d1=this[_0x53505b(0x258)][_0xe6b6d2]-this[_0x53505b(0x292)](_0xe6b6d2,_0xa2622a);return(_0x3968d1/_0x4f605f)[_0x53505b(0x37e)](0x0,0x1);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x2e8)]=function(){const _0x118ccf=_0x45bccd;for(;;){const _0x5dccdc=DataManager['checkForNewUnlockedClasses'](this);if(_0x5dccdc['length']>0x0){if(_0x118ccf(0x4a5)===_0x118ccf(0x2b8)){function _0x402a00(){const _0x570cca=_0x118ccf;_0x2d983f=_0x376064[_0x570cca(0x1d8)](_0x42f2ac);}}else for(const _0x38e87b of _0x5dccdc){if(_0x118ccf(0x23d)===_0x118ccf(0x214)){function _0x47bf1c(){const _0x4c2661=_0x118ccf;this[_0x4c2661(0x2e6)]=_0x1ed030(_0x23a895['$1']);}}else this[_0x118ccf(0x45c)](_0x38e87b);}}else break;}},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x242)]=function(){const _0x57dd86=_0x45bccd;let _0x3da532=[];const _0x265e23=VisuMZ['ClassChangeSystem']['RegExp'],_0xd9f727=this[_0x57dd86(0x43f)]()[_0x57dd86(0x323)],_0xd6aad7=_0xd9f727[_0x57dd86(0x264)](_0x265e23[_0x57dd86(0x490)]);if(_0xd6aad7)for(const _0x2f9fed of _0xd6aad7){if('sCGIF'!=='ymqBc'){if(!_0x2f9fed)continue;_0x2f9fed[_0x57dd86(0x264)](_0x265e23['RestrictClassChangeTier']);const _0x543355=String(RegExp['$1'])[_0x57dd86(0x1f5)](',')[_0x57dd86(0x361)](_0x54fbb7=>Number(_0x54fbb7));_0x3da532=_0x3da532['concat'](_0x543355);}else{function _0x4341f8(){const _0x1e086a=_0x57dd86,_0xd7985f=_0x411a0c(_0xfd1dee['$1']);this[_0x1e086a(0x3c7)](_0xd7985f);}}}_0x3da532=_0x3da532[_0x57dd86(0x287)]((_0x4991db,_0x165958,_0x73a70b)=>_0x73a70b[_0x57dd86(0x165)](_0x4991db)===_0x165958),_0x3da532[_0x57dd86(0x306)](null)['remove'](undefined),_0x3da532[_0x57dd86(0x2fb)]((_0x4f6729,_0x3f05d4)=>_0x4f6729-_0x3f05d4),this[_0x57dd86(0x366)]=_0x3da532;},Game_Actor[_0x45bccd(0x404)]['isClassChangeTierRestricted']=function(_0xdef388){const _0x4f1159=_0x45bccd;return this['_classChangeTierRestrictions']===undefined&&this[_0x4f1159(0x242)](),this[_0x4f1159(0x366)][_0x4f1159(0x3f5)](_0xdef388);},Game_Actor[_0x45bccd(0x404)][_0x45bccd(0x1f4)]=function(_0x4f7a09){const _0x4a292a=_0x45bccd;if(this[_0x4a292a(0x366)]===undefined){if(_0x4a292a(0x4b1)!==_0x4a292a(0x359))this[_0x4a292a(0x242)]();else{function _0x2465ba(){const _0x1eddc9=_0x4a292a;_0x48f098[_0x1eddc9(0x2e8)]();}}}if(this['_classChangeTierRestrictions']['includes'](_0x4f7a09))return;this[_0x4a292a(0x366)][_0x4a292a(0x3ab)](_0x4f7a09),this[_0x4a292a(0x366)][_0x4a292a(0x2fb)]((_0x51e8a2,_0x5bf00f)=>_0x51e8a2-_0x5bf00f);},Game_Actor[_0x45bccd(0x404)]['removeClassChangeTierRestriction']=function(_0x942f52){const _0x4d44a4=_0x45bccd;this[_0x4d44a4(0x366)]===undefined&&this[_0x4d44a4(0x242)]();if(!this[_0x4d44a4(0x366)][_0x4d44a4(0x3f5)](_0x942f52))return;this[_0x4d44a4(0x366)]['remove'](_0x942f52),this['_classChangeTierRestrictions'][_0x4d44a4(0x2fb)]((_0x22afc2,_0x2ba733)=>_0x22afc2-_0x2ba733);},Game_Enemy[_0x45bccd(0x404)][_0x45bccd(0x26f)]=function(){const _0x1721f4=_0x45bccd,_0x2c0fa9=VisuMZ[_0x1721f4(0x3ac)]['Settings'][_0x1721f4(0x28e)],_0x3154ff=VisuMZ[_0x1721f4(0x3ac)][_0x1721f4(0x411)],_0x1a2e12=this['enemy']()['note'];if(_0x1a2e12[_0x1721f4(0x264)](_0x3154ff[_0x1721f4(0x365)]))try{return eval(RegExp['$1']);}catch(_0x444234){if($gameTemp[_0x1721f4(0x2fa)]())console[_0x1721f4(0x210)](_0x444234);return 0x0;}try{return eval(_0x2c0fa9[_0x1721f4(0x298)]);}catch(_0x1cb124){if(_0x1721f4(0x3e2)!==_0x1721f4(0x3e2)){function _0x5159b1(){const _0x297b07=_0x1721f4,_0x28ff78=_0x4fa9e9[_0x297b07(0x41e)][_0x297b07(0x303)];if(_0x28ff78)_0x28ff78['refreshActorPortrait'](this);}}else{if($gameTemp[_0x1721f4(0x2fa)]())console[_0x1721f4(0x210)](_0x1cb124);return 0x0;}}},Game_Enemy['prototype'][_0x45bccd(0x4ad)]=function(){const _0x34d4d6=_0x45bccd,_0x28566a=VisuMZ[_0x34d4d6(0x3ac)][_0x34d4d6(0x38c)][_0x34d4d6(0x1ea)],_0x144a07=VisuMZ[_0x34d4d6(0x3ac)][_0x34d4d6(0x411)],_0x4c4f76=this[_0x34d4d6(0x4bc)]()['note'];if(_0x4c4f76[_0x34d4d6(0x264)](_0x144a07[_0x34d4d6(0x423)])){if(_0x34d4d6(0x32d)===_0x34d4d6(0x32d))try{return eval(RegExp['$1']);}catch(_0x6d3761){if($gameTemp[_0x34d4d6(0x2fa)]())console[_0x34d4d6(0x210)](_0x6d3761);return 0x0;}else{function _0x27cdf1(){const _0x585d2e=_0x34d4d6;_0x5bb82a[_0x585d2e(0x3ac)][_0x585d2e(0x2c4)][_0x585d2e(0x42e)](this,_0x11b6af);for(const _0x555529 of _0x354e87[_0x585d2e(0x340)]()){_0x555529[_0x585d2e(0x2e8)]();}}}}try{return eval(_0x28566a['PerEnemy']);}catch(_0x311a48){if(_0x34d4d6(0x4a7)!==_0x34d4d6(0x4a7)){function _0xe23283(){const _0x2b7d0b=_0x34d4d6;_0xf6ed65=this[_0x2b7d0b(0x349)][_0x2b7d0b(0x26c)](_0x29c0ca),_0x2a93e6=this[_0x2b7d0b(0x310)][_0x2b7d0b(0x26c)](_0x32cd53),_0x2e787a=_0xb4e146%0x1!==0x0||_0x2c32c4%0x1!==0x0;}}else{if($gameTemp[_0x34d4d6(0x2fa)]())console['log'](_0x311a48);return 0x0;}}},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x437)]=Game_Party[_0x45bccd(0x404)][_0x45bccd(0x2b5)],Game_Party[_0x45bccd(0x404)][_0x45bccd(0x2b5)]=function(){const _0x2ad293=_0x45bccd;VisuMZ[_0x2ad293(0x3ac)]['Game_Party_initialize'][_0x2ad293(0x42e)](this),this['initClassChangeUnlocks']();},Game_Party[_0x45bccd(0x404)]['initClassChangeUnlocks']=function(){this['_unlockedClasses']=[];},Game_Party[_0x45bccd(0x404)][_0x45bccd(0x29b)]=function(){const _0x3a6d60=_0x45bccd;if(this['_unlockedClasses']===undefined)this['initClassChangeUnlocks']();return this[_0x3a6d60(0x20a)];},Game_Party['prototype'][_0x45bccd(0x45c)]=function(_0x917bf){const _0x3c6d4f=_0x45bccd;for(const _0x4c9a2a of this[_0x3c6d4f(0x340)]()){if(!_0x4c9a2a)continue;_0x4c9a2a[_0x3c6d4f(0x45c)](_0x917bf);}if(this[_0x3c6d4f(0x20a)]===undefined)this[_0x3c6d4f(0x458)]();if(this[_0x3c6d4f(0x20a)][_0x3c6d4f(0x3f5)](_0x917bf))return;this[_0x3c6d4f(0x20a)][_0x3c6d4f(0x3ab)](_0x917bf),this[_0x3c6d4f(0x20a)]['sort'](function(_0x58b498,_0x4f1e5){return _0x58b498-_0x4f1e5;});},Game_Party[_0x45bccd(0x404)][_0x45bccd(0x3f0)]=function(_0x237e3a){const _0x2e2c19=_0x45bccd;for(const _0x331b0f of this[_0x2e2c19(0x340)]()){if('hjpZx'!==_0x2e2c19(0x1f3)){function _0x3a1374(){_0x349eb7+=0x0;}}else{if(!_0x331b0f)continue;_0x331b0f[_0x2e2c19(0x3f0)](_0x237e3a);}}if(this[_0x2e2c19(0x20a)]===undefined)this[_0x2e2c19(0x458)]();if(!this['_unlockedClasses'][_0x2e2c19(0x3f5)](_0x237e3a))return;this[_0x2e2c19(0x20a)][_0x2e2c19(0x306)](_0x237e3a)[_0x2e2c19(0x306)](null),this[_0x2e2c19(0x20a)]['sort'](function(_0x4a0857,_0x3b17a6){const _0x360899=_0x2e2c19;if(_0x360899(0x364)==='jImQg'){function _0xa416f6(){return![];}}else return _0x4a0857-_0x3b17a6;});},Game_Party['prototype']['highestMulticlassTier']=function(){const _0x33559c=_0x45bccd,_0x4f597f=this[_0x33559c(0x340)]();return Math[_0x33559c(0x2b0)](...this['members']()[_0x33559c(0x361)](_0x2823c1=>_0x2823c1[_0x33559c(0x295)]()));},Game_Troop[_0x45bccd(0x404)][_0x45bccd(0x257)]=function(){const _0x505469=_0x45bccd;return this[_0x505469(0x447)]()[_0x505469(0x31b)]((_0x281108,_0x13ca41)=>_0x281108+_0x13ca41[_0x505469(0x26f)](),0x0);},Game_Troop[_0x45bccd(0x404)][_0x45bccd(0x28a)]=function(){const _0xfbd780=_0x45bccd;return this[_0xfbd780(0x447)]()[_0xfbd780(0x31b)]((_0x310eb4,_0x22b787)=>_0x310eb4+_0x22b787['jobPoints'](),0x0);},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x360)]=Scene_Menu['prototype'][_0x45bccd(0x348)],Scene_Menu[_0x45bccd(0x404)]['createCommandWindow']=function(){const _0x3cb51e=_0x45bccd;VisuMZ[_0x3cb51e(0x3ac)][_0x3cb51e(0x360)][_0x3cb51e(0x42e)](this);const _0x38d6f3=this[_0x3cb51e(0x2f2)];_0x38d6f3[_0x3cb51e(0x48b)](_0x3cb51e(0x3ac),this[_0x3cb51e(0x34b)][_0x3cb51e(0x187)](this));},VisuMZ[_0x45bccd(0x3ac)][_0x45bccd(0x280)]=Scene_Menu['prototype'][_0x45bccd(0x44d)],Scene_Menu[_0x45bccd(0x404)][_0x45bccd(0x44d)]=function(){const _0x395165=_0x45bccd;this[_0x395165(0x2f2)][_0x395165(0x491)]()==='ClassChangeSystem'?SceneManager[_0x395165(0x3ab)](Scene_ClassChange):VisuMZ[_0x395165(0x3ac)][_0x395165(0x280)]['call'](this);};function Scene_ClassChange(){const _0xb48649=_0x45bccd;this[_0xb48649(0x2b5)](...arguments);}Scene_ClassChange[_0x45bccd(0x404)]=Object[_0x45bccd(0x1ec)](Scene_MenuBase[_0x45bccd(0x404)]),Scene_ClassChange[_0x45bccd(0x404)]['constructor']=Scene_ClassChange,Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x2b5)]=function(){const _0x319322=_0x45bccd;Scene_MenuBase[_0x319322(0x404)][_0x319322(0x2b5)][_0x319322(0x42e)](this),this[_0x319322(0x188)]=this[_0x319322(0x188)]||[];},Scene_ClassChange['prototype'][_0x45bccd(0x333)]=function(){return!![];},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x31c)]=function(){const _0x4286c9=_0x45bccd;if(this[_0x4286c9(0x39b)]()>0x1)return this[_0x4286c9(0x253)]&&this[_0x4286c9(0x253)]['active'];else{if(_0x4286c9(0x488)!==_0x4286c9(0x488)){function _0x4b9e32(){_0x17c124=0x0;}}else return this[_0x4286c9(0x3ed)]&&this[_0x4286c9(0x3ed)][_0x4286c9(0x4a9)];}},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x1e8)]=function(){const _0xdda942=_0x45bccd;Scene_MenuBase['prototype']['update'][_0xdda942(0x42e)](this),this['updateClassChangeAnimations']();},Scene_ClassChange['prototype'][_0x45bccd(0x468)]=function(){return!![];},Scene_ClassChange['prototype'][_0x45bccd(0x474)]=function(){const _0x26b2dc=_0x45bccd;if(ConfigManager[_0x26b2dc(0x3c0)]&&ConfigManager['uiHelpPosition']!==undefined){if(_0x26b2dc(0x28f)!==_0x26b2dc(0x38e))return ConfigManager['uiHelpPosition'];else{function _0x4f91b6(){const _0x443ce0=_0x26b2dc;this[_0x443ce0(0x47a)](![]);const _0x152589=_0x132247[_0x443ce0(0x304)](_0x546e78['y']+this[_0x443ce0(0x376)]()+(_0x3e4986[_0x443ce0(0x49e)]-this[_0x443ce0(0x376)]()*0x2)/0x2);this[_0x443ce0(0x1c5)](_0x28d692[_0x443ce0(0x321)],_0x6db603['x'],_0x152589,_0x2c9199[_0x443ce0(0x2c7)],'center');return;}}}else{if(this['isUseSkillsStatesCoreUpdatedLayout']())return this[_0x26b2dc(0x37a)]()[_0x26b2dc(0x264)](/LOWER/i);else{if(_0x26b2dc(0x3aa)!==_0x26b2dc(0x435))Scene_MenuBase[_0x26b2dc(0x404)][_0x26b2dc(0x3b4)][_0x26b2dc(0x42e)](this);else{function _0x18bd95(){const _0xc6213e=_0x26b2dc;_0x58686e=_0x1ac9c4[_0xc6213e(0x1d8)](_0x4c3a3e);}}}}},Scene_ClassChange['prototype'][_0x45bccd(0x3b4)]=function(){const _0xfb6065=_0x45bccd;if(ConfigManager[_0xfb6065(0x3c0)]&&ConfigManager[_0xfb6065(0x286)]!==undefined)return ConfigManager[_0xfb6065(0x286)];else{if(this[_0xfb6065(0x1b6)]())return this[_0xfb6065(0x37a)]()[_0xfb6065(0x264)](/RIGHT/i);else{if(_0xfb6065(0x316)!==_0xfb6065(0x1a8))Scene_MenuBase[_0xfb6065(0x404)][_0xfb6065(0x3b4)][_0xfb6065(0x42e)](this);else{function _0x1a31d6(){_0x2dde87=_0x447b45(_0x54018d['$1']);}}}}},Scene_ClassChange['prototype'][_0x45bccd(0x37a)]=function(){const _0x3e24c6=_0x45bccd;return VisuMZ[_0x3e24c6(0x3ac)][_0x3e24c6(0x38c)]['Window'][_0x3e24c6(0x2b2)];},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x1b6)]=function(){const _0x462406=_0x45bccd;return VisuMZ['ClassChangeSystem']['Settings'][_0x462406(0x239)][_0x462406(0x2ce)];},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x1ec)]=function(){const _0x1d578b=_0x45bccd;Scene_MenuBase[_0x1d578b(0x404)][_0x1d578b(0x1ec)]['call'](this),this[_0x1d578b(0x30d)](),this['createStatusWindow'](),this[_0x1d578b(0x2cc)](),this[_0x1d578b(0x2db)](),this[_0x1d578b(0x309)](),this[_0x1d578b(0x2f5)]();},Scene_ClassChange[_0x45bccd(0x404)]['createStatusWindow']=function(){const _0x232877=_0x45bccd,_0x5900a9=this[_0x232877(0x487)]();this[_0x232877(0x303)]=new Window_ClassStatus(_0x5900a9),this[_0x232877(0x217)](this[_0x232877(0x303)]),this[_0x232877(0x303)][_0x232877(0x2bf)](VisuMZ[_0x232877(0x3ac)][_0x232877(0x38c)]['Window'][_0x232877(0x472)]);},Scene_ClassChange['prototype'][_0x45bccd(0x487)]=function(){const _0x5edc5a=_0x45bccd,_0x4603f1=VisuMZ[_0x5edc5a(0x3ac)][_0x5edc5a(0x38c)][_0x5edc5a(0x239)];if(_0x4603f1[_0x5edc5a(0x3a4)])return _0x4603f1['Window_ClassStatus_RectJS']['call'](this);const _0x1fc786=Math[_0x5edc5a(0x382)](Graphics['boxWidth']/0x2),_0x4acd88=this[_0x5edc5a(0x4ac)](),_0x286aa0=this[_0x5edc5a(0x3b4)]()?0x0:_0x1fc786,_0x2c3b56=this[_0x5edc5a(0x274)]();return new Rectangle(_0x286aa0,_0x2c3b56,_0x1fc786,_0x4acd88);},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x2cc)]=function(){const _0x50aa6b=_0x45bccd,_0x36717e=this[_0x50aa6b(0x1b0)](),_0x516093=new Window_ClassTier(_0x36717e);_0x516093[_0x50aa6b(0x22e)](this[_0x50aa6b(0x372)]),_0x516093[_0x50aa6b(0x2bf)](VisuMZ[_0x50aa6b(0x3ac)]['Settings'][_0x50aa6b(0x239)][_0x50aa6b(0x312)]),this['addWindow'](_0x516093),this[_0x50aa6b(0x253)]=_0x516093,_0x516093['setHandler']('cancel',this['popScene']['bind'](this));if(this[_0x50aa6b(0x39b)]()>0x1){if('NXbEz'!==_0x50aa6b(0x18e))_0x516093[_0x50aa6b(0x48b)](_0x50aa6b(0x3d1),this[_0x50aa6b(0x167)][_0x50aa6b(0x187)](this)),_0x516093[_0x50aa6b(0x48b)]('pageup',this[_0x50aa6b(0x25f)][_0x50aa6b(0x187)](this));else{function _0x427a82(){const _0x480bde=_0x50aa6b;_0x47bc6f!==''?(this[_0x480bde(0x48d)]=_0x774c79,this[_0x480bde(0x330)]=_0x4c4e27):(this[_0x480bde(0x48d)]=_0x39cfe8,this[_0x480bde(0x330)]=_0x44b4ad);}}}_0x516093['setHandler']('tier',this['onMulticlassOk'][_0x50aa6b(0x187)](this));},Scene_ClassChange[_0x45bccd(0x404)]['classTierWindowRect']=function(){const _0x47d20a=_0x45bccd,_0x570f22=VisuMZ[_0x47d20a(0x3ac)]['Settings'][_0x47d20a(0x239)];if(_0x570f22[_0x47d20a(0x1cb)])return _0x570f22['Window_ClassTier_RectJS']['call'](this);const _0x42aa50=Graphics[_0x47d20a(0x41b)]-this[_0x47d20a(0x303)][_0x47d20a(0x2c7)],_0x17e52d=this[_0x47d20a(0x4ac)](),_0x4a4b7d=this[_0x47d20a(0x3b4)]()?_0x42aa50:0x0,_0x48379a=this[_0x47d20a(0x274)]();return new Rectangle(_0x4a4b7d,_0x48379a,_0x42aa50,_0x17e52d);},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x2db)]=function(){const _0x23965f=_0x45bccd,_0x27b00d=this[_0x23965f(0x1c6)](),_0x373951=new Window_ClassList(_0x27b00d);_0x373951['setHelpWindow'](this[_0x23965f(0x372)]),_0x373951['setStatusWindow'](this[_0x23965f(0x303)]),_0x373951[_0x23965f(0x2bf)](VisuMZ[_0x23965f(0x3ac)]['Settings'][_0x23965f(0x239)]['Window_ClassList_BgType']),this['addWindow'](_0x373951),this['_classListWindow']=_0x373951,_0x373951['setHandler'](_0x23965f(0x3e9),this[_0x23965f(0x1d5)][_0x23965f(0x187)](this)),this[_0x23965f(0x39b)]()<=0x1&&(_0x373951[_0x23965f(0x48b)](_0x23965f(0x3d1),this[_0x23965f(0x167)][_0x23965f(0x187)](this)),_0x373951[_0x23965f(0x48b)](_0x23965f(0x407),this[_0x23965f(0x25f)][_0x23965f(0x187)](this))),_0x373951[_0x23965f(0x48b)](_0x23965f(0x344),this[_0x23965f(0x4a2)]['bind'](this));},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x1c6)]=function(){const _0x18362f=_0x45bccd,_0x304c5a=VisuMZ[_0x18362f(0x3ac)][_0x18362f(0x38c)][_0x18362f(0x239)];if(_0x304c5a['Window_ClassList_RectJS']){if(_0x18362f(0x206)!==_0x18362f(0x185))return _0x304c5a[_0x18362f(0x4b9)][_0x18362f(0x42e)](this);else{function _0x4d9a9f(){const _0x4c37cc=_0x18362f;_0xcfc256[_0x4c37cc(0x45c)](_0x2b5d9a);}}}const _0x2fc718=Graphics['boxWidth']-this[_0x18362f(0x303)][_0x18362f(0x2c7)],_0x3b2b8e=this[_0x18362f(0x4ac)](),_0x55e3d7=this[_0x18362f(0x3b4)]()?_0x2fc718:0x0,_0x260319=this['mainAreaTop']();return new Rectangle(_0x55e3d7,_0x260319,_0x2fc718,_0x3b2b8e);},Scene_ClassChange['prototype'][_0x45bccd(0x39b)]=function(){const _0x3bd955=_0x45bccd;if(this['_highestTier']!==undefined)return this['_highestTier'];return this[_0x3bd955(0x24b)]=$gameParty[_0x3bd955(0x1ff)](),this['_highestTier'];},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x309)]=function(){const _0x116142=_0x45bccd;if(this[_0x116142(0x39b)]()>0x1){if('exAhg'===_0x116142(0x3d5))this['_classTierWindow'][_0x116142(0x420)](0x0),this[_0x116142(0x253)][_0x116142(0x18f)](),this['_classTierWindow']['activate'](),this['_classListWindow'][_0x116142(0x275)](),this[_0x116142(0x3ed)][_0x116142(0x489)]();else{function _0x187817(){const _0x4c8a1e=_0x116142;return _0x4592b7[_0x4c8a1e(0x3f1)](_0x4c8a1e(0x440));}}}else{if('UbNNv'==='UbNNv')this[_0x116142(0x3ed)][_0x116142(0x420)](0x0),this[_0x116142(0x3ed)][_0x116142(0x229)](0x1),this[_0x116142(0x3ed)]['show'](),this[_0x116142(0x3ed)]['activate'](),this[_0x116142(0x253)]['hide'](),this[_0x116142(0x253)][_0x116142(0x489)]();else{function _0x5072b4(){return this['_wordWrap'];}}}},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x2f5)]=function(){const _0x12c66e=_0x45bccd,_0xf7fffe=this[_0x12c66e(0x43f)]();_0xf7fffe['checkForAutoClassUnlocks'](),this[_0x12c66e(0x303)][_0x12c66e(0x205)](_0xf7fffe),this[_0x12c66e(0x253)][_0x12c66e(0x205)](_0xf7fffe),this[_0x12c66e(0x3ed)][_0x12c66e(0x205)](_0xf7fffe);},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x325)]=function(){const _0x1c4bc8=_0x45bccd;Scene_MenuBase[_0x1c4bc8(0x404)][_0x1c4bc8(0x325)][_0x1c4bc8(0x42e)](this),this[_0x1c4bc8(0x2f5)](),this[_0x1c4bc8(0x309)]();},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x4b7)]=function(){const _0x477b6f=_0x45bccd,_0x8caa57=this[_0x477b6f(0x253)][_0x477b6f(0x176)]();this['_classListWindow']['setTier'](_0x8caa57),this[_0x477b6f(0x3ed)][_0x477b6f(0x18f)](),this[_0x477b6f(0x3ed)][_0x477b6f(0x220)](),this['_classListWindow'][_0x477b6f(0x420)](0x0),this[_0x477b6f(0x253)]['hide'](),this[_0x477b6f(0x253)][_0x477b6f(0x489)](),this[_0x477b6f(0x182)]();},Scene_ClassChange['prototype'][_0x45bccd(0x1d5)]=function(){const _0x252c76=_0x45bccd;this[_0x252c76(0x39b)]()>0x1?(this['_classTierWindow']['show'](),this[_0x252c76(0x253)][_0x252c76(0x220)](),this[_0x252c76(0x3ed)][_0x252c76(0x275)](),this[_0x252c76(0x3ed)][_0x252c76(0x489)](),this['_statusWindow'][_0x252c76(0x2a6)](null)):this[_0x252c76(0x1d0)]();},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x4a2)]=function(){const _0x20e244=_0x45bccd,_0x4890c1=this['_classListWindow'][_0x20e244(0x4b8)],_0x11920e=this['_classListWindow']['currentExt'](),_0x2952ef=this[_0x20e244(0x3ed)]['index'](),_0x57310d=_0x11920e?_0x11920e['id']:0x0;this[_0x20e244(0x349)][_0x20e244(0x215)](_0x57310d,_0x4890c1),this['_classTierWindow'][_0x20e244(0x3d3)](),this[_0x20e244(0x3ed)][_0x20e244(0x3d3)](),this[_0x20e244(0x303)][_0x20e244(0x2a6)](null),this[_0x20e244(0x179)](_0x57310d,_0x4890c1),this[_0x20e244(0x309)]();if(this[_0x20e244(0x253)][_0x20e244(0x4a9)]){if(_0x20e244(0x3a9)!==_0x20e244(0x234))this[_0x20e244(0x253)]['smoothSelect'](_0x4890c1-0x1);else{function _0x580936(){const _0x239de3=_0x20e244;_0x68fa30[_0x239de3(0x3b6)](_0x2c8064,_0x34b490);}}}else this['_classListWindow'][_0x20e244(0x4a9)]&&this[_0x20e244(0x3ed)][_0x20e244(0x193)](_0x2952ef);},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x179)]=function(_0xa2cd8c,_0x5ea6e6){const _0x32eca1=_0x45bccd,_0xfed978=this[_0x32eca1(0x384)](_0x5ea6e6);this['createClassChangeAnimation'](_0xa2cd8c,_0x5ea6e6,_0xfed978);},Scene_ClassChange['prototype']['createAnimationDummySprite']=function(_0x3b2719){const _0x483ac9=_0x45bccd,_0x1a5c6f=new Sprite(),_0x5c60b6=VisuMZ[_0x483ac9(0x3ac)][_0x483ac9(0x38c)][_0x483ac9(0x239)];if(_0x3b2719<=0x1){const _0x28f87e=this[_0x483ac9(0x303)];_0x1a5c6f['x']=_0x28f87e['x']+Math[_0x483ac9(0x304)](_0x28f87e[_0x483ac9(0x2c7)]/0x2),_0x1a5c6f['y']=_0x28f87e['y']+Math[_0x483ac9(0x304)](_0x28f87e[_0x483ac9(0x49e)]/0x2),_0x1a5c6f['x']+=_0x5c60b6[_0x483ac9(0x392)]||0x0,_0x1a5c6f['y']+=_0x5c60b6[_0x483ac9(0x26e)]||0x0;}else{const _0x4ff162=this['_classTierWindow'],_0x2f7464=_0x4ff162[_0x483ac9(0x21e)](_0x4ff162[_0x483ac9(0x36a)]()),_0x1fec88=_0x4ff162[_0x483ac9(0x410)]||0x0;_0x1a5c6f['x']=_0x4ff162['x']+_0x2f7464['x']+Math[_0x483ac9(0x304)](_0x2f7464['width']/0x2)+_0x1fec88,_0x1a5c6f['y']=_0x4ff162['y']+_0x2f7464['y']+Math[_0x483ac9(0x304)](_0x2f7464['height']/0x2)+_0x1fec88,_0x1a5c6f['x']+=_0x5c60b6['ConfirmAniSubclassOffsetX']||0x0,_0x1a5c6f['y']+=_0x5c60b6[_0x483ac9(0x17f)]||0x0;}return _0x1a5c6f['x']+=this[_0x483ac9(0x2a9)]['x'],_0x1a5c6f['y']+=this[_0x483ac9(0x2a9)]['y'],_0x1a5c6f;},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x3f4)]=function(_0x1b6e86,_0x1f3035,_0x3200b1){const _0x30ca09=_0x45bccd,_0x295336=this[_0x30ca09(0x427)](_0x1b6e86),_0x4a9040=$dataAnimations[_0x295336];if(!_0x4a9040)return;const _0x3e2f5e=this[_0x30ca09(0x237)](_0x4a9040),_0x244391=new(_0x3e2f5e?Sprite_AnimationMV:Sprite_Animation)(),_0x1f9356=[_0x3200b1],_0x380ba8=0x0;_0x244391[_0x30ca09(0x40f)](_0x1f9356,_0x4a9040,![],_0x380ba8,null),_0x244391['_classChangeTier']=_0x1f3035,this[_0x30ca09(0x324)](_0x3200b1),this[_0x30ca09(0x324)](_0x244391),this[_0x30ca09(0x188)][_0x30ca09(0x3ab)](_0x244391);},Scene_ClassChange['prototype'][_0x45bccd(0x427)]=function(_0x50c653){const _0x1347ef=_0x45bccd,_0x5a1b26=$dataClasses[_0x50c653];if(_0x5a1b26){const _0x57a90c=VisuMZ[_0x1347ef(0x3ac)][_0x1347ef(0x411)],_0x5a1a55=_0x5a1b26[_0x1347ef(0x323)];if(_0x5a1a55['match'](_0x57a90c[_0x1347ef(0x3ae)])){if(_0x1347ef(0x1c9)!=='lpGDR')return Number(RegExp['$1']);else{function _0x111349(){_0x4f353c=_0x29f2fc(_0x3de0ff['$1']);}}}}return VisuMZ[_0x1347ef(0x3ac)][_0x1347ef(0x38c)][_0x1347ef(0x239)][_0x1347ef(0x3cf)];},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x237)]=function(_0x339928){const _0x449657=_0x45bccd;return!!_0x339928[_0x449657(0x19e)];},Scene_ClassChange[_0x45bccd(0x404)]['updateClassChangeAnimations']=function(){const _0x4e840d=_0x45bccd,_0x573604=[];for(const _0x2be628 of this[_0x4e840d(0x188)]){if(_0x4e840d(0x332)===_0x4e840d(0x332)){if(!_0x2be628)continue;if(_0x2be628[_0x4e840d(0x3e7)]())continue;_0x573604[_0x4e840d(0x3ab)](_0x2be628);}else{function _0x35e662(){const _0x258f25=_0x4e840d;if(_0x3ceb36['isPlaytest']())_0xf22626[_0x258f25(0x210)](_0x179e86);}}}for(const _0x3fd1b0 of _0x573604){if(!_0x3fd1b0)continue;for(const _0x2017ac of _0x3fd1b0[_0x4e840d(0x3fd)]){if(_0x4e840d(0x49d)!==_0x4e840d(0x49d)){function _0x5b3930(){const _0x280f84=_0x4e840d;this[_0x280f84(0x2cd)]={};}}else this['removeChild'](_0x2017ac);}this[_0x4e840d(0x188)][_0x4e840d(0x306)](_0x3fd1b0),this['removeChild'](_0x3fd1b0);};},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x182)]=function(){const _0x1159f8=_0x45bccd,_0x57a4f5=[];for(const _0x11b8ed of this[_0x1159f8(0x188)]){if(!_0x11b8ed)continue;if(_0x11b8ed['_classChangeTier']<=0x1)continue;_0x57a4f5[_0x1159f8(0x3ab)](_0x11b8ed);}for(const _0x5b4813 of _0x57a4f5){if(!_0x5b4813)continue;for(const _0x5a7fd6 of _0x5b4813[_0x1159f8(0x3fd)]){this[_0x1159f8(0x47c)](_0x5a7fd6);}this[_0x1159f8(0x188)][_0x1159f8(0x306)](_0x5b4813),this[_0x1159f8(0x47c)](_0x5b4813);};},Scene_ClassChange['prototype']['buttonAssistSlotWindowShift']=function(){const _0x2ee1d2=_0x45bccd;if(!this['_classTierWindow'])return![];if(!this[_0x2ee1d2(0x253)][_0x2ee1d2(0x4a9)])return![];return this[_0x2ee1d2(0x253)]['isShiftRemoveShortcutEnabled']();},Scene_ClassChange[_0x45bccd(0x404)]['buttonAssistKey3']=function(){const _0x543fc3=_0x45bccd;if(this['buttonAssistSlotWindowShift']())return TextManager[_0x543fc3(0x3f1)](_0x543fc3(0x440));return Scene_MenuBase['prototype'][_0x543fc3(0x311)][_0x543fc3(0x42e)](this);},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x174)]=function(){const _0x897704=_0x45bccd;if(this['buttonAssistSlotWindowShift']())return TextManager[_0x897704(0x398)];return Scene_MenuBase['prototype'][_0x897704(0x174)]['call'](this);},Scene_ClassChange['prototype'][_0x45bccd(0x3fb)]=function(){const _0x2e71eb=_0x45bccd;if(this[_0x2e71eb(0x408)]())return this[_0x2e71eb(0x2e9)][_0x2e71eb(0x2c7)]/0x5/-0x3;return Scene_MenuBase[_0x2e71eb(0x404)][_0x2e71eb(0x3fb)][_0x2e71eb(0x42e)](this);},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x235)]=function(){const _0x1dc3c6=_0x45bccd;Scene_MenuBase[_0x1dc3c6(0x404)][_0x1dc3c6(0x235)]['call'](this),this[_0x1dc3c6(0x21c)](this[_0x1dc3c6(0x18a)]()),this[_0x1dc3c6(0x42d)]();},Scene_ClassChange[_0x45bccd(0x404)]['getBackgroundOpacity']=function(){const _0x29988e=_0x45bccd;return VisuMZ[_0x29988e(0x3ac)][_0x29988e(0x38c)]['BgSettings']['SnapshotOpacity'];},Scene_ClassChange[_0x45bccd(0x404)][_0x45bccd(0x42d)]=function(){const _0x5885b1=_0x45bccd,_0x42c34b=VisuMZ['ClassChangeSystem'][_0x5885b1(0x38c)][_0x5885b1(0x46f)];_0x42c34b&&(_0x42c34b[_0x5885b1(0x401)]!==''||_0x42c34b[_0x5885b1(0x278)]!=='')&&(this['_backSprite1']=new Sprite(ImageManager['loadTitle1'](_0x42c34b[_0x5885b1(0x401)]||'')),this[_0x5885b1(0x394)]=new Sprite(ImageManager[_0x5885b1(0x2d1)](_0x42c34b[_0x5885b1(0x278)]||'')),this[_0x5885b1(0x324)](this[_0x5885b1(0x351)]),this[_0x5885b1(0x324)](this['_backSprite2']),this[_0x5885b1(0x351)]['bitmap'][_0x5885b1(0x469)](this[_0x5885b1(0x1d3)][_0x5885b1(0x187)](this,this[_0x5885b1(0x351)])),this[_0x5885b1(0x394)][_0x5885b1(0x3d8)][_0x5885b1(0x469)](this[_0x5885b1(0x1d3)][_0x5885b1(0x187)](this,this[_0x5885b1(0x394)])));},Scene_ClassChange['prototype'][_0x45bccd(0x1d3)]=function(_0x3301dc){const _0x54d776=_0x45bccd;this[_0x54d776(0x219)](_0x3301dc),this[_0x54d776(0x3bc)](_0x3301dc);},Window_Base[_0x45bccd(0x404)][_0x45bccd(0x244)]=function(_0x2dede5,_0xb1bd6e,_0x57911d,_0xa49ebe,_0x4f7c0b){const _0x594b0a=_0x45bccd;_0x4f7c0b=_0x4f7c0b||_0x594b0a(0x2d6);const _0x4acc14=_0x594b0a(0x355)[_0x594b0a(0x3dd)](ImageManager[_0x594b0a(0x1fc)]),_0x3a0b86=TextManager[_0x594b0a(0x473)],_0x39189b=_0x3a0b86[_0x594b0a(0x3dd)](_0x2dede5,TextManager['classPointsAbbr'],_0x4acc14,TextManager['classPointsFull']),_0x1276e9=this[_0x594b0a(0x2fe)](_0x39189b)['width'];if(_0x4f7c0b==='left')_0xb1bd6e+=0x0;else _0x4f7c0b===_0x594b0a(0x2c9)?_0xb1bd6e+=Math['round']((_0xa49ebe-_0x1276e9)/0x2):_0xb1bd6e+=_0xa49ebe-_0x1276e9;this[_0x594b0a(0x245)](_0x39189b,_0xb1bd6e,_0x57911d);},Window_Base[_0x45bccd(0x404)][_0x45bccd(0x273)]=function(_0x42ea44,_0x542d7d,_0x151041,_0x200476,_0x26fe1c,_0x3485f7){const _0x5d6eac=_0x42ea44['getClassPoints'](_0x542d7d);this['drawClassPoints'](_0x5d6eac,_0x151041,_0x200476,_0x26fe1c,_0x3485f7);},Window_Base[_0x45bccd(0x404)][_0x45bccd(0x24c)]=function(_0x22a696,_0x54e054,_0x16ab54,_0x2b6a05,_0x1f2578){const _0x2efc2d=_0x45bccd;_0x1f2578=_0x1f2578||_0x2efc2d(0x2d6);const _0x17c8a5=_0x2efc2d(0x355)['format'](ImageManager[_0x2efc2d(0x1e0)]),_0x5ec38c=TextManager[_0x2efc2d(0x1cd)],_0x5a4cf8=_0x5ec38c['format'](_0x22a696,TextManager[_0x2efc2d(0x1fe)],_0x17c8a5,TextManager[_0x2efc2d(0x3d0)]),_0x240ec0=this['textSizeEx'](_0x5a4cf8)[_0x2efc2d(0x2c7)];if(_0x1f2578===_0x2efc2d(0x2d6)){if(_0x2efc2d(0x49b)==='UGEpF')_0x54e054+=0x0;else{function _0x69a9b9(){const _0x290ba9=_0x2efc2d,_0x381843=this[_0x290ba9(0x1b0)](),_0x16d570=new _0x9984c1(_0x381843);_0x16d570[_0x290ba9(0x22e)](this['_helpWindow']),_0x16d570[_0x290ba9(0x2bf)](_0x3906e7[_0x290ba9(0x3ac)][_0x290ba9(0x38c)][_0x290ba9(0x239)][_0x290ba9(0x312)]),this[_0x290ba9(0x217)](_0x16d570),this[_0x290ba9(0x253)]=_0x16d570,_0x16d570[_0x290ba9(0x48b)](_0x290ba9(0x3e9),this[_0x290ba9(0x1d0)][_0x290ba9(0x187)](this)),this[_0x290ba9(0x39b)]()>0x1&&(_0x16d570['setHandler']('pagedown',this['nextActor'][_0x290ba9(0x187)](this)),_0x16d570[_0x290ba9(0x48b)](_0x290ba9(0x407),this['previousActor'][_0x290ba9(0x187)](this))),_0x16d570[_0x290ba9(0x48b)](_0x290ba9(0x31f),this[_0x290ba9(0x4b7)][_0x290ba9(0x187)](this));}}}else _0x1f2578===_0x2efc2d(0x2c9)?_0x54e054+=Math[_0x2efc2d(0x304)]((_0x2b6a05-_0x240ec0)/0x2):_0x54e054+=_0x2b6a05-_0x240ec0;this['drawTextEx'](_0x5a4cf8,_0x54e054,_0x16ab54);},Window_Base['prototype'][_0x45bccd(0x19f)]=function(_0x2504dc,_0x42e01f,_0x2c61ca,_0x2dcd2c,_0x283564,_0xc54fb6){const _0x3b7d9c=_0x45bccd,_0x335acd=_0x2504dc[_0x3b7d9c(0x3e8)](_0x42e01f);this['drawJobPoints'](_0x335acd,_0x2c61ca,_0x2dcd2c,_0x283564,_0xc54fb6);},Window_Base[_0x45bccd(0x404)][_0x45bccd(0x460)]=function(_0x3af3cf,_0x429e55,_0x221393,_0x459351){const _0x14370a=_0x45bccd;if(!$dataClasses[_0x429e55])return;this['isClassExpGaugeDrawn']()&&this[_0x14370a(0x2f7)](_0x3af3cf,_0x429e55,_0x221393,_0x459351),this[_0x14370a(0x43d)](ColorManager['systemColor']()),this[_0x14370a(0x1c5)](TextManager[_0x14370a(0x1e3)],_0x221393,_0x459351,0x30),this[_0x14370a(0x26a)](),this[_0x14370a(0x1c5)](_0x3af3cf['classLevel'](_0x429e55),_0x221393+0x54,_0x459351,0x24,_0x14370a(0x16b));},Window_Base[_0x45bccd(0x404)][_0x45bccd(0x24f)]=function(){const _0x1e8fbb=_0x45bccd;return Imported[_0x1e8fbb(0x495)]&&VisuMZ[_0x1e8fbb(0x3e0)][_0x1e8fbb(0x38c)]['UI'][_0x1e8fbb(0x289)];},Window_Base[_0x45bccd(0x404)][_0x45bccd(0x2f7)]=function(_0x3feb51,_0x491f9f,_0x1d7d08,_0x1815b4){const _0x40cf7e=_0x45bccd;if(!_0x3feb51)return;if(!_0x3feb51[_0x40cf7e(0x3a1)]())return;const _0x55ce11=0x80,_0x115fcc=_0x3feb51[_0x40cf7e(0x204)](_0x491f9f);let _0xbb59f3=ColorManager['expGaugeColor1'](),_0x3ca8b2=ColorManager[_0x40cf7e(0x329)]();if(_0x115fcc>=0x1){if('teDra'!==_0x40cf7e(0x178)){function _0x2d77b2(){const _0x234c5c=_0x40cf7e;return _0x284fb7[_0x234c5c(0x398)];}}else _0xbb59f3=ColorManager[_0x40cf7e(0x479)](),_0x3ca8b2=ColorManager[_0x40cf7e(0x4af)]();}this['drawGauge'](_0x1d7d08,_0x1815b4,_0x55ce11,_0x115fcc,_0xbb59f3,_0x3ca8b2);},VisuMZ['ClassChangeSystem'][_0x45bccd(0x416)]=Window_MenuCommand[_0x45bccd(0x404)][_0x45bccd(0x29f)],Window_MenuCommand[_0x45bccd(0x404)]['addOriginalCommands']=function(){const _0x1ef460=_0x45bccd;VisuMZ['ClassChangeSystem']['Window_MenuCommand_addOriginalCommands'][_0x1ef460(0x42e)](this),this['addClassChangeSystemCommand']();},Window_MenuCommand[_0x45bccd(0x404)][_0x45bccd(0x378)]=function(){const _0x1e61a7=_0x45bccd;if(!this[_0x1e61a7(0x352)]())return;if(!this[_0x1e61a7(0x451)]())return;const _0x2d205b=TextManager[_0x1e61a7(0x476)],_0x147ff9=this[_0x1e61a7(0x25d)]();this['addCommand'](_0x2d205b,_0x1e61a7(0x3ac),_0x147ff9);},Window_MenuCommand[_0x45bccd(0x404)][_0x45bccd(0x352)]=function(){const _0x544732=_0x45bccd;return Imported[_0x544732(0x484)]?![]:!![];},Window_MenuCommand[_0x45bccd(0x404)]['isClassChangeCommandVisible']=function(){const _0x17f845=_0x45bccd;return $gameSystem[_0x17f845(0x222)]();},Window_MenuCommand[_0x45bccd(0x404)][_0x45bccd(0x25d)]=function(){const _0x723a1f=_0x45bccd;return $gameSystem[_0x723a1f(0x36e)]();};function Window_ClassStatus(){this['initialize'](...arguments);}Window_ClassStatus[_0x45bccd(0x404)]=Object['create'](Window_StatusBase['prototype']),Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x30a)]=Window_ClassStatus,Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x2b5)]=function(_0x4216bd){const _0x178b8d=_0x45bccd;Window_StatusBase['prototype'][_0x178b8d(0x2b5)][_0x178b8d(0x42e)](this,_0x4216bd),this['_actor']=null,this['_tempActor']=null,this['refresh']();},Window_ClassStatus['prototype'][_0x45bccd(0x205)]=function(_0x5a388c){const _0x4b9b5a=_0x45bccd;this['_actor']!==_0x5a388c&&(this[_0x4b9b5a(0x349)]=_0x5a388c,this[_0x4b9b5a(0x3d3)]());},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x27c)]=function(){return 0x0;},Window_ClassStatus[_0x45bccd(0x404)]['setTempActor']=function(_0x118102){const _0xe90a78=_0x45bccd;if(this[_0xe90a78(0x310)]!==_0x118102){if(_0xe90a78(0x368)!==_0xe90a78(0x37f))this[_0xe90a78(0x310)]=_0x118102,this['refresh']();else{function _0x576e66(){const _0x52ac7c=_0xe90a78;this[_0x52ac7c(0x43d)](_0x4f8157[_0x52ac7c(0x27a)](_0x323794[_0x52ac7c(0x4b0)])),this[_0x52ac7c(0x1c5)](_0x780364[_0x52ac7c(0x433)],_0x567fec['x'],_0x3feb2a['y'],_0x21c86d['width'],_0x52ac7c(0x2c9)),this[_0x52ac7c(0x26a)]();}}}},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x3d3)]=function(){const _0x10995f=_0x45bccd;this[_0x10995f(0x262)](),this[_0x10995f(0x47f)]();if(this[_0x10995f(0x349)])this[_0x10995f(0x349)]['refresh']();this['prepareRefreshItemsEquipsCoreLayout']();},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x1ba)]=function(){const _0x17ec8d=_0x45bccd;this[_0x17ec8d(0x31e)]['clear']();if(!this[_0x17ec8d(0x349)])return;if(this[_0x17ec8d(0x3c5)]()){const _0x589744=ImageManager[_0x17ec8d(0x429)](this['_actor'][_0x17ec8d(0x462)]());_0x589744[_0x17ec8d(0x469)](this[_0x17ec8d(0x1d2)]['bind'](this));}else this[_0x17ec8d(0x29c)]();},Window_ClassStatus[_0x45bccd(0x404)]['isMainMenuCoreMenuImageOptionAvailable']=function(){const _0x7abb50=_0x45bccd;return Imported[_0x7abb50(0x484)]&&this['_actor']['getMenuImage']()!==''&&VisuMZ['ClassChangeSystem'][_0x7abb50(0x38c)][_0x7abb50(0x239)][_0x7abb50(0x4b3)];},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x1d2)]=function(){const _0x3d99e0=_0x45bccd;VisuMZ[_0x3d99e0(0x3ac)][_0x3d99e0(0x38c)][_0x3d99e0(0x239)][_0x3d99e0(0x38f)][_0x3d99e0(0x42e)](this),this['drawParameterList']();},Window_ClassStatus['prototype']['drawItemActorMenuImage']=function(_0x5e1d4e,_0x1db933,_0x3749cb,_0x3b8b14,_0x9cf500){const _0x3748d0=_0x45bccd,_0x4767f1=ImageManager[_0x3748d0(0x429)](_0x5e1d4e['getMenuImage']()),_0x511fa5=this['innerWidth']-_0x4767f1[_0x3748d0(0x2c7)];_0x1db933+=_0x511fa5/0x2;if(_0x511fa5<0x0)_0x3b8b14-=_0x511fa5;Window_StatusBase[_0x3748d0(0x404)][_0x3748d0(0x36d)][_0x3748d0(0x42e)](this,_0x5e1d4e,_0x1db933,_0x3749cb,_0x3b8b14,_0x9cf500);},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x29c)]=function(){const _0x4a1166=_0x45bccd;VisuMZ[_0x4a1166(0x3ac)][_0x4a1166(0x38c)][_0x4a1166(0x239)][_0x4a1166(0x3a3)]['call'](this),this['drawParameterList']();},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x3e5)]=function(){const _0x19f071=_0x45bccd;this['resetFontSettings'](),VisuMZ['ClassChangeSystem']['Settings'][_0x19f071(0x239)][_0x19f071(0x3b2)][_0x19f071(0x42e)](this);},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x1ee)]=function(){const _0xc54fd4=_0x45bccd;if(Imported[_0xc54fd4(0x495)])return VisuMZ[_0xc54fd4(0x3e0)][_0xc54fd4(0x38c)][_0xc54fd4(0x4a8)][_0xc54fd4(0x17c)];else{if(_0xc54fd4(0x3b7)!==_0xc54fd4(0x3b7)){function _0x5b7226(){const _0x111167=_0xc54fd4;return _0xf9425b['ClassChangeSystem'][_0x111167(0x38c)][_0x111167(0x338)][_0x111167(0x1b4)];}}else return[0x0,0x1,0x2,0x3,0x4,0x5,0x6,0x7];}},Window_ClassStatus['prototype'][_0x45bccd(0x265)]=function(){const _0x5d8232=_0x45bccd;return VisuMZ[_0x5d8232(0x3ac)][_0x5d8232(0x38c)][_0x5d8232(0x239)]['ParamValueFontSize'];},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x39c)]=function(){const _0x511aba=_0x45bccd;return Imported[_0x511aba(0x495)]&&VisuMZ[_0x511aba(0x3e0)][_0x511aba(0x38c)][_0x511aba(0x4a8)][_0x511aba(0x2ae)];},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x3b1)]=function(_0x113bd1,_0x40faa5,_0x5ddf58,_0xfb0e82,_0x2cec23){const _0x3e6c81=_0x45bccd;if(Imported[_0x3e6c81(0x484)]){if('vgKXc'!=='ItOZO')switch(this[_0x3e6c81(0x30b)]()){case'none':break;case'sprite':this[_0x3e6c81(0x373)](_0x113bd1,_0x40faa5,_0x5ddf58,_0xfb0e82,_0x2cec23);break;case _0x3e6c81(0x430):this[_0x3e6c81(0x4ae)](_0x113bd1,_0x40faa5,_0x5ddf58,_0xfb0e82,_0x2cec23);break;default:Window_StatusBase[_0x3e6c81(0x404)]['drawActorFace'][_0x3e6c81(0x42e)](this,_0x113bd1,_0x40faa5,_0x5ddf58,_0xfb0e82,_0x2cec23);break;}else{function _0xfb1334(){return _0x5919f7(_0x128286['$1']);}}}else{if(_0x3e6c81(0x299)==='WazDu'){function _0x15ad13(){const _0x1fead7=_0x3e6c81;this[_0x1fead7(0x310)]=_0x37b6a1,this[_0x1fead7(0x3d3)]();}}else Window_StatusBase[_0x3e6c81(0x404)][_0x3e6c81(0x3b1)][_0x3e6c81(0x42e)](this,_0x113bd1,_0x40faa5,_0x5ddf58,_0xfb0e82,_0x2cec23);}},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x1e4)]=function(_0x166efd,_0x18b51b){const _0x14ca1b=_0x45bccd,_0x33b930=this[_0x14ca1b(0x2d8)]();this[_0x14ca1b(0x43d)](ColorManager['systemColor']());if(Imported[_0x14ca1b(0x495)]){if(_0x14ca1b(0x2fc)===_0x14ca1b(0x16d)){function _0x2c2c88(){const _0x80699=_0x14ca1b,_0x5ac481=_0x2de087[_0x80699(0x429)](this[_0x80699(0x349)][_0x80699(0x462)]());_0x5ac481['addLoadListener'](this[_0x80699(0x1d2)][_0x80699(0x187)](this));}}else{const _0x382a97=VisuMZ[_0x14ca1b(0x3e0)][_0x14ca1b(0x38c)]['UI']['ParamArrow'];this[_0x14ca1b(0x1c5)](_0x382a97,_0x166efd,_0x18b51b,_0x33b930,_0x14ca1b(0x2c9));}}else this[_0x14ca1b(0x1c5)]('→',_0x166efd,_0x18b51b,_0x33b930,_0x14ca1b(0x2c9));},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x2d8)]=function(){return 0x20;},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x1bc)]=function(_0x5b27f1,_0x20e0db,_0x24ce22,_0x351f17){const _0x243e16=_0x45bccd,_0x1059f4=this[_0x243e16(0x213)]();if(Imported['VisuMZ_0_CoreEngine'])this[_0x243e16(0x23a)](_0x20e0db+_0x1059f4,_0x24ce22,_0x351f17,_0x5b27f1,![]);else{if(_0x243e16(0x2ca)===_0x243e16(0x2ca))this[_0x243e16(0x43d)](ColorManager[_0x243e16(0x2e3)]()),this[_0x243e16(0x1c5)](TextManager[_0x243e16(0x26c)](_0x5b27f1),_0x20e0db+_0x1059f4,_0x24ce22,_0x351f17),this[_0x243e16(0x26a)]();else{function _0x39091a(){const _0x3b7d21=_0x243e16;this[_0x3b7d21(0x25a)][_0x4ca7e8]=_0x3b2c15(_0x2cd838);}}}},Window_ClassStatus['prototype'][_0x45bccd(0x413)]=function(_0x2d86ee,_0x1a8d71,_0x4bc155,_0x3a30f3){const _0x45c3c9=_0x45bccd,_0x5d7110=this[_0x45c3c9(0x213)]();let _0xace44e=0x0;if(Imported[_0x45c3c9(0x495)])_0xace44e=this[_0x45c3c9(0x349)][_0x45c3c9(0x218)](_0x2d86ee,!![]);else{if(_0x45c3c9(0x496)===_0x45c3c9(0x34f)){function _0x2020bd(){const _0x5dcd30=_0x45c3c9;this[_0x5dcd30(0x20a)]=[];}}else _0xace44e=this[_0x45c3c9(0x349)]['param'](_0x2d86ee);}const _0x17d126=_0xace44e;this[_0x45c3c9(0x1c5)](_0xace44e,_0x1a8d71,_0x4bc155,_0x3a30f3-_0x5d7110,'right'),this[_0x45c3c9(0x26a)]();},Window_ClassStatus['prototype'][_0x45bccd(0x1aa)]=function(_0x5cfcc3,_0x105687,_0x2ff1bc,_0x1b1f79){const _0x4fb749=_0x45bccd,_0x5ea10f=this[_0x4fb749(0x213)]();let _0x146598=0x0,_0x5ab6e6=0x0,_0x1827b8='';if(this['_tempActor']){Imported['VisuMZ_0_CoreEngine']?(_0x146598=this[_0x4fb749(0x349)][_0x4fb749(0x218)](_0x5cfcc3,![]),_0x5ab6e6=this[_0x4fb749(0x310)][_0x4fb749(0x218)](_0x5cfcc3,![]),_0x1827b8=this['_tempActor'][_0x4fb749(0x218)](_0x5cfcc3,!![])):(_0x146598=this[_0x4fb749(0x349)][_0x4fb749(0x26c)](_0x5cfcc3),_0x5ab6e6=this[_0x4fb749(0x310)][_0x4fb749(0x26c)](_0x5cfcc3),_0x1827b8=this[_0x4fb749(0x310)]['param'](_0x5cfcc3));const _0x4ada63=_0x146598,_0x3fe0ef=_0x5ab6e6;diffValue=_0x3fe0ef-_0x4ada63,this[_0x4fb749(0x43d)](ColorManager['paramchangeTextColor'](diffValue)),this[_0x4fb749(0x1c5)](_0x1827b8,_0x105687,_0x2ff1bc,_0x1b1f79-_0x5ea10f,_0x4fb749(0x16b));}this[_0x4fb749(0x26a)]();},Window_ClassStatus[_0x45bccd(0x404)]['drawUpdatedParamValueDiff']=function(_0x1ebbc7,_0x2cd239,_0x5e6ff6,_0x51109c){const _0xd9b4bf=_0x45bccd,_0x6af7a2=this[_0xd9b4bf(0x213)]();let _0x22d537=0x0,_0x442e00=0x0,_0x19a4b9=![];if(this[_0xd9b4bf(0x310)]){Imported[_0xd9b4bf(0x495)]?(_0x22d537=this[_0xd9b4bf(0x349)][_0xd9b4bf(0x218)](_0x1ebbc7,![]),_0x442e00=this[_0xd9b4bf(0x310)]['paramValueByName'](_0x1ebbc7,![]),_0x19a4b9=String(this[_0xd9b4bf(0x349)][_0xd9b4bf(0x218)](_0x1ebbc7,!![]))[_0xd9b4bf(0x264)](/([%％])/i)):(_0x22d537=this[_0xd9b4bf(0x349)][_0xd9b4bf(0x26c)](_0x1ebbc7),_0x442e00=this['_tempActor'][_0xd9b4bf(0x26c)](_0x1ebbc7),_0x19a4b9=_0x22d537%0x1!==0x0||_0x442e00%0x1!==0x0);const _0x1ab311=_0x22d537,_0x3c7959=_0x442e00,_0x1599b4=_0x3c7959-_0x1ab311;let _0x5d5f87=_0x1599b4;if(_0x19a4b9)_0x5d5f87=Math[_0xd9b4bf(0x304)](_0x1599b4*0x64)+'%';_0x1599b4!==0x0&&(this[_0xd9b4bf(0x43d)](ColorManager[_0xd9b4bf(0x399)](_0x1599b4)),_0x5d5f87=(_0x1599b4>0x0?_0xd9b4bf(0x3cb):_0xd9b4bf(0x389))[_0xd9b4bf(0x3dd)](_0x5d5f87),this['drawText'](_0x5d5f87,_0x2cd239+_0x6af7a2,_0x5e6ff6,_0x51109c,_0xd9b4bf(0x2d6)));}this[_0xd9b4bf(0x26a)]();},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x3c4)]=function(_0x168439,_0x28a4d9,_0x13aae7,_0x463b3e,_0x555343){const _0x1ded3e=_0x45bccd;if(VisuMZ['ClassChangeSystem'][_0x1ded3e(0x38c)][_0x1ded3e(0x239)][_0x1ded3e(0x1ef)]===![])return;_0x555343=Math['max'](_0x555343||0x1,0x1);while(_0x555343--){_0x463b3e=_0x463b3e||this[_0x1ded3e(0x376)](),this['contents'][_0x1ded3e(0x16c)]=0xa0;const _0x1eed9f=ColorManager[_0x1ded3e(0x22c)]();this[_0x1ded3e(0x31e)]['fillRect'](_0x168439+0x1,_0x28a4d9+0x1,_0x13aae7-0x2,_0x463b3e-0x2,_0x1eed9f),this['contents'][_0x1ded3e(0x16c)]=0xff;}},ColorManager[_0x45bccd(0x22c)]=function(){const _0x24bffa=_0x45bccd,_0x26ec92=VisuMZ[_0x24bffa(0x3ac)][_0x24bffa(0x38c)][_0x24bffa(0x239)];let _0x59981e=_0x26ec92[_0x24bffa(0x2f0)]!==undefined?_0x26ec92[_0x24bffa(0x2f0)]:0x13;return ColorManager[_0x24bffa(0x27a)](_0x59981e);},Window_ClassStatus[_0x45bccd(0x404)][_0x45bccd(0x417)]=function(_0x14844b,_0x11455f,_0x12c535){const _0x5b3a86=_0x45bccd,_0x20d733=VisuMZ[_0x5b3a86(0x3ac)]['Settings']['Window']['DisplayedResources'],_0x8f8390=this['_actor']['currentClass']()['id'];for(const _0x4a91f6 of _0x20d733){switch(_0x4a91f6[_0x5b3a86(0x377)]()['trim']()){case'AP':if(!Imported[_0x5b3a86(0x4a1)])continue;this[_0x5b3a86(0x456)](this[_0x5b3a86(0x349)],_0x8f8390,_0x14844b,_0x11455f,_0x12c535,'right'),_0x11455f+=this[_0x5b3a86(0x376)]();break;case'CP':if(!Imported[_0x5b3a86(0x3da)])continue;this[_0x5b3a86(0x273)](this[_0x5b3a86(0x349)],_0x8f8390,_0x14844b,_0x11455f,_0x12c535,_0x5b3a86(0x16b)),_0x11455f+=this[_0x5b3a86(0x376)]();break;case'JP':if(!Imported['VisuMZ_2_ClassChangeSystem'])continue;this[_0x5b3a86(0x19f)](this[_0x5b3a86(0x349)],_0x8f8390,_0x14844b,_0x11455f,_0x12c535,_0x5b3a86(0x16b)),_0x11455f+=this[_0x5b3a86(0x376)]();break;case'SP':if(!Imported[_0x5b3a86(0x4a1)])continue;this['drawActorSkillPoints'](this['_actor'],_0x8f8390,_0x14844b,_0x11455f,_0x12c535,_0x5b3a86(0x16b)),_0x11455f+=this[_0x5b3a86(0x376)]();break;}}};function Window_ClassCommand(){const _0x5bc85e=_0x45bccd;this[_0x5bc85e(0x2b5)](...arguments);}Window_ClassCommand[_0x45bccd(0x404)]=Object['create'](Window_Command[_0x45bccd(0x404)]),Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x30a)]=Window_ClassCommand,Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x2b5)]=function(_0x52b3e3){const _0x333526=_0x45bccd;Window_Command[_0x333526(0x404)][_0x333526(0x2b5)][_0x333526(0x42e)](this,_0x52b3e3),this['deselect'](),this[_0x333526(0x489)]();},Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x24e)]=function(){return this['lineHeight']()*0x3+0x8;},Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x205)]=function(_0x525914){const _0x1343f6=_0x45bccd;this[_0x1343f6(0x349)]!==_0x525914&&(this[_0x1343f6(0x349)]=_0x525914,this[_0x1343f6(0x3d3)]());},Window_ClassCommand[_0x45bccd(0x404)]['refresh']=function(){const _0x39d970=_0x45bccd;Window_Command[_0x39d970(0x404)][_0x39d970(0x3d3)][_0x39d970(0x42e)](this),this['refreshCursor']();if(this[_0x39d970(0x4a9)])this[_0x39d970(0x249)]();},Window_ClassCommand['prototype'][_0x45bccd(0x31d)]=function(_0x194d28,_0x26c5ca){const _0xd758c8=_0x45bccd;_0x26c5ca=_0x26c5ca||0x1,this[_0xd758c8(0x47a)](![]);const _0x566d80=ColorManager[_0xd758c8(0x40c)](),_0x5b2fdf=ColorManager[_0xd758c8(0x3db)](),_0x5b6e6d=_0x194d28['width']/0x2,_0x357885=this[_0xd758c8(0x376)]();while(_0x26c5ca--){this[_0xd758c8(0x31e)][_0xd758c8(0x499)](_0x194d28['x'],_0x194d28['y'],_0x5b6e6d,_0x357885,_0x5b2fdf,_0x566d80),this[_0xd758c8(0x31e)][_0xd758c8(0x499)](_0x194d28['x']+_0x5b6e6d,_0x194d28['y'],_0x5b6e6d,_0x357885,_0x566d80,_0x5b2fdf);}this['changePaintOpacity'](!![]);},Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x4bd)]=function(_0x3ec50f,_0x3cff6f,_0x3518a9){const _0x3eb0f8=_0x45bccd;if(!_0x3cff6f)return;const _0x4527c8=VisuMZ['ClassChangeSystem'][_0x3eb0f8(0x411)],_0x2a133c=_0x3cff6f[_0x3eb0f8(0x323)];let _0xe3f970='';if(_0x2a133c[_0x3eb0f8(0x264)](_0x4527c8[_0x3eb0f8(0x2a4)]))_0xe3f970=String(RegExp['$1']);else _0x2a133c[_0x3eb0f8(0x264)](_0x4527c8[_0x3eb0f8(0x170)])&&(_0xe3f970=String(RegExp['$1']));if(_0xe3f970){if(_0x3eb0f8(0x35b)!==_0x3eb0f8(0x17a)){const _0x20a7c7=ImageManager[_0x3eb0f8(0x429)](_0xe3f970);_0x20a7c7[_0x3eb0f8(0x469)](this[_0x3eb0f8(0x290)][_0x3eb0f8(0x187)](this,_0x3ec50f,_0x20a7c7));}else{function _0x295ba6(){const _0x140c39=_0x3eb0f8;_0x343120[_0x140c39(0x3ac)]['BattleManager_gainExp']['call'](this);const _0x4726b7=this[_0x140c39(0x44b)][_0x140c39(0x412)];for(const _0x58b990 of _0x44192e[_0x140c39(0x340)]()){_0x58b990['gainMulticlassExp'](_0x4726b7);}}}}else{if(_0x3eb0f8(0x421)==='fTFge')this[_0x3eb0f8(0x45f)](_0x3cff6f,_0x3518a9);else{function _0x427ca2(){_0x22c306=_0x516ddb||this['currentClass']()['id'];}}}},Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x290)]=function(_0x1e0c87,_0x3b628e){const _0x1c4baf=_0x45bccd,_0x5a692c=this['itemRectWithPadding'](_0x1e0c87);let _0x20733e=_0x5a692c['x']+this[_0x1c4baf(0x213)](),_0x3fdcd9=_0x5a692c['y']+0x4,_0x5ace42=_0x5a692c[_0x1c4baf(0x2c7)]-this[_0x1c4baf(0x213)]()*0x2,_0x5a40b4=Math[_0x1c4baf(0x3eb)](this[_0x1c4baf(0x376)]()*0x3,_0x5a692c[_0x1c4baf(0x49e)])-0x4,_0x121897=Math[_0x1c4baf(0x3eb)](_0x5ace42,_0x5a40b4);const _0x52bbc3=_0x121897/_0x3b628e[_0x1c4baf(0x2c7)],_0x34c77e=_0x121897/_0x3b628e[_0x1c4baf(0x49e)],_0x11d53d=Math[_0x1c4baf(0x3eb)](_0x52bbc3,_0x34c77e,0x1);let _0x88bfc7=Math[_0x1c4baf(0x304)](_0x3b628e[_0x1c4baf(0x2c7)]*_0x11d53d),_0x2c0c82=Math[_0x1c4baf(0x304)](_0x3b628e['height']*_0x11d53d);_0x20733e+=Math[_0x1c4baf(0x304)]((_0x121897-_0x88bfc7)/0x2),_0x3fdcd9+=Math['round']((_0x121897-_0x2c0c82)/0x2);const _0x21a9c9=_0x3b628e[_0x1c4baf(0x2c7)],_0x35b3da=_0x3b628e[_0x1c4baf(0x49e)];this[_0x1c4baf(0x31e)]['_context'][_0x1c4baf(0x2a3)]=!![],this['contents']['blt'](_0x3b628e,0x0,0x0,_0x21a9c9,_0x35b3da,_0x20733e,_0x3fdcd9,_0x88bfc7,_0x2c0c82),this['contents'][_0x1c4baf(0x1e1)][_0x1c4baf(0x2a3)]=!![];},Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x45f)]=function(_0xdfadaa,_0x3aec8a){const _0x4f394f=_0x45bccd;if(!_0xdfadaa)return;const _0x40d579=_0xdfadaa[_0x4f394f(0x34a)];let _0x39e587=_0x3aec8a['x']+this['itemPadding'](),_0x2cf830=_0x3aec8a['y']+0x4,_0x2e58b2=_0x3aec8a[_0x4f394f(0x2c7)]-this[_0x4f394f(0x213)]()*0x2,_0xf1d0e4=Math[_0x4f394f(0x3eb)](this[_0x4f394f(0x376)]()*0x3,_0x3aec8a[_0x4f394f(0x49e)]),_0x2077f2=Math[_0x4f394f(0x3eb)](_0x2e58b2,_0xf1d0e4);_0x2077f2=Math[_0x4f394f(0x382)](_0x2077f2/ImageManager[_0x4f394f(0x444)])*ImageManager['iconWidth'],_0x2cf830+=(_0xf1d0e4-_0x2077f2)/0x2;const _0x13d910=ImageManager[_0x4f394f(0x302)](_0x4f394f(0x453)),_0x2fa486=ImageManager[_0x4f394f(0x444)],_0x456994=ImageManager['iconHeight'],_0x4e8cb3=_0x40d579%0x10*_0x2fa486,_0x11c04b=Math[_0x4f394f(0x382)](_0x40d579/0x10)*_0x456994;this[_0x4f394f(0x31e)]['_context'][_0x4f394f(0x2a3)]=![],this['contents'][_0x4f394f(0x277)](_0x13d910,_0x4e8cb3,_0x11c04b,_0x2fa486,_0x456994,_0x39e587,_0x2cf830,_0x2077f2,_0x2077f2),this[_0x4f394f(0x31e)][_0x4f394f(0x1e1)][_0x4f394f(0x2a3)]=!![];},Window_ClassCommand[_0x45bccd(0x404)][_0x45bccd(0x1a9)]=function(){const _0x806ce3=_0x45bccd;return VisuMZ[_0x806ce3(0x3ac)][_0x806ce3(0x38c)]['Window'][_0x806ce3(0x47d)]||[];},Window_ClassCommand[_0x45bccd(0x404)]['drawClassResources']=function(_0x485bc3,_0x105d23){const _0x409730=_0x45bccd,_0x56c731=this[_0x409730(0x1a9)]();let _0x423c92=_0x105d23['y']+this[_0x409730(0x376)](),_0x4c914f=0x0;const _0x2508c7=_0x105d23[_0x409730(0x2c7)]-this[_0x409730(0x213)]()*0x2;for(const _0x52e3e8 of _0x56c731){if(_0x409730(0x3af)!==_0x409730(0x3af)){function _0x21c47b(){const _0xa3a178=_0x409730,_0x2be024=_0x2942b6(_0x5f1c6c['$1']);_0x2570fb[_0xa3a178(0x2ea)](_0x2be024);}}else{if(_0x4c914f>=0x2)return;switch(_0x52e3e8){case'AP':if(!Imported[_0x409730(0x4a1)])continue;let _0x25cd78=VisuMZ[_0x409730(0x3cc)][_0x409730(0x38c)]['AbilityPoints'];if(!_0x25cd78)continue;if(_0x25cd78[_0x409730(0x191)])continue;this[_0x409730(0x456)](this[_0x409730(0x349)],_0x485bc3,_0x105d23['x'],_0x423c92,_0x2508c7,_0x409730(0x16b)),_0x423c92+=this[_0x409730(0x376)](),_0x4c914f++;break;case'CP':if(!Imported[_0x409730(0x3da)])continue;let _0x630a6f=VisuMZ[_0x409730(0x3ac)][_0x409730(0x38c)]['ClassPoints'];if(!_0x630a6f)continue;if(_0x630a6f['SharedResource'])continue;this[_0x409730(0x273)](this['_actor'],_0x485bc3,_0x105d23['x'],_0x423c92,_0x2508c7,_0x409730(0x16b)),_0x423c92+=this['lineHeight'](),_0x4c914f++;break;case'JP':if(!Imported['VisuMZ_2_ClassChangeSystem'])continue;let _0x555b29=VisuMZ['ClassChangeSystem'][_0x409730(0x38c)][_0x409730(0x1ea)];if(!_0x555b29)continue;if(_0x555b29[_0x409730(0x191)])continue;this[_0x409730(0x19f)](this[_0x409730(0x349)],_0x485bc3,_0x105d23['x'],_0x423c92,_0x2508c7,'right'),_0x423c92+=this[_0x409730(0x376)](),_0x4c914f++;break;case'SP':if(!Imported[_0x409730(0x4a1)])continue;let _0x3d5caf=VisuMZ[_0x409730(0x3cc)][_0x409730(0x38c)][_0x409730(0x20d)];if(!_0x3d5caf)continue;if(_0x3d5caf['SharedResource'])continue;this['drawActorSkillPoints'](this['_actor'],_0x485bc3,_0x105d23['x'],_0x423c92,_0x2508c7,_0x409730(0x16b)),_0x423c92+=this[_0x409730(0x376)](),_0x4c914f++;break;}}}};function _0x2d2d(_0x469d1f,_0x1b1cab){_0x469d1f=_0x469d1f-0x165;let _0x34786c=_0x3478[_0x469d1f];return _0x34786c;}function Window_ClassTier(){const _0x26433b=_0x45bccd;this[_0x26433b(0x2b5)](...arguments);}Window_ClassTier[_0x45bccd(0x404)]=Object[_0x45bccd(0x1ec)](Window_ClassCommand[_0x45bccd(0x404)]),Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x30a)]=Window_ClassTier,Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x2b5)]=function(_0xc4fb6f){const _0x7d8365=_0x45bccd;Window_ClassCommand[_0x7d8365(0x404)][_0x7d8365(0x2b5)][_0x7d8365(0x42e)](this,_0xc4fb6f);},Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x2cf)]=function(){const _0x375aff=_0x45bccd;return this[_0x375aff(0x212)];},Window_ClassTier[_0x45bccd(0x404)]['itemHeight']=function(){const _0x302692=_0x45bccd;let _0x757a17=Window_ClassCommand[_0x302692(0x404)]['itemHeight']['call'](this);if(this[_0x302692(0x349)]){const _0x2cdb06=this[_0x302692(0x349)]['totalMulticlass']()||0x1;_0x757a17=Math[_0x302692(0x2b0)](_0x757a17,this['innerHeight']/_0x2cdb06);}return _0x757a17;},Window_ClassTier[_0x45bccd(0x404)]['updateHelp']=function(){const _0x258473=_0x45bccd;if(this['_helpWindow']){if('uHoBx'!==_0x258473(0x471)){function _0x4b653e(){const _0x2902c7=_0x258473;let _0x54511d=_0x107534['ClassChangeSystem'][_0x2902c7(0x48f)]['call'](this);return this[_0x2902c7(0x232)]&&(_0x54511d=this['applyMulticlassObjects'](_0x54511d)),this[_0x2902c7(0x232)]=_0x476f72,_0x54511d;}}else{if(this[_0x258473(0x176)]()){if(_0x258473(0x375)!==_0x258473(0x375)){function _0x481646(){const _0x1e3091=_0x258473;this[_0x1e3091(0x2b5)](...arguments);}}else{const _0x4d4490=VisuMZ[_0x258473(0x3ac)][_0x258473(0x38c)][_0x258473(0x2bd)];if(!_0x4d4490)return;const _0x299edf=_0x4d4490[this[_0x258473(0x176)]()-0x1];if(!_0x299edf)return;this[_0x258473(0x372)]['setText'](_0x299edf[_0x258473(0x36b)]);}}else this['_helpWindow'][_0x258473(0x183)]('');}}},Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x2b7)]=function(){const _0x4d59b5=_0x45bccd;if(!this[_0x4d59b5(0x349)])return;const _0x2130da=this[_0x4d59b5(0x349)][_0x4d59b5(0x295)](),_0x1d6fd8=VisuMZ[_0x4d59b5(0x3ac)][_0x4d59b5(0x38c)][_0x4d59b5(0x2bd)];for(let _0x28f0b2=0x0;_0x28f0b2<_0x2130da;_0x28f0b2++){const _0x4cb752=_0x1d6fd8[_0x28f0b2];if(!_0x4cb752)continue;const _0x5ba5ca=_0x4cb752[_0x4d59b5(0x433)],_0x3772a3=_0x28f0b2+0x1,_0x25f27f=this[_0x4d59b5(0x32e)](_0x3772a3);this[_0x4d59b5(0x2de)](_0x5ba5ca,_0x4d59b5(0x31f),_0x25f27f,_0x3772a3);}},Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x32e)]=function(_0x59ee82){const _0x3f4674=_0x45bccd;if(this[_0x3f4674(0x349)]['isClassChangeTierRestricted'](_0x59ee82))return![];return _0x59ee82>0x0;},Window_ClassTier['prototype'][_0x45bccd(0x240)]=function(_0x1c0b23){const _0x3eb7d6=_0x45bccd;if(!this[_0x3eb7d6(0x349)])return;const _0xea6f95=this[_0x3eb7d6(0x1cc)](_0x1c0b23),_0x3a586a=this[_0x3eb7d6(0x43a)][_0x1c0b23][_0x3eb7d6(0x2ab)]||0x1,_0x73c931=this['_actor'][_0x3eb7d6(0x2da)](_0x3a586a),_0x56d3dd=_0x73c931?_0x73c931['id']:0x0,_0x454c08=VisuMZ[_0x3eb7d6(0x3ac)]['Settings'][_0x3eb7d6(0x2bd)];if(!_0x454c08)return;const _0x2bf88b=_0x454c08[_0x3a586a-0x1];if(!_0x2bf88b)return;let _0x5d3056=_0xea6f95['x'],_0x101a54=_0xea6f95['y'],_0x17c48b=_0xea6f95['width']-this[_0x3eb7d6(0x213)]()*0x2,_0x50508f=_0xea6f95[_0x3eb7d6(0x49e)],_0x511b0c=Math['min'](_0x17c48b,_0x50508f,this[_0x3eb7d6(0x376)]()*0x3);_0x511b0c=Math['floor'](_0x511b0c/ImageManager[_0x3eb7d6(0x444)])*ImageManager[_0x3eb7d6(0x444)],_0x5d3056+=_0x511b0c+this[_0x3eb7d6(0x213)]()*0x4,this['resetFontSettings'](),this[_0x3eb7d6(0x26a)](),this[_0x3eb7d6(0x31d)](_0xea6f95),this['changePaintOpacity'](this[_0x3eb7d6(0x32e)](_0x3a586a)),this[_0x3eb7d6(0x4bd)](_0x1c0b23,_0x73c931,_0xea6f95),this[_0x3eb7d6(0x43d)](ColorManager['getColor'](_0x2bf88b[_0x3eb7d6(0x4b0)])),this[_0x3eb7d6(0x1c5)](_0x2bf88b['Name'],_0xea6f95['x'],_0xea6f95['y'],_0xea6f95[_0x3eb7d6(0x2c7)],_0x3eb7d6(0x2c9)),this[_0x3eb7d6(0x26a)]();if(!_0x73c931){if(_0x3eb7d6(0x3ee)!==_0x3eb7d6(0x1ac)){this[_0x3eb7d6(0x47a)](![]);const _0x33e3dc=Math[_0x3eb7d6(0x304)](_0xea6f95['y']+this['lineHeight']()+(_0xea6f95[_0x3eb7d6(0x49e)]-this[_0x3eb7d6(0x376)]()*0x2)/0x2);this[_0x3eb7d6(0x1c5)](TextManager['classChange_multiclass_noClass'],_0xea6f95['x'],_0x33e3dc,_0xea6f95[_0x3eb7d6(0x2c7)],_0x3eb7d6(0x2c9));return;}else{function _0x250a66(){const _0x3ce4ff=_0x3eb7d6;this[_0x3ce4ff(0x2de)]('',_0x3ce4ff(0x344),!![],null);}}}_0x101a54+=this[_0x3eb7d6(0x376)](),this['drawText'](_0x73c931[_0x3eb7d6(0x3ea)],_0x5d3056,_0x101a54,_0xea6f95[_0x3eb7d6(0x2c7)]-_0x5d3056),_0x101a54+=this[_0x3eb7d6(0x376)](),this[_0x3eb7d6(0x460)](this[_0x3eb7d6(0x349)],_0x56d3dd,_0x5d3056,_0x101a54-0x4),_0x101a54+=this[_0x3eb7d6(0x376)](),this[_0x3eb7d6(0x2ec)](_0x56d3dd,_0xea6f95),this[_0x3eb7d6(0x39f)](_0x56d3dd,_0x3a586a,_0x2bf88b,_0xea6f95);},Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x39f)]=function(){const _0x3a745c=_0x45bccd,_0x29b34f=VisuMZ['ClassChangeSystem'][_0x3a745c(0x38c)][_0x3a745c(0x239)][_0x3a745c(0x363)];if(_0x29b34f){_0x29b34f[_0x3a745c(0x2ee)](this,arguments);return;}const _0x597bbe=arguments[0x0],_0x1e5a57=arguments[0x1],_0x13754e=arguments[0x2],_0x35f663=arguments[0x3],_0xf8daf8=$dataClasses[_0x597bbe],_0x17287c=Imported[_0x3a745c(0x3d9)],_0x30452c=!![],_0x49056b=0x16;let _0x3b80d5=_0x35f663['x']+this[_0x3a745c(0x213)]()*0x4,_0x246b2f=_0x35f663['y']+this['lineHeight']()*3.25,_0x5bae64=_0x35f663[_0x3a745c(0x2c7)]-this[_0x3a745c(0x213)]()*0x8;if(_0x13754e[_0x3a745c(0x37c)]&&_0x246b2f+this[_0x3a745c(0x376)]()<=_0x35f663['y']+_0x35f663[_0x3a745c(0x49e)]){if(_0x3a745c(0x432)==='uwihl'){let _0x4e300b=_0xf8daf8[_0x3a745c(0x1bd)]['filter'](_0x559e91=>_0x559e91['code']===Game_BattlerBase[_0x3a745c(0x4b6)])[_0x3a745c(0x361)](_0x52712e=>$dataSystem['skillTypes'][_0x52712e[_0x3a745c(0x2f1)]])[_0x3a745c(0x46b)](',\x20'),_0x305be9=_0x3a745c(0x2ad)[_0x3a745c(0x3dd)](TextManager[_0x3a745c(0x17b)],_0x4e300b,_0x49056b||0x16);if(_0x30452c)_0x305be9=_0x305be9[_0x3a745c(0x203)](/\\I\[(\d+)\]/gi,'');if(_0x17287c)_0x305be9=_0x3a745c(0x1a0)+_0x305be9;this[_0x3a745c(0x245)](_0x305be9,_0x3b80d5,_0x246b2f,_0x5bae64),_0x246b2f+=this[_0x3a745c(0x376)]();}else{function _0x4d6ff7(){const _0x4e32ae=_0x3a745c,_0x5731e7=_0x32d635(_0x14c9c6['$1']);_0x27a89c[_0x4e32ae(0x3c7)](_0x5731e7);}}}if(_0x13754e[_0x3a745c(0x21a)]&&_0x246b2f+this[_0x3a745c(0x376)]()<=_0x35f663['y']+_0x35f663['height']){let _0x3f07ea=_0xf8daf8['traits'][_0x3a745c(0x287)](_0x534486=>_0x534486['code']===Game_BattlerBase[_0x3a745c(0x269)])[_0x3a745c(0x361)](_0x5f2966=>$dataSystem['weaponTypes'][_0x5f2966[_0x3a745c(0x2f1)]])['join'](',\x20'),_0x53d50f=_0x3a745c(0x2ad)[_0x3a745c(0x3dd)](TextManager[_0x3a745c(0x1e5)],_0x3f07ea,_0x49056b||0x16);if(_0x30452c)_0x53d50f=_0x53d50f[_0x3a745c(0x203)](/\\I\[(\d+)\]/gi,'');if(_0x17287c)_0x53d50f=_0x3a745c(0x1a0)+_0x53d50f;this[_0x3a745c(0x245)](_0x53d50f,_0x3b80d5,_0x246b2f,_0x5bae64),_0x246b2f+=this['lineHeight']();}if(_0x13754e[_0x3a745c(0x2f3)]&&_0x246b2f+this['lineHeight']()<=_0x35f663['y']+_0x35f663[_0x3a745c(0x49e)]){if(_0x3a745c(0x1df)!=='gtjqf'){let _0x1937ea=_0xf8daf8[_0x3a745c(0x1bd)][_0x3a745c(0x287)](_0x4381ab=>_0x4381ab['code']===Game_BattlerBase['TRAIT_EQUIP_ATYPE'])[_0x3a745c(0x361)](_0x2bd78f=>$dataSystem[_0x3a745c(0x455)][_0x2bd78f['dataId']])[_0x3a745c(0x46b)](',\x20'),_0x2a0ddc=_0x3a745c(0x2ad)[_0x3a745c(0x3dd)](TextManager[_0x3a745c(0x236)],_0x1937ea,_0x49056b||0x16);if(_0x30452c)_0x2a0ddc=_0x2a0ddc['replace'](/\\I\[(\d+)\]/gi,'');if(_0x17287c)_0x2a0ddc=_0x3a745c(0x1a0)+_0x2a0ddc;this[_0x3a745c(0x245)](_0x2a0ddc,_0x3b80d5,_0x246b2f,_0x5bae64),_0x246b2f+=this[_0x3a745c(0x376)]();}else{function _0x1d5afc(){const _0x4eff8a=_0x3a745c;this[_0x4eff8a(0x2b5)](...arguments);}}}},Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x3ef)]=function(){const _0x4427de=_0x45bccd;Window_ClassCommand['prototype'][_0x4427de(0x3ef)]['call'](this),this[_0x4427de(0x16e)]();},Window_ClassTier['prototype'][_0x45bccd(0x16e)]=function(){const _0x33886a=_0x45bccd;if(!this['isShiftRemoveShortcutEnabled']())return;if(!this[_0x33886a(0x349)])return;if(Input['isTriggered'](_0x33886a(0x440))){if(this['_actor']){if(this[_0x33886a(0x2eb)](this[_0x33886a(0x36a)]()))this[_0x33886a(0x461)](),this[_0x33886a(0x249)]();else{if(_0x33886a(0x35c)!==_0x33886a(0x35a))this[_0x33886a(0x48c)]();else{function _0x51ef0f(){return!![];}}}}}},Window_ClassTier[_0x45bccd(0x404)][_0x45bccd(0x463)]=function(){const _0x5d2071=_0x45bccd;if(!this[_0x5d2071(0x4a9)])return![];if(!VisuMZ[_0x5d2071(0x3ac)][_0x5d2071(0x38c)][_0x5d2071(0x239)][_0x5d2071(0x428)])return![];return!![];},Window_ClassTier['prototype']['canShiftRemoveClass']=function(_0x2a085f){const _0x4f18b0=_0x45bccd;if(!this[_0x4f18b0(0x349)])return;const _0x39cb7f=this[_0x4f18b0(0x36a)]()+0x1;if(_0x39cb7f<=0x1)return![];if(this['_actor']['isClassChangeTierRestricted'](_0x39cb7f))return![];if(!this['_actor'][_0x4f18b0(0x2da)](_0x39cb7f)){if('cUiPm'!=='cUiPm'){function _0x3d0763(){const _0x1bbfad=_0x4f18b0;if(this[_0x1bbfad(0x2e6)]===_0x335f3a)this[_0x1bbfad(0x3a6)]();this['_multiclassTiers']=_0xc8e85f[_0x1bbfad(0x37e)](0x1,_0x39d070['ClassChangeSystem']['Settings']['Multiclass'][_0x1bbfad(0x38a)]||0x1);}}else return![];}return!![];;},Window_ClassTier[_0x45bccd(0x404)]['processShiftRemoveShortcut']=function(){const _0x5d9841=_0x45bccd;SoundManager[_0x5d9841(0x243)](),this[_0x5d9841(0x349)]['changeMulticlass'](0x0,this[_0x5d9841(0x36a)]()+0x1),this[_0x5d9841(0x3d3)](),SceneManager[_0x5d9841(0x41e)]['_statusWindow']['refresh']();};function Window_ClassList(){this['initialize'](...arguments);}Window_ClassList[_0x45bccd(0x404)]=Object[_0x45bccd(0x1ec)](Window_ClassCommand[_0x45bccd(0x404)]),Window_ClassList[_0x45bccd(0x404)]['constructor']=Window_ClassList,Window_ClassList[_0x45bccd(0x404)]['initialize']=function(_0x1f43a3){const _0x59e9b0=_0x45bccd;this[_0x59e9b0(0x4b8)]=0x1,Window_ClassCommand[_0x59e9b0(0x404)][_0x59e9b0(0x2b5)][_0x59e9b0(0x42e)](this,_0x1f43a3);},Window_ClassList[_0x45bccd(0x404)][_0x45bccd(0x45a)]=function(){const _0x16d838=_0x45bccd;SoundManager[_0x16d838(0x243)]();},Window_ClassList[_0x45bccd(0x404)][_0x45bccd(0x383)]=function(_0x2de2cf){const _0x1d3080=_0x45bccd;this[_0x1d3080(0x303)]=_0x2de2cf,this[_0x1d3080(0x226)]();},Window_ClassList[_0x45bccd(0x404)][_0x45bccd(0x249)]=function(){const _0x2da190=_0x45bccd;if(this['_helpWindow']){if(this['currentExt']())this[_0x2da190(0x372)]['setItem'](this[_0x2da190(0x176)]());else{if(_0x2da190(0x2c0)===_0x2da190(0x2c0))this[_0x2da190(0x372)]['setText'](TextManager[_0x2da190(0x42a)]);else{function _0x18b074(){const _0x1287e7=_0x2da190;return this[_0x1287e7(0x20f)];}}}}this[_0x2da190(0x349)]&&this[_0x2da190(0x303)]&&this['updateStatusWindow']();},Window_ClassList[_0x45bccd(0x404)][_0x45bccd(0x43c)]=function(){const _0x29b08a=_0x45bccd,_0x5f13ea=this[_0x29b08a(0x176)](),_0x3add79=JsonEx[_0x29b08a(0x2d3)](this[_0x29b08a(0x349)]);_0x3add79[_0x29b08a(0x310)]=!![];if(_0x5f13ea!==this['_actor']['currentClass']()){if(_0x29b08a(0x3b5)===_0x29b08a(0x3b5))_0x5f13ea?_0x3add79[_0x29b08a(0x215)](_0x5f13ea['id'],this[_0x29b08a(0x4b8)]):_0x3add79[_0x29b08a(0x215)](0x0,this['_tier']);else{function _0x301af4(){const _0xde8974=_0x29b08a,_0x14a5ea=_0xde8974(0x241)[_0xde8974(0x3dd)](_0x313785,_0x3f2a23);_0x42bef5[_0xde8974(0x445)][_0x14a5ea]=_0xe818fe;}}}this['_statusWindow'][_0x29b08a(0x2a6)](_0x3add79);},Window_ClassList[_0x45bccd(0x404)][_0x45bccd(0x229)]=function(_0x428e18){const _0x3ff9be=_0x45bccd;this[_0x3ff9be(0x4b8)]!==_0x428e18&&(this[_0x3ff9be(0x4b8)]=_0x428e18,this[_0x3ff9be(0x3d3)]());},Window_ClassList[_0x45bccd(0x404)][_0x45bccd(0x2b7)]=function(){const _0x4e96f5=_0x45bccd;if(!this[_0x4e96f5(0x349)])return;if(this[_0x4e96f5(0x4b8)]<=0x0)return;const _0x4cba79=DataManager[_0x4e96f5(0x276)](this[_0x4e96f5(0x349)]);for(const _0x3fde0f of _0x4cba79){if('OiYaL'===_0x4e96f5(0x296)){function _0x5888c2(){const _0x14cbf9=_0x4e96f5;return _0x103699&&_0x8725a1[_0x14cbf9(0x323)][_0x14cbf9(0x264)](_0x3d303a[_0x14cbf9(0x3ac)][_0x14cbf9(0x411)][_0x14cbf9(0x37d)])?_0x20734*(_0x130a52(_0x2e05a3['$1'])*0.01):_0x6a3063;}}else{if(!_0x3fde0f)continue;const _0x233cb4=_0x3fde0f[_0x4e96f5(0x3ea)],_0x2063a0=this[_0x4e96f5(0x32e)](_0x3fde0f);this['addCommand'](_0x233cb4,_0x4e96f5(0x344),_0x2063a0,_0x3fde0f);}}this[_0x4e96f5(0x4b8)]>0x1&&this[_0x4e96f5(0x2de)]('',_0x4e96f5(0x344),!![],null);},Window_ClassList[_0x45bccd(0x404)]['isEnabled']=function(_0x226390){const _0x45c0ed=_0x45bccd;if(this[_0x45c0ed(0x349)][_0x45c0ed(0x268)](this[_0x45c0ed(0x4b8)]))return![];if(this[_0x45c0ed(0x4b8)]>0x1&&_0x226390===this[_0x45c0ed(0x349)][_0x45c0ed(0x2b3)]())return![];if(_0x226390){const _0x3924ba=this['_actor'][_0x45c0ed(0x33c)](_0x226390['id']);if(_0x3924ba>0x0&&this[_0x45c0ed(0x349)]['isClassChangeTierRestricted'](_0x3924ba))return![];const _0x37bf20=DataManager[_0x45c0ed(0x2ed)](_0x226390);if(!_0x37bf20[_0x45c0ed(0x3f5)](this[_0x45c0ed(0x4b8)]))return![];}return this[_0x45c0ed(0x4b8)]>0x0;},Window_ClassList[_0x45bccd(0x404)]['drawItem']=function(_0x1796e2){const _0x1277ff=_0x45bccd;if(!this['_actor'])return;const _0x2de2f1=this['itemRectWithPadding'](_0x1796e2),_0x4ad0c5=this[_0x1277ff(0x4b8)],_0x193c22=this['_list'][_0x1796e2]['ext'],_0x169b66=_0x193c22?_0x193c22['id']:0x0,_0x30443a=VisuMZ[_0x1277ff(0x3ac)][_0x1277ff(0x38c)]['Multiclass'];if(!_0x30443a)return;const _0x193f27=_0x30443a[_0x4ad0c5-0x1];if(!_0x193f27)return;let _0x1acd8=_0x2de2f1['x'],_0x2c1600=_0x2de2f1['y'],_0xa9c570=_0x2de2f1[_0x1277ff(0x2c7)]-this['itemPadding']()*0x2,_0x1e73da=_0x2de2f1[_0x1277ff(0x49e)],_0xb70f53=Math[_0x1277ff(0x3eb)](_0xa9c570,_0x1e73da,this[_0x1277ff(0x376)]()*0x3);_0xb70f53=Math['floor'](_0xb70f53/ImageManager[_0x1277ff(0x444)])*ImageManager['iconWidth'],_0x1acd8+=_0xb70f53+this[_0x1277ff(0x213)]()*0x4,this[_0x1277ff(0x47f)](),this['resetTextColor'](),this[_0x1277ff(0x31d)](_0x2de2f1),this['changePaintOpacity'](this['isEnabled'](_0x193c22));if(!_0x193c22){this[_0x1277ff(0x47a)](![]);const _0x57d9e7=Math[_0x1277ff(0x304)](_0x2de2f1['y']+this['lineHeight']()+(_0x2de2f1[_0x1277ff(0x49e)]-this[_0x1277ff(0x376)]()*0x2)/0x2);this[_0x1277ff(0x1c5)](TextManager[_0x1277ff(0x321)],_0x2de2f1['x'],_0x57d9e7,_0x2de2f1[_0x1277ff(0x2c7)],_0x1277ff(0x2c9));return;}this[_0x1277ff(0x4bd)](_0x1796e2,_0x193c22,_0x2de2f1);const _0x283c50=this[_0x1277ff(0x349)][_0x1277ff(0x33c)](_0x169b66);if(_0x283c50>0x0){if('yWMne'!==_0x1277ff(0x1e7)){const _0x179831=_0x30443a[_0x283c50-0x1];_0x179831&&(this[_0x1277ff(0x43d)](ColorManager[_0x1277ff(0x27a)](_0x179831[_0x1277ff(0x4b0)])),this['drawText'](_0x179831[_0x1277ff(0x433)],_0x2de2f1['x'],_0x2de2f1['y'],_0x2de2f1['width'],'center'),this[_0x1277ff(0x26a)]());}else{function _0xc8b1b(){_0x4c446a*=this['jobPointsRate']();}}}this[_0x1277ff(0x47a)](this['isEnabled'](_0x193c22)),_0x2c1600+=this[_0x1277ff(0x376)](),this['drawText'](_0x193c22[_0x1277ff(0x3ea)],_0x1acd8,_0x2c1600,_0x2de2f1['width']-_0x1acd8),_0x2c1600+=this['lineHeight'](),this[_0x1277ff(0x460)](this[_0x1277ff(0x349)],_0x169b66,_0x1acd8,_0x2c1600-0x4),_0x2c1600+=this[_0x1277ff(0x376)](),this[_0x1277ff(0x2ec)](_0x169b66,_0x2de2f1);};